import { DirectionalLight } from 'three/src/lights/DirectionalLight';
import { AmbientLight } from 'three/src/lights/AmbientLight';
import { WebGLRenderer } from 'three/src/renderers/WebGLRenderer';
import { Scene } from 'three/src/scenes/Scene';
import { Vector3 } from 'three/src/math/Vector3';
import { GridLayer } from './layer/GridLayer';
import { DemLayer } from './layer/DemLayer';
import { Map } from './core/Map';
import * as domEvent from './core/domEvent';
import { Coordinates } from './controls/Coordinates';
import { NorthArrow } from './controls/NorthArrow';
import { LayerControl } from './controls/LayerControl';
import { BasemapControl } from './controls/BasemapControl';
import { SliderControl } from './controls/SliderControl';
import { Mesh } from 'three/src/objects/Mesh';
import { SphereGeometry } from 'three/src/geometries/SphereGeometry';
import { MeshLambertMaterial } from 'three/src/materials/MeshLambertMaterial';
import * as util from './core/utilities';
import * as browser from './core/browser';
import * as domUtil from './core/domUtil';
import { PickingTool } from './clip/PickingTool';
import { ShowModal } from './components/ShowModal';

import { Selection } from './clip/Selection';
import _ from "lodash";

import '../css/page_bulma.scss'; /* style loader will import it */

class Application {

    constructor(container) {
        this.container = container;
        this.running = false;
        this.wireframeMode = false;
        this.canvas;
        this._canvasImageUrl;
        this.downloadButton;
        this.showCaps = true;

        this.objects = [];

        if (container.clientWidth && container.clientHeight) {
            this.width = container.clientWidth;
            this.height = container.clientHeight;
            this._fullWindow = false;
        } else {
            this.width = window.innerWidth;
            this.height = window.innerHeight;
            this._fullWindow = true;
        }

        // this.canvas = document.querySelector('#imgCanvas');
        // this.$topTextInput = document.querySelector('#topText');
        // this.$bottomTextInput = document.querySelector('#bottomText');
        // this.$imageInput = document.querySelector('#image');
        this.downloadButton = document.querySelector('#menu-dowload-button');
        this.menuIcon = document.querySelector('#menu-icon');
        this.navigation = document.getElementsByClassName('navigation')[0];
        // this.addEventListeners();

        let parentContainer = document.getElementById("app");
        this.dialog = new ShowModal("Help", parentContainer, { klass: "fm_about" });

        // this.dialog = new MobileDialog("Help", container, { klass: "fm_about" });
        this.aboutIcon = document.querySelector('#menu-about-icon');


        // this.createScene();
        // this.addEventListeners();
    }
    async build() {
        await this.createScene();
        this.addEventListeners();
    }

    async createScene() {

        var dirNode = document.getElementsByTagName("body")[0];
        if (browser.touch == true && browser.mobile == true) {
            //dirNode.setAttribute("dir", "ltr");
            domUtil.addClass(dirNode, "touch");
        } else {
            domUtil.addClass(dirNode, "notouch");
        }

        // let opt = { r: 200, c: 0x38eeff, o: 0.8 };
        var opt = { r: 5, c: 0xffff00, o: 1 };
        this.queryMarker = new Mesh(new SphereGeometry(opt.r),
            new MeshLambertMaterial({ color: opt.c, opacity: opt.o, transparent: (opt.o < 1) }));

        this.queryMarker.visible = true;
        // this.queryMarker.position.set(4282010, 2302070, -13616.3);
        /* Renderer */
        // let bgcolor = 0xfdfdfd;
        this.renderer = new WebGLRenderer({ alpha: true, antialias: true, preserveDrawingBuffer: true });
        this.renderer.setPixelRatio(window.devicePixelRatio);
        this.renderer.setSize(this.width, this.height);
        this.renderer.autoClear = false;
        this.renderer.setClearColor(0x000000, 0.0); // second param is opacity, 0 => transparent 

        // enable clipping
        // let Empty = Object.freeze([]);
        // this.renderer.clippingPlanes = Empty; // GUI sets it to globalPlanes
        this.renderer.localClippingEnabled = true;
        this.container.appendChild(this.renderer.domElement);

        /* Scene: that will hold all our elements such as objects, cameras and lights. */
        this.scene = new Scene();
        this.capsScene = new Scene();
        // this.backStencil = new Scene();
        // this.frontStencil = new Scene();
        this._buildDefaultLights(this.scene);
        //app.scene.autoUpdate = false;
        //// show axes in the screen
        //app.scene.add(new THREE.AxisHelper(100));
        this.scene.add(this.queryMarker);

        /* Camera */
        var angle = 45;
        var aspect = this.width / this.height;
        var near = 0.1; //This is the distance at which the camera will start rendering scene objects
        var far = 2000; //Anything beyond this distance will not be rendered                
        // this.camera = new PerspectiveCamera(angle, aspect, near, far);
        // this.camera.position.set(0, -0.1, 150);
        // this.camera.lookAt(new Vector3(0, 0, 0));

        // this.camera = new PerspectiveCamera(30, this.width / this.height, 100, 100000);
        // let x = { min: 4415940, max: 4508490, avg: 4463830 };       
        // let y = { min: 2350280, max: 2475820, avg: 2412360 };       
        // let z = { min: -8798.15, max: 1401.92, avg: -177.74 };
        // const center = new Vector3((x.min + x.max) / 2, (y.min + y.max) / 2, 0);       
        // const size = Math.max(x.max - x.min, y.max - y.min, z.max - z.min);
        // let baseExtent = {
        //     x: x,
        //     y: y
        // };
        // const camDirection = new Vector3(-0.5, -Math.SQRT1_2, 0.5);
        // // const camDirection = new Vector3(0, 0, 1);
        // const camOffset = camDirection.multiplyScalar(size * 2);
        // this.camera.position.copy(center);
        // this.camera.position.add(camOffset);
        // this.camera.near = size * 0.1;
        // this.camera.far = size * 25;
        // this.camera.updateProjectionMatrix();


        /* Camera */
        // // const center = new Vector3();
        // var angle = 45;
        // var aspect = this.width / this.height;
        // var near = 0.1; //This is the distance at which the camera will start rendering scene objects
        // var far = 2000; //Anything beyond this distance will not be rendered                
        // this.camera = new PerspectiveCamera(angle, aspect, near, far);        
        // this.camera.position.set(0, -0.1, 150);
        // this.camera.lookAt(new Vector3(0, 0, 0));

        let map = this.map = await Map.build(this.scene, this.container, 'https://geusegdi01.geus.dk/meta3d/rpc/model_meta_all?modelid=20');
        this.mapTitle = document.querySelector('#map-title');
        this.mapTitle.innerHTML += map.title;
        map.on('ready', () => {
            this.selectionBox.setUniforms();
            this.animate();
        }, this);

        this.selectionBox = new Selection(
            // new Vector3(-7, -14, -14),
            // new Vector3(14, 9, 3)
            { name: 'Slicing Box' },
            new Vector3(this.map.x.min, this.map.y.min, this.map.z.min),
            new Vector3(this.map.x.max, this.map.y.max, this.map.z.max)
        );
        this.map.addLayer(this.selectionBox);

        this.map.picking = new PickingTool(this.map.size, this.map.center, this);


        // let boxLayer = new BoxLayer({ 
        //     width: 10000, height: 10000, depth: 10000, name: 'center-box', color: 800080 , center: center
        // });
        // this.map.addLayer(boxLayer);

        //add map controls:
        if (util.hasTouch() == false) {
            let coordinates = new Coordinates({ camera: this.map.camera, crs: "EPSG:3034" }).addTo(this.map);
            // coordinates.addListener('onPoint', (args) => {
            //     let vector = args[0];
            //     this.queryMarker.position.set(vector.x, vector.y, vector.z);
            //     // this.queryMarker.updateMatrixWorld();
            //     this.animate();
            // }, this);
        }
        this.northArrow = new NorthArrow({ headLength: 1, headWidth: 1 }).addTo(this.map);

        let demLayer = new DemLayer({
            q: 0, shading: true, type: 'dem', name: 'DEM Layer', color: 16382457, "baseExtent": this.map.baseExtent,
            "materialParameter": [{
                "i": 0,
                "materialtypee": 0,
                "ds": 1,
                "bottomZ": 3000,
                "o": 0.7
            }]
        });
        demLayer.addBlock({
            "width": 299, //267,
            "plane": {
                "width": this.map.x.max - this.map.x.min, //100.0,
                // "offsetX": 0,
                // "offsetY": 0,
                // "offsetX" : x.avg,
                // "offsetY" : y.avg,
                "offsetX": this.map.center.x,
                "offsetY": this.map.center.y,
                "height": this.map.y.max - this.map.y.min, //78.59618717504333
            },
            "dem_values": [758.75,
                752.05,
                742.79,
                732.38,
                721.94,
                711.48,
                701.0,
                690.49,
                679.95,
                669.39,
                658.8,
                648.19,
                637.56,
                626.9,
                616.21,
                605.5,
                594.77,
                584.01,
                573.22,
                562.41,
                551.57,
                545.5,
                542.66,
                539.83,
                537.0,
                534.18,
                531.37,
                528.56,
                525.76,
                522.96,
                520.17,
                517.38,
                514.6,
                511.83,
                509.06,
                506.3,
                503.54,
                500.79,
                498.05,
                495.31,
                491.92,
                487.87,
                483.83,
                479.8,
                475.79,
                471.78,
                467.79,
                463.8,
                459.83,
                455.87,
                451.91,
                447.97,
                444.04,
                440.12,
                436.21,
                432.31,
                428.42,
                424.55,
                420.68,
                418.37,
                418.46,
                418.55,
                418.64,
                418.71,
                418.78,
                418.85,
                418.9,
                418.96,
                419.0,
                419.04,
                419.07,
                419.1,
                419.12,
                419.14,
                419.15,
                419.15,
                419.14,
                419.13,
                418.83,
                417.8,
                416.78,
                415.75,
                414.73,
                413.7,
                412.68,
                411.66,
                410.64,
                409.62,
                408.6,
                407.58,
                406.57,
                405.55,
                404.54,
                403.52,
                402.51,
                401.5,
                400.49,
                399.42,
                398.05,
                396.68,
                395.32,
                393.95,
                392.59,
                391.24,
                389.88,
                388.53,
                387.19,
                385.84,
                384.5,
                383.16,
                381.83,
                380.49,
                379.16,
                377.84,
                376.51,
                375.19,
                374.07,
                376.22,
                378.38,
                380.55,
                382.74,
                384.95,
                387.17,
                389.4,
                391.65,
                393.91,
                396.19,
                398.48,
                400.78,
                403.1,
                405.44,
                407.79,
                410.15,
                412.53,
                414.92,
                417.33,
                420.53,
                423.79,
                427.05,
                430.31,
                433.58,
                436.84,
                440.11,
                443.38,
                446.66,
                449.93,
                453.21,
                456.48,
                459.76,
                463.05,
                466.33,
                469.62,
                472.9,
                476.33,
                479.76,
                479.35,
                478.04,
                476.7,
                475.35,
                473.97,
                472.57,
                471.15,
                469.71,
                468.25,
                466.76,
                465.25,
                463.72,
                462.17,
                460.6,
                459.0,
                457.38,
                455.75,
                454.08,
                452.4,
                449.47,
                445.98,
                442.51,
                439.03,
                435.57,
                432.11,
                428.66,
                425.21,
                421.77,
                418.34,
                414.91,
                411.49,
                408.08,
                404.67,
                401.27,
                397.88,
                394.49,
                391.11,
                387.74,
                385.27,
                383.53,
                381.78,
                380.03,
                378.28,
                376.52,
                374.77,
                373.01,
                371.26,
                369.5,
                367.74,
                365.98,
                364.22,
                362.45,
                360.69,
                358.92,
                357.16,
                355.39,
                353.62,
                352.52,
                352.31,
                352.09,
                351.86,
                351.6,
                351.32,
                351.03,
                350.71,
                350.38,
                350.03,
                349.66,
                349.27,
                348.86,
                348.44,
                347.99,
                347.53,
                347.05,
                346.54,
                346.02,
                345.65,
                345.7,
                345.75,
                345.81,
                345.87,
                345.94,
                346.01,
                346.09,
                346.17,
                346.26,
                346.35,
                346.44,
                346.54,
                346.65,
                346.76,
                346.87,
                346.99,
                347.11,
                347.24,
                347.63,
                349.58,
                351.54,
                353.52,
                355.51,
                357.53,
                359.55,
                361.6,
                363.66,
                365.74,
                367.83,
                369.94,
                372.07,
                374.21,
                376.37,
                378.55,
                380.74,
                382.95,
                385.18,
                387.42,
                391.32,
                395.23,
                399.16,
                403.1,
                407.06,
                411.02,
                415.0,
                418.99,
                423.0,
                427.01,
                431.04,
                435.08,
                439.14,
                443.2,
                447.28,
                451.37,
                455.48,
                459.59,
                463.72,
                466.58,
                469.14,
                471.63,
                474.04,
                476.38,
                478.66,
                480.86,
                482.99,
                485.05,
                487.03,
                760.78,
                753.81,
                745.08,
                734.2,
                723.3,
                712.37,
                701.42,
                690.45,
                679.45,
                668.42,
                657.37,
                646.3,
                635.2,
                624.07,
                612.92,
                601.75,
                590.55,
                579.32,
                568.07,
                556.79,
                545.49,
                537.2,
                534.47,
                531.75,
                529.03,
                526.32,
                523.62,
                520.92,
                518.22,
                515.54,
                512.85,
                510.18,
                507.51,
                504.84,
                502.18,
                499.53,
                496.88,
                494.24,
                491.6,
                488.97,
                486.04,
                482.19,
                478.35,
                474.51,
                470.69,
                466.88,
                463.09,
                459.3,
                455.52,
                451.75,
                447.99,
                444.25,
                440.51,
                436.79,
                433.08,
                429.37,
                425.68,
                422.0,
                418.33,
                415.18,
                415.17,
                415.15,
                415.12,
                415.09,
                415.05,
                415.01,
                414.96,
                414.9,
                414.84,
                414.77,
                414.69,
                414.61,
                414.52,
                414.43,
                414.33,
                414.22,
                414.11,
                413.99,
                413.84,
                412.84,
                411.83,
                410.83,
                409.83,
                408.82,
                407.82,
                406.82,
                405.82,
                404.83,
                403.83,
                402.83,
                401.84,
                400.85,
                399.85,
                398.86,
                397.87,
                396.89,
                395.9,
                394.91,
                393.62,
                392.31,
                391.0,
                389.69,
                388.38,
                387.08,
                385.78,
                384.49,
                383.19,
                381.9,
                380.62,
                379.33,
                378.05,
                376.77,
                375.5,
                374.23,
                372.96,
                371.69,
                370.43,
                372.08,
                374.53,
                377.19,
                379.88,
                382.61,
                385.37,
                388.17,
                391.01,
                393.88,
                396.79,
                399.74,
                402.73,
                405.75,
                408.8,
                411.9,
                415.03,
                418.2,
                421.4,
                424.64,
                428.04,
                431.47,
                434.88,
                438.26,
                441.62,
                444.95,
                448.26,
                451.54,
                454.8,
                458.03,
                461.24,
                464.42,
                467.58,
                470.71,
                473.81,
                476.89,
                479.95,
                482.98,
                485.98,
                486.42,
                484.75,
                483.04,
                481.32,
                479.58,
                477.81,
                476.02,
                474.21,
                472.38,
                470.53,
                468.66,
                466.76,
                464.84,
                462.9,
                460.94,
                458.95,
                456.95,
                454.92,
                452.87,
                450.25,
                446.88,
                443.51,
                440.14,
                436.79,
                433.43,
                430.09,
                426.75,
                423.42,
                420.1,
                416.78,
                413.47,
                410.17,
                406.87,
                403.58,
                400.29,
                397.01,
                393.74,
                390.48,
                387.64,
                385.87,
                384.1,
                382.33,
                380.56,
                378.78,
                377.01,
                375.23,
                373.45,
                371.67,
                369.89,
                368.11,
                366.33,
                364.54,
                362.76,
                360.97,
                359.18,
                357.39,
                355.6,
                354.0,
                353.49,
                352.97,
                352.43,
                351.87,
                351.29,
                350.69,
                350.08,
                349.44,
                348.79,
                348.12,
                347.43,
                346.72,
                345.99,
                345.24,
                344.48,
                343.69,
                342.89,
                342.06,
                341.23,
                341.35,
                341.48,
                341.61,
                341.75,
                341.89,
                342.04,
                342.19,
                342.35,
                342.51,
                342.68,
                342.85,
                343.02,
                343.2,
                343.39,
                343.58,
                343.77,
                343.97,
                344.17,
                344.38,
                346.3,
                348.52,
                350.76,
                353.02,
                355.29,
                357.57,
                359.88,
                362.2,
                364.53,
                366.88,
                369.25,
                371.64,
                374.04,
                376.46,
                378.89,
                381.34,
                383.81,
                386.3,
                388.8,
                392.43,
                396.54,
                400.66,
                404.8,
                408.94,
                413.1,
                417.28,
                421.46,
                425.66,
                429.87,
                434.09,
                438.32,
                442.57,
                446.83,
                451.1,
                455.39,
                459.68,
                463.99,
                468.32,
                471.09,
                472.55,
                473.94,
                475.25,
                476.5,
                477.67,
                478.77,
                479.8,
                480.76,
                481.65,
                762.94,
                755.7,
                747.59,
                736.25,
                724.88,
                713.49,
                702.07,
                690.63,
                679.17,
                667.68,
                656.16,
                644.62,
                633.06,
                621.47,
                609.86,
                598.22,
                586.55,
                574.86,
                563.15,
                551.41,
                539.64,
                528.85,
                526.23,
                523.62,
                521.01,
                518.41,
                515.81,
                513.22,
                510.64,
                508.06,
                505.48,
                502.92,
                500.35,
                497.8,
                495.25,
                492.7,
                490.16,
                487.63,
                485.1,
                482.58,
                480.06,
                476.41,
                472.76,
                469.13,
                465.5,
                461.89,
                458.29,
                454.69,
                451.11,
                447.54,
                443.98,
                440.43,
                436.89,
                433.36,
                429.84,
                426.33,
                422.84,
                419.35,
                415.88,
                412.41,
                411.92,
                411.79,
                411.66,
                411.52,
                411.37,
                411.22,
                411.06,
                410.9,
                410.73,
                410.55,
                410.37,
                410.18,
                409.98,
                409.78,
                409.57,
                409.36,
                409.14,
                408.91,
                408.68,
                407.86,
                406.88,
                405.89,
                404.91,
                403.98,
                403.09,
                402.2,
                401.32,
                400.44,
                399.56,
                398.69,
                397.82,
                396.96,
                396.1,
                395.24,
                394.39,
                393.54,
                392.7,
                391.85,
                390.85,
                389.78,
                388.72,
                387.67,
                386.63,
                385.6,
                384.59,
                383.59,
                382.6,
                381.63,
                380.66,
                379.71,
                378.77,
                377.85,
                376.93,
                376.03,
                375.15,
                374.27,
                373.41,
                374.73,
                377.97,
                381.26,
                384.57,
                387.93,
                391.32,
                394.75,
                398.21,
                401.71,
                405.25,
                408.82,
                412.44,
                416.08,
                419.77,
                423.49,
                427.25,
                431.04,
                434.87,
                438.74,
                442.28,
                445.28,
                448.25,
                451.2,
                454.13,
                457.03,
                459.9,
                462.75,
                465.58,
                468.38,
                471.15,
                473.9,
                476.62,
                479.32,
                481.99,
                484.64,
                487.26,
                489.86,
                492.43,
                493.7,
                491.65,
                489.58,
                487.5,
                485.38,
                483.25,
                481.1,
                478.92,
                476.72,
                474.5,
                472.26,
                469.99,
                467.71,
                465.4,
                463.07,
                460.72,
                458.35,
                455.95,
                453.53,
                450.98,
                447.71,
                444.45,
                441.19,
                437.94,
                434.7,
                431.47,
                428.24,
                425.01,
                421.8,
                418.59,
                415.39,
                412.19,
                409.0,
                405.82,
                402.64,
                399.47,
                396.31,
                393.16,
                390.02,
                388.23,
                386.43,
                384.64,
                382.85,
                381.05,
                379.25,
                377.46,
                375.66,
                373.86,
                372.05,
                370.25,
                368.45,
                366.64,
                364.83,
                363.03,
                361.22,
                359.41,
                357.59,
                355.78,
                354.84,
                354.01,
                353.17,
                352.31,
                351.43,
                350.53,
                349.61,
                348.68,
                347.72,
                346.75,
                345.75,
                344.74,
                343.71,
                342.66,
                341.59,
                340.51,
                339.4,
                338.28,
                337.13,
                336.96,
                337.16,
                337.37,
                337.58,
                337.8,
                338.02,
                338.25,
                338.48,
                338.72,
                338.96,
                339.21,
                339.46,
                339.71,
                339.98,
                340.24,
                340.51,
                340.78,
                341.06,
                341.35,
                342.88,
                345.36,
                347.86,
                350.37,
                352.9,
                355.44,
                358.0,
                360.58,
                363.18,
                365.79,
                368.41,
                371.06,
                373.72,
                376.39,
                379.09,
                381.8,
                384.52,
                387.26,
                390.02,
                393.43,
                397.73,
                402.05,
                406.38,
                410.72,
                415.07,
                419.44,
                423.81,
                428.2,
                432.61,
                437.02,
                441.45,
                445.89,
                450.34,
                454.81,
                459.29,
                463.78,
                468.28,
                472.79,
                476.25,
                476.61,
                476.89,
                477.11,
                477.26,
                477.33,
                477.34,
                477.27,
                477.13,
                476.92,
                765.23,
                757.72,
                750.21,
                738.52,
                726.69,
                714.83,
                702.95,
                691.05,
                679.12,
                667.16,
                655.18,
                643.18,
                631.15,
                619.1,
                607.02,
                594.91,
                582.79,
                570.63,
                558.45,
                546.25,
                534.02,
                521.76,
                517.94,
                515.43,
                512.93,
                510.44,
                507.95,
                505.47,
                503.0,
                500.53,
                498.06,
                495.6,
                493.15,
                490.7,
                488.26,
                485.82,
                483.39,
                480.97,
                478.55,
                476.14,
                473.73,
                470.53,
                467.08,
                463.64,
                460.21,
                456.8,
                453.39,
                450.14,
                446.95,
                443.76,
                440.57,
                437.36,
                434.15,
                430.93,
                427.71,
                424.48,
                421.24,
                417.99,
                414.74,
                411.48,
                410.21,
                410.06,
                409.9,
                409.73,
                409.55,
                409.36,
                409.17,
                408.96,
                408.75,
                408.53,
                408.3,
                408.06,
                407.81,
                407.55,
                407.28,
                407.0,
                406.71,
                406.42,
                406.11,
                405.53,
                404.69,
                403.85,
                403.02,
                402.19,
                401.37,
                400.55,
                399.73,
                398.92,
                398.11,
                397.3,
                396.5,
                395.7,
                394.9,
                394.11,
                393.32,
                392.54,
                391.76,
                390.98,
                390.17,
                389.31,
                388.46,
                387.63,
                386.81,
                386.0,
                385.21,
                384.42,
                383.65,
                382.89,
                382.14,
                381.41,
                380.69,
                379.98,
                379.28,
                378.6,
                377.93,
                377.27,
                376.62,
                377.21,
                381.08,
                384.99,
                388.94,
                392.92,
                396.94,
                400.99,
                405.08,
                409.21,
                413.37,
                417.58,
                421.81,
                426.09,
                430.4,
                434.74,
                439.13,
                443.55,
                448.01,
                452.5,
                456.75,
                459.32,
                461.86,
                464.38,
                466.87,
                469.34,
                471.78,
                474.2,
                476.59,
                478.95,
                481.29,
                483.61,
                485.9,
                488.17,
                490.4,
                492.62,
                494.81,
                496.97,
                499.11,
                501.17,
                498.76,
                496.32,
                493.87,
                491.39,
                488.89,
                486.37,
                483.82,
                481.26,
                478.67,
                476.06,
                473.43,
                470.78,
                468.1,
                465.41,
                462.69,
                459.95,
                457.19,
                454.4,
                451.6,
                448.48,
                445.33,
                442.18,
                439.04,
                435.91,
                432.78,
                429.66,
                426.55,
                423.44,
                420.34,
                417.24,
                414.16,
                411.08,
                408.0,
                404.94,
                401.87,
                398.82,
                395.77,
                392.73,
                390.59,
                388.78,
                386.97,
                385.15,
                383.33,
                381.51,
                379.7,
                377.87,
                376.05,
                374.23,
                372.41,
                370.58,
                368.75,
                366.92,
                365.1,
                363.26,
                361.43,
                359.6,
                357.76,
                356.35,
                355.23,
                354.08,
                352.92,
                351.74,
                350.54,
                349.32,
                348.08,
                346.82,
                345.54,
                344.25,
                342.94,
                341.6,
                340.25,
                338.88,
                337.5,
                336.09,
                334.66,
                333.22,
                332.52,
                332.8,
                333.08,
                333.37,
                333.67,
                333.97,
                334.27,
                334.58,
                334.89,
                335.2,
                335.53,
                335.85,
                336.18,
                336.52,
                336.86,
                337.2,
                337.55,
                337.91,
                338.27,
                339.31,
                342.05,
                344.8,
                347.57,
                350.36,
                353.16,
                355.98,
                358.82,
                361.67,
                364.54,
                367.43,
                370.33,
                373.25,
                376.18,
                379.13,
                382.1,
                385.08,
                388.08,
                391.1,
                394.32,
                398.82,
                403.33,
                407.85,
                412.38,
                416.93,
                421.48,
                426.06,
                430.64,
                435.23,
                439.84,
                444.46,
                449.1,
                453.74,
                458.4,
                463.07,
                467.75,
                472.45,
                477.16,
                481.88,
                481.31,
                480.5,
                479.62,
                478.67,
                477.65,
                476.55,
                475.38,
                474.15,
                472.84,
                767.65,
                759.88,
                752.09,
                741.01,
                728.72,
                716.4,
                704.05,
                691.68,
                679.29,
                666.87,
                654.22,
                641.56,
                628.95,
                616.39,
                603.88,
                591.43,
                579.02,
                566.67,
                554.37,
                542.12,
                529.92,
                517.77,
                511.84,
                509.7,
                507.57,
                505.45,
                503.35,
                501.27,
                499.21,
                497.16,
                495.13,
                493.11,
                491.11,
                489.13,
                487.17,
                485.22,
                483.29,
                481.37,
                479.47,
                477.59,
                475.72,
                473.15,
                469.88,
                466.61,
                463.32,
                460.03,
                456.73,
                453.43,
                450.12,
                446.8,
                443.47,
                440.14,
                436.8,
                433.45,
                430.09,
                426.73,
                423.36,
                419.99,
                416.6,
                413.21,
                411.02,
                410.71,
                410.38,
                410.05,
                409.71,
                409.36,
                409.01,
                408.64,
                408.26,
                407.88,
                407.48,
                407.08,
                406.67,
                406.25,
                405.82,
                405.38,
                404.93,
                404.47,
                404.0,
                403.45,
                402.67,
                401.9,
                401.14,
                400.37,
                399.61,
                398.86,
                398.11,
                397.36,
                396.61,
                395.87,
                395.13,
                394.4,
                393.67,
                392.94,
                392.22,
                391.5,
                390.78,
                390.07,
                389.37,
                388.73,
                388.1,
                387.49,
                386.88,
                386.29,
                385.71,
                385.14,
                384.59,
                384.04,
                383.51,
                383.0,
                382.49,
                382.0,
                381.52,
                381.05,
                380.6,
                380.15,
                379.72,
                379.37,
                383.87,
                388.4,
                392.97,
                397.58,
                402.23,
                406.91,
                411.62,
                416.38,
                421.17,
                425.99,
                430.86,
                435.76,
                440.7,
                445.67,
                450.68,
                455.73,
                460.81,
                465.93,
                471.09,
                473.59,
                475.7,
                477.78,
                479.84,
                481.88,
                483.89,
                485.87,
                487.83,
                489.76,
                491.67,
                493.55,
                495.41,
                497.24,
                499.05,
                500.83,
                502.59,
                504.32,
                506.02,
                507.7,
                506.06,
                503.26,
                500.44,
                497.59,
                494.72,
                491.84,
                488.93,
                485.99,
                483.04,
                480.06,
                477.07,
                474.05,
                471.0,
                467.94,
                464.86,
                461.75,
                458.62,
                455.47,
                452.3,
                449.19,
                446.15,
                443.11,
                440.08,
                437.05,
                434.04,
                431.02,
                428.02,
                425.02,
                422.03,
                419.04,
                416.06,
                413.09,
                410.13,
                407.17,
                404.21,
                401.27,
                398.33,
                395.4,
                392.97,
                391.14,
                389.3,
                387.46,
                385.63,
                383.79,
                381.95,
                380.1,
                378.26,
                376.42,
                374.57,
                372.72,
                370.88,
                369.03,
                367.18,
                365.32,
                363.47,
                361.62,
                359.76,
                358.04,
                356.61,
                355.16,
                353.7,
                352.22,
                350.71,
                349.19,
                347.65,
                346.09,
                344.51,
                342.92,
                341.3,
                339.67,
                338.02,
                336.34,
                334.65,
                332.95,
                331.22,
                329.47,
                328.04,
                328.4,
                328.76,
                329.12,
                329.49,
                329.86,
                330.24,
                330.62,
                331.01,
                331.4,
                331.8,
                332.2,
                332.61,
                333.02,
                333.44,
                333.86,
                334.28,
                334.71,
                335.15,
                335.59,
                338.59,
                341.6,
                344.63,
                347.67,
                350.73,
                353.81,
                356.91,
                360.02,
                363.14,
                366.29,
                369.45,
                372.62,
                375.82,
                379.02,
                382.25,
                385.49,
                388.75,
                392.03,
                395.32,
                399.79,
                404.49,
                409.2,
                413.93,
                418.67,
                423.42,
                428.18,
                432.96,
                437.75,
                442.55,
                447.36,
                452.19,
                457.03,
                461.88,
                466.74,
                471.62,
                476.51,
                481.41,
                486.32,
                486.66,
                484.76,
                482.78,
                480.73,
                478.61,
                476.41,
                474.15,
                471.82,
                469.41,
                752.33,
                744.52,
                736.75,
                726.87,
                714.82,
                702.81,
                690.85,
                678.95,
                667.09,
                655.29,
                643.53,
                631.83,
                620.18,
                608.57,
                597.02,
                585.53,
                574.08,
                562.68,
                551.34,
                540.04,
                528.8,
                517.61,
                509.97,
                508.12,
                506.3,
                504.49,
                502.69,
                500.92,
                499.15,
                497.41,
                495.68,
                493.97,
                492.28,
                490.6,
                488.94,
                487.29,
                485.66,
                484.05,
                482.45,
                480.87,
                479.31,
                477.29,
                473.89,
                470.48,
                467.07,
                463.65,
                460.22,
                456.78,
                453.34,
                449.89,
                446.44,
                442.98,
                439.51,
                436.03,
                432.54,
                429.05,
                425.56,
                422.05,
                418.54,
                415.02,
                411.91,
                411.44,
                410.95,
                410.46,
                409.96,
                409.45,
                408.93,
                408.4,
                407.86,
                407.31,
                406.76,
                406.19,
                405.61,
                405.03,
                404.44,
                403.84,
                403.22,
                402.6,
                401.98,
                401.34,
                400.63,
                399.92,
                399.22,
                398.52,
                397.83,
                397.14,
                396.45,
                395.77,
                395.09,
                394.41,
                393.74,
                393.07,
                392.4,
                391.74,
                391.08,
                390.43,
                389.78,
                389.13,
                388.49,
                388.04,
                387.63,
                387.23,
                386.84,
                386.46,
                386.1,
                385.75,
                385.41,
                385.08,
                384.77,
                384.47,
                384.18,
                383.9,
                383.64,
                383.39,
                383.15,
                382.92,
                382.71,
                382.51,
                386.32,
                391.48,
                396.68,
                401.91,
                407.18,
                412.49,
                417.83,
                423.21,
                428.63,
                434.08,
                439.57,
                445.1,
                450.66,
                456.26,
                461.9,
                467.57,
                473.28,
                479.03,
                484.81,
                488.09,
                489.77,
                491.42,
                493.05,
                494.65,
                496.23,
                497.78,
                499.3,
                500.8,
                502.28,
                503.73,
                505.15,
                506.55,
                507.93,
                509.28,
                510.6,
                511.9,
                513.17,
                514.42,
                513.56,
                510.4,
                507.21,
                503.99,
                500.76,
                497.5,
                494.23,
                490.93,
                487.61,
                484.26,
                480.9,
                477.51,
                474.11,
                470.68,
                467.22,
                463.75,
                460.26,
                456.74,
                453.2,
                449.84,
                446.91,
                443.98,
                441.06,
                438.14,
                435.23,
                432.33,
                429.43,
                426.54,
                423.66,
                420.78,
                417.91,
                415.05,
                412.19,
                409.34,
                406.5,
                403.66,
                400.83,
                398.0,
                395.36,
                393.51,
                391.65,
                389.79,
                387.93,
                386.07,
                384.21,
                382.35,
                380.48,
                378.62,
                376.75,
                374.88,
                373.01,
                371.14,
                369.27,
                367.4,
                365.52,
                363.65,
                361.77,
                359.9,
                358.17,
                356.42,
                354.65,
                352.86,
                351.06,
                349.24,
                347.39,
                345.53,
                343.65,
                341.76,
                339.84,
                337.9,
                335.95,
                333.98,
                331.98,
                329.97,
                327.94,
                325.9,
                323.83,
                323.95,
                324.38,
                324.82,
                325.27,
                325.72,
                326.17,
                326.63,
                327.09,
                327.56,
                328.03,
                328.51,
                328.99,
                329.48,
                329.97,
                330.47,
                330.97,
                331.47,
                331.98,
                332.5,
                334.98,
                338.25,
                341.53,
                344.84,
                348.16,
                351.49,
                354.85,
                358.21,
                361.6,
                365.0,
                368.42,
                371.85,
                375.3,
                378.77,
                382.25,
                385.75,
                389.27,
                392.8,
                396.35,
                400.64,
                405.54,
                410.45,
                415.37,
                420.3,
                425.24,
                430.2,
                435.17,
                440.15,
                445.14,
                450.15,
                455.17,
                460.2,
                465.25,
                470.3,
                475.37,
                480.45,
                485.55,
                490.66,
                492.66,
                489.66,
                486.58,
                483.43,
                480.21,
                476.93,
                473.56,
                470.13,
                466.63,
                730.96,
                723.96,
                717.01,
                709.03,
                697.93,
                686.88,
                675.88,
                664.93,
                654.03,
                643.18,
                632.38,
                621.63,
                610.94,
                600.29,
                589.7,
                579.16,
                568.66,
                558.22,
                547.83,
                537.5,
                527.21,
                516.97,
                507.94,
                506.4,
                504.88,
                503.37,
                501.88,
                500.41,
                498.95,
                497.51,
                496.09,
                494.68,
                493.29,
                491.91,
                490.55,
                489.21,
                487.89,
                486.58,
                485.29,
                484.01,
                482.75,
                481.48,
                477.95,
                474.42,
                470.87,
                467.33,
                463.77,
                460.2,
                456.63,
                453.06,
                449.47,
                445.88,
                442.28,
                438.67,
                435.06,
                431.44,
                427.81,
                424.18,
                420.54,
                416.89,
                413.23,
                412.25,
                411.61,
                410.95,
                410.29,
                409.61,
                408.93,
                408.24,
                407.54,
                406.83,
                406.11,
                405.38,
                404.64,
                403.9,
                403.14,
                402.38,
                401.6,
                400.82,
                400.03,
                399.23,
                398.55,
                397.91,
                397.27,
                396.64,
                396.01,
                395.38,
                394.76,
                394.14,
                393.53,
                392.92,
                392.31,
                391.7,
                391.1,
                390.51,
                389.91,
                389.32,
                388.74,
                388.16,
                387.58,
                387.24,
                387.04,
                386.85,
                386.68,
                386.52,
                386.38,
                386.24,
                386.12,
                386.01,
                385.91,
                385.83,
                385.76,
                385.7,
                385.65,
                385.61,
                385.59,
                385.58,
                385.58,
                385.6,
                388.44,
                394.23,
                400.05,
                405.91,
                411.81,
                417.74,
                423.71,
                429.72,
                435.76,
                441.84,
                447.95,
                454.11,
                460.3,
                466.52,
                472.78,
                479.08,
                485.42,
                491.79,
                498.2,
                502.83,
                504.07,
                505.29,
                506.48,
                507.65,
                508.8,
                509.91,
                511.01,
                512.08,
                513.12,
                514.14,
                515.13,
                516.1,
                517.04,
                517.96,
                518.85,
                519.71,
                520.55,
                521.37,
                521.26,
                517.73,
                514.17,
                510.59,
                506.99,
                503.37,
                499.73,
                496.06,
                492.37,
                488.67,
                484.93,
                481.18,
                477.41,
                473.61,
                469.79,
                465.95,
                462.09,
                458.21,
                454.3,
                450.44,
                447.61,
                444.79,
                441.98,
                439.17,
                436.37,
                433.57,
                430.78,
                428.0,
                425.23,
                422.46,
                419.7,
                416.94,
                414.19,
                411.45,
                408.72,
                405.99,
                403.26,
                400.55,
                397.84,
                395.89,
                394.01,
                392.13,
                390.25,
                388.37,
                386.48,
                384.6,
                382.71,
                380.83,
                378.94,
                377.05,
                375.16,
                373.27,
                371.37,
                369.48,
                367.58,
                365.69,
                363.79,
                361.89,
                359.89,
                357.84,
                355.77,
                353.68,
                351.57,
                349.45,
                347.31,
                345.14,
                342.96,
                340.76,
                338.55,
                336.31,
                334.05,
                331.78,
                329.48,
                327.17,
                324.84,
                322.49,
                320.12,
                319.46,
                319.97,
                320.49,
                321.01,
                321.53,
                322.06,
                322.59,
                323.13,
                323.67,
                324.22,
                324.77,
                325.33,
                325.89,
                326.46,
                327.03,
                327.61,
                328.19,
                328.77,
                329.36,
                331.22,
                334.75,
                338.29,
                341.85,
                345.43,
                349.02,
                352.63,
                356.26,
                359.9,
                363.56,
                367.24,
                370.93,
                374.64,
                378.36,
                382.11,
                385.86,
                389.64,
                393.43,
                397.23,
                401.29,
                406.23,
                411.16,
                416.07,
                420.96,
                425.83,
                430.68,
                435.51,
                440.32,
                445.11,
                449.89,
                454.64,
                459.37,
                464.08,
                468.78,
                473.45,
                478.11,
                482.74,
                487.36,
                491.2,
                487.71,
                484.26,
                480.83,
                477.43,
                474.06,
                470.72,
                467.4,
                464.12,
                709.19,
                703.01,
                696.87,
                690.73,
                680.58,
                670.48,
                660.44,
                650.45,
                640.5,
                630.61,
                620.77,
                610.98,
                601.24,
                591.55,
                581.91,
                572.32,
                562.78,
                553.3,
                543.86,
                534.48,
                525.15,
                515.87,
                506.64,
                504.53,
                503.31,
                502.11,
                500.92,
                499.75,
                498.6,
                497.46,
                496.34,
                495.24,
                494.15,
                493.08,
                492.02,
                490.98,
                489.96,
                488.96,
                487.97,
                487.0,
                486.04,
                485.1,
                482.09,
                478.42,
                474.75,
                471.07,
                467.38,
                463.69,
                459.99,
                456.28,
                452.57,
                448.85,
                445.12,
                441.38,
                437.64,
                433.89,
                430.13,
                426.37,
                422.6,
                418.82,
                415.03,
                413.15,
                412.34,
                411.52,
                410.69,
                409.86,
                409.01,
                408.16,
                407.3,
                406.42,
                405.54,
                404.65,
                403.75,
                402.85,
                401.93,
                401.0,
                400.07,
                399.12,
                398.17,
                397.21,
                396.43,
                395.86,
                395.29,
                394.72,
                394.15,
                393.59,
                393.04,
                392.48,
                391.93,
                391.39,
                390.84,
                390.31,
                389.77,
                389.24,
                388.71,
                388.19,
                387.67,
                387.15,
                386.64,
                386.32,
                386.34,
                386.37,
                386.42,
                386.47,
                386.54,
                386.62,
                386.72,
                386.82,
                386.94,
                387.08,
                387.22,
                387.38,
                387.54,
                387.73,
                387.92,
                388.13,
                388.34,
                388.58,
                390.23,
                396.64,
                403.09,
                409.58,
                416.1,
                422.66,
                429.26,
                435.89,
                442.56,
                449.26,
                456.01,
                462.78,
                469.6,
                476.45,
                483.34,
                490.26,
                497.22,
                504.22,
                511.26,
                517.79,
                518.6,
                519.39,
                520.15,
                520.88,
                521.6,
                522.28,
                522.94,
                523.58,
                524.19,
                524.78,
                525.34,
                525.87,
                526.38,
                526.87,
                527.33,
                527.76,
                528.17,
                528.55,
                528.91,
                525.26,
                521.34,
                517.39,
                513.42,
                509.44,
                505.43,
                501.39,
                497.34,
                493.26,
                489.17,
                485.05,
                480.91,
                476.74,
                472.56,
                468.35,
                464.12,
                459.87,
                455.6,
                451.31,
                448.25,
                445.54,
                442.84,
                440.14,
                437.44,
                434.76,
                432.08,
                429.4,
                426.74,
                424.08,
                421.42,
                418.78,
                416.14,
                413.5,
                410.88,
                408.26,
                405.64,
                403.03,
                400.43,
                398.28,
                396.38,
                394.48,
                392.58,
                390.68,
                388.77,
                386.87,
                384.96,
                383.05,
                381.14,
                379.23,
                377.32,
                375.41,
                373.49,
                371.58,
                369.66,
                367.74,
                365.82,
                363.9,
                361.78,
                359.43,
                357.06,
                354.67,
                352.26,
                349.83,
                347.39,
                344.93,
                342.44,
                339.94,
                337.42,
                334.88,
                332.33,
                329.75,
                327.16,
                324.54,
                321.91,
                319.26,
                316.59,
                314.93,
                315.51,
                316.1,
                316.7,
                317.3,
                317.9,
                318.72,
                319.57,
                320.4,
                321.21,
                322.0,
                322.78,
                323.54,
                324.28,
                325.0,
                325.71,
                326.4,
                327.07,
                327.72,
                328.82,
                332.55,
                336.27,
                339.99,
                343.7,
                347.4,
                351.1,
                354.8,
                358.48,
                362.16,
                365.84,
                369.51,
                373.17,
                376.82,
                380.47,
                384.12,
                387.75,
                391.38,
                395.01,
                398.63,
                403.25,
                407.87,
                412.48,
                417.07,
                421.63,
                426.18,
                430.71,
                435.22,
                439.71,
                444.18,
                448.63,
                453.06,
                457.47,
                461.86,
                466.24,
                470.59,
                474.92,
                479.23,
                483.53,
                481.93,
                478.91,
                475.91,
                472.94,
                470.0,
                467.09,
                464.21,
                461.35,
                687.03,
                681.66,
                676.34,
                671.05,
                662.77,
                653.63,
                644.54,
                635.5,
                626.51,
                617.58,
                608.69,
                599.85,
                591.07,
                582.33,
                573.65,
                565.02,
                556.44,
                547.91,
                539.43,
                531.0,
                522.62,
                514.3,
                506.02,
                502.51,
                501.59,
                500.69,
                499.81,
                498.94,
                498.09,
                497.26,
                496.44,
                495.64,
                494.86,
                494.09,
                493.34,
                492.6,
                491.89,
                491.18,
                490.5,
                489.83,
                489.18,
                488.54,
                486.28,
                482.49,
                478.69,
                474.88,
                471.06,
                467.24,
                463.41,
                459.57,
                455.73,
                451.88,
                448.02,
                444.16,
                440.29,
                436.41,
                432.52,
                428.63,
                424.73,
                420.82,
                416.9,
                414.13,
                413.15,
                412.17,
                411.19,
                410.19,
                409.18,
                408.16,
                407.14,
                406.1,
                405.06,
                404.01,
                402.95,
                401.88,
                400.8,
                399.71,
                398.61,
                397.51,
                396.39,
                395.26,
                394.29,
                393.78,
                393.27,
                392.77,
                392.27,
                391.77,
                391.28,
                390.79,
                390.31,
                389.83,
                389.35,
                388.87,
                388.4,
                387.94,
                387.47,
                387.01,
                386.56,
                386.11,
                385.66,
                385.29,
                385.53,
                385.77,
                386.04,
                386.31,
                386.59,
                386.89,
                387.2,
                387.53,
                387.86,
                388.21,
                388.57,
                388.94,
                389.33,
                389.72,
                390.13,
                390.56,
                390.99,
                391.44,
                391.9,
                398.73,
                405.81,
                412.92,
                420.07,
                427.25,
                434.48,
                441.73,
                449.03,
                456.36,
                463.73,
                471.13,
                478.57,
                486.05,
                493.56,
                501.11,
                508.7,
                516.32,
                523.98,
                531.68,
                533.36,
                533.72,
                534.04,
                534.35,
                534.63,
                534.88,
                535.11,
                535.32,
                535.49,
                535.65,
                535.78,
                535.88,
                535.96,
                536.01,
                536.04,
                536.04,
                536.01,
                535.96,
                535.89,
                532.99,
                528.7,
                524.39,
                520.05,
                515.7,
                511.32,
                506.92,
                502.5,
                498.06,
                493.6,
                489.11,
                484.61,
                480.08,
                475.53,
                470.95,
                466.36,
                461.74,
                457.1,
                452.44,
                448.84,
                446.23,
                443.64,
                441.05,
                438.46,
                435.88,
                433.31,
                430.75,
                428.19,
                425.64,
                423.09,
                420.55,
                418.02,
                415.5,
                412.98,
                410.46,
                407.96,
                405.46,
                402.97,
                400.69,
                398.76,
                396.84,
                394.92,
                393.0,
                391.07,
                389.14,
                387.21,
                385.29,
                383.35,
                381.42,
                379.45,
                377.41,
                375.35,
                373.28,
                371.2,
                369.11,
                367.0,
                364.88,
                362.66,
                360.09,
                357.55,
                355.04,
                352.56,
                350.12,
                347.7,
                345.31,
                342.95,
                340.62,
                338.32,
                336.06,
                333.82,
                331.61,
                329.43,
                327.29,
                325.17,
                323.08,
                321.03,
                319.09,
                319.76,
                320.41,
                321.05,
                321.67,
                322.27,
                322.85,
                323.42,
                323.97,
                324.5,
                325.01,
                325.51,
                325.99,
                326.45,
                326.89,
                327.32,
                327.73,
                328.12,
                328.49,
                328.85,
                332.03,
                335.65,
                339.28,
                342.89,
                346.5,
                350.1,
                353.69,
                357.28,
                360.87,
                364.44,
                368.01,
                371.58,
                375.14,
                378.69,
                382.23,
                385.77,
                389.31,
                392.83,
                396.35,
                400.44,
                404.76,
                409.07,
                413.35,
                417.62,
                421.86,
                426.09,
                430.29,
                434.48,
                438.65,
                442.8,
                446.93,
                451.04,
                455.13,
                459.2,
                463.25,
                467.28,
                471.29,
                475.28,
                475.89,
                473.3,
                470.74,
                468.2,
                465.69,
                463.21,
                460.75,
                458.33,
                664.48,
                659.92,
                655.41,
                650.94,
                644.49,
                636.31,
                628.18,
                620.09,
                612.06,
                604.08,
                596.15,
                588.26,
                580.43,
                572.66,
                564.93,
                557.25,
                549.62,
                542.05,
                534.52,
                527.05,
                519.63,
                512.26,
                504.94,
                500.34,
                499.73,
                499.13,
                498.55,
                497.99,
                497.44,
                496.91,
                496.4,
                495.9,
                495.42,
                494.95,
                494.51,
                494.08,
                493.66,
                493.26,
                492.88,
                492.52,
                492.17,
                491.83,
                490.54,
                486.62,
                482.69,
                478.75,
                474.81,
                470.85,
                466.9,
                462.93,
                458.96,
                454.98,
                450.99,
                447.0,
                443.0,
                438.99,
                434.97,
                430.95,
                426.92,
                422.88,
                418.84,
                415.18,
                414.05,
                412.91,
                411.76,
                410.6,
                409.43,
                408.25,
                407.06,
                405.87,
                404.66,
                403.45,
                402.22,
                400.99,
                399.75,
                398.5,
                397.24,
                395.97,
                394.69,
                393.41,
                392.11,
                391.66,
                391.22,
                390.78,
                390.35,
                389.92,
                389.49,
                389.07,
                388.65,
                388.23,
                387.82,
                387.41,
                387.0,
                386.6,
                386.2,
                385.81,
                385.42,
                385.03,
                384.65,
                384.27,
                384.6,
                385.07,
                385.54,
                386.03,
                386.53,
                387.05,
                387.58,
                388.11,
                388.67,
                389.23,
                389.81,
                390.39,
                390.99,
                391.61,
                392.23,
                392.87,
                393.52,
                394.19,
                394.86,
                400.49,
                408.19,
                415.93,
                423.71,
                431.52,
                439.36,
                447.25,
                455.17,
                463.12,
                471.11,
                479.14,
                487.21,
                495.31,
                503.45,
                511.62,
                519.84,
                528.09,
                536.37,
                544.69,
                548.35,
                548.27,
                548.17,
                548.04,
                547.89,
                547.71,
                547.51,
                547.28,
                547.03,
                546.75,
                546.45,
                546.12,
                545.76,
                545.38,
                544.98,
                544.55,
                544.09,
                543.61,
                543.11,
                540.92,
                536.26,
                531.58,
                526.88,
                522.16,
                517.42,
                512.65,
                507.87,
                503.06,
                498.23,
                493.38,
                488.51,
                483.61,
                478.69,
                473.75,
                468.79,
                463.69,
                458.51,
                453.35,
                448.83,
                446.21,
                443.61,
                441.03,
                438.47,
                435.93,
                433.4,
                430.89,
                428.4,
                425.93,
                423.48,
                421.04,
                418.63,
                416.23,
                413.85,
                411.49,
                409.15,
                406.82,
                404.52,
                402.24,
                400.13,
                398.01,
                395.87,
                393.72,
                391.56,
                389.39,
                387.2,
                385.01,
                382.79,
                380.57,
                378.34,
                376.09,
                373.83,
                371.55,
                369.27,
                366.97,
                364.66,
                362.33,
                359.99,
                357.89,
                355.84,
                353.82,
                351.82,
                349.86,
                347.93,
                346.02,
                344.15,
                342.31,
                340.5,
                338.71,
                336.96,
                335.24,
                333.55,
                331.88,
                330.25,
                328.65,
                327.08,
                325.54,
                325.45,
                325.83,
                326.18,
                326.52,
                326.84,
                327.15,
                327.43,
                327.7,
                327.95,
                328.19,
                328.4,
                328.6,
                328.78,
                328.95,
                329.09,
                329.22,
                329.33,
                329.43,
                329.5,
                331.57,
                335.1,
                338.62,
                342.14,
                345.65,
                349.15,
                352.65,
                356.14,
                359.62,
                363.1,
                366.58,
                370.04,
                373.51,
                376.96,
                380.41,
                383.85,
                387.29,
                390.72,
                394.14,
                397.8,
                401.82,
                405.83,
                409.81,
                413.77,
                417.72,
                421.64,
                425.55,
                429.43,
                433.3,
                437.14,
                440.97,
                444.78,
                448.56,
                452.33,
                456.08,
                459.81,
                463.52,
                467.21,
                469.6,
                467.44,
                465.31,
                463.2,
                461.12,
                459.07,
                457.05,
                455.05,
                641.53,
                637.79,
                634.09,
                630.43,
                625.76,
                618.53,
                611.35,
                604.22,
                597.14,
                590.11,
                583.14,
                576.21,
                569.34,
                562.51,
                555.74,
                549.02,
                542.34,
                535.72,
                529.15,
                522.63,
                516.17,
                509.75,
                503.39,
                498.02,
                497.71,
                497.42,
                497.14,
                496.88,
                496.64,
                496.41,
                496.2,
                496.01,
                495.83,
                495.67,
                495.52,
                495.4,
                495.28,
                495.19,
                495.11,
                495.05,
                495.0,
                494.97,
                494.87,
                490.81,
                486.75,
                482.69,
                478.61,
                474.53,
                470.45,
                466.35,
                462.25,
                458.14,
                454.02,
                449.9,
                445.77,
                441.63,
                437.49,
                433.34,
                429.18,
                425.01,
                420.84,
                416.66,
                415.03,
                413.73,
                412.41,
                411.09,
                409.76,
                408.42,
                407.07,
                405.71,
                404.34,
                402.97,
                401.58,
                400.19,
                398.78,
                397.37,
                395.95,
                394.52,
                393.08,
                391.63,
                390.17,
                389.51,
                389.14,
                388.76,
                388.39,
                388.03,
                387.67,
                387.31,
                386.95,
                386.6,
                386.25,
                385.91,
                385.57,
                385.23,
                384.9,
                384.57,
                384.24,
                383.92,
                383.6,
                383.29,
                383.56,
                384.25,
                384.94,
                385.64,
                386.36,
                387.09,
                387.83,
                388.59,
                389.36,
                390.14,
                390.93,
                391.73,
                392.55,
                393.38,
                394.22,
                395.08,
                395.94,
                396.82,
                397.72,
                401.92,
                410.25,
                418.61,
                427.01,
                435.45,
                443.92,
                452.43,
                460.97,
                469.55,
                478.17,
                486.82,
                495.52,
                504.24,
                513.01,
                521.81,
                530.64,
                539.52,
                548.43,
                557.38,
                563.57,
                562.75,
                561.74,
                560.71,
                559.65,
                558.57,
                557.46,
                556.33,
                555.18,
                554.01,
                552.81,
                551.59,
                550.34,
                549.07,
                547.78,
                546.46,
                545.12,
                543.75,
                542.36,
                540.44,
                535.28,
                530.13,
                524.99,
                519.86,
                514.75,
                509.65,
                504.55,
                499.47,
                494.4,
                489.34,
                484.3,
                479.26,
                474.24,
                469.23,
                464.23,
                459.24,
                454.26,
                449.3,
                444.34,
                441.96,
                439.67,
                437.39,
                435.13,
                432.89,
                430.67,
                428.46,
                426.28,
                424.11,
                421.96,
                419.82,
                417.71,
                415.61,
                413.54,
                411.48,
                409.44,
                407.42,
                405.41,
                403.43,
                401.17,
                398.85,
                396.51,
                394.15,
                391.79,
                389.41,
                387.02,
                384.62,
                382.2,
                379.77,
                377.33,
                374.88,
                372.42,
                369.94,
                367.45,
                364.94,
                362.43,
                359.9,
                357.36,
                355.42,
                353.86,
                352.32,
                350.81,
                349.33,
                347.88,
                346.47,
                345.08,
                343.72,
                342.39,
                341.1,
                339.83,
                338.59,
                337.38,
                336.21,
                335.06,
                333.94,
                332.86,
                331.8,
                331.31,
                331.4,
                331.47,
                331.53,
                331.57,
                331.6,
                331.6,
                331.59,
                331.56,
                331.52,
                331.45,
                331.37,
                331.27,
                331.16,
                331.02,
                330.87,
                330.7,
                330.52,
                330.31,
                331.16,
                334.59,
                338.02,
                341.44,
                344.85,
                348.26,
                351.66,
                355.05,
                358.44,
                361.82,
                365.2,
                368.57,
                371.93,
                375.29,
                378.64,
                381.98,
                385.32,
                388.65,
                391.98,
                395.34,
                399.06,
                402.76,
                406.44,
                410.11,
                413.75,
                417.37,
                420.97,
                424.56,
                428.12,
                431.67,
                435.19,
                438.7,
                442.18,
                445.65,
                449.1,
                452.52,
                455.93,
                459.32,
                462.69,
                461.33,
                459.63,
                457.95,
                456.3,
                454.68,
                453.09,
                451.52,
                618.19,
                615.26,
                612.38,
                609.54,
                606.56,
                600.28,
                594.06,
                587.89,
                581.76,
                575.69,
                569.67,
                563.69,
                557.77,
                551.9,
                546.08,
                540.32,
                534.6,
                528.93,
                523.32,
                517.75,
                512.24,
                506.78,
                501.36,
                496.0,
                495.55,
                495.56,
                495.58,
                495.63,
                495.69,
                495.76,
                495.86,
                495.97,
                496.09,
                496.23,
                496.39,
                496.57,
                496.76,
                496.97,
                497.19,
                497.43,
                497.69,
                497.96,
                498.25,
                495.07,
                490.89,
                486.69,
                482.49,
                478.28,
                474.06,
                469.84,
                465.6,
                461.37,
                457.12,
                452.87,
                448.61,
                444.34,
                440.07,
                435.79,
                431.5,
                427.21,
                422.9,
                418.6,
                416.09,
                414.63,
                413.15,
                411.67,
                410.17,
                408.67,
                407.16,
                405.64,
                404.11,
                402.57,
                401.02,
                399.47,
                397.9,
                396.33,
                394.74,
                393.15,
                391.55,
                389.94,
                388.32,
                387.33,
                387.02,
                386.71,
                386.41,
                386.11,
                385.81,
                385.52,
                385.23,
                384.94,
                384.66,
                384.38,
                384.1,
                383.83,
                383.56,
                383.3,
                383.04,
                382.78,
                382.53,
                382.28,
                382.42,
                383.31,
                384.22,
                385.14,
                386.33,
                387.78,
                389.23,
                390.67,
                392.11,
                393.54,
                394.97,
                396.4,
                397.82,
                399.23,
                400.64,
                402.05,
                403.45,
                404.84,
                406.24,
                408.97,
                417.57,
                426.11,
                434.6,
                443.03,
                451.41,
                459.73,
                468.0,
                476.22,
                484.38,
                492.49,
                500.54,
                508.54,
                516.48,
                524.37,
                532.2,
                539.98,
                547.7,
                555.37,
                562.72,
                561.34,
                559.93,
                558.49,
                557.04,
                555.56,
                554.05,
                552.52,
                550.97,
                549.4,
                547.8,
                546.18,
                544.53,
                542.87,
                541.17,
                539.46,
                537.72,
                535.95,
                534.16,
                532.35,
                527.8,
                522.84,
                517.9,
                512.97,
                508.05,
                503.14,
                498.24,
                493.36,
                488.48,
                483.62,
                478.77,
                473.93,
                469.1,
                464.28,
                459.48,
                454.69,
                449.9,
                445.13,
                440.37,
                437.55,
                435.56,
                433.59,
                431.63,
                429.69,
                427.77,
                425.87,
                423.98,
                422.12,
                420.27,
                418.44,
                416.63,
                414.83,
                413.06,
                411.3,
                409.56,
                407.84,
                406.14,
                404.46,
                402.33,
                399.8,
                397.25,
                394.7,
                392.13,
                389.54,
                386.95,
                384.34,
                381.72,
                379.09,
                376.45,
                373.79,
                371.12,
                368.44,
                365.74,
                363.03,
                360.31,
                357.58,
                354.84,
                352.68,
                351.6,
                350.55,
                349.52,
                348.53,
                347.57,
                346.64,
                345.73,
                344.86,
                344.02,
                343.2,
                342.42,
                341.67,
                340.95,
                340.26,
                339.59,
                338.96,
                338.36,
                337.79,
                337.32,
                337.13,
                336.93,
                336.7,
                336.47,
                336.21,
                335.94,
                335.65,
                335.34,
                335.01,
                334.67,
                334.31,
                333.93,
                333.53,
                333.12,
                332.69,
                332.24,
                331.77,
                331.29,
                330.81,
                334.15,
                337.48,
                340.8,
                344.11,
                347.42,
                350.72,
                354.02,
                357.31,
                360.6,
                363.87,
                367.15,
                370.41,
                373.67,
                376.93,
                380.17,
                383.41,
                386.65,
                389.88,
                393.1,
                396.48,
                399.88,
                403.26,
                406.62,
                409.96,
                413.28,
                416.58,
                419.86,
                423.12,
                426.36,
                429.59,
                432.79,
                435.98,
                439.14,
                442.29,
                445.41,
                448.52,
                451.6,
                454.67,
                454.96,
                453.69,
                452.44,
                451.22,
                450.03,
                448.87,
                447.74,
                594.46,
                592.34,
                590.27,
                588.24,
                586.26,
                581.58,
                576.31,
                571.09,
                565.92,
                560.8,
                555.73,
                550.71,
                545.75,
                540.83,
                535.96,
                531.15,
                526.39,
                521.67,
                517.01,
                512.4,
                507.84,
                503.33,
                498.88,
                494.47,
                493.23,
                493.55,
                493.88,
                494.22,
                494.59,
                494.97,
                495.36,
                495.77,
                496.2,
                496.65,
                497.11,
                497.59,
                498.08,
                498.59,
                499.12,
                499.66,
                500.22,
                500.8,
                501.4,
                499.4,
                495.08,
                490.76,
                486.42,
                482.08,
                477.74,
                473.39,
                469.03,
                464.66,
                460.28,
                455.9,
                451.51,
                447.12,
                442.72,
                438.31,
                433.89,
                429.47,
                425.03,
                420.6,
                417.23,
                415.61,
                413.97,
                412.32,
                410.67,
                409.0,
                407.39,
                406.05,
                404.72,
                403.41,
                402.1,
                400.81,
                399.53,
                398.26,
                397.0,
                395.76,
                394.52,
                393.3,
                392.09,
                391.24,
                391.52,
                391.81,
                392.12,
                392.43,
                392.76,
                393.11,
                393.46,
                393.83,
                394.21,
                394.6,
                395.01,
                395.43,
                395.86,
                396.3,
                396.76,
                397.22,
                397.7,
                398.2,
                398.78,
                400.16,
                401.54,
                402.91,
                404.28,
                405.65,
                407.01,
                408.37,
                409.72,
                411.07,
                412.41,
                413.75,
                415.08,
                416.41,
                417.73,
                419.05,
                420.37,
                421.68,
                422.98,
                424.28,
                431.46,
                439.06,
                446.61,
                454.1,
                461.54,
                468.92,
                476.25,
                483.52,
                490.75,
                497.91,
                505.02,
                512.08,
                519.08,
                526.03,
                532.92,
                539.76,
                546.54,
                553.27,
                559.94,
                560.13,
                558.33,
                556.49,
                554.64,
                552.76,
                550.85,
                548.93,
                546.98,
                545.01,
                543.01,
                540.99,
                538.94,
                536.87,
                534.78,
                532.67,
                530.53,
                528.37,
                526.18,
                523.97,
                520.21,
                515.45,
                510.71,
                505.97,
                501.24,
                496.53,
                491.83,
                487.14,
                482.46,
                477.79,
                473.13,
                468.49,
                463.86,
                459.23,
                454.62,
                450.03,
                445.44,
                440.86,
                436.3,
                432.98,
                431.29,
                429.61,
                427.96,
                426.32,
                424.7,
                423.1,
                421.52,
                419.96,
                418.41,
                416.88,
                415.38,
                413.88,
                412.41,
                410.96,
                409.52,
                408.1,
                406.7,
                405.32,
                403.6,
                400.86,
                398.11,
                395.35,
                392.58,
                389.79,
                386.99,
                384.18,
                381.36,
                378.52,
                375.67,
                372.81,
                369.94,
                367.05,
                364.15,
                361.24,
                358.31,
                355.38,
                352.43,
                349.67,
                349.07,
                348.5,
                347.97,
                347.46,
                346.98,
                346.53,
                346.11,
                345.73,
                345.37,
                345.04,
                344.74,
                344.47,
                344.24,
                344.03,
                343.85,
                343.71,
                343.59,
                343.5,
                343.45,
                343.02,
                342.54,
                342.04,
                341.52,
                340.98,
                340.43,
                339.86,
                339.27,
                338.66,
                338.04,
                337.4,
                336.74,
                336.06,
                335.37,
                334.66,
                333.93,
                333.18,
                332.42,
                331.64,
                333.76,
                336.99,
                340.21,
                343.43,
                346.64,
                349.85,
                353.05,
                356.24,
                359.43,
                362.61,
                365.78,
                368.95,
                372.11,
                375.27,
                378.42,
                381.56,
                384.7,
                387.83,
                390.96,
                394.07,
                397.17,
                400.24,
                403.3,
                406.34,
                409.36,
                412.36,
                415.34,
                418.3,
                421.24,
                424.16,
                427.06,
                429.95,
                432.81,
                435.65,
                438.48,
                441.28,
                444.07,
                446.83,
                448.34,
                447.5,
                446.68,
                445.9,
                445.13,
                444.4,
                443.7,
                570.34,
                569.03,
                567.78,
                566.56,
                565.38,
                562.41,
                558.1,
                553.83,
                549.61,
                545.45,
                541.33,
                537.27,
                533.26,
                529.29,
                525.38,
                521.52,
                517.71,
                513.95,
                510.24,
                506.58,
                502.98,
                499.42,
                495.92,
                492.47,
                490.77,
                491.39,
                492.02,
                492.67,
                493.34,
                494.02,
                494.72,
                495.43,
                496.2,
                496.96,
                497.72,
                498.46,
                499.19,
                499.92,
                500.63,
                501.34,
                502.03,
                502.72,
                503.39,
                502.65,
                498.35,
                494.08,
                489.84,
                485.63,
                481.45,
                477.3,
                473.18,
                469.1,
                465.04,
                461.02,
                457.03,
                453.07,
                449.14,
                445.24,
                441.37,
                437.54,
                433.73,
                429.96,
                426.55,
                425.35,
                424.16,
                422.97,
                421.81,
                420.65,
                419.5,
                418.37,
                417.25,
                416.14,
                415.04,
                413.95,
                412.88,
                411.81,
                410.76,
                409.72,
                408.7,
                407.68,
                406.68,
                405.68,
                406.15,
                406.66,
                407.18,
                407.71,
                408.26,
                408.82,
                409.39,
                409.97,
                410.57,
                411.18,
                411.8,
                412.43,
                413.08,
                413.74,
                414.41,
                415.1,
                415.79,
                416.5,
                417.23,
                418.42,
                419.72,
                421.0,
                422.29,
                423.57,
                424.84,
                426.11,
                427.37,
                428.63,
                429.89,
                431.14,
                432.39,
                433.63,
                434.87,
                436.1,
                437.33,
                438.55,
                439.77,
                440.98,
                445.85,
                452.51,
                459.11,
                465.66,
                472.16,
                478.6,
                484.99,
                491.33,
                497.61,
                503.83,
                510.0,
                516.12,
                522.18,
                528.19,
                534.14,
                540.04,
                545.88,
                551.67,
                557.4,
                559.15,
                556.94,
                554.71,
                552.45,
                550.17,
                547.87,
                545.55,
                543.2,
                540.82,
                538.43,
                536.01,
                533.57,
                531.1,
                528.61,
                526.09,
                523.56,
                520.99,
                518.41,
                515.8,
                512.52,
                507.96,
                503.41,
                498.86,
                494.33,
                489.82,
                485.31,
                480.81,
                476.33,
                471.85,
                467.39,
                462.94,
                458.5,
                454.08,
                449.66,
                445.26,
                440.87,
                436.49,
                432.12,
                428.24,
                426.85,
                425.48,
                424.13,
                422.79,
                421.47,
                420.18,
                418.9,
                417.63,
                416.39,
                415.17,
                413.96,
                412.77,
                411.6,
                410.45,
                409.31,
                408.2,
                407.1,
                406.02,
                404.96,
                402.04,
                399.09,
                396.12,
                393.14,
                390.15,
                387.15,
                384.13,
                381.11,
                378.07,
                375.01,
                371.95,
                368.87,
                365.78,
                362.67,
                359.56,
                356.43,
                353.29,
                350.13,
                346.97,
                346.27,
                346.19,
                346.13,
                346.11,
                346.12,
                346.15,
                346.22,
                346.32,
                346.44,
                346.6,
                346.79,
                347.01,
                347.25,
                347.53,
                347.84,
                348.18,
                348.54,
                348.94,
                349.37,
                349.07,
                348.31,
                347.53,
                346.73,
                345.91,
                345.08,
                344.23,
                343.36,
                342.47,
                341.57,
                340.65,
                339.71,
                338.76,
                337.78,
                336.79,
                335.78,
                334.76,
                333.72,
                332.65,
                333.42,
                336.56,
                339.68,
                342.8,
                345.92,
                349.03,
                352.13,
                355.22,
                358.31,
                361.4,
                364.47,
                367.54,
                370.61,
                373.67,
                376.72,
                379.77,
                382.81,
                385.84,
                388.87,
                391.84,
                394.63,
                397.41,
                400.16,
                402.9,
                405.62,
                408.32,
                410.99,
                413.65,
                416.29,
                418.91,
                421.51,
                424.09,
                426.66,
                429.2,
                431.72,
                434.22,
                436.71,
                439.17,
                441.47,
                441.05,
                440.67,
                440.31,
                439.98,
                439.68,
                439.4,
                550.67,
                550.82,
                551.0,
                551.18,
                551.38,
                550.53,
                547.5,
                544.46,
                541.41,
                538.36,
                535.31,
                532.25,
                529.18,
                526.11,
                523.03,
                519.94,
                516.85,
                513.76,
                510.66,
                507.55,
                504.44,
                501.32,
                498.2,
                495.07,
                492.63,
                493.29,
                493.94,
                494.58,
                495.21,
                495.83,
                496.44,
                497.04,
                497.63,
                498.21,
                498.78,
                499.34,
                499.89,
                500.43,
                500.96,
                501.48,
                502.0,
                502.5,
                502.99,
                503.32,
                499.58,
                495.87,
                492.2,
                488.55,
                484.93,
                481.35,
                477.8,
                474.27,
                470.78,
                467.32,
                463.89,
                460.49,
                457.13,
                453.79,
                450.49,
                447.21,
                443.97,
                440.76,
                437.58,
                436.33,
                435.34,
                434.37,
                433.41,
                432.45,
                431.51,
                430.59,
                429.67,
                428.77,
                427.87,
                426.99,
                426.12,
                425.26,
                424.42,
                423.58,
                422.76,
                421.95,
                421.15,
                420.37,
                420.67,
                421.4,
                422.13,
                422.88,
                423.65,
                424.42,
                425.21,
                426.01,
                426.82,
                427.64,
                428.48,
                429.33,
                430.19,
                431.07,
                431.96,
                432.86,
                433.77,
                434.7,
                435.63,
                436.73,
                437.94,
                439.14,
                440.33,
                441.53,
                442.71,
                443.9,
                445.08,
                446.25,
                447.42,
                448.58,
                449.74,
                450.9,
                452.05,
                453.2,
                454.34,
                455.47,
                456.61,
                457.73,
                460.73,
                466.45,
                472.11,
                477.72,
                483.28,
                488.78,
                494.23,
                499.62,
                504.96,
                510.25,
                515.48,
                520.66,
                525.78,
                530.85,
                535.86,
                540.82,
                545.72,
                550.57,
                555.36,
                558.37,
                555.76,
                553.13,
                550.48,
                547.8,
                545.1,
                542.38,
                539.63,
                536.86,
                534.06,
                531.24,
                528.4,
                525.54,
                522.65,
                519.74,
                516.8,
                513.84,
                510.85,
                507.85,
                504.73,
                500.36,
                496.0,
                491.65,
                487.32,
                482.99,
                478.68,
                474.38,
                470.09,
                465.81,
                461.55,
                457.29,
                453.05,
                448.82,
                444.6,
                440.39,
                436.19,
                432.0,
                427.83,
                423.67,
                422.25,
                421.18,
                420.13,
                419.09,
                418.08,
                417.08,
                416.1,
                415.14,
                414.2,
                413.28,
                412.37,
                411.49,
                410.62,
                409.77,
                408.94,
                408.12,
                407.33,
                406.55,
                405.79,
                403.33,
                400.18,
                397.01,
                393.82,
                390.63,
                387.42,
                384.2,
                380.97,
                377.72,
                374.47,
                371.2,
                367.91,
                364.62,
                361.31,
                357.99,
                354.66,
                351.31,
                347.96,
                344.59,
                343.2,
                343.6,
                344.03,
                344.49,
                344.98,
                345.5,
                346.06,
                346.64,
                347.25,
                347.89,
                348.56,
                349.26,
                349.99,
                350.76,
                351.55,
                352.37,
                353.22,
                354.1,
                355.02,
                355.28,
                354.24,
                353.17,
                352.1,
                351.0,
                349.89,
                348.76,
                347.61,
                346.45,
                345.26,
                344.06,
                342.84,
                341.61,
                340.36,
                339.09,
                337.8,
                336.49,
                335.17,
                333.83,
                333.14,
                336.18,
                339.21,
                342.23,
                345.25,
                348.26,
                351.26,
                354.26,
                357.26,
                360.24,
                363.22,
                366.2,
                369.16,
                372.12,
                375.08,
                378.03,
                380.97,
                383.91,
                386.84,
                389.76,
                392.27,
                394.75,
                397.2,
                399.64,
                402.05,
                404.45,
                406.83,
                409.18,
                411.52,
                413.84,
                416.14,
                418.42,
                420.68,
                422.92,
                425.14,
                427.34,
                429.53,
                431.69,
                433.83,
                434.36,
                434.4,
                434.47,
                434.57,
                434.7,
                434.86,
                554.53,
                554.96,
                555.4,
                555.86,
                556.33,
                556.49,
                553.35,
                550.21,
                547.06,
                543.9,
                540.73,
                537.57,
                534.39,
                531.21,
                528.03,
                524.83,
                521.64,
                518.43,
                515.22,
                512.01,
                508.79,
                505.56,
                502.33,
                499.1,
                495.85,
                496.1,
                496.57,
                497.03,
                497.47,
                497.91,
                498.33,
                498.75,
                499.16,
                499.55,
                499.94,
                500.32,
                500.68,
                501.04,
                501.39,
                501.72,
                502.05,
                502.37,
                502.67,
                502.97,
                500.53,
                497.39,
                494.27,
                491.19,
                488.14,
                485.11,
                482.12,
                479.16,
                476.24,
                473.34,
                470.47,
                467.64,
                464.83,
                462.06,
                459.32,
                456.61,
                453.93,
                451.28,
                448.66,
                447.21,
                446.43,
                445.66,
                444.9,
                444.16,
                443.42,
                442.7,
                441.99,
                441.29,
                440.6,
                439.93,
                439.26,
                438.61,
                437.97,
                437.34,
                436.72,
                436.12,
                435.53,
                434.94,
                435.08,
                436.02,
                436.98,
                437.94,
                438.92,
                439.91,
                440.91,
                441.93,
                442.96,
                444.0,
                445.05,
                446.12,
                447.2,
                448.29,
                449.39,
                450.51,
                451.63,
                452.78,
                453.93,
                455.08,
                456.2,
                457.32,
                458.43,
                459.53,
                460.63,
                461.73,
                462.82,
                463.91,
                464.99,
                466.07,
                467.14,
                468.21,
                469.28,
                470.34,
                471.39,
                472.44,
                473.49,
                474.53,
                476.1,
                480.88,
                485.61,
                490.28,
                494.89,
                499.46,
                503.97,
                508.42,
                512.82,
                517.17,
                521.46,
                525.69,
                529.88,
                534.0,
                538.08,
                542.09,
                546.06,
                549.96,
                553.82,
                557.62,
                554.8,
                551.77,
                548.72,
                545.64,
                542.54,
                539.42,
                536.27,
                533.1,
                529.91,
                526.69,
                523.45,
                520.19,
                516.9,
                513.59,
                510.26,
                506.9,
                503.51,
                500.11,
                496.68,
                492.65,
                488.49,
                484.34,
                480.2,
                476.07,
                471.95,
                467.84,
                463.75,
                459.67,
                455.59,
                451.53,
                447.48,
                443.45,
                439.42,
                435.41,
                431.4,
                427.41,
                423.43,
                419.47,
                417.48,
                416.71,
                415.96,
                415.23,
                414.52,
                413.82,
                413.15,
                412.49,
                411.85,
                411.23,
                410.62,
                410.04,
                409.47,
                408.92,
                408.39,
                407.88,
                407.39,
                406.91,
                406.45,
                404.74,
                401.38,
                398.0,
                394.61,
                391.22,
                387.8,
                384.38,
                380.94,
                377.5,
                374.03,
                370.56,
                367.07,
                363.57,
                360.06,
                356.54,
                353.0,
                349.45,
                345.89,
                342.32,
                339.85,
                340.74,
                341.65,
                342.6,
                343.58,
                344.58,
                345.62,
                346.68,
                347.78,
                348.9,
                350.06,
                351.25,
                352.46,
                353.71,
                354.98,
                356.29,
                357.63,
                358.99,
                360.39,
                361.65,
                360.32,
                358.98,
                357.63,
                356.25,
                354.86,
                353.45,
                352.02,
                350.58,
                349.11,
                347.63,
                346.14,
                344.62,
                343.09,
                341.54,
                339.97,
                338.39,
                336.79,
                335.17,
                333.53,
                335.86,
                338.79,
                341.72,
                344.64,
                347.55,
                350.46,
                353.36,
                356.25,
                359.14,
                362.03,
                364.9,
                367.77,
                370.64,
                373.5,
                376.35,
                379.19,
                382.03,
                384.86,
                387.69,
                390.09,
                392.26,
                394.41,
                396.55,
                398.66,
                400.76,
                402.83,
                404.89,
                406.93,
                408.94,
                410.94,
                412.92,
                414.88,
                416.82,
                418.74,
                420.64,
                422.72,
                424.8,
                426.87,
                428.27,
                428.89,
                429.52,
                430.15,
                430.78,
                431.41,
                558.27,
                558.97,
                559.68,
                560.41,
                561.15,
                561.9,
                559.26,
                556.01,
                552.75,
                549.49,
                546.21,
                542.94,
                539.66,
                536.37,
                533.07,
                529.78,
                526.47,
                523.16,
                519.84,
                516.52,
                513.19,
                509.86,
                506.52,
                503.18,
                499.83,
                499.01,
                499.29,
                499.56,
                499.82,
                500.08,
                500.32,
                500.55,
                500.77,
                500.99,
                501.19,
                501.38,
                501.56,
                501.74,
                501.9,
                502.05,
                502.19,
                502.33,
                502.45,
                502.56,
                501.2,
                498.62,
                496.07,
                493.55,
                491.06,
                488.6,
                486.17,
                483.77,
                481.41,
                479.07,
                476.77,
                474.5,
                472.26,
                470.05,
                467.87,
                465.72,
                463.6,
                461.52,
                459.46,
                457.98,
                457.41,
                456.85,
                456.29,
                455.75,
                455.22,
                454.71,
                454.2,
                453.71,
                453.23,
                452.76,
                452.3,
                451.85,
                451.42,
                450.99,
                450.58,
                450.18,
                449.79,
                449.42,
                449.38,
                450.54,
                451.71,
                452.89,
                454.08,
                455.29,
                456.51,
                457.74,
                458.98,
                460.24,
                461.51,
                462.79,
                464.09,
                465.39,
                466.71,
                468.04,
                469.39,
                470.75,
                472.12,
                473.48,
                474.51,
                475.54,
                476.56,
                477.58,
                478.6,
                479.61,
                480.61,
                481.61,
                482.61,
                483.6,
                484.59,
                485.57,
                486.55,
                487.52,
                488.49,
                489.45,
                490.41,
                491.37,
                492.32,
                495.81,
                499.6,
                503.33,
                507.0,
                510.63,
                514.2,
                517.71,
                521.17,
                524.58,
                527.93,
                531.23,
                534.47,
                537.66,
                540.79,
                543.87,
                546.89,
                549.86,
                552.78,
                555.64,
                554.05,
                550.62,
                547.17,
                543.7,
                540.2,
                536.68,
                533.13,
                529.56,
                525.97,
                522.36,
                518.72,
                515.06,
                511.37,
                507.66,
                503.93,
                500.17,
                496.39,
                492.58,
                488.76,
                484.84,
                480.87,
                476.91,
                472.97,
                469.03,
                465.11,
                461.2,
                457.3,
                453.41,
                449.54,
                445.67,
                441.82,
                437.97,
                434.14,
                430.32,
                426.51,
                422.72,
                418.93,
                415.16,
                412.54,
                412.08,
                411.63,
                411.2,
                410.79,
                410.4,
                410.02,
                409.67,
                409.33,
                409.01,
                408.71,
                408.42,
                408.16,
                407.91,
                407.68,
                407.47,
                407.28,
                407.11,
                406.95,
                406.25,
                402.69,
                399.11,
                395.52,
                391.92,
                388.3,
                384.67,
                381.03,
                377.38,
                373.72,
                370.04,
                366.35,
                362.64,
                358.93,
                355.2,
                351.46,
                347.71,
                343.94,
                340.16,
                336.37,
                337.61,
                339.01,
                340.44,
                341.9,
                343.39,
                344.91,
                346.46,
                348.04,
                349.65,
                351.29,
                352.96,
                354.65,
                356.38,
                358.14,
                359.94,
                361.76,
                363.61,
                365.49,
                367.4,
                366.57,
                364.95,
                363.31,
                361.66,
                359.99,
                358.3,
                356.59,
                354.87,
                353.12,
                351.37,
                349.59,
                347.8,
                345.98,
                344.15,
                342.31,
                340.44,
                338.56,
                336.66,
                334.75,
                335.6,
                338.43,
                341.26,
                344.08,
                346.91,
                349.74,
                352.58,
                355.42,
                358.26,
                361.1,
                363.95,
                366.8,
                369.66,
                372.52,
                375.38,
                378.24,
                381.11,
                383.98,
                386.86,
                389.43,
                391.51,
                393.59,
                395.66,
                397.72,
                399.78,
                401.84,
                403.89,
                405.93,
                407.97,
                410.01,
                412.04,
                414.07,
                416.09,
                418.11,
                420.12,
                422.12,
                424.13,
                426.12,
                427.88,
                428.54,
                429.2,
                429.86,
                430.52,
                431.18,
                561.87,
                562.84,
                563.83,
                564.82,
                565.84,
                566.86,
                565.23,
                561.87,
                558.5,
                555.13,
                551.75,
                548.36,
                544.97,
                541.58,
                538.18,
                534.77,
                531.36,
                527.94,
                524.51,
                521.09,
                517.65,
                514.21,
                510.76,
                507.31,
                503.85,
                502.0,
                502.1,
                502.19,
                502.27,
                502.34,
                502.39,
                502.44,
                502.48,
                502.51,
                502.53,
                502.54,
                502.54,
                502.52,
                502.5,
                502.47,
                502.43,
                502.38,
                502.32,
                502.25,
                501.59,
                499.57,
                497.58,
                495.63,
                493.7,
                491.8,
                489.94,
                488.1,
                486.3,
                484.53,
                482.79,
                481.08,
                479.4,
                477.75,
                476.13,
                474.55,
                472.99,
                471.47,
                469.98,
                468.66,
                468.29,
                467.93,
                467.58,
                467.25,
                466.92,
                466.61,
                466.31,
                466.02,
                465.75,
                465.48,
                465.23,
                464.99,
                464.76,
                464.54,
                464.33,
                464.14,
                463.96,
                463.78,
                463.63,
                464.94,
                466.33,
                467.73,
                469.14,
                470.56,
                471.99,
                473.44,
                474.9,
                476.37,
                477.86,
                479.35,
                480.86,
                482.39,
                483.92,
                485.47,
                487.03,
                488.6,
                490.19,
                491.79,
                492.87,
                493.81,
                494.75,
                495.68,
                496.61,
                497.53,
                498.45,
                499.36,
                500.27,
                501.18,
                502.08,
                502.97,
                503.86,
                504.75,
                505.63,
                506.51,
                507.38,
                508.25,
                509.11,
                511.23,
                514.08,
                516.87,
                519.61,
                522.29,
                524.92,
                527.5,
                530.02,
                532.49,
                534.9,
                537.26,
                539.56,
                541.81,
                544.01,
                546.15,
                548.23,
                550.26,
                552.24,
                554.16,
                553.51,
                549.69,
                545.84,
                541.96,
                538.07,
                534.15,
                530.21,
                526.24,
                522.25,
                518.23,
                514.2,
                510.14,
                506.05,
                501.94,
                497.81,
                493.66,
                489.48,
                485.28,
                481.05,
                476.92,
                473.15,
                469.39,
                465.64,
                461.9,
                458.17,
                454.45,
                450.75,
                447.05,
                443.37,
                439.7,
                436.04,
                432.39,
                428.76,
                425.13,
                421.52,
                417.92,
                414.33,
                410.75,
                407.44,
                407.28,
                407.13,
                407.01,
                406.9,
                406.81,
                406.73,
                406.68,
                406.64,
                406.63,
                406.63,
                406.64,
                406.68,
                406.73,
                406.81,
                406.9,
                407.01,
                407.14,
                407.28,
                407.45,
                404.12,
                400.33,
                396.54,
                392.73,
                388.91,
                385.08,
                381.24,
                377.38,
                373.51,
                369.63,
                365.73,
                361.83,
                357.91,
                353.98,
                350.03,
                346.07,
                342.11,
                338.12,
                334.13,
                334.2,
                336.09,
                338.0,
                339.95,
                341.92,
                343.92,
                345.96,
                348.02,
                350.11,
                352.27,
                354.43,
                356.58,
                358.72,
                360.84,
                362.96,
                365.06,
                367.15,
                369.23,
                371.3,
                371.35,
                369.39,
                367.42,
                365.45,
                363.48,
                361.51,
                359.53,
                357.55,
                355.57,
                353.59,
                351.6,
                349.61,
                347.61,
                345.62,
                343.62,
                341.62,
                339.62,
                337.61,
                335.6,
                335.05,
                337.92,
                340.79,
                343.67,
                346.55,
                349.43,
                352.32,
                355.21,
                358.11,
                361.01,
                363.91,
                366.81,
                369.72,
                372.63,
                375.55,
                378.46,
                381.39,
                384.31,
                387.24,
                390.09,
                392.09,
                394.09,
                396.08,
                398.07,
                400.06,
                402.04,
                404.01,
                405.98,
                407.95,
                409.91,
                411.86,
                413.82,
                415.76,
                417.7,
                419.64,
                421.57,
                423.5,
                425.42,
                427.33,
                428.16,
                428.85,
                429.55,
                430.24,
                430.94,
                565.35,
                566.59,
                567.84,
                569.11,
                570.39,
                571.69,
                571.24,
                567.77,
                564.3,
                560.82,
                557.33,
                553.84,
                550.34,
                546.84,
                543.33,
                539.82,
                536.3,
                532.77,
                529.24,
                525.7,
                522.16,
                518.61,
                515.06,
                511.5,
                507.93,
                505.09,
                505.0,
                504.91,
                504.8,
                504.68,
                504.56,
                504.42,
                504.28,
                504.12,
                503.96,
                503.78,
                503.6,
                503.4,
                503.2,
                502.98,
                502.76,
                502.52,
                502.28,
                502.02,
                501.7,
                500.25,
                498.82,
                497.42,
                496.06,
                494.72,
                493.42,
                492.15,
                490.91,
                489.7,
                488.52,
                487.37,
                486.25,
                485.17,
                484.11,
                483.09,
                482.1,
                481.14,
                480.21,
                479.31,
                479.06,
                478.91,
                478.77,
                478.64,
                478.52,
                478.41,
                478.32,
                478.23,
                478.16,
                478.1,
                478.05,
                478.02,
                477.99,
                477.98,
                477.98,
                477.99,
                478.01,
                478.05,
                478.09,
                479.24,
                480.84,
                482.45,
                484.08,
                485.72,
                487.37,
                489.03,
                490.7,
                492.39,
                494.09,
                495.81,
                497.53,
                499.27,
                501.02,
                502.78,
                504.56,
                506.35,
                508.15,
                509.96,
                511.27,
                512.12,
                512.97,
                513.82,
                514.66,
                515.5,
                516.33,
                517.16,
                517.98,
                518.8,
                519.61,
                520.42,
                521.23,
                522.03,
                522.82,
                523.61,
                524.4,
                525.18,
                525.96,
                527.15,
                529.06,
                530.91,
                532.71,
                534.46,
                536.15,
                537.78,
                539.37,
                540.89,
                542.37,
                543.79,
                545.15,
                546.46,
                547.72,
                548.92,
                550.07,
                551.16,
                552.19,
                553.18,
                553.19,
                548.96,
                544.71,
                540.44,
                536.15,
                531.83,
                527.49,
                523.13,
                518.74,
                514.33,
                509.89,
                505.43,
                500.95,
                496.44,
                491.91,
                487.36,
                482.78,
                478.18,
                473.56,
                468.91,
                465.32,
                461.76,
                458.2,
                454.65,
                451.12,
                447.6,
                444.09,
                440.59,
                437.1,
                433.62,
                430.16,
                426.71,
                423.26,
                419.83,
                416.42,
                413.01,
                409.61,
                406.23,
                402.86,
                402.32,
                402.47,
                402.65,
                402.84,
                403.05,
                403.28,
                403.53,
                403.79,
                404.08,
                404.38,
                404.7,
                405.03,
                405.39,
                405.76,
                406.21,
                406.67,
                407.09,
                407.48,
                407.84,
                405.56,
                401.65,
                397.75,
                393.87,
                390.01,
                386.16,
                382.32,
                378.5,
                374.7,
                370.91,
                367.13,
                363.38,
                359.63,
                355.9,
                352.19,
                348.49,
                344.81,
                341.14,
                337.49,
                336.24,
                338.32,
                340.39,
                342.44,
                344.49,
                346.52,
                348.54,
                350.55,
                352.55,
                354.54,
                356.51,
                358.48,
                360.43,
                362.37,
                364.3,
                366.22,
                368.13,
                370.03,
                371.91,
                372.95,
                370.94,
                368.93,
                366.92,
                364.9,
                362.89,
                360.87,
                358.85,
                356.82,
                354.79,
                352.76,
                350.73,
                348.7,
                346.66,
                344.62,
                342.57,
                340.53,
                338.48,
                336.43,
                334.44,
                337.36,
                340.29,
                343.22,
                346.16,
                349.09,
                352.03,
                354.98,
                357.93,
                360.88,
                363.83,
                366.79,
                369.75,
                372.72,
                375.69,
                378.66,
                381.63,
                384.61,
                387.59,
                390.58,
                392.71,
                394.64,
                396.55,
                398.47,
                400.38,
                402.28,
                404.18,
                406.08,
                407.97,
                409.85,
                411.73,
                413.61,
                415.48,
                417.34,
                419.2,
                421.06,
                422.91,
                424.76,
                426.6,
                427.77,
                428.49,
                429.22,
                429.95,
                430.67,
                568.69,
                570.2,
                571.73,
                573.27,
                574.82,
                576.38,
                577.31,
                573.73,
                570.15,
                566.56,
                562.97,
                559.37,
                555.76,
                552.15,
                548.54,
                544.92,
                541.29,
                537.65,
                534.02,
                530.37,
                526.72,
                523.06,
                519.4,
                515.73,
                512.06,
                508.38,
                507.99,
                507.71,
                507.42,
                507.12,
                506.82,
                506.5,
                506.17,
                505.83,
                505.48,
                505.12,
                504.75,
                504.37,
                503.98,
                503.59,
                503.18,
                502.76,
                502.33,
                501.89,
                501.44,
                500.64,
                499.77,
                498.94,
                498.14,
                497.37,
                496.62,
                495.91,
                495.24,
                494.59,
                493.97,
                493.38,
                492.83,
                492.31,
                491.81,
                491.35,
                490.92,
                490.52,
                490.15,
                489.82,
                489.73,
                489.78,
                489.85,
                489.92,
                490.01,
                490.11,
                490.22,
                490.34,
                490.47,
                490.62,
                490.78,
                490.94,
                491.13,
                491.32,
                491.52,
                491.74,
                491.97,
                492.2,
                492.46,
                493.42,
                495.24,
                497.07,
                498.91,
                500.76,
                502.63,
                504.51,
                506.4,
                508.3,
                510.22,
                512.15,
                514.09,
                516.04,
                518.01,
                519.99,
                521.98,
                523.98,
                526.0,
                528.03,
                529.71,
                530.48,
                531.25,
                532.0,
                532.76,
                533.51,
                534.26,
                535.0,
                535.73,
                536.47,
                537.19,
                537.92,
                538.63,
                539.35,
                540.06,
                540.76,
                541.46,
                542.16,
                542.85,
                543.56,
                544.53,
                545.45,
                546.31,
                547.11,
                547.86,
                548.56,
                549.21,
                549.8,
                550.33,
                550.81,
                551.24,
                551.61,
                551.93,
                552.19,
                552.4,
                552.55,
                552.65,
                552.7,
                552.68,
                548.45,
                543.81,
                539.14,
                534.44,
                529.73,
                524.99,
                520.23,
                515.44,
                510.63,
                505.8,
                500.94,
                496.06,
                491.15,
                486.23,
                481.28,
                476.3,
                471.3,
                466.28,
                461.31,
                457.73,
                454.65,
                451.59,
                448.57,
                445.57,
                442.61,
                439.68,
                436.78,
                433.91,
                431.07,
                428.26,
                425.48,
                422.73,
                420.02,
                417.34,
                414.68,
                412.06,
                409.47,
                406.91,
                405.94,
                406.31,
                406.66,
                406.97,
                407.24,
                407.49,
                407.71,
                407.89,
                408.05,
                408.17,
                408.27,
                408.33,
                408.36,
                408.36,
                408.32,
                408.26,
                408.16,
                408.04,
                407.88,
                406.51,
                402.85,
                399.2,
                395.57,
                391.95,
                388.35,
                384.76,
                381.19,
                377.64,
                374.09,
                370.57,
                367.06,
                363.56,
                360.08,
                356.62,
                353.17,
                349.73,
                346.31,
                342.91,
                340.26,
                342.15,
                344.04,
                345.91,
                347.77,
                349.62,
                351.46,
                353.28,
                355.1,
                356.9,
                358.7,
                360.48,
                362.25,
                364.01,
                365.76,
                367.49,
                369.22,
                370.93,
                372.63,
                374.32,
                372.51,
                370.46,
                368.41,
                366.35,
                364.29,
                362.23,
                360.17,
                358.1,
                356.03,
                353.96,
                351.88,
                349.8,
                347.72,
                345.64,
                343.55,
                341.46,
                339.37,
                337.28,
                335.18,
                336.78,
                339.76,
                342.74,
                345.73,
                348.72,
                351.72,
                354.71,
                357.72,
                360.72,
                363.73,
                366.74,
                369.75,
                372.77,
                375.79,
                378.82,
                381.85,
                384.88,
                387.91,
                390.95,
                393.38,
                395.23,
                397.07,
                398.91,
                400.74,
                402.57,
                404.39,
                406.21,
                408.03,
                409.84,
                411.64,
                413.44,
                415.24,
                417.03,
                418.81,
                420.59,
                422.37,
                424.14,
                425.9,
                427.36,
                428.12,
                428.87,
                429.63,
                430.39,
                571.91,
                573.69,
                575.48,
                577.29,
                579.11,
                580.95,
                582.8,
                579.74,
                576.05,
                572.36,
                568.66,
                564.95,
                561.24,
                557.52,
                553.8,
                550.07,
                546.33,
                542.59,
                538.84,
                535.09,
                531.33,
                527.57,
                523.8,
                520.03,
                516.25,
                512.46,
                511.07,
                510.61,
                510.14,
                509.65,
                509.16,
                508.66,
                508.15,
                507.62,
                507.09,
                506.55,
                506.0,
                505.43,
                504.86,
                504.28,
                503.69,
                503.08,
                502.47,
                501.85,
                501.22,
                500.75,
                500.45,
                500.18,
                499.94,
                499.73,
                499.55,
                499.4,
                499.28,
                499.2,
                499.14,
                499.12,
                499.12,
                499.16,
                499.23,
                499.33,
                499.46,
                499.62,
                499.82,
                500.04,
                500.29,
                500.55,
                500.82,
                501.1,
                501.39,
                501.7,
                502.01,
                502.34,
                502.68,
                503.03,
                503.39,
                503.77,
                504.15,
                504.55,
                504.96,
                505.38,
                505.81,
                506.26,
                506.71,
                507.5,
                509.53,
                511.57,
                513.63,
                515.7,
                517.78,
                519.87,
                521.98,
                524.1,
                526.23,
                528.37,
                530.53,
                532.7,
                534.88,
                537.08,
                539.28,
                541.5,
                543.73,
                545.98,
                548.2,
                548.88,
                549.56,
                550.24,
                550.9,
                551.57,
                552.23,
                552.88,
                553.53,
                554.18,
                554.82,
                555.45,
                556.09,
                556.71,
                557.34,
                557.95,
                558.57,
                559.18,
                559.78,
                560.38,
                560.5,
                560.47,
                560.4,
                559.92,
                559.27,
                558.61,
                557.94,
                557.26,
                556.56,
                555.86,
                555.14,
                554.41,
                553.68,
                552.92,
                552.16,
                551.39,
                550.6,
                549.81,
                549.0,
                545.58,
                540.77,
                535.98,
                531.21,
                526.46,
                521.73,
                517.03,
                512.34,
                507.68,
                503.03,
                498.41,
                493.81,
                489.23,
                484.67,
                480.13,
                475.61,
                471.12,
                466.64,
                462.19,
                458.6,
                456.03,
                453.49,
                450.98,
                448.51,
                446.06,
                443.65,
                441.26,
                438.91,
                436.59,
                434.3,
                432.04,
                429.81,
                427.61,
                425.44,
                423.31,
                421.2,
                419.13,
                417.09,
                415.57,
                415.43,
                415.25,
                415.04,
                414.81,
                414.54,
                414.24,
                413.91,
                413.54,
                413.15,
                412.73,
                412.27,
                411.78,
                411.27,
                410.72,
                410.14,
                409.52,
                408.88,
                408.21,
                407.33,
                403.91,
                400.51,
                397.13,
                393.76,
                390.41,
                387.07,
                383.74,
                380.44,
                377.14,
                373.86,
                370.6,
                367.35,
                364.12,
                360.91,
                357.7,
                354.52,
                351.35,
                348.19,
                345.05,
                346.09,
                347.79,
                349.48,
                351.16,
                352.82,
                354.48,
                356.12,
                357.75,
                359.38,
                360.99,
                362.58,
                364.17,
                365.75,
                367.31,
                368.86,
                370.41,
                371.94,
                373.46,
                374.96,
                374.11,
                372.02,
                369.92,
                367.82,
                365.72,
                363.62,
                361.51,
                359.4,
                357.29,
                355.17,
                353.05,
                350.93,
                348.81,
                346.68,
                344.55,
                342.42,
                340.29,
                338.15,
                336.01,
                336.17,
                339.2,
                342.24,
                345.28,
                348.32,
                351.37,
                354.42,
                357.47,
                360.53,
                363.59,
                366.66,
                369.72,
                372.8,
                375.87,
                378.95,
                382.03,
                385.11,
                388.2,
                391.29,
                394.09,
                395.86,
                397.63,
                399.39,
                401.15,
                402.9,
                404.65,
                406.39,
                408.13,
                409.87,
                411.6,
                413.32,
                415.04,
                416.75,
                418.46,
                420.17,
                421.87,
                423.56,
                425.25,
                426.93,
                427.72,
                428.51,
                429.3,
                430.09,
                574.99,
                577.04,
                579.11,
                581.18,
                583.28,
                585.38,
                587.51,
                585.81,
                582.01,
                578.21,
                574.4,
                570.58,
                566.76,
                562.94,
                559.11,
                555.27,
                551.43,
                547.58,
                543.73,
                539.87,
                536.0,
                532.13,
                528.25,
                524.37,
                520.48,
                516.59,
                514.25,
                513.6,
                512.94,
                512.28,
                511.6,
                510.91,
                510.22,
                509.51,
                508.79,
                508.07,
                507.33,
                506.59,
                505.83,
                505.06,
                504.29,
                503.5,
                502.71,
                501.9,
                501.08,
                500.58,
                500.84,
                501.13,
                501.45,
                501.81,
                502.19,
                502.6,
                503.05,
                503.52,
                504.03,
                504.57,
                505.13,
                505.73,
                506.37,
                507.03,
                507.72,
                508.44,
                509.2,
                509.99,
                510.76,
                511.22,
                511.69,
                512.18,
                512.68,
                513.19,
                513.71,
                514.24,
                514.78,
                515.34,
                515.91,
                516.48,
                517.08,
                517.68,
                518.29,
                518.92,
                519.56,
                520.21,
                520.87,
                521.54,
                523.71,
                525.97,
                528.24,
                530.52,
                532.82,
                535.13,
                537.26,
                539.38,
                541.5,
                543.63,
                545.76,
                547.89,
                550.03,
                552.18,
                554.33,
                556.48,
                558.64,
                560.8,
                562.97,
                563.56,
                563.64,
                563.68,
                563.68,
                563.64,
                563.57,
                563.46,
                563.31,
                563.13,
                562.9,
                562.64,
                562.34,
                562.01,
                561.63,
                561.22,
                560.77,
                560.28,
                559.75,
                559.19,
                558.48,
                557.67,
                556.85,
                556.02,
                555.18,
                554.33,
                553.46,
                552.58,
                551.7,
                550.8,
                549.89,
                548.97,
                548.03,
                547.09,
                546.13,
                545.17,
                544.19,
                543.2,
                542.19,
                539.85,
                535.4,
                530.96,
                526.55,
                522.16,
                517.79,
                513.44,
                509.11,
                504.8,
                500.51,
                496.25,
                492.0,
                487.78,
                483.58,
                479.39,
                475.23,
                471.09,
                466.97,
                462.88,
                459.18,
                457.13,
                455.11,
                453.12,
                451.16,
                449.23,
                447.33,
                445.47,
                443.63,
                441.83,
                440.05,
                438.31,
                436.6,
                434.92,
                433.27,
                431.65,
                430.06,
                428.51,
                426.98,
                425.49,
                424.82,
                424.13,
                423.41,
                422.65,
                421.87,
                421.05,
                420.2,
                419.32,
                418.41,
                417.47,
                416.5,
                415.5,
                414.46,
                413.4,
                412.3,
                411.17,
                410.01,
                408.82,
                407.6,
                404.84,
                401.69,
                398.55,
                395.43,
                392.32,
                389.23,
                386.16,
                383.1,
                380.05,
                377.02,
                374.01,
                371.01,
                368.02,
                365.05,
                362.1,
                359.16,
                356.24,
                353.33,
                350.44,
                350.13,
                351.65,
                353.15,
                354.65,
                356.13,
                357.6,
                359.06,
                360.51,
                361.95,
                363.38,
                364.79,
                366.2,
                367.59,
                368.97,
                370.34,
                371.7,
                373.05,
                374.38,
                375.71,
                375.74,
                373.6,
                371.46,
                369.32,
                367.17,
                365.03,
                362.88,
                360.72,
                358.57,
                356.41,
                354.25,
                352.09,
                349.92,
                347.75,
                345.58,
                343.41,
                341.23,
                339.05,
                336.87,
                335.52,
                338.61,
                341.7,
                344.79,
                347.89,
                350.99,
                354.09,
                357.2,
                360.31,
                363.43,
                366.54,
                369.66,
                372.79,
                375.92,
                379.05,
                382.18,
                385.32,
                388.46,
                391.61,
                394.75,
                396.54,
                398.23,
                399.92,
                401.6,
                403.28,
                404.95,
                406.62,
                408.28,
                409.94,
                411.6,
                413.24,
                414.89,
                416.53,
                418.16,
                419.79,
                421.42,
                423.04,
                424.65,
                426.26,
                427.3,
                428.12,
                428.95,
                429.77,
                577.95,
                580.27,
                582.6,
                584.95,
                587.31,
                589.69,
                592.08,
                591.92,
                588.02,
                584.11,
                580.19,
                576.27,
                572.34,
                568.41,
                564.47,
                560.53,
                556.58,
                552.62,
                548.66,
                544.69,
                540.72,
                536.74,
                532.76,
                528.77,
                524.77,
                520.77,
                517.51,
                516.68,
                515.84,
                514.99,
                514.13,
                513.26,
                512.38,
                511.49,
                510.59,
                509.68,
                508.76,
                507.83,
                506.89,
                505.94,
                504.98,
                504.01,
                503.03,
                502.04,
                501.04,
                500.14,
                500.96,
                501.81,
                502.69,
                503.61,
                504.55,
                505.53,
                506.53,
                507.57,
                508.71,
                509.86,
                511.01,
                512.16,
                513.31,
                514.48,
                515.64,
                516.81,
                517.99,
                519.16,
                520.35,
                520.96,
                521.49,
                522.03,
                522.56,
                523.09,
                523.62,
                524.15,
                524.68,
                525.21,
                525.74,
                526.27,
                526.8,
                527.33,
                527.85,
                528.38,
                528.91,
                529.43,
                529.96,
                530.48,
                532.15,
                534.31,
                536.48,
                538.66,
                540.84,
                543.02,
                545.21,
                547.4,
                549.6,
                551.8,
                554.01,
                556.22,
                558.44,
                560.66,
                562.88,
                565.11,
                567.34,
                569.58,
                571.82,
                572.65,
                572.07,
                571.45,
                570.79,
                570.1,
                569.37,
                568.6,
                567.79,
                566.95,
                566.07,
                565.15,
                564.19,
                563.2,
                562.17,
                561.1,
                559.99,
                558.84,
                557.66,
                556.44,
                555.27,
                554.26,
                553.25,
                552.23,
                551.19,
                550.14,
                549.08,
                548.01,
                546.93,
                545.84,
                544.74,
                543.62,
                542.49,
                541.35,
                540.2,
                539.04,
                537.87,
                536.69,
                535.49,
                533.93,
                529.83,
                525.76,
                521.7,
                517.67,
                513.65,
                509.66,
                505.69,
                501.74,
                497.8,
                493.89,
                490.01,
                486.14,
                482.29,
                478.46,
                474.66,
                470.88,
                467.11,
                463.37,
                459.65,
                457.95,
                456.45,
                454.98,
                453.53,
                452.12,
                450.74,
                449.39,
                448.07,
                446.79,
                445.53,
                444.31,
                443.11,
                441.95,
                440.81,
                439.71,
                438.64,
                437.6,
                436.6,
                435.62,
                434.5,
                433.29,
                432.05,
                430.78,
                429.48,
                428.14,
                426.78,
                425.38,
                423.96,
                422.5,
                421.01,
                419.49,
                417.94,
                416.36,
                414.75,
                413.1,
                411.43,
                409.72,
                407.98,
                405.63,
                402.72,
                399.83,
                396.96,
                394.1,
                391.26,
                388.43,
                385.62,
                382.82,
                380.04,
                377.27,
                374.52,
                371.79,
                369.07,
                366.36,
                363.67,
                360.99,
                358.33,
                355.69,
                354.27,
                355.61,
                356.93,
                358.24,
                359.54,
                360.83,
                362.11,
                363.37,
                364.63,
                365.87,
                367.1,
                368.33,
                369.54,
                370.73,
                371.92,
                373.1,
                374.26,
                375.41,
                376.56,
                377.38,
                375.21,
                373.02,
                370.84,
                368.65,
                366.46,
                364.27,
                362.07,
                359.88,
                357.68,
                355.47,
                353.27,
                351.06,
                348.85,
                346.63,
                344.42,
                342.2,
                339.98,
                337.75,
                335.52,
                337.98,
                341.13,
                344.28,
                347.43,
                350.58,
                353.74,
                356.9,
                360.06,
                363.23,
                366.4,
                369.57,
                372.75,
                375.93,
                379.12,
                382.3,
                385.49,
                388.69,
                391.89,
                395.09,
                397.26,
                398.88,
                400.49,
                402.1,
                403.7,
                405.3,
                406.89,
                408.48,
                410.06,
                411.64,
                413.21,
                414.78,
                416.34,
                417.9,
                419.46,
                421.01,
                422.55,
                424.09,
                425.63,
                426.87,
                427.72,
                428.57,
                429.43,
                580.77,
                583.36,
                585.97,
                588.58,
                591.22,
                593.86,
                596.52,
                598.09,
                594.08,
                590.06,
                586.04,
                582.01,
                577.97,
                573.93,
                569.89,
                565.83,
                561.76,
                557.57,
                553.42,
                549.28,
                545.18,
                541.09,
                537.04,
                533.01,
                529.0,
                525.02,
                521.06,
                520.13,
                519.24,
                518.36,
                517.49,
                516.63,
                515.77,
                514.92,
                514.08,
                513.25,
                512.43,
                511.62,
                510.81,
                510.01,
                509.22,
                508.44,
                507.67,
                506.91,
                506.15,
                505.4,
                506.22,
                507.41,
                508.59,
                509.79,
                510.99,
                512.19,
                513.4,
                514.61,
                515.82,
                517.04,
                518.27,
                519.49,
                520.73,
                521.96,
                523.2,
                524.45,
                525.7,
                526.95,
                528.21,
                529.01,
                529.53,
                530.05,
                530.57,
                531.09,
                531.61,
                532.13,
                532.65,
                533.17,
                533.69,
                534.21,
                534.73,
                535.24,
                535.76,
                536.28,
                536.79,
                537.31,
                537.82,
                538.34,
                539.61,
                541.85,
                544.1,
                546.35,
                548.6,
                550.86,
                553.13,
                555.39,
                557.67,
                559.94,
                562.22,
                564.51,
                566.8,
                569.1,
                571.39,
                573.7,
                576.01,
                578.32,
                580.64,
                582.07,
                580.84,
                579.56,
                578.25,
                576.89,
                575.51,
                574.08,
                572.62,
                571.12,
                569.58,
                568.0,
                566.39,
                564.73,
                563.05,
                561.32,
                559.55,
                557.75,
                555.91,
                554.03,
                552.15,
                550.96,
                549.75,
                548.53,
                547.3,
                546.06,
                544.81,
                543.55,
                542.27,
                540.98,
                539.69,
                538.38,
                537.06,
                535.72,
                534.38,
                533.03,
                531.66,
                530.28,
                528.89,
                527.49,
                524.08,
                520.36,
                516.66,
                512.99,
                509.33,
                505.69,
                502.07,
                498.48,
                494.9,
                491.35,
                487.82,
                484.31,
                480.81,
                477.34,
                473.9,
                470.47,
                467.06,
                463.67,
                460.31,
                458.49,
                457.51,
                456.55,
                455.63,
                454.73,
                453.87,
                453.04,
                452.24,
                451.47,
                450.73,
                450.02,
                449.34,
                448.69,
                448.08,
                447.49,
                446.94,
                446.42,
                445.93,
                445.47,
                444.46,
                442.73,
                440.98,
                439.19,
                437.37,
                435.52,
                433.64,
                431.73,
                429.79,
                427.82,
                425.81,
                423.77,
                421.71,
                419.61,
                417.48,
                415.32,
                413.13,
                410.9,
                408.65,
                406.28,
                403.62,
                400.98,
                398.36,
                395.75,
                393.15,
                390.57,
                388.01,
                385.46,
                382.92,
                380.4,
                377.9,
                375.41,
                372.94,
                370.48,
                368.04,
                365.61,
                363.2,
                360.8,
                358.52,
                359.67,
                360.81,
                361.94,
                363.05,
                364.16,
                365.25,
                366.34,
                367.41,
                368.47,
                369.52,
                370.56,
                371.58,
                372.6,
                373.6,
                374.6,
                375.58,
                376.55,
                377.51,
                378.46,
                376.83,
                374.61,
                372.38,
                370.15,
                367.92,
                365.69,
                363.45,
                361.21,
                358.96,
                356.72,
                354.47,
                352.22,
                349.96,
                347.71,
                345.45,
                343.19,
                340.92,
                338.66,
                336.39,
                337.33,
                340.53,
                343.73,
                346.93,
                350.14,
                353.35,
                356.56,
                359.78,
                363.0,
                366.22,
                369.45,
                372.68,
                375.92,
                379.15,
                382.39,
                385.64,
                388.89,
                392.14,
                395.39,
                398.03,
                399.57,
                401.11,
                402.64,
                404.16,
                405.69,
                407.2,
                408.72,
                410.22,
                411.73,
                413.22,
                414.72,
                416.21,
                417.69,
                419.17,
                420.64,
                422.11,
                423.57,
                425.03,
                426.41,
                427.3,
                428.18,
                429.07,
                579.64,
                582.0,
                584.34,
                586.67,
                588.98,
                591.28,
                593.56,
                595.83,
                592.35,
                588.44,
                584.55,
                580.68,
                576.84,
                573.02,
                569.23,
                565.47,
                561.72,
                558.01,
                554.32,
                550.65,
                547.01,
                543.39,
                539.8,
                536.23,
                532.69,
                529.17,
                525.68,
                524.24,
                523.5,
                522.78,
                522.05,
                521.34,
                520.64,
                519.94,
                519.25,
                518.57,
                517.9,
                517.24,
                516.59,
                515.94,
                515.3,
                514.67,
                514.05,
                513.44,
                512.83,
                512.24,
                512.69,
                513.95,
                515.21,
                516.48,
                517.76,
                519.03,
                520.32,
                521.6,
                522.89,
                524.19,
                525.49,
                526.79,
                528.1,
                529.41,
                530.73,
                532.05,
                533.37,
                534.7,
                536.04,
                537.06,
                537.57,
                538.08,
                538.59,
                539.1,
                539.61,
                540.12,
                540.63,
                541.14,
                541.65,
                542.15,
                542.66,
                543.17,
                543.67,
                544.18,
                544.68,
                545.19,
                545.69,
                546.19,
                547.03,
                549.35,
                551.67,
                554.0,
                556.33,
                558.66,
                561.0,
                563.34,
                565.69,
                568.04,
                570.4,
                572.76,
                575.13,
                577.5,
                579.87,
                582.25,
                584.63,
                587.02,
                589.41,
                591.81,
                589.95,
                588.01,
                586.04,
                584.03,
                581.99,
                579.9,
                577.78,
                575.62,
                573.43,
                571.2,
                568.92,
                566.62,
                564.27,
                561.88,
                559.46,
                557.0,
                554.5,
                551.97,
                549.39,
                547.75,
                546.35,
                544.94,
                543.52,
                542.08,
                540.64,
                539.18,
                537.71,
                536.23,
                534.74,
                533.24,
                531.72,
                530.2,
                528.66,
                527.11,
                525.55,
                523.98,
                522.4,
                520.8,
                518.14,
                514.78,
                511.44,
                508.11,
                504.81,
                501.53,
                498.27,
                495.03,
                491.81,
                488.61,
                485.44,
                482.28,
                479.15,
                476.03,
                472.94,
                469.87,
                466.82,
                463.79,
                460.78,
                458.76,
                458.29,
                457.85,
                457.44,
                457.06,
                456.72,
                456.4,
                456.12,
                455.86,
                455.64,
                455.45,
                455.29,
                455.16,
                455.06,
                454.99,
                454.95,
                454.95,
                454.97,
                455.03,
                454.7,
                452.46,
                450.19,
                447.88,
                445.55,
                443.18,
                440.79,
                438.36,
                435.9,
                433.41,
                430.89,
                428.34,
                425.76,
                423.14,
                420.5,
                417.82,
                415.11,
                412.38,
                409.6,
                406.8,
                404.38,
                401.99,
                399.61,
                397.25,
                394.9,
                392.57,
                390.25,
                387.95,
                385.67,
                383.39,
                381.14,
                378.9,
                376.67,
                374.46,
                372.27,
                370.09,
                367.92,
                365.77,
                363.64,
                363.83,
                364.79,
                365.73,
                366.67,
                367.59,
                368.5,
                369.4,
                370.29,
                371.17,
                372.04,
                372.89,
                373.74,
                374.57,
                375.39,
                376.2,
                377.0,
                377.79,
                378.56,
                379.33,
                378.49,
                376.22,
                373.95,
                371.68,
                369.4,
                367.13,
                364.85,
                362.56,
                360.28,
                357.99,
                355.7,
                353.4,
                351.11,
                348.81,
                346.51,
                344.2,
                341.9,
                339.59,
                337.27,
                336.65,
                339.9,
                343.15,
                346.41,
                349.67,
                352.93,
                356.2,
                359.47,
                362.74,
                366.02,
                369.3,
                372.58,
                375.87,
                379.16,
                382.45,
                385.75,
                389.05,
                392.36,
                395.66,
                398.84,
                400.3,
                401.76,
                403.22,
                404.67,
                406.12,
                407.56,
                409.0,
                410.43,
                411.86,
                413.28,
                414.7,
                416.11,
                417.52,
                418.92,
                420.32,
                421.72,
                423.1,
                424.49,
                425.87,
                426.86,
                427.77,
                428.7,
                575.83,
                577.92,
                579.99,
                582.04,
                584.09,
                586.11,
                588.13,
                590.12,
                588.37,
                584.92,
                581.5,
                578.1,
                574.72,
                571.37,
                568.04,
                564.74,
                561.47,
                558.21,
                554.99,
                551.79,
                548.61,
                545.46,
                542.33,
                539.23,
                536.15,
                533.1,
                530.07,
                528.28,
                527.69,
                527.11,
                526.54,
                525.98,
                525.43,
                524.89,
                524.35,
                523.82,
                523.3,
                522.79,
                522.29,
                521.79,
                521.31,
                520.83,
                520.36,
                519.9,
                519.44,
                519.0,
                519.12,
                520.46,
                521.8,
                523.14,
                524.49,
                525.84,
                527.2,
                528.56,
                529.93,
                531.3,
                532.67,
                534.05,
                535.43,
                536.82,
                538.21,
                539.61,
                541.01,
                542.41,
                543.82,
                545.12,
                545.62,
                546.12,
                546.62,
                547.12,
                547.62,
                548.12,
                548.61,
                549.11,
                549.61,
                550.1,
                550.6,
                551.09,
                551.59,
                552.08,
                552.58,
                553.07,
                553.56,
                554.06,
                554.55,
                556.81,
                559.21,
                561.61,
                564.01,
                566.42,
                568.84,
                571.26,
                573.68,
                576.11,
                578.54,
                580.97,
                583.41,
                585.86,
                588.31,
                590.76,
                593.22,
                595.68,
                598.15,
                600.62,
                599.4,
                596.81,
                594.18,
                591.51,
                588.81,
                586.07,
                583.29,
                580.48,
                577.62,
                574.73,
                571.81,
                568.84,
                565.84,
                562.79,
                559.71,
                556.6,
                553.44,
                550.25,
                547.02,
                544.65,
                543.06,
                541.45,
                539.84,
                538.21,
                536.57,
                534.92,
                533.25,
                531.58,
                529.89,
                528.2,
                526.49,
                524.77,
                523.04,
                521.3,
                519.54,
                517.78,
                516.0,
                514.21,
                512.01,
                509.01,
                506.02,
                503.05,
                500.11,
                497.18,
                494.28,
                491.39,
                488.53,
                485.69,
                482.87,
                480.07,
                477.29,
                474.53,
                471.79,
                469.08,
                466.38,
                463.71,
                461.05,
                458.74,
                458.79,
                458.87,
                458.97,
                459.11,
                459.28,
                459.49,
                459.72,
                459.98,
                460.27,
                460.6,
                460.95,
                461.34,
                461.76,
                462.21,
                462.69,
                463.2,
                463.74,
                464.31,
                464.92,
                462.47,
                459.68,
                456.86,
                454.01,
                451.13,
                448.22,
                445.27,
                442.3,
                439.29,
                436.26,
                433.19,
                430.09,
                426.96,
                423.8,
                420.61,
                417.39,
                414.13,
                410.84,
                407.53,
                405.01,
                402.86,
                400.73,
                398.62,
                396.52,
                394.43,
                392.36,
                390.31,
                388.27,
                386.25,
                384.24,
                382.25,
                380.27,
                378.31,
                376.36,
                374.43,
                372.51,
                370.61,
                368.72,
                368.1,
                368.87,
                369.64,
                370.39,
                371.13,
                371.86,
                372.57,
                373.28,
                373.98,
                374.66,
                375.33,
                375.99,
                376.64,
                377.28,
                377.91,
                378.53,
                379.13,
                379.72,
                380.31,
                380.17,
                377.86,
                375.54,
                373.23,
                370.91,
                368.59,
                366.27,
                363.94,
                361.61,
                359.28,
                356.95,
                354.61,
                352.27,
                349.93,
                347.59,
                345.24,
                342.89,
                340.54,
                338.19,
                335.93,
                339.24,
                342.54,
                345.85,
                349.17,
                352.48,
                355.8,
                359.13,
                362.45,
                365.78,
                369.12,
                372.45,
                375.79,
                379.14,
                382.48,
                385.83,
                389.19,
                392.55,
                395.91,
                399.27,
                401.09,
                402.54,
                404.04,
                405.59,
                407.2,
                408.87,
                410.59,
                412.36,
                414.19,
                416.07,
                418.01,
                420.0,
                422.05,
                424.15,
                426.3,
                428.52,
                430.78,
                433.1,
                435.48,
                437.77,
                440.02,
                442.33,
                572.15,
                573.96,
                575.76,
                577.55,
                579.32,
                581.08,
                582.82,
                584.55,
                584.17,
                581.18,
                578.22,
                575.29,
                572.38,
                569.49,
                566.63,
                563.79,
                560.98,
                558.2,
                555.43,
                552.7,
                549.98,
                547.3,
                544.64,
                542.0,
                539.39,
                536.8,
                534.24,
                532.24,
                531.81,
                531.38,
                530.96,
                530.55,
                530.15,
                529.76,
                529.37,
                528.99,
                528.63,
                528.26,
                527.91,
                527.57,
                527.23,
                526.91,
                526.59,
                526.28,
                525.98,
                525.68,
                525.51,
                526.93,
                528.34,
                529.76,
                531.18,
                532.61,
                534.05,
                535.48,
                536.92,
                538.37,
                539.82,
                541.27,
                542.73,
                544.19,
                545.66,
                547.13,
                548.61,
                550.09,
                551.57,
                553.06,
                553.68,
                554.16,
                554.65,
                555.14,
                555.63,
                556.11,
                556.6,
                557.09,
                557.57,
                558.06,
                558.54,
                559.03,
                559.51,
                560.0,
                560.48,
                560.96,
                561.44,
                561.92,
                562.41,
                564.23,
                566.71,
                569.18,
                571.66,
                574.15,
                576.63,
                579.13,
                581.63,
                584.13,
                586.63,
                589.15,
                591.66,
                594.18,
                596.71,
                599.23,
                601.77,
                604.31,
                606.85,
                609.39,
                609.19,
                605.94,
                602.66,
                599.34,
                595.98,
                592.58,
                589.14,
                585.67,
                582.16,
                578.61,
                575.03,
                571.41,
                567.75,
                564.05,
                560.31,
                556.54,
                552.73,
                548.88,
                544.99,
                541.65,
                539.87,
                538.07,
                536.25,
                534.43,
                532.6,
                530.75,
                528.9,
                527.03,
                525.15,
                523.26,
                521.36,
                519.45,
                517.52,
                515.59,
                513.64,
                511.68,
                509.71,
                507.73,
                505.7,
                503.04,
                500.41,
                497.8,
                495.21,
                492.64,
                490.09,
                487.56,
                485.06,
                482.57,
                480.11,
                477.66,
                475.24,
                472.83,
                470.45,
                468.09,
                465.75,
                463.43,
                461.14,
                458.86,
                459.01,
                459.6,
                460.23,
                460.89,
                461.57,
                462.29,
                463.04,
                463.82,
                464.63,
                465.47,
                466.34,
                467.24,
                468.18,
                469.14,
                470.14,
                471.17,
                472.22,
                473.31,
                474.43,
                472.76,
                469.46,
                466.12,
                462.75,
                459.36,
                455.93,
                452.47,
                448.98,
                445.46,
                441.91,
                438.32,
                434.71,
                431.06,
                427.39,
                423.68,
                419.94,
                416.17,
                412.37,
                408.54,
                405.49,
                403.59,
                401.71,
                399.85,
                397.99,
                396.16,
                394.34,
                392.53,
                390.74,
                388.96,
                387.2,
                385.46,
                383.73,
                382.01,
                380.31,
                378.63,
                376.96,
                375.3,
                373.66,
                372.47,
                373.06,
                373.64,
                374.21,
                374.77,
                375.31,
                375.85,
                376.37,
                376.88,
                377.38,
                377.87,
                378.35,
                378.82,
                379.28,
                379.72,
                380.15,
                380.58,
                380.99,
                381.39,
                381.77,
                379.52,
                377.16,
                374.8,
                372.44,
                370.08,
                367.71,
                365.29,
                362.77,
                360.29,
                357.83,
                355.4,
                352.99,
                350.61,
                348.26,
                345.94,
                343.64,
                341.37,
                339.13,
                336.91,
                338.79,
                342.15,
                345.51,
                348.87,
                352.23,
                355.59,
                358.95,
                362.32,
                365.68,
                369.04,
                372.41,
                375.77,
                379.14,
                382.51,
                385.87,
                389.24,
                392.61,
                395.98,
                399.35,
                402.16,
                404.46,
                406.81,
                409.21,
                411.67,
                414.18,
                416.74,
                419.36,
                422.04,
                424.77,
                427.55,
                430.39,
                433.28,
                436.23,
                439.24,
                442.29,
                445.41,
                448.57,
                451.8,
                455.08,
                458.43,
                461.85,
                568.6,
                570.14,
                571.67,
                573.19,
                574.69,
                576.18,
                577.65,
                579.11,
                579.74,
                577.22,
                574.72,
                572.25,
                569.81,
                567.39,
                564.99,
                562.62,
                560.27,
                557.95,
                555.65,
                553.38,
                551.13,
                548.91,
                546.71,
                544.54,
                542.39,
                540.27,
                538.17,
                536.13,
                535.84,
                535.57,
                535.3,
                535.04,
                534.79,
                534.55,
                534.32,
                534.09,
                533.87,
                533.66,
                533.46,
                533.27,
                533.09,
                532.91,
                532.74,
                532.59,
                532.43,
                532.29,
                532.16,
                533.36,
                534.85,
                536.34,
                537.84,
                539.35,
                540.85,
                542.37,
                543.88,
                545.4,
                546.93,
                548.46,
                549.99,
                551.53,
                553.07,
                554.62,
                556.17,
                557.72,
                559.28,
                560.85,
                561.73,
                562.21,
                562.69,
                563.17,
                563.64,
                564.12,
                564.6,
                565.07,
                565.55,
                566.02,
                566.49,
                566.97,
                567.44,
                567.91,
                568.39,
                568.86,
                569.33,
                569.8,
                570.27,
                571.62,
                574.16,
                576.72,
                579.27,
                581.83,
                584.39,
                586.96,
                589.54,
                592.11,
                594.69,
                597.28,
                599.87,
                602.47,
                605.07,
                607.67,
                610.28,
                612.89,
                615.51,
                618.13,
                619.32,
                615.42,
                611.48,
                607.5,
                603.48,
                599.43,
                595.34,
                591.21,
                587.04,
                582.84,
                578.6,
                574.32,
                570.0,
                565.65,
                561.25,
                556.82,
                552.36,
                547.85,
                543.31,
                538.76,
                536.77,
                534.78,
                532.78,
                530.76,
                528.73,
                526.7,
                524.65,
                522.58,
                520.51,
                518.43,
                516.33,
                514.23,
                512.11,
                509.98,
                507.84,
                505.69,
                503.53,
                501.35,
                499.16,
                496.89,
                494.61,
                492.36,
                490.12,
                487.91,
                485.72,
                483.54,
                481.39,
                479.26,
                477.15,
                475.06,
                473.0,
                470.95,
                468.92,
                466.92,
                464.93,
                462.97,
                461.03,
                459.11,
                458.95,
                460.06,
                461.2,
                462.38,
                463.58,
                464.81,
                466.08,
                467.37,
                468.7,
                470.06,
                471.45,
                472.86,
                474.31,
                475.8,
                477.31,
                478.85,
                480.43,
                482.03,
                483.67,
                483.33,
                479.51,
                475.66,
                471.78,
                467.87,
                463.92,
                459.95,
                455.94,
                451.91,
                447.84,
                443.74,
                439.61,
                435.45,
                431.26,
                427.04,
                422.78,
                418.5,
                414.18,
                409.83,
                405.84,
                404.19,
                402.56,
                400.94,
                399.33,
                397.74,
                396.17,
                394.61,
                393.07,
                391.54,
                390.03,
                388.53,
                387.05,
                385.31,
                383.52,
                381.71,
                379.88,
                378.03,
                376.15,
                374.25,
                374.14,
                374.2,
                374.27,
                374.34,
                374.41,
                374.49,
                374.58,
                374.66,
                374.76,
                374.85,
                374.95,
                375.06,
                375.17,
                375.28,
                375.4,
                375.53,
                375.65,
                375.79,
                375.92,
                374.42,
                372.17,
                369.96,
                367.77,
                365.6,
                363.47,
                361.36,
                359.27,
                357.22,
                355.19,
                353.19,
                351.21,
                349.26,
                347.34,
                345.45,
                343.58,
                341.74,
                339.93,
                338.14,
                338.68,
                342.05,
                345.42,
                348.79,
                352.16,
                355.53,
                358.9,
                362.28,
                365.65,
                369.02,
                372.4,
                375.77,
                379.15,
                382.53,
                385.9,
                389.28,
                392.66,
                396.04,
                399.42,
                402.74,
                405.89,
                409.08,
                412.33,
                415.63,
                418.99,
                422.4,
                425.87,
                429.39,
                432.97,
                436.6,
                440.28,
                444.02,
                447.82,
                451.67,
                455.57,
                459.53,
                463.55,
                467.62,
                471.74,
                476.19,
                480.71,
                565.18,
                566.45,
                567.71,
                568.96,
                570.19,
                571.4,
                572.61,
                573.79,
                574.97,
                573.03,
                571.0,
                568.99,
                567.01,
                565.05,
                563.12,
                561.22,
                559.33,
                557.48,
                555.64,
                553.84,
                552.05,
                550.3,
                548.56,
                546.86,
                545.17,
                543.52,
                541.88,
                540.28,
                539.81,
                539.69,
                539.57,
                539.46,
                539.36,
                539.27,
                539.19,
                539.11,
                539.05,
                538.99,
                538.94,
                538.9,
                538.87,
                538.84,
                538.83,
                538.82,
                538.82,
                538.83,
                538.84,
                539.75,
                541.32,
                542.89,
                544.46,
                546.04,
                547.62,
                549.21,
                550.8,
                552.4,
                554.0,
                555.6,
                557.21,
                558.83,
                560.44,
                562.06,
                563.69,
                565.32,
                566.96,
                568.6,
                569.8,
                570.27,
                570.73,
                571.2,
                571.66,
                572.13,
                572.59,
                573.06,
                573.52,
                573.99,
                574.45,
                574.91,
                575.37,
                575.84,
                576.3,
                576.76,
                577.22,
                577.68,
                578.14,
                578.96,
                581.59,
                584.21,
                586.84,
                589.48,
                592.12,
                594.76,
                597.41,
                600.06,
                602.72,
                605.38,
                608.04,
                610.71,
                613.39,
                616.06,
                618.75,
                621.44,
                624.13,
                626.82,
                629.52,
                625.24,
                620.64,
                616.0,
                611.33,
                606.62,
                601.87,
                597.09,
                592.27,
                587.4,
                582.51,
                577.57,
                572.6,
                567.59,
                562.54,
                557.45,
                552.33,
                547.17,
                541.97,
                536.73,
                533.78,
                531.6,
                529.4,
                527.19,
                524.97,
                522.74,
                520.5,
                518.24,
                515.98,
                513.7,
                511.41,
                509.11,
                506.8,
                504.48,
                502.14,
                499.8,
                497.44,
                495.07,
                492.69,
                490.55,
                488.63,
                486.73,
                484.85,
                482.99,
                481.15,
                479.34,
                477.54,
                475.76,
                474.01,
                472.28,
                470.56,
                468.87,
                467.2,
                465.55,
                463.92,
                462.32,
                460.73,
                459.16,
                458.62,
                460.24,
                461.9,
                463.59,
                465.31,
                467.06,
                468.84,
                470.65,
                472.49,
                474.37,
                476.27,
                478.21,
                480.17,
                482.17,
                484.2,
                486.26,
                488.35,
                490.4,
                492.07,
                493.12,
                488.36,
                483.61,
                478.87,
                474.16,
                469.46,
                464.77,
                460.1,
                455.44,
                450.8,
                446.18,
                441.57,
                436.98,
                432.4,
                427.84,
                423.29,
                418.76,
                414.25,
                409.75,
                405.26,
                403.06,
                401.18,
                399.27,
                397.35,
                395.41,
                393.44,
                391.45,
                389.44,
                387.41,
                385.35,
                383.28,
                381.18,
                379.06,
                376.92,
                374.75,
                372.57,
                370.36,
                368.13,
                365.88,
                365.14,
                365.28,
                365.42,
                365.56,
                365.71,
                365.87,
                366.02,
                366.19,
                366.35,
                366.53,
                366.7,
                366.88,
                367.07,
                367.26,
                367.45,
                367.65,
                367.85,
                368.06,
                368.27,
                367.67,
                365.85,
                364.06,
                362.3,
                360.57,
                358.86,
                357.18,
                355.53,
                353.9,
                352.3,
                350.73,
                349.19,
                347.67,
                346.18,
                344.71,
                343.27,
                341.86,
                340.48,
                339.12,
                338.56,
                341.94,
                345.32,
                348.7,
                352.08,
                355.47,
                358.85,
                362.23,
                365.61,
                369.0,
                372.38,
                375.77,
                379.15,
                382.54,
                385.93,
                389.32,
                392.71,
                396.1,
                399.49,
                402.88,
                406.82,
                410.86,
                414.96,
                419.11,
                423.31,
                427.57,
                431.88,
                436.25,
                440.67,
                445.15,
                449.68,
                454.27,
                458.91,
                463.61,
                468.36,
                473.16,
                478.02,
                482.94,
                487.91,
                493.3,
                498.92,
                561.89,
                562.89,
                563.88,
                564.85,
                565.82,
                566.76,
                567.69,
                568.61,
                569.51,
                568.61,
                567.05,
                565.51,
                563.99,
                562.5,
                561.03,
                559.59,
                558.17,
                556.78,
                555.41,
                554.07,
                552.75,
                551.46,
                550.19,
                548.94,
                547.73,
                546.53,
                545.36,
                544.22,
                543.7,
                543.73,
                543.76,
                543.81,
                543.86,
                543.92,
                543.99,
                544.06,
                544.15,
                544.24,
                544.34,
                544.45,
                544.57,
                544.7,
                544.83,
                544.97,
                545.13,
                545.29,
                545.45,
                546.11,
                547.75,
                549.39,
                551.04,
                552.7,
                554.36,
                556.02,
                557.69,
                559.36,
                561.03,
                562.71,
                564.4,
                566.08,
                567.78,
                569.47,
                571.18,
                572.88,
                574.59,
                576.31,
                577.87,
                578.32,
                578.78,
                579.24,
                579.69,
                580.15,
                580.6,
                581.05,
                581.51,
                581.96,
                582.41,
                582.86,
                583.31,
                583.76,
                584.22,
                584.67,
                585.11,
                585.56,
                586.01,
                586.46,
                588.97,
                591.67,
                594.37,
                597.08,
                599.8,
                602.52,
                605.24,
                607.97,
                610.7,
                613.43,
                616.17,
                618.92,
                621.67,
                624.42,
                627.18,
                629.94,
                632.71,
                635.48,
                638.26,
                635.4,
                630.14,
                624.85,
                619.52,
                614.15,
                608.75,
                603.31,
                597.83,
                592.31,
                586.76,
                581.17,
                575.54,
                569.87,
                564.17,
                558.43,
                552.65,
                546.83,
                540.97,
                535.08,
                530.9,
                528.52,
                526.13,
                523.72,
                521.31,
                518.88,
                516.45,
                514.0,
                511.54,
                509.07,
                506.59,
                504.1,
                501.59,
                499.08,
                496.55,
                494.01,
                491.46,
                488.9,
                486.32,
                484.01,
                482.45,
                480.92,
                479.53,
                478.13,
                476.72,
                475.3,
                473.88,
                472.45,
                471.01,
                469.57,
                468.12,
                466.66,
                465.2,
                463.72,
                462.25,
                460.76,
                459.27,
                457.77,
                456.42,
                458.09,
                459.75,
                461.39,
                463.03,
                464.65,
                466.27,
                467.87,
                469.46,
                471.04,
                472.6,
                474.16,
                475.7,
                477.23,
                478.75,
                480.26,
                481.75,
                483.24,
                484.71,
                486.17,
                482.72,
                478.23,
                473.76,
                469.3,
                464.86,
                460.43,
                456.02,
                451.62,
                447.24,
                442.87,
                438.52,
                434.19,
                429.87,
                425.57,
                421.28,
                417.01,
                412.75,
                408.51,
                404.29,
                401.26,
                399.02,
                396.76,
                394.49,
                392.19,
                389.87,
                387.52,
                385.16,
                382.77,
                380.36,
                377.93,
                375.48,
                373.01,
                370.51,
                367.99,
                365.45,
                362.89,
                360.31,
                357.7,
                356.1,
                356.31,
                356.52,
                356.74,
                356.97,
                357.2,
                357.43,
                357.67,
                357.91,
                358.16,
                358.41,
                358.66,
                358.92,
                359.19,
                359.46,
                359.73,
                360.01,
                360.29,
                360.58,
                360.67,
                359.28,
                357.93,
                356.6,
                355.29,
                354.01,
                352.76,
                351.54,
                350.34,
                349.17,
                348.03,
                346.91,
                345.83,
                344.76,
                343.73,
                342.72,
                341.74,
                340.78,
                339.86,
                338.96,
                341.83,
                345.22,
                348.61,
                352.0,
                355.39,
                358.79,
                362.18,
                365.57,
                368.97,
                372.36,
                375.76,
                379.15,
                382.55,
                385.95,
                389.35,
                392.75,
                396.14,
                399.55,
                402.95,
                407.26,
                412.15,
                417.09,
                422.09,
                427.14,
                432.24,
                437.4,
                442.62,
                447.89,
                453.21,
                458.59,
                464.02,
                469.51,
                475.05,
                480.64,
                486.3,
                492.0,
                497.76,
                503.58,
                509.75,
                516.48,
                558.73,
                559.46,
                560.18,
                560.88,
                561.57,
                562.25,
                562.91,
                563.56,
                564.19,
                563.97,
                562.87,
                561.79,
                560.74,
                559.71,
                558.71,
                557.73,
                556.78,
                555.85,
                554.95,
                554.07,
                553.22,
                552.39,
                551.58,
                550.81,
                550.05,
                549.32,
                548.62,
                547.94,
                547.52,
                547.7,
                547.88,
                548.08,
                548.28,
                548.49,
                548.71,
                548.94,
                549.17,
                549.42,
                549.67,
                549.93,
                550.2,
                550.48,
                550.76,
                551.06,
                551.36,
                551.67,
                551.99,
                552.43,
                554.14,
                555.86,
                557.59,
                559.32,
                561.05,
                562.79,
                564.53,
                566.28,
                568.03,
                569.78,
                571.54,
                573.31,
                575.07,
                576.85,
                578.62,
                580.4,
                582.19,
                583.98,
                585.77,
                586.39,
                586.83,
                587.28,
                587.72,
                588.17,
                588.61,
                589.05,
                589.49,
                589.94,
                590.38,
                590.82,
                591.26,
                591.7,
                592.14,
                592.58,
                593.02,
                593.45,
                593.89,
                594.33,
                596.31,
                599.09,
                601.87,
                604.65,
                607.44,
                610.23,
                613.03,
                615.83,
                618.64,
                621.45,
                624.27,
                627.09,
                629.91,
                632.74,
                635.57,
                638.41,
                641.25,
                644.1,
                646.95,
                645.9,
                639.98,
                634.04,
                628.05,
                622.03,
                615.97,
                609.59,
                603.21,
                596.88,
                590.58,
                584.32,
                578.1,
                571.93,
                565.79,
                559.69,
                553.63,
                547.61,
                541.63,
                535.69,
                530.59,
                528.19,
                525.78,
                523.36,
                520.93,
                518.48,
                516.02,
                513.55,
                511.07,
                508.58,
                506.07,
                503.56,
                501.03,
                498.49,
                495.94,
                493.37,
                490.8,
                488.21,
                485.61,
                483.0,
                481.51,
                480.0,
                478.49,
                476.97,
                475.44,
                473.91,
                472.37,
                470.82,
                469.26,
                467.7,
                466.13,
                464.56,
                462.97,
                461.38,
                459.79,
                458.18,
                456.57,
                454.95,
                453.33,
                454.13,
                455.59,
                457.05,
                458.49,
                459.92,
                461.34,
                462.74,
                464.14,
                465.52,
                466.9,
                468.26,
                469.61,
                470.94,
                472.27,
                473.58,
                474.88,
                476.18,
                477.45,
                478.72,
                476.94,
                472.71,
                468.5,
                464.3,
                460.11,
                455.94,
                451.79,
                447.65,
                443.53,
                439.42,
                435.33,
                431.26,
                427.2,
                423.15,
                419.12,
                415.11,
                411.11,
                407.13,
                403.16,
                399.65,
                397.06,
                394.45,
                391.82,
                389.16,
                386.49,
                383.79,
                381.07,
                378.33,
                375.57,
                372.79,
                369.98,
                367.15,
                364.3,
                361.43,
                358.54,
                355.62,
                352.68,
                349.73,
                347.01,
                347.3,
                347.59,
                347.88,
                348.18,
                348.49,
                348.79,
                349.11,
                349.42,
                349.75,
                350.07,
                350.4,
                350.74,
                351.08,
                351.42,
                351.77,
                352.13,
                352.48,
                352.85,
                353.21,
                352.47,
                351.55,
                350.64,
                349.77,
                348.92,
                348.1,
                347.31,
                346.54,
                345.8,
                345.08,
                344.4,
                343.74,
                343.1,
                342.5,
                341.92,
                341.37,
                340.84,
                340.34,
                339.87,
                341.71,
                345.11,
                348.51,
                351.91,
                355.31,
                358.72,
                362.12,
                365.52,
                368.93,
                372.33,
                375.74,
                379.15,
                382.55,
                385.96,
                389.37,
                392.78,
                396.19,
                399.6,
                403.01,
                407.21,
                412.94,
                418.73,
                424.57,
                430.47,
                436.42,
                442.43,
                448.49,
                454.6,
                460.77,
                466.99,
                473.27,
                479.6,
                485.99,
                492.43,
                498.93,
                505.48,
                512.09,
                518.75,
                525.56,
                533.4,
                555.69,
                556.16,
                556.61,
                557.04,
                557.46,
                557.87,
                558.26,
                558.64,
                559.0,
                559.11,
                558.47,
                557.86,
                557.27,
                556.71,
                556.17,
                555.65,
                555.17,
                554.7,
                554.26,
                553.85,
                553.46,
                553.09,
                552.75,
                552.44,
                552.15,
                551.88,
                551.64,
                551.43,
                551.26,
                551.59,
                551.93,
                552.27,
                552.63,
                552.99,
                553.36,
                553.74,
                554.12,
                554.52,
                554.92,
                555.33,
                555.75,
                556.18,
                556.62,
                557.06,
                557.52,
                557.98,
                558.45,
                558.93,
                560.5,
                562.3,
                564.1,
                565.9,
                567.71,
                569.52,
                571.34,
                573.16,
                574.99,
                576.82,
                578.65,
                580.49,
                582.33,
                584.18,
                586.03,
                587.89,
                589.75,
                591.61,
                593.48,
                594.46,
                594.89,
                595.33,
                595.76,
                596.19,
                596.62,
                597.06,
                597.49,
                597.59,
                597.44,
                597.31,
                597.21,
                597.14,
                597.09,
                597.07,
                597.07,
                597.1,
                597.15,
                597.23,
                598.23,
                600.56,
                602.88,
                605.2,
                607.51,
                609.81,
                612.11,
                614.41,
                616.7,
                618.99,
                621.27,
                623.54,
                625.81,
                628.08,
                630.34,
                632.59,
                634.84,
                637.09,
                639.33,
                640.01,
                634.09,
                628.21,
                622.36,
                616.56,
                610.8,
                605.07,
                599.38,
                593.74,
                588.13,
                582.56,
                577.04,
                571.55,
                566.1,
                560.69,
                555.32,
                549.99,
                544.7,
                539.45,
                534.24,
                531.56,
                528.94,
                526.32,
                523.68,
                521.03,
                518.37,
                515.7,
                513.01,
                510.31,
                507.6,
                504.88,
                502.15,
                499.41,
                496.65,
                493.88,
                491.1,
                488.31,
                485.51,
                482.69,
                480.76,
                479.14,
                477.51,
                475.87,
                474.23,
                472.57,
                470.91,
                469.25,
                467.58,
                465.9,
                464.21,
                462.52,
                460.82,
                459.11,
                457.39,
                455.67,
                453.94,
                452.21,
                450.46,
                450.27,
                451.54,
                452.8,
                454.05,
                455.29,
                456.51,
                457.73,
                458.93,
                460.12,
                461.3,
                462.46,
                463.62,
                464.76,
                465.89,
                467.01,
                468.12,
                469.22,
                470.3,
                471.38,
                471.02,
                467.05,
                463.1,
                459.15,
                455.23,
                451.32,
                447.42,
                443.54,
                439.68,
                435.83,
                432.0,
                428.18,
                424.38,
                420.6,
                416.82,
                413.07,
                409.33,
                405.61,
                401.9,
                398.24,
                395.3,
                392.33,
                389.35,
                386.34,
                383.31,
                380.26,
                377.19,
                374.09,
                370.97,
                367.84,
                364.68,
                361.5,
                358.29,
                355.07,
                351.82,
                348.55,
                345.26,
                341.95,
                338.61,
                338.24,
                338.61,
                338.98,
                339.35,
                339.73,
                340.12,
                340.5,
                340.9,
                341.29,
                341.7,
                342.1,
                342.51,
                342.93,
                343.35,
                343.77,
                344.2,
                344.63,
                345.07,
                345.51,
                345.42,
                344.92,
                344.45,
                344.0,
                343.58,
                343.19,
                342.83,
                342.49,
                342.18,
                341.89,
                341.64,
                341.4,
                341.2,
                341.02,
                340.87,
                340.75,
                340.66,
                340.59,
                340.54,
                341.59,
                345.0,
                348.41,
                351.82,
                355.23,
                358.64,
                362.06,
                365.47,
                368.88,
                372.3,
                375.72,
                379.13,
                382.55,
                385.97,
                389.39,
                392.81,
                396.23,
                399.65,
                403.07,
                406.67,
                413.25,
                419.88,
                426.57,
                433.31,
                440.11,
                446.96,
                453.86,
                460.82,
                467.84,
                474.9,
                482.03,
                489.21,
                496.44,
                503.73,
                511.07,
                518.47,
                525.92,
                533.42,
                540.99,
                549.66,
                552.79,
                552.99,
                553.16,
                553.33,
                553.48,
                553.62,
                553.74,
                553.85,
                553.94,
                554.02,
                553.84,
                553.7,
                553.57,
                553.47,
                553.4,
                553.35,
                553.33,
                553.33,
                553.35,
                553.4,
                553.47,
                553.57,
                553.7,
                553.85,
                554.02,
                554.22,
                554.44,
                554.69,
                554.97,
                555.41,
                555.9,
                556.39,
                556.9,
                557.41,
                557.93,
                558.46,
                559.0,
                559.54,
                560.06,
                560.32,
                560.55,
                560.76,
                560.94,
                561.08,
                561.21,
                561.3,
                561.37,
                561.4,
                562.16,
                563.43,
                564.68,
                565.91,
                567.11,
                568.3,
                569.46,
                570.6,
                571.72,
                572.81,
                573.89,
                574.94,
                575.98,
                576.99,
                577.97,
                578.94,
                579.89,
                580.81,
                581.71,
                582.3,
                582.4,
                582.53,
                582.69,
                582.87,
                583.07,
                583.3,
                583.55,
                583.83,
                584.14,
                584.46,
                584.82,
                585.2,
                585.6,
                586.03,
                586.48,
                586.96,
                587.47,
                588.0,
                588.81,
                591.05,
                593.28,
                595.51,
                597.74,
                599.96,
                602.17,
                604.38,
                606.58,
                608.78,
                610.98,
                613.17,
                615.35,
                617.53,
                619.7,
                621.87,
                624.03,
                626.19,
                628.34,
                630.49,
                625.76,
                620.57,
                615.42,
                610.3,
                605.23,
                600.19,
                595.2,
                590.24,
                585.33,
                580.45,
                575.61,
                570.81,
                566.05,
                561.33,
                556.65,
                552.02,
                547.42,
                542.86,
                538.34,
                535.03,
                532.21,
                529.38,
                526.54,
                523.69,
                520.82,
                517.95,
                515.06,
                512.16,
                509.24,
                506.32,
                503.38,
                500.43,
                497.47,
                494.5,
                491.52,
                488.52,
                485.51,
                482.49,
                480.08,
                478.34,
                476.59,
                474.84,
                473.07,
                471.3,
                469.53,
                467.74,
                465.95,
                464.15,
                462.35,
                460.54,
                458.72,
                456.89,
                455.06,
                453.22,
                451.37,
                449.52,
                447.66,
                446.52,
                447.6,
                448.66,
                449.72,
                450.76,
                451.79,
                452.81,
                453.82,
                454.82,
                455.8,
                456.77,
                457.74,
                458.69,
                459.62,
                460.55,
                461.47,
                462.37,
                463.26,
                464.14,
                464.96,
                461.25,
                457.55,
                453.87,
                450.2,
                446.55,
                442.91,
                439.29,
                435.69,
                432.1,
                428.52,
                424.97,
                421.42,
                417.9,
                414.38,
                410.89,
                407.41,
                403.94,
                400.49,
                397.06,
                393.73,
                390.41,
                387.07,
                383.71,
                380.33,
                376.92,
                373.5,
                370.05,
                366.58,
                363.09,
                359.57,
                356.04,
                352.48,
                348.9,
                345.3,
                341.68,
                338.03,
                334.37,
                330.68,
                329.15,
                329.59,
                330.03,
                330.48,
                330.94,
                331.4,
                331.86,
                332.33,
                332.8,
                333.27,
                333.76,
                334.24,
                334.73,
                335.23,
                335.73,
                336.23,
                336.74,
                337.25,
                337.77,
                338.12,
                338.05,
                338.01,
                337.99,
                338.0,
                338.04,
                338.1,
                338.19,
                338.31,
                338.46,
                338.63,
                338.83,
                339.05,
                339.3,
                339.58,
                339.89,
                340.22,
                340.58,
                340.97,
                341.46,
                344.88,
                348.3,
                351.72,
                355.14,
                358.56,
                361.99,
                365.41,
                368.84,
                372.26,
                375.69,
                379.11,
                382.54,
                385.97,
                389.4,
                392.83,
                396.26,
                399.69,
                403.12,
                406.55,
                413.05,
                420.53,
                428.07,
                435.65,
                443.3,
                450.99,
                458.74,
                466.55,
                474.41,
                482.32,
                490.29,
                498.31,
                506.39,
                514.52,
                522.71,
                530.95,
                539.25,
                547.6,
                556.01,
                565.27,
                550.02,
                549.9,
                549.76,
                549.63,
                549.49,
                549.35,
                549.22,
                549.08,
                548.94,
                548.8,
                548.95,
                549.21,
                549.47,
                549.76,
                550.05,
                550.36,
                550.68,
                551.02,
                551.37,
                551.73,
                552.11,
                552.5,
                552.9,
                553.32,
                553.75,
                554.2,
                554.66,
                555.13,
                555.62,
                555.86,
                555.86,
                555.83,
                555.78,
                555.7,
                555.59,
                555.45,
                555.29,
                555.1,
                554.88,
                554.63,
                554.36,
                554.05,
                553.73,
                553.37,
                552.98,
                552.57,
                552.13,
                551.66,
                551.64,
                552.52,
                553.38,
                554.22,
                555.03,
                555.83,
                556.6,
                557.35,
                558.08,
                558.79,
                559.48,
                560.15,
                560.79,
                561.41,
                562.01,
                562.59,
                563.15,
                563.68,
                564.2,
                564.7,
                565.25,
                565.83,
                566.44,
                567.07,
                567.73,
                568.41,
                569.12,
                569.85,
                570.6,
                571.39,
                572.19,
                573.02,
                573.88,
                574.76,
                575.67,
                576.6,
                577.56,
                578.54,
                579.54,
                581.58,
                583.73,
                585.87,
                588.01,
                590.14,
                592.27,
                594.4,
                596.51,
                598.63,
                600.73,
                602.83,
                604.93,
                607.02,
                609.11,
                611.19,
                613.27,
                615.34,
                617.41,
                619.47,
                617.08,
                612.58,
                608.11,
                603.69,
                599.3,
                594.96,
                590.65,
                586.39,
                582.16,
                577.97,
                573.82,
                569.71,
                565.64,
                561.62,
                557.63,
                553.68,
                549.76,
                545.89,
                542.06,
                538.61,
                535.59,
                532.56,
                529.51,
                526.45,
                523.38,
                520.3,
                517.21,
                514.11,
                510.99,
                507.86,
                504.72,
                501.57,
                498.4,
                495.23,
                492.04,
                488.84,
                485.63,
                482.41,
                479.47,
                477.61,
                475.74,
                473.87,
                471.99,
                470.1,
                468.2,
                466.3,
                464.39,
                462.48,
                460.55,
                458.62,
                456.69,
                454.74,
                452.79,
                450.84,
                448.87,
                446.9,
                444.92,
                442.93,
                443.76,
                444.63,
                445.49,
                446.34,
                447.18,
                448.0,
                448.82,
                449.62,
                450.41,
                451.19,
                451.96,
                452.72,
                453.46,
                454.19,
                454.91,
                455.62,
                456.32,
                457.01,
                457.68,
                455.31,
                451.87,
                448.44,
                445.03,
                441.64,
                438.26,
                434.9,
                431.55,
                428.22,
                424.91,
                421.61,
                418.32,
                415.05,
                411.8,
                408.56,
                405.34,
                402.13,
                398.94,
                395.77,
                392.36,
                388.69,
                384.99,
                381.28,
                377.54,
                373.78,
                370.0,
                366.2,
                362.38,
                358.53,
                354.66,
                350.77,
                346.86,
                342.93,
                338.98,
                335.0,
                331.0,
                326.98,
                322.94,
                320.01,
                320.52,
                321.04,
                321.57,
                322.1,
                322.63,
                323.17,
                323.71,
                324.26,
                324.81,
                325.37,
                325.93,
                326.49,
                327.06,
                327.64,
                328.22,
                328.8,
                329.39,
                329.98,
                330.58,
                330.94,
                331.32,
                331.74,
                332.18,
                332.64,
                333.13,
                333.65,
                334.2,
                334.77,
                335.37,
                336.0,
                336.66,
                337.34,
                338.04,
                338.78,
                339.54,
                340.33,
                341.15,
                341.99,
                344.75,
                348.18,
                351.61,
                355.04,
                358.48,
                361.91,
                365.34,
                368.78,
                372.22,
                375.65,
                379.09,
                382.53,
                385.96,
                389.4,
                392.84,
                396.28,
                399.72,
                403.16,
                406.61,
                412.37,
                420.7,
                429.07,
                437.51,
                445.99,
                454.54,
                463.13,
                471.78,
                480.48,
                489.24,
                498.06,
                506.92,
                515.85,
                524.82,
                533.86,
                542.94,
                552.08,
                561.28,
                570.53,
                580.23,
                544.82,
                544.69,
                544.55,
                544.41,
                544.28,
                544.14,
                544.0,
                543.86,
                543.73,
                543.59,
                543.79,
                544.3,
                544.82,
                545.36,
                545.92,
                546.48,
                547.06,
                547.66,
                548.27,
                548.89,
                549.52,
                550.17,
                550.84,
                551.51,
                552.21,
                552.91,
                553.63,
                554.36,
                555.11,
                555.48,
                554.98,
                554.45,
                553.88,
                553.3,
                552.68,
                552.04,
                551.37,
                550.67,
                549.94,
                549.19,
                548.41,
                547.6,
                546.77,
                545.9,
                545.01,
                544.09,
                543.14,
                542.17,
                541.3,
                541.8,
                542.27,
                542.72,
                543.15,
                543.55,
                543.94,
                544.3,
                544.65,
                544.97,
                545.27,
                545.54,
                545.8,
                546.03,
                546.25,
                546.44,
                546.61,
                546.75,
                546.88,
                546.98,
                547.87,
                548.91,
                549.97,
                551.05,
                552.16,
                553.29,
                554.45,
                555.64,
                556.84,
                558.08,
                559.34,
                560.62,
                561.93,
                563.26,
                564.62,
                566.0,
                567.41,
                568.85,
                570.31,
                572.16,
                574.22,
                576.28,
                578.33,
                580.38,
                582.42,
                584.45,
                586.49,
                588.51,
                590.53,
                592.55,
                594.56,
                596.56,
                598.56,
                600.56,
                602.55,
                604.54,
                606.52,
                608.49,
                608.04,
                604.22,
                600.45,
                596.72,
                593.02,
                589.36,
                585.75,
                582.17,
                578.63,
                575.13,
                571.68,
                568.26,
                564.88,
                561.54,
                558.23,
                554.97,
                551.75,
                548.57,
                545.43,
                542.3,
                539.08,
                535.84,
                532.59,
                529.33,
                526.05,
                522.77,
                519.47,
                516.16,
                512.84,
                509.51,
                506.17,
                502.81,
                499.44,
                496.06,
                492.67,
                489.27,
                485.85,
                482.43,
                478.99,
                476.94,
                474.95,
                472.96,
                470.96,
                468.95,
                466.94,
                464.92,
                462.9,
                460.86,
                458.82,
                456.77,
                454.72,
                452.66,
                450.59,
                448.51,
                446.43,
                444.34,
                442.24,
                440.14,
                440.02,
                440.7,
                441.37,
                442.02,
                442.67,
                443.3,
                443.92,
                444.53,
                445.13,
                445.71,
                446.29,
                446.85,
                447.4,
                447.94,
                448.47,
                448.99,
                449.49,
                449.98,
                450.46,
                449.22,
                446.04,
                442.88,
                439.73,
                436.59,
                433.47,
                430.37,
                427.28,
                424.21,
                421.15,
                418.11,
                415.08,
                412.07,
                409.07,
                406.09,
                403.13,
                400.18,
                397.25,
                394.33,
                391.18,
                387.16,
                383.11,
                379.04,
                374.95,
                370.84,
                366.7,
                362.55,
                358.37,
                354.17,
                349.95,
                345.71,
                341.45,
                337.16,
                332.85,
                328.52,
                324.17,
                319.8,
                315.4,
                310.98,
                311.42,
                312.02,
                312.61,
                313.22,
                313.83,
                314.44,
                315.06,
                315.68,
                316.31,
                316.94,
                317.57,
                318.21,
                318.86,
                319.51,
                320.16,
                320.82,
                321.48,
                322.15,
                322.82,
                323.58,
                324.39,
                325.24,
                326.1,
                327.0,
                327.92,
                328.87,
                329.85,
                330.85,
                331.88,
                332.93,
                334.01,
                335.12,
                336.26,
                337.42,
                338.62,
                339.83,
                341.08,
                342.35,
                344.62,
                348.06,
                351.5,
                354.94,
                358.39,
                361.83,
                365.27,
                368.72,
                372.16,
                375.61,
                379.06,
                382.5,
                385.95,
                389.4,
                392.85,
                396.3,
                399.75,
                403.2,
                406.66,
                411.2,
                420.37,
                429.59,
                438.87,
                448.2,
                457.58,
                467.02,
                476.52,
                486.07,
                495.67,
                505.33,
                515.04,
                524.81,
                534.63,
                544.5,
                554.44,
                564.42,
                574.7,
                585.21,
                595.79,
                539.61,
                539.47,
                539.34,
                539.2,
                539.06,
                538.92,
                538.79,
                538.65,
                538.51,
                538.37,
                538.49,
                539.26,
                540.05,
                540.84,
                541.66,
                542.48,
                543.32,
                544.17,
                545.04,
                545.92,
                546.82,
                547.73,
                548.65,
                549.58,
                550.53,
                551.5,
                552.47,
                553.47,
                554.47,
                555.36,
                554.35,
                553.31,
                552.24,
                551.15,
                550.02,
                548.88,
                547.7,
                546.49,
                545.26,
                544.0,
                542.72,
                541.4,
                540.06,
                538.69,
                537.29,
                535.86,
                534.41,
                532.93,
                531.42,
                531.27,
                531.35,
                531.41,
                531.46,
                531.47,
                531.47,
                531.45,
                531.4,
                531.34,
                531.25,
                531.14,
                531.0,
                530.85,
                530.68,
                530.48,
                530.26,
                530.02,
                529.76,
                529.47,
                530.27,
                531.75,
                533.26,
                534.8,
                536.36,
                537.95,
                539.56,
                541.19,
                542.85,
                544.54,
                546.25,
                547.98,
                549.74,
                551.53,
                553.34,
                555.18,
                557.04,
                558.92,
                560.83,
                562.78,
                564.76,
                566.73,
                568.69,
                570.65,
                572.61,
                574.56,
                576.5,
                578.44,
                580.38,
                582.31,
                584.23,
                586.15,
                588.06,
                589.97,
                591.88,
                593.77,
                595.67,
                597.56,
                598.64,
                595.51,
                592.43,
                589.38,
                586.38,
                583.41,
                580.48,
                577.6,
                574.75,
                571.94,
                569.17,
                566.44,
                563.75,
                561.09,
                558.48,
                555.91,
                553.38,
                550.89,
                548.43,
                546.02,
                542.67,
                539.23,
                535.77,
                532.31,
                528.83,
                525.34,
                521.84,
                518.33,
                514.81,
                511.27,
                507.72,
                504.16,
                500.59,
                497.01,
                493.41,
                489.8,
                486.19,
                482.56,
                478.91,
                476.33,
                474.22,
                472.11,
                470.0,
                467.87,
                465.74,
                463.61,
                461.46,
                459.31,
                457.15,
                454.99,
                452.81,
                450.63,
                448.45,
                446.25,
                444.05,
                441.85,
                439.63,
                437.41,
                436.39,
                436.87,
                437.35,
                437.81,
                438.26,
                438.7,
                439.13,
                439.54,
                439.95,
                440.34,
                440.72,
                441.09,
                441.45,
                441.79,
                442.13,
                442.45,
                442.76,
                443.06,
                443.35,
                443.0,
                440.07,
                437.17,
                434.27,
                431.4,
                428.54,
                425.69,
                422.86,
                420.05,
                417.25,
                414.46,
                411.69,
                408.94,
                406.2,
                403.48,
                400.78,
                398.09,
                395.41,
                392.75,
                390.11,
                385.82,
                381.42,
                377.0,
                372.56,
                368.09,
                363.6,
                359.1,
                354.57,
                350.01,
                345.44,
                340.84,
                336.23,
                331.59,
                326.93,
                322.24,
                317.54,
                312.81,
                308.06,
                303.29,
                302.27,
                302.94,
                303.62,
                304.3,
                304.98,
                305.67,
                306.36,
                307.06,
                307.76,
                308.47,
                309.18,
                309.89,
                310.61,
                311.34,
                312.06,
                312.8,
                313.53,
                314.28,
                315.02,
                315.98,
                317.22,
                318.49,
                319.79,
                321.11,
                322.46,
                323.84,
                325.25,
                326.68,
                328.13,
                329.62,
                331.13,
                332.67,
                334.23,
                335.82,
                337.44,
                339.09,
                340.76,
                342.46,
                344.48,
                347.93,
                351.38,
                354.84,
                358.3,
                361.84,
                365.41,
                369.03,
                372.67,
                376.36,
                380.08,
                383.84,
                387.63,
                391.46,
                395.33,
                399.23,
                403.17,
                407.14,
                411.15,
                415.2,
                425.0,
                435.48,
                446.02,
                456.63,
                467.29,
                478.02,
                488.81,
                499.67,
                510.58,
                521.56,
                532.61,
                543.71,
                554.88,
                566.12,
                577.41,
                588.77,
                600.19,
                611.68,
                623.23,
                534.39,
                534.26,
                534.12,
                533.98,
                533.85,
                533.71,
                533.57,
                533.44,
                533.3,
                533.16,
                533.07,
                534.1,
                535.14,
                536.2,
                537.27,
                538.36,
                539.45,
                540.57,
                541.69,
                542.83,
                543.98,
                545.15,
                546.33,
                547.53,
                548.73,
                549.96,
                551.19,
                552.44,
                553.71,
                554.98,
                553.97,
                552.42,
                550.85,
                549.25,
                547.62,
                545.96,
                544.28,
                542.57,
                540.83,
                539.06,
                537.27,
                535.45,
                533.6,
                531.72,
                529.82,
                527.89,
                525.93,
                523.94,
                521.93,
                520.93,
                520.63,
                520.3,
                519.96,
                519.59,
                519.2,
                518.79,
                518.35,
                517.9,
                517.42,
                516.92,
                516.4,
                515.86,
                515.3,
                514.71,
                514.11,
                513.48,
                512.83,
                512.16,
                512.43,
                514.37,
                516.33,
                518.32,
                520.33,
                522.37,
                524.43,
                526.52,
                528.63,
                530.77,
                532.93,
                535.12,
                537.33,
                539.57,
                541.83,
                544.12,
                546.43,
                548.77,
                551.13,
                553.45,
                555.34,
                557.22,
                559.1,
                560.97,
                562.84,
                564.71,
                566.56,
                568.42,
                570.27,
                572.11,
                573.95,
                575.78,
                577.61,
                579.43,
                581.25,
                583.06,
                584.87,
                586.67,
                588.46,
                586.44,
                584.05,
                581.69,
                579.38,
                577.1,
                574.86,
                572.66,
                570.5,
                568.38,
                566.3,
                564.26,
                562.26,
                560.29,
                558.37,
                556.49,
                554.64,
                552.84,
                551.08,
                549.35,
                546.37,
                542.72,
                539.06,
                535.4,
                531.72,
                528.02,
                524.32,
                520.6,
                516.87,
                513.14,
                509.38,
                505.62,
                501.85,
                498.06,
                494.26,
                490.45,
                486.63,
                482.79,
                478.95,
                475.78,
                473.56,
                471.33,
                469.1,
                466.86,
                464.61,
                462.35,
                460.09,
                457.82,
                455.55,
                453.26,
                450.97,
                448.67,
                446.37,
                444.06,
                441.74,
                439.42,
                437.08,
                434.74,
                432.86,
                433.15,
                433.43,
                433.7,
                433.96,
                434.21,
                434.44,
                434.66,
                434.87,
                435.07,
                435.26,
                435.44,
                435.6,
                435.75,
                435.89,
                436.02,
                436.14,
                436.25,
                436.34,
                436.42,
                433.97,
                431.32,
                428.68,
                426.06,
                423.46,
                420.87,
                418.3,
                415.74,
                413.2,
                410.68,
                408.17,
                405.67,
                403.19,
                400.73,
                398.28,
                395.85,
                393.43,
                391.03,
                388.65,
                384.69,
                379.93,
                375.16,
                370.36,
                365.54,
                360.7,
                355.84,
                350.95,
                346.05,
                341.12,
                336.17,
                331.2,
                326.21,
                321.2,
                316.16,
                311.1,
                306.02,
                300.92,
                295.8,
                293.08,
                293.83,
                294.58,
                295.33,
                296.09,
                296.85,
                297.62,
                298.39,
                299.17,
                299.95,
                300.81,
                301.84,
                302.86,
                303.88,
                304.89,
                305.89,
                306.89,
                307.89,
                308.87,
                309.97,
                311.73,
                313.5,
                315.27,
                317.05,
                318.83,
                320.61,
                322.41,
                324.21,
                326.01,
                327.82,
                329.63,
                331.45,
                333.28,
                335.11,
                336.94,
                338.78,
                340.63,
                342.48,
                344.34,
                348.06,
                352.06,
                356.1,
                360.17,
                364.28,
                368.42,
                372.6,
                376.81,
                381.07,
                385.35,
                389.68,
                394.04,
                398.44,
                402.87,
                407.34,
                411.84,
                416.38,
                420.96,
                425.58,
                434.37,
                445.82,
                457.34,
                468.91,
                480.55,
                492.25,
                504.02,
                515.84,
                527.73,
                539.69,
                551.7,
                563.78,
                575.92,
                588.13,
                600.39,
                612.72,
                625.12,
                637.58,
                650.1,
                529.18,
                529.04,
                528.91,
                528.77,
                528.63,
                528.5,
                528.36,
                528.22,
                528.08,
                527.95,
                527.81,
                528.81,
                530.12,
                531.43,
                532.76,
                534.1,
                535.46,
                536.83,
                538.21,
                539.61,
                541.02,
                542.45,
                543.89,
                545.34,
                546.81,
                548.29,
                549.78,
                551.29,
                552.81,
                554.35,
                553.83,
                551.78,
                549.7,
                547.59,
                545.46,
                543.3,
                541.11,
                538.89,
                536.65,
                534.38,
                532.08,
                529.75,
                527.39,
                525.01,
                522.6,
                520.16,
                517.7,
                515.21,
                512.68,
                510.79,
                510.1,
                509.39,
                508.65,
                507.9,
                507.12,
                506.32,
                505.5,
                504.65,
                503.79,
                502.9,
                502.0,
                501.07,
                500.12,
                499.15,
                498.15,
                497.14,
                496.1,
                495.04,
                494.37,
                496.76,
                499.17,
                501.61,
                504.08,
                506.57,
                509.08,
                511.62,
                514.18,
                516.77,
                519.39,
                522.03,
                524.69,
                527.38,
                530.09,
                532.83,
                535.6,
                538.39,
                541.2,
                544.04,
                545.96,
                547.76,
                549.55,
                551.34,
                553.12,
                554.9,
                556.67,
                558.44,
                560.2,
                561.95,
                563.71,
                565.45,
                567.19,
                568.93,
                570.66,
                572.39,
                574.11,
                575.82,
                577.53,
                577.02,
                575.31,
                573.65,
                572.02,
                570.43,
                568.88,
                567.37,
                565.9,
                564.46,
                563.07,
                561.72,
                560.41,
                559.13,
                557.9,
                556.7,
                555.55,
                554.43,
                553.36,
                552.32,
                550.17,
                546.32,
                542.46,
                538.59,
                534.71,
                530.81,
                526.9,
                522.98,
                519.05,
                515.11,
                511.15,
                507.19,
                503.21,
                499.22,
                495.22,
                491.2,
                487.18,
                483.14,
                479.09,
                475.3,
                472.96,
                470.61,
                468.26,
                465.9,
                463.54,
                461.16,
                458.78,
                456.4,
                454.0,
                451.6,
                449.19,
                446.78,
                444.36,
                441.93,
                439.49,
                437.05,
                434.6,
                432.14,
                429.68,
                429.54,
                429.62,
                429.7,
                429.76,
                429.82,
                429.86,
                429.89,
                429.9,
                429.91,
                429.9,
                429.89,
                429.86,
                429.82,
                429.77,
                429.7,
                429.63,
                429.54,
                429.44,
                429.33,
                427.72,
                425.32,
                422.95,
                420.59,
                418.24,
                415.91,
                413.6,
                411.3,
                409.02,
                406.75,
                404.5,
                402.26,
                400.04,
                397.83,
                395.64,
                393.47,
                391.29,
                389.11,
                386.91,
                383.57,
                378.51,
                373.46,
                368.44,
                363.42,
                358.42,
                353.43,
                348.46,
                343.51,
                338.56,
                333.64,
                328.72,
                323.82,
                318.94,
                314.07,
                309.21,
                304.37,
                299.55,
                294.73,
                290.72,
                291.71,
                292.69,
                293.67,
                294.65,
                295.62,
                296.58,
                297.53,
                298.48,
                299.43,
                300.37,
                301.3,
                302.23,
                303.15,
                304.06,
                304.97,
                305.87,
                306.77,
                307.66,
                308.54,
                310.26,
                312.11,
                313.97,
                315.83,
                317.7,
                319.57,
                321.45,
                323.33,
                325.22,
                327.11,
                329.01,
                330.92,
                332.83,
                334.74,
                336.67,
                338.59,
                340.52,
                342.46,
                344.4,
                347.9,
                352.47,
                357.07,
                361.71,
                366.39,
                371.1,
                375.84,
                380.63,
                385.45,
                390.3,
                395.19,
                400.12,
                405.08,
                410.08,
                415.12,
                420.19,
                425.3,
                430.44,
                435.62,
                443.18,
                455.6,
                468.08,
                480.63,
                493.24,
                505.92,
                518.65,
                531.45,
                544.31,
                557.24,
                570.22,
                583.27,
                596.39,
                609.56,
                622.8,
                636.11,
                649.47,
                662.9,
                676.4,
                523.97,
                523.83,
                523.69,
                523.56,
                523.42,
                523.28,
                523.14,
                523.01,
                522.87,
                522.73,
                522.59,
                523.4,
                524.96,
                526.54,
                528.12,
                529.73,
                531.34,
                532.97,
                534.61,
                536.27,
                537.94,
                539.62,
                541.32,
                543.03,
                544.76,
                546.5,
                548.25,
                550.02,
                551.8,
                553.59,
                553.95,
                551.39,
                548.8,
                546.19,
                543.55,
                540.88,
                538.19,
                535.47,
                532.72,
                529.94,
                527.13,
                524.3,
                521.44,
                518.55,
                515.63,
                512.69,
                509.72,
                506.72,
                503.69,
                500.84,
                499.76,
                498.66,
                497.54,
                496.4,
                495.23,
                494.04,
                492.84,
                491.61,
                490.35,
                489.08,
                487.79,
                486.47,
                485.13,
                483.77,
                482.39,
                480.99,
                479.56,
                478.12,
                476.65,
                478.92,
                481.78,
                484.68,
                487.59,
                490.53,
                493.5,
                496.49,
                499.51,
                502.55,
                505.61,
                508.7,
                511.82,
                514.96,
                518.12,
                521.31,
                524.53,
                527.77,
                531.04,
                534.33,
                536.63,
                538.34,
                540.05,
                541.75,
                543.45,
                545.14,
                546.82,
                548.5,
                550.18,
                551.85,
                553.51,
                555.17,
                556.83,
                558.48,
                560.12,
                561.76,
                563.39,
                565.02,
                566.65,
                567.24,
                566.22,
                565.24,
                564.3,
                563.4,
                562.54,
                561.72,
                560.93,
                560.19,
                559.49,
                558.82,
                558.2,
                557.61,
                557.06,
                556.56,
                556.09,
                555.66,
                555.28,
                554.93,
                554.09,
                550.03,
                545.97,
                541.89,
                537.81,
                533.71,
                529.59,
                525.47,
                521.34,
                517.19,
                513.03,
                508.86,
                504.68,
                500.49,
                496.28,
                492.06,
                487.83,
                483.59,
                479.34,
                475.07,
                472.42,
                469.96,
                467.49,
                465.01,
                462.53,
                460.04,
                457.54,
                455.03,
                452.52,
                450.0,
                447.48,
                444.94,
                442.41,
                439.86,
                437.3,
                434.74,
                432.18,
                429.6,
                427.02,
                426.03,
                425.91,
                425.79,
                425.7,
                425.61,
                425.54,
                425.48,
                425.43,
                425.4,
                425.38,
                425.38,
                425.38,
                425.41,
                425.44,
                425.49,
                425.55,
                425.63,
                425.71,
                425.82,
                425.05,
                422.86,
                420.65,
                418.44,
                416.21,
                413.98,
                411.73,
                409.47,
                407.2,
                404.92,
                402.63,
                400.32,
                398.01,
                395.69,
                393.35,
                391.0,
                388.64,
                386.27,
                383.89,
                381.21,
                376.38,
                371.58,
                366.79,
                362.01,
                357.24,
                352.5,
                347.76,
                343.04,
                338.33,
                333.64,
                328.97,
                324.3,
                319.66,
                315.02,
                310.4,
                305.8,
                301.21,
                296.64,
                292.07,
                292.19,
                293.08,
                293.96,
                294.84,
                295.71,
                296.58,
                297.44,
                298.29,
                299.14,
                299.98,
                300.81,
                301.64,
                302.47,
                303.28,
                304.1,
                304.9,
                305.7,
                306.49,
                307.28,
                308.74,
                310.68,
                312.62,
                314.57,
                316.52,
                318.48,
                320.44,
                322.41,
                324.38,
                326.36,
                328.35,
                330.34,
                332.33,
                334.33,
                336.34,
                338.35,
                340.37,
                342.39,
                344.42,
                347.42,
                352.56,
                357.72,
                362.93,
                368.17,
                373.45,
                378.76,
                384.11,
                389.5,
                394.92,
                400.38,
                405.87,
                411.4,
                416.97,
                422.57,
                428.21,
                433.88,
                439.59,
                445.34,
                451.41,
                464.81,
                478.27,
                491.79,
                505.37,
                519.01,
                532.72,
                546.49,
                560.32,
                574.22,
                588.18,
                602.2,
                616.29,
                630.43,
                644.64,
                658.92,
                673.26,
                687.66,
                702.12,
                518.75,
                518.61,
                518.48,
                518.34,
                518.2,
                518.07,
                517.93,
                517.79,
                517.66,
                517.52,
                517.38,
                517.87,
                519.68,
                521.52,
                523.36,
                525.22,
                527.1,
                528.98,
                530.88,
                532.8,
                534.73,
                536.67,
                538.62,
                540.59,
                542.58,
                544.58,
                546.59,
                548.61,
                550.65,
                552.7,
                554.31,
                551.25,
                548.16,
                545.04,
                541.89,
                538.72,
                535.52,
                532.29,
                529.03,
                525.75,
                522.44,
                519.1,
                515.73,
                512.34,
                508.92,
                505.47,
                501.99,
                498.49,
                494.95,
                491.39,
                489.62,
                488.13,
                486.62,
                485.09,
                483.54,
                481.96,
                480.37,
                478.75,
                477.11,
                475.45,
                473.77,
                472.07,
                470.34,
                468.59,
                466.82,
                465.03,
                463.22,
                461.39,
                459.53,
                460.85,
                464.17,
                467.51,
                470.88,
                474.27,
                477.69,
                481.13,
                484.6,
                488.09,
                491.61,
                495.15,
                498.72,
                502.31,
                505.92,
                509.57,
                513.23,
                516.92,
                520.64,
                524.38,
                527.35,
                528.97,
                530.59,
                532.2,
                533.81,
                535.42,
                537.02,
                538.61,
                540.2,
                541.78,
                543.36,
                544.93,
                546.5,
                548.07,
                549.63,
                551.18,
                552.73,
                554.27,
                555.81,
                557.1,
                556.77,
                556.48,
                556.22,
                556.01,
                555.84,
                555.71,
                555.61,
                555.56,
                555.54,
                555.56,
                555.63,
                555.73,
                555.87,
                556.05,
                556.27,
                556.53,
                556.83,
                557.17,
                557.55,
                553.85,
                549.58,
                545.3,
                541.01,
                536.67,
                532.01,
                527.37,
                522.74,
                518.12,
                513.51,
                508.92,
                504.35,
                499.79,
                495.24,
                490.7,
                486.18,
                481.67,
                477.18,
                472.7,
                469.37,
                466.74,
                464.11,
                461.49,
                458.87,
                456.27,
                453.67,
                451.08,
                448.49,
                445.92,
                443.35,
                440.79,
                438.23,
                435.69,
                433.15,
                430.62,
                428.1,
                425.58,
                423.07,
                421.53,
                421.63,
                421.75,
                421.87,
                422.01,
                422.16,
                422.33,
                422.51,
                422.7,
                422.91,
                423.13,
                423.36,
                423.61,
                423.87,
                424.14,
                424.43,
                424.73,
                425.05,
                425.37,
                425.41,
                423.04,
                420.65,
                418.26,
                415.85,
                413.43,
                411.0,
                408.56,
                406.11,
                403.64,
                401.17,
                398.68,
                396.19,
                393.68,
                391.16,
                388.63,
                386.09,
                383.54,
                380.98,
                378.41,
                374.13,
                369.56,
                365.0,
                360.46,
                355.94,
                351.42,
                346.93,
                342.44,
                337.97,
                333.52,
                329.08,
                324.65,
                320.24,
                315.85,
                311.46,
                307.1,
                302.74,
                298.4,
                294.08,
                292.72,
                293.52,
                294.3,
                295.08,
                295.86,
                296.63,
                297.39,
                298.15,
                298.9,
                299.64,
                300.38,
                301.12,
                301.84,
                302.56,
                303.28,
                303.99,
                304.69,
                305.39,
                306.08,
                307.17,
                309.19,
                311.22,
                313.26,
                315.29,
                317.34,
                319.39,
                321.44,
                323.5,
                325.56,
                327.63,
                329.71,
                331.79,
                333.88,
                335.97,
                338.06,
                340.17,
                342.27,
                344.39,
                346.61,
                352.31,
                358.05,
                363.82,
                369.63,
                375.47,
                381.35,
                387.27,
                393.22,
                399.21,
                405.23,
                411.29,
                417.39,
                423.52,
                429.69,
                435.9,
                442.14,
                448.41,
                454.73,
                461.08,
                473.45,
                487.88,
                502.37,
                516.92,
                531.54,
                546.22,
                560.96,
                575.77,
                590.63,
                605.56,
                620.56,
                635.61,
                650.73,
                665.91,
                681.16,
                696.47,
                711.84,
                727.27,
                513.54,
                513.4,
                513.26,
                513.13,
                512.99,
                512.85,
                512.72,
                512.58,
                512.44,
                512.3,
                512.17,
                512.2,
                514.28,
                516.37,
                518.48,
                520.59,
                522.73,
                524.87,
                527.03,
                529.2,
                531.39,
                533.59,
                535.8,
                538.03,
                540.27,
                542.53,
                544.8,
                547.08,
                549.38,
                551.69,
                554.01,
                551.35,
                547.76,
                544.13,
                540.48,
                536.8,
                533.1,
                529.36,
                525.6,
                521.81,
                517.99,
                514.15,
                510.28,
                506.38,
                502.45,
                498.5,
                494.51,
                490.5,
                486.47,
                482.4,
                479.67,
                477.79,
                475.9,
                473.98,
                472.04,
                470.08,
                468.09,
                466.09,
                464.06,
                462.02,
                459.95,
                457.86,
                455.74,
                453.61,
                451.45,
                449.28,
                447.08,
                444.85,
                442.61,
                442.55,
                446.32,
                450.12,
                453.94,
                457.78,
                461.65,
                465.54,
                469.46,
                473.4,
                477.37,
                481.37,
                485.38,
                489.43,
                493.49,
                497.59,
                501.71,
                505.85,
                510.02,
                514.21,
                518.1,
                519.64,
                521.17,
                522.7,
                524.22,
                525.74,
                527.26,
                528.76,
                530.25,
                531.69,
                533.11,
                534.52,
                535.91,
                537.29,
                538.66,
                540.01,
                541.35,
                542.67,
                543.98,
                545.27,
                545.49,
                545.55,
                545.62,
                545.71,
                545.79,
                545.89,
                545.99,
                546.1,
                546.21,
                546.34,
                546.47,
                546.6,
                546.75,
                546.9,
                547.06,
                547.22,
                547.4,
                547.58,
                547.77,
                545.06,
                540.58,
                536.12,
                531.67,
                527.24,
                522.82,
                518.41,
                514.02,
                509.64,
                505.27,
                500.92,
                496.58,
                492.25,
                487.94,
                483.64,
                479.36,
                475.09,
                470.83,
                466.58,
                462.98,
                460.47,
                457.97,
                455.48,
                453.0,
                450.52,
                448.05,
                445.59,
                443.13,
                440.69,
                438.25,
                435.82,
                433.39,
                430.97,
                428.57,
                426.16,
                423.77,
                421.38,
                419.0,
                416.91,
                417.24,
                417.57,
                417.93,
                418.29,
                418.67,
                419.06,
                419.47,
                419.89,
                420.32,
                420.76,
                421.22,
                421.69,
                422.18,
                422.68,
                423.19,
                423.72,
                424.26,
                424.81,
                425.37,
                423.32,
                420.75,
                418.17,
                415.58,
                412.98,
                410.37,
                407.75,
                405.11,
                402.47,
                399.81,
                397.14,
                394.47,
                391.78,
                389.08,
                386.37,
                383.64,
                380.91,
                378.17,
                375.41,
                371.74,
                367.41,
                363.09,
                358.79,
                354.5,
                350.22,
                345.96,
                341.71,
                337.48,
                333.26,
                329.06,
                324.87,
                320.7,
                316.54,
                312.39,
                308.26,
                304.14,
                300.04,
                295.95,
                293.31,
                294.01,
                294.7,
                295.38,
                296.06,
                296.73,
                297.4,
                298.06,
                298.72,
                299.36,
                300.01,
                300.64,
                301.27,
                301.9,
                302.52,
                303.13,
                303.74,
                304.34,
                304.93,
                305.56,
                307.66,
                309.78,
                311.89,
                314.02,
                316.15,
                318.28,
                320.42,
                322.56,
                324.71,
                326.87,
                329.03,
                331.2,
                333.37,
                335.55,
                337.73,
                339.92,
                342.11,
                344.31,
                346.51,
                351.75,
                358.05,
                364.39,
                370.76,
                377.17,
                383.62,
                390.1,
                396.62,
                403.17,
                409.76,
                416.39,
                423.05,
                429.75,
                436.48,
                443.26,
                450.06,
                456.91,
                463.79,
                470.7,
                481.53,
                496.93,
                512.39,
                527.92,
                543.5,
                559.15,
                574.87,
                590.64,
                606.48,
                622.38,
                638.34,
                654.37,
                670.46,
                686.61,
                702.83,
                719.11,
                735.45,
                751.86,
                508.32,
                508.19,
                508.05,
                507.91,
                507.78,
                507.64,
                507.5,
                507.36,
                507.23,
                507.09,
                506.95,
                506.81,
                508.75,
                511.1,
                513.46,
                515.84,
                518.23,
                520.63,
                523.05,
                525.48,
                527.93,
                530.39,
                532.86,
                535.34,
                537.84,
                540.36,
                542.88,
                545.43,
                547.98,
                550.55,
                553.13,
                551.71,
                547.61,
                543.48,
                539.32,
                535.13,
                530.92,
                526.68,
                522.42,
                518.12,
                513.8,
                509.45,
                505.07,
                500.67,
                496.24,
                491.78,
                487.29,
                482.77,
                478.23,
                473.66,
                469.91,
                467.65,
                465.36,
                463.06,
                460.73,
                458.38,
                456.01,
                453.62,
                451.21,
                448.77,
                446.38,
                444.08,
                441.8,
                439.53,
                437.28,
                435.04,
                432.81,
                430.6,
                428.4,
                426.97,
                431.32,
                435.67,
                440.0,
                444.33,
                448.65,
                452.95,
                457.25,
                461.54,
                465.82,
                470.08,
                474.34,
                478.59,
                482.83,
                487.06,
                491.28,
                495.48,
                499.68,
                503.87,
                508.05,
                509.73,
                511.02,
                512.3,
                513.56,
                514.8,
                516.04,
                517.25,
                518.46,
                519.65,
                520.82,
                521.98,
                523.13,
                524.26,
                525.38,
                526.48,
                527.57,
                528.65,
                529.71,
                530.76,
                531.27,
                531.47,
                531.67,
                531.88,
                532.09,
                532.32,
                532.55,
                532.79,
                533.03,
                533.28,
                533.54,
                533.81,
                534.08,
                534.36,
                534.65,
                534.94,
                535.25,
                535.56,
                535.87,
                534.55,
                530.32,
                526.09,
                521.88,
                517.68,
                513.5,
                509.33,
                505.17,
                501.03,
                496.9,
                492.78,
                488.68,
                484.59,
                480.52,
                476.46,
                472.41,
                468.37,
                464.35,
                460.35,
                456.51,
                454.14,
                451.77,
                449.4,
                447.05,
                444.7,
                442.36,
                440.03,
                437.7,
                435.39,
                433.08,
                430.77,
                428.48,
                426.19,
                423.91,
                421.64,
                419.37,
                417.12,
                414.87,
                412.62,
                412.72,
                413.28,
                413.86,
                414.45,
                415.05,
                415.67,
                416.3,
                416.94,
                417.6,
                418.27,
                418.96,
                419.65,
                420.36,
                421.09,
                421.83,
                422.58,
                423.34,
                424.12,
                424.91,
                423.7,
                420.95,
                418.19,
                415.41,
                412.63,
                409.84,
                407.03,
                404.22,
                401.39,
                398.55,
                395.71,
                392.85,
                389.97,
                387.09,
                384.2,
                381.3,
                378.38,
                375.45,
                372.52,
                369.23,
                365.13,
                361.05,
                356.98,
                352.93,
                348.89,
                344.86,
                340.85,
                336.86,
                332.87,
                328.91,
                324.95,
                321.02,
                317.09,
                313.18,
                309.29,
                305.41,
                301.54,
                297.69,
                293.96,
                294.56,
                295.15,
                295.74,
                296.32,
                296.9,
                297.47,
                298.03,
                298.59,
                299.14,
                299.68,
                300.22,
                300.76,
                301.29,
                301.81,
                302.32,
                302.83,
                303.34,
                303.84,
                304.33,
                306.08,
                308.28,
                310.49,
                312.69,
                314.91,
                317.13,
                319.35,
                321.58,
                323.82,
                326.06,
                328.3,
                330.56,
                332.81,
                335.07,
                337.34,
                339.61,
                341.89,
                344.18,
                346.47,
                350.85,
                357.72,
                364.63,
                371.57,
                378.54,
                385.56,
                392.6,
                399.69,
                406.81,
                413.96,
                421.16,
                428.38,
                435.65,
                442.95,
                450.29,
                457.66,
                465.07,
                472.52,
                480.0,
                489.04,
                505.41,
                521.84,
                538.34,
                554.9,
                571.52,
                588.2,
                604.95,
                621.76,
                638.63,
                655.56,
                672.56,
                689.62,
                706.74,
                723.93,
                741.18,
                758.49,
                775.87,
                503.11,
                502.97,
                502.83,
                502.7,
                502.56,
                502.42,
                502.29,
                502.15,
                502.01,
                501.88,
                501.74,
                501.6,
                503.1,
                505.71,
                508.33,
                510.96,
                513.61,
                516.27,
                518.95,
                521.64,
                524.34,
                527.06,
                529.79,
                532.35,
                534.67,
                536.97,
                539.27,
                541.55,
                543.83,
                546.09,
                548.34,
                548.02,
                543.15,
                538.3,
                533.47,
                528.67,
                523.89,
                519.13,
                514.4,
                509.69,
                505.01,
                500.35,
                495.72,
                491.11,
                486.53,
                481.97,
                477.43,
                472.92,
                468.43,
                463.97,
                459.79,
                457.6,
                455.44,
                453.28,
                451.14,
                449.01,
                446.9,
                444.8,
                442.71,
                440.63,
                438.57,
                436.53,
                434.49,
                432.47,
                430.46,
                428.47,
                426.49,
                424.53,
                422.57,
                420.63,
                423.99,
                428.16,
                432.33,
                436.48,
                440.62,
                444.76,
                448.88,
                453.0,
                457.1,
                461.2,
                465.28,
                469.36,
                473.42,
                477.48,
                481.52,
                485.56,
                489.59,
                493.6,
                497.61,
                499.8,
                500.84,
                501.87,
                502.88,
                503.88,
                504.87,
                505.84,
                506.79,
                507.74,
                508.66,
                509.58,
                510.48,
                511.36,
                512.23,
                513.09,
                513.93,
                514.76,
                515.57,
                516.37,
                516.99,
                517.31,
                517.65,
                517.98,
                518.33,
                518.68,
                519.04,
                519.41,
                519.78,
                520.16,
                520.55,
                520.94,
                521.35,
                521.76,
                522.17,
                522.6,
                523.03,
                523.47,
                523.91,
                523.92,
                519.92,
                515.94,
                511.96,
                508.0,
                504.06,
                500.12,
                496.2,
                492.3,
                488.4,
                484.52,
                480.66,
                476.81,
                472.97,
                469.15,
                465.33,
                461.54,
                457.75,
                453.98,
                450.23,
                447.73,
                445.49,
                443.26,
                441.03,
                438.82,
                436.6,
                434.4,
                432.21,
                430.02,
                427.84,
                425.66,
                423.5,
                421.34,
                419.19,
                417.04,
                414.91,
                412.78,
                410.66,
                408.55,
                408.08,
                408.87,
                409.67,
                410.49,
                411.31,
                412.16,
                413.01,
                413.88,
                414.76,
                415.66,
                416.57,
                417.49,
                418.43,
                419.38,
                420.34,
                421.31,
                422.3,
                423.31,
                424.32,
                424.18,
                421.24,
                418.3,
                415.35,
                412.38,
                409.41,
                406.42,
                403.42,
                400.42,
                397.4,
                394.37,
                391.32,
                388.27,
                385.21,
                382.13,
                379.05,
                375.95,
                372.84,
                369.72,
                366.58,
                362.72,
                358.87,
                355.04,
                351.22,
                347.42,
                343.63,
                339.86,
                336.1,
                332.35,
                328.62,
                324.91,
                321.21,
                317.52,
                313.85,
                310.19,
                306.54,
                302.92,
                299.3,
                295.7,
                295.16,
                295.66,
                296.15,
                296.63,
                297.11,
                297.58,
                298.05,
                298.51,
                298.97,
                299.42,
                299.86,
                300.3,
                300.73,
                301.16,
                301.57,
                301.99,
                302.4,
                302.8,
                303.19,
                304.46,
                306.74,
                309.03,
                311.32,
                313.62,
                315.93,
                318.24,
                320.55,
                322.87,
                325.2,
                327.53,
                329.87,
                332.21,
                334.55,
                336.91,
                339.27,
                341.63,
                344.0,
                346.37,
                349.63,
                357.07,
                364.54,
                372.05,
                379.59,
                387.17,
                394.78,
                402.43,
                410.12,
                417.84,
                425.6,
                433.39,
                441.22,
                449.09,
                456.99,
                464.93,
                472.9,
                480.92,
                488.96,
                497.05,
                513.33,
                530.73,
                548.2,
                565.73,
                583.32,
                600.97,
                618.69,
                636.47,
                654.31,
                672.21,
                690.18,
                708.21,
                726.3,
                744.46,
                762.68,
                780.96,
                799.31,
                493.68,
                493.27,
                492.86,
                492.45,
                492.04,
                491.63,
                491.22,
                490.81,
                490.4,
                489.98,
                489.57,
                489.16,
                489.7,
                491.94,
                494.16,
                496.38,
                498.58,
                500.77,
                502.95,
                505.12,
                507.28,
                509.43,
                511.56,
                513.69,
                515.8,
                517.9,
                519.99,
                522.07,
                524.14,
                526.2,
                528.24,
                529.54,
                525.11,
                520.72,
                516.34,
                511.99,
                507.66,
                503.36,
                499.09,
                494.83,
                490.6,
                486.4,
                482.22,
                478.06,
                473.93,
                469.82,
                465.74,
                461.68,
                457.64,
                453.63,
                449.65,
                447.44,
                445.52,
                443.62,
                441.72,
                439.84,
                437.98,
                436.12,
                434.28,
                432.46,
                430.64,
                428.84,
                427.06,
                425.28,
                423.53,
                421.78,
                420.05,
                418.33,
                416.62,
                414.93,
                416.75,
                420.75,
                424.74,
                428.72,
                432.69,
                436.65,
                440.6,
                444.54,
                448.48,
                452.4,
                456.31,
                460.21,
                464.11,
                467.99,
                471.86,
                475.73,
                479.58,
                483.42,
                487.26,
                490.0,
                490.79,
                491.57,
                492.34,
                493.09,
                493.82,
                494.55,
                495.26,
                495.95,
                496.63,
                497.3,
                497.95,
                498.59,
                499.21,
                499.82,
                500.41,
                500.99,
                501.56,
                502.11,
                502.64,
                503.1,
                503.55,
                504.02,
                504.5,
                504.98,
                505.47,
                505.96,
                506.46,
                506.97,
                507.49,
                508.01,
                508.54,
                509.08,
                509.63,
                510.18,
                510.74,
                511.31,
                511.88,
                512.46,
                509.41,
                505.66,
                501.92,
                498.2,
                494.49,
                490.79,
                487.11,
                483.44,
                479.78,
                476.14,
                472.51,
                468.9,
                465.3,
                461.71,
                458.13,
                454.57,
                451.03,
                447.49,
                443.98,
                441.26,
                439.15,
                437.05,
                434.95,
                432.86,
                430.78,
                428.7,
                426.64,
                424.58,
                422.53,
                420.48,
                418.44,
                416.42,
                414.39,
                412.38,
                410.37,
                408.37,
                406.38,
                404.4,
                403.32,
                404.33,
                405.36,
                406.4,
                407.45,
                408.52,
                409.6,
                410.7,
                411.8,
                412.92,
                414.06,
                415.21,
                416.37,
                417.54,
                418.73,
                419.93,
                421.14,
                422.37,
                423.61,
                424.75,
                421.64,
                418.52,
                415.38,
                412.23,
                409.08,
                405.91,
                402.73,
                399.54,
                396.34,
                393.13,
                389.9,
                386.67,
                383.42,
                380.17,
                376.9,
                373.62,
                370.33,
                367.03,
                363.72,
                360.17,
                356.56,
                352.97,
                349.39,
                345.82,
                342.27,
                338.73,
                335.21,
                331.7,
                328.21,
                324.73,
                321.26,
                317.81,
                314.38,
                310.95,
                307.55,
                304.15,
                300.78,
                297.41,
                295.81,
                296.21,
                296.61,
                297.0,
                297.38,
                297.76,
                298.13,
                298.49,
                298.85,
                299.2,
                299.55,
                299.89,
                300.23,
                300.56,
                300.88,
                301.2,
                301.51,
                301.81,
                302.11,
                302.78,
                305.15,
                307.52,
                309.9,
                312.28,
                314.67,
                317.07,
                319.47,
                321.88,
                324.29,
                326.7,
                329.13,
                331.55,
                333.99,
                336.42,
                338.87,
                341.32,
                343.77,
                346.23,
                348.69,
                356.09,
                364.13,
                372.2,
                380.31,
                388.45,
                396.63,
                404.85,
                413.1,
                421.39,
                429.71,
                438.07,
                446.47,
                454.9,
                463.37,
                471.87,
                480.41,
                488.99,
                497.6,
                506.25,
                520.68,
                539.05,
                557.49,
                575.99,
                594.55,
                613.17,
                631.86,
                650.61,
                669.42,
                688.29,
                707.23,
                726.23,
                745.29,
                764.42,
                783.61,
                802.86,
                822.18,
                478.04,
                477.63,
                477.22,
                476.81,
                476.4,
                475.99,
                475.58,
                475.16,
                474.75,
                474.34,
                473.93,
                473.52,
                473.39,
                475.43,
                477.45,
                479.46,
                481.46,
                483.44,
                485.42,
                487.38,
                489.34,
                491.28,
                493.21,
                495.13,
                497.04,
                498.93,
                500.82,
                502.69,
                504.56,
                506.41,
                508.25,
                510.08,
                506.86,
                502.91,
                498.99,
                495.09,
                491.22,
                487.37,
                483.55,
                479.74,
                475.97,
                472.22,
                468.49,
                464.78,
                461.1,
                457.45,
                453.82,
                450.21,
                446.63,
                443.07,
                439.54,
                437.16,
                435.49,
                433.83,
                432.18,
                430.55,
                428.93,
                427.32,
                425.73,
                424.15,
                422.59,
                421.04,
                419.5,
                417.97,
                416.46,
                414.96,
                413.48,
                412.01,
                410.55,
                409.11,
                409.59,
                413.42,
                417.24,
                421.04,
                424.84,
                428.63,
                432.41,
                436.18,
                439.94,
                443.69,
                447.43,
                451.16,
                454.88,
                458.59,
                462.29,
                465.98,
                469.66,
                473.33,
                476.99,
                480.32,
                480.86,
                481.4,
                481.92,
                482.42,
                482.91,
                483.39,
                483.85,
                484.29,
                484.73,
                485.15,
                485.55,
                485.94,
                486.32,
                486.68,
                487.03,
                487.36,
                487.68,
                487.98,
                488.27,
                488.81,
                489.4,
                489.99,
                490.6,
                491.21,
                491.82,
                492.45,
                493.08,
                493.72,
                494.36,
                495.02,
                495.68,
                496.34,
                497.02,
                497.7,
                498.39,
                499.08,
                499.79,
                500.5,
                498.77,
                495.25,
                491.75,
                488.27,
                484.79,
                481.33,
                477.89,
                474.46,
                471.04,
                467.63,
                464.24,
                460.86,
                457.5,
                454.15,
                450.81,
                447.49,
                444.18,
                440.88,
                437.6,
                434.72,
                432.74,
                430.76,
                428.8,
                426.84,
                424.88,
                422.94,
                421.0,
                419.07,
                417.15,
                415.23,
                413.32,
                411.42,
                409.53,
                407.65,
                405.77,
                403.9,
                402.04,
                400.18,
                398.43,
                399.67,
                400.93,
                402.19,
                403.47,
                404.76,
                406.07,
                407.39,
                408.72,
                410.07,
                411.43,
                412.8,
                414.18,
                415.58,
                417.0,
                418.42,
                419.86,
                421.32,
                422.78,
                424.26,
                422.14,
                418.83,
                415.51,
                412.19,
                408.85,
                405.5,
                402.14,
                398.76,
                395.38,
                391.99,
                388.58,
                385.17,
                381.74,
                378.3,
                374.85,
                371.39,
                367.92,
                364.44,
                360.95,
                357.5,
                354.13,
                350.77,
                347.42,
                344.09,
                340.78,
                337.48,
                334.19,
                330.92,
                327.66,
                324.42,
                321.19,
                317.97,
                314.77,
                311.59,
                308.42,
                305.26,
                302.12,
                298.99,
                296.52,
                296.83,
                297.13,
                297.42,
                297.7,
                297.98,
                298.26,
                298.53,
                298.79,
                299.05,
                299.3,
                299.54,
                299.78,
                300.01,
                300.24,
                300.46,
                300.67,
                300.88,
                301.09,
                301.28,
                303.51,
                305.97,
                308.43,
                310.9,
                313.38,
                315.86,
                318.34,
                320.83,
                323.33,
                325.83,
                328.34,
                330.85,
                333.37,
                335.89,
                338.42,
                340.95,
                343.49,
                346.04,
                348.59,
                354.79,
                363.39,
                372.03,
                380.7,
                389.41,
                398.15,
                406.94,
                415.75,
                424.61,
                433.5,
                442.42,
                451.38,
                460.38,
                469.41,
                478.48,
                487.59,
                496.73,
                505.91,
                515.12,
                527.47,
                546.82,
                566.69,
                586.35,
                605.8,
                625.05,
                644.1,
                662.93,
                681.56,
                699.99,
                718.2,
                736.21,
                754.01,
                771.6,
                788.99,
                806.17,
                823.14,
                462.4,
                461.99,
                461.58,
                461.17,
                460.76,
                460.34,
                459.93,
                459.52,
                459.11,
                458.7,
                458.29,
                457.87,
                457.46,
                459.01,
                460.83,
                462.64,
                464.43,
                466.21,
                467.98,
                469.74,
                471.49,
                473.23,
                474.95,
                476.67,
                478.37,
                480.07,
                481.75,
                483.42,
                485.07,
                486.72,
                488.36,
                489.98,
                488.38,
                484.89,
                481.42,
                477.97,
                474.55,
                471.16,
                467.78,
                464.44,
                461.11,
                457.81,
                454.54,
                451.28,
                448.06,
                444.85,
                441.68,
                438.52,
                435.39,
                432.29,
                429.21,
                426.75,
                425.33,
                423.92,
                422.52,
                421.13,
                419.76,
                418.4,
                417.06,
                415.73,
                414.41,
                413.11,
                411.81,
                410.54,
                409.27,
                408.02,
                406.79,
                405.56,
                404.35,
                403.16,
                402.52,
                406.18,
                409.82,
                413.46,
                417.08,
                420.7,
                424.31,
                427.9,
                431.49,
                435.07,
                438.63,
                442.19,
                445.74,
                449.28,
                452.8,
                456.32,
                459.83,
                463.33,
                466.82,
                470.29,
                471.07,
                471.35,
                471.62,
                471.88,
                472.12,
                472.35,
                472.56,
                472.76,
                472.95,
                473.12,
                473.28,
                473.42,
                473.55,
                473.67,
                473.77,
                473.85,
                473.93,
                473.98,
                474.03,
                474.46,
                475.17,
                475.9,
                476.63,
                477.37,
                478.11,
                478.87,
                479.63,
                480.4,
                481.17,
                481.95,
                482.74,
                483.54,
                484.34,
                485.15,
                485.97,
                486.79,
                487.62,
                488.46,
                488.0,
                484.72,
                481.46,
                478.21,
                474.98,
                471.75,
                468.54,
                465.35,
                462.17,
                459.0,
                455.84,
                452.7,
                449.57,
                446.46,
                443.36,
                440.27,
                437.2,
                434.14,
                431.09,
                428.11,
                426.26,
                424.41,
                422.57,
                420.74,
                418.92,
                417.1,
                415.29,
                413.49,
                411.7,
                409.91,
                408.13,
                406.36,
                404.6,
                402.84,
                401.09,
                399.35,
                397.62,
                395.89,
                394.18,
                394.89,
                396.37,
                397.86,
                399.37,
                400.88,
                402.42,
                403.96,
                405.52,
                407.09,
                408.67,
                410.27,
                411.88,
                413.5,
                415.14,
                416.79,
                418.46,
                420.14,
                421.83,
                423.53,
                422.73,
                419.24,
                415.75,
                412.24,
                408.72,
                405.18,
                401.64,
                398.09,
                394.52,
                390.95,
                387.36,
                383.76,
                380.16,
                376.54,
                372.91,
                369.26,
                365.61,
                361.95,
                358.27,
                354.7,
                351.56,
                348.44,
                345.33,
                342.23,
                339.16,
                336.09,
                333.04,
                330.0,
                326.98,
                323.98,
                320.98,
                318.0,
                315.04,
                312.09,
                309.16,
                306.24,
                303.33,
                300.44,
                297.56,
                297.49,
                297.7,
                297.89,
                298.08,
                298.27,
                298.44,
                298.62,
                298.78,
                298.94,
                299.1,
                299.24,
                299.39,
                299.52,
                299.65,
                299.78,
                299.9,
                300.01,
                300.11,
                300.21,
                301.82,
                304.36,
                306.91,
                309.47,
                312.03,
                314.59,
                317.16,
                319.74,
                322.45,
                325.22,
                328.02,
                330.86,
                333.73,
                336.65,
                339.6,
                342.59,
                345.61,
                348.67,
                351.77,
                357.02,
                366.81,
                376.68,
                386.64,
                396.67,
                406.79,
                416.99,
                427.27,
                437.63,
                448.08,
                458.61,
                469.22,
                479.92,
                490.7,
                501.56,
                512.5,
                523.53,
                534.64,
                545.83,
                557.22,
                574.07,
                590.71,
                607.15,
                623.39,
                639.42,
                655.24,
                670.85,
                686.26,
                701.47,
                716.46,
                731.25,
                745.83,
                760.21,
                774.37,
                788.33,
                802.08,
                446.75,
                446.34,
                445.93,
                445.52,
                445.11,
                444.7,
                444.29,
                443.88,
                443.47,
                443.06,
                442.64,
                442.23,
                441.82,
                442.7,
                444.31,
                445.91,
                447.5,
                449.08,
                450.65,
                452.2,
                453.75,
                455.28,
                456.8,
                458.31,
                459.81,
                461.3,
                462.77,
                464.24,
                465.69,
                467.13,
                468.56,
                469.98,
                469.68,
                466.64,
                463.62,
                460.63,
                457.66,
                454.72,
                451.8,
                448.9,
                446.03,
                443.18,
                440.36,
                437.56,
                434.79,
                432.04,
                429.31,
                426.61,
                423.93,
                421.28,
                418.65,
                416.22,
                415.04,
                413.88,
                412.73,
                411.59,
                410.47,
                409.36,
                408.26,
                407.18,
                406.11,
                405.05,
                404.01,
                402.98,
                401.96,
                400.96,
                399.97,
                398.99,
                398.03,
                397.08,
                396.15,
                399.02,
                402.5,
                405.96,
                409.41,
                412.85,
                416.29,
                419.71,
                423.13,
                426.53,
                429.93,
                433.31,
                436.68,
                440.05,
                443.41,
                446.75,
                450.09,
                453.41,
                456.73,
                460.03,
                461.39,
                461.43,
                461.45,
                461.46,
                461.46,
                461.44,
                461.41,
                461.36,
                461.3,
                461.23,
                461.14,
                461.03,
                460.91,
                460.78,
                460.64,
                460.47,
                460.3,
                460.11,
                459.91,
                460.04,
                460.88,
                461.74,
                462.6,
                463.46,
                464.34,
                465.22,
                466.11,
                467.01,
                467.91,
                468.82,
                469.74,
                470.66,
                471.59,
                472.53,
                473.48,
                474.43,
                475.39,
                476.36,
                477.11,
                474.07,
                471.04,
                468.03,
                465.03,
                462.05,
                459.07,
                456.11,
                453.17,
                450.24,
                447.32,
                444.41,
                441.52,
                438.65,
                435.78,
                432.93,
                430.09,
                427.27,
                424.46,
                421.67,
                419.71,
                417.99,
                416.28,
                414.58,
                412.88,
                411.2,
                409.52,
                407.85,
                406.18,
                404.52,
                402.87,
                401.23,
                399.6,
                397.97,
                396.35,
                394.74,
                393.13,
                391.54,
                389.95,
                389.99,
                391.7,
                393.41,
                395.14,
                396.88,
                398.64,
                400.41,
                402.19,
                403.99,
                405.79,
                407.62,
                409.45,
                411.3,
                413.16,
                415.04,
                416.93,
                418.83,
                420.75,
                422.68,
                423.43,
                419.76,
                416.08,
                412.39,
                408.68,
                404.97,
                401.25,
                397.51,
                393.77,
                390.01,
                386.24,
                382.46,
                378.67,
                374.87,
                371.06,
                367.24,
                363.4,
                359.56,
                355.7,
                351.83,
                348.86,
                345.97,
                343.1,
                340.24,
                337.4,
                334.57,
                331.76,
                328.96,
                326.17,
                323.4,
                320.64,
                317.9,
                315.17,
                312.45,
                309.73,
                307.02,
                304.33,
                301.65,
                298.98,
                298.06,
                298.16,
                298.25,
                298.34,
                298.43,
                298.52,
                298.61,
                298.69,
                298.77,
                298.85,
                298.93,
                299.01,
                299.08,
                299.15,
                299.22,
                299.29,
                299.35,
                299.42,
                299.48,
                300.54,
                303.6,
                306.69,
                309.83,
                312.99,
                316.2,
                319.44,
                322.72,
                326.04,
                329.39,
                332.78,
                336.21,
                339.67,
                343.17,
                346.71,
                350.29,
                353.9,
                357.55,
                361.24,
                365.22,
                376.3,
                387.47,
                398.71,
                410.04,
                421.45,
                432.94,
                444.52,
                456.17,
                467.91,
                479.73,
                491.64,
                503.62,
                515.69,
                527.84,
                540.08,
                552.4,
                564.8,
                577.28,
                589.85,
                603.19,
                616.62,
                629.84,
                642.85,
                655.66,
                668.26,
                680.66,
                692.85,
                704.83,
                716.61,
                728.18,
                739.54,
                750.7,
                761.64,
                772.38,
                782.91,
                431.11,
                430.7,
                430.29,
                429.88,
                429.47,
                429.06,
                428.65,
                428.24,
                427.83,
                427.41,
                427.0,
                426.59,
                426.18,
                426.49,
                427.9,
                429.29,
                430.68,
                432.05,
                433.41,
                434.76,
                436.1,
                437.43,
                438.74,
                440.05,
                441.34,
                442.63,
                443.9,
                445.16,
                446.41,
                447.64,
                448.87,
                450.08,
                450.76,
                448.17,
                445.61,
                443.07,
                440.55,
                438.06,
                435.59,
                433.15,
                430.73,
                428.33,
                425.96,
                423.61,
                421.29,
                418.99,
                416.72,
                414.47,
                412.24,
                410.04,
                407.87,
                405.72,
                404.64,
                403.72,
                402.82,
                401.93,
                401.05,
                400.19,
                399.34,
                398.5,
                397.68,
                396.87,
                396.08,
                395.29,
                394.52,
                393.77,
                393.03,
                392.3,
                391.58,
                390.88,
                390.2,
                391.96,
                395.26,
                398.55,
                401.83,
                405.1,
                408.36,
                411.61,
                414.85,
                418.08,
                421.31,
                424.52,
                427.72,
                430.91,
                434.1,
                437.27,
                440.43,
                443.58,
                446.73,
                449.86,
                451.85,
                451.64,
                451.41,
                451.18,
                450.93,
                450.66,
                450.38,
                450.09,
                449.78,
                449.46,
                449.12,
                448.77,
                448.4,
                448.03,
                447.63,
                447.22,
                446.8,
                446.37,
                445.91,
                445.55,
                446.53,
                447.51,
                448.5,
                449.49,
                450.5,
                451.51,
                452.52,
                453.55,
                454.58,
                455.62,
                456.67,
                457.72,
                458.78,
                459.85,
                460.92,
                462.0,
                463.09,
                464.19,
                465.29,
                463.29,
                460.5,
                457.73,
                454.96,
                452.22,
                449.48,
                446.76,
                444.05,
                441.35,
                438.67,
                436.0,
                433.35,
                430.71,
                428.08,
                425.47,
                422.87,
                420.28,
                417.71,
                415.15,
                413.09,
                411.5,
                409.92,
                408.35,
                406.78,
                405.22,
                403.67,
                402.13,
                400.6,
                399.07,
                397.55,
                396.03,
                394.53,
                393.03,
                391.54,
                390.06,
                388.58,
                387.11,
                385.65,
                384.97,
                386.9,
                388.84,
                390.79,
                392.76,
                394.74,
                396.73,
                398.74,
                400.76,
                402.8,
                404.84,
                406.9,
                408.98,
                411.06,
                413.16,
                415.28,
                417.41,
                419.55,
                421.7,
                423.66,
                419.83,
                415.67,
                411.54,
                407.43,
                403.34,
                399.28,
                395.25,
                391.24,
                387.26,
                383.3,
                379.37,
                375.46,
                371.58,
                367.72,
                363.89,
                360.08,
                356.3,
                352.54,
                348.81,
                345.76,
                343.09,
                340.42,
                337.77,
                335.13,
                332.51,
                329.9,
                327.3,
                324.71,
                322.14,
                319.58,
                317.03,
                314.5,
                311.98,
                309.47,
                306.98,
                304.5,
                302.03,
                299.58,
                298.02,
                298.09,
                298.15,
                298.21,
                298.27,
                298.32,
                298.37,
                298.43,
                298.48,
                298.52,
                298.57,
                298.61,
                298.66,
                298.69,
                298.73,
                298.77,
                298.8,
                298.83,
                298.86,
                299.09,
                302.73,
                306.42,
                310.14,
                313.89,
                317.69,
                321.52,
                325.38,
                329.29,
                333.23,
                337.21,
                341.22,
                345.27,
                349.36,
                353.49,
                357.65,
                361.85,
                366.09,
                370.36,
                374.68,
                385.05,
                397.51,
                410.05,
                422.67,
                435.37,
                448.15,
                461.02,
                473.96,
                486.99,
                500.11,
                513.3,
                526.58,
                539.94,
                553.38,
                566.91,
                580.52,
                594.21,
                607.99,
                621.84,
                634.19,
                644.39,
                654.4,
                664.19,
                673.78,
                683.17,
                692.34,
                701.32,
                710.08,
                718.64,
                726.99,
                735.14,
                743.07,
                750.8,
                758.33,
                765.64,
                415.47,
                415.06,
                414.65,
                414.24,
                413.83,
                413.42,
                413.01,
                412.6,
                412.18,
                411.77,
                411.36,
                410.95,
                410.53,
                410.38,
                411.58,
                412.77,
                413.95,
                415.12,
                416.27,
                417.42,
                418.55,
                419.68,
                420.79,
                421.89,
                422.98,
                424.06,
                425.12,
                426.18,
                427.22,
                428.26,
                429.28,
                430.29,
                431.29,
                429.48,
                427.37,
                425.28,
                423.21,
                421.17,
                419.16,
                417.17,
                415.2,
                413.26,
                411.34,
                409.44,
                407.57,
                405.73,
                403.9,
                402.11,
                400.33,
                398.58,
                396.86,
                395.16,
                394.11,
                393.44,
                392.78,
                392.14,
                391.51,
                390.9,
                390.29,
                389.71,
                389.13,
                388.57,
                388.02,
                387.48,
                386.96,
                386.46,
                385.96,
                385.48,
                385.01,
                384.56,
                384.12,
                384.98,
                388.1,
                391.22,
                394.33,
                397.43,
                400.52,
                403.6,
                406.66,
                409.72,
                412.77,
                415.81,
                418.84,
                421.86,
                424.87,
                427.87,
                430.86,
                433.85,
                436.82,
                439.78,
                442.43,
                441.97,
                441.5,
                441.02,
                440.52,
                440.01,
                439.48,
                438.94,
                438.38,
                437.81,
                437.23,
                436.63,
                436.02,
                435.4,
                434.76,
                434.1,
                433.43,
                432.75,
                432.05,
                431.34,
                432.1,
                433.21,
                434.33,
                435.46,
                436.59,
                437.73,
                438.87,
                440.03,
                441.19,
                442.35,
                443.53,
                444.71,
                445.9,
                447.09,
                448.3,
                449.51,
                450.72,
                451.95,
                453.18,
                452.39,
                449.84,
                447.3,
                444.77,
                442.26,
                439.76,
                437.27,
                434.8,
                432.34,
                429.9,
                427.47,
                425.05,
                422.64,
                420.25,
                417.87,
                415.51,
                413.16,
                410.82,
                408.5,
                406.4,
                404.94,
                403.49,
                402.05,
                400.58,
                399.1,
                397.63,
                396.16,
                394.7,
                393.24,
                391.79,
                390.34,
                388.91,
                387.47,
                386.05,
                384.63,
                383.22,
                381.81,
                380.41,
                379.01,
                380.73,
                382.55,
                384.36,
                386.15,
                387.92,
                389.67,
                391.41,
                393.13,
                394.83,
                396.51,
                398.18,
                399.83,
                401.46,
                403.07,
                404.67,
                406.25,
                407.81,
                409.35,
                410.88,
                408.85,
                405.11,
                401.4,
                397.71,
                394.04,
                390.4,
                386.79,
                383.2,
                379.63,
                376.09,
                372.58,
                369.09,
                365.62,
                362.18,
                358.77,
                355.38,
                352.02,
                348.68,
                345.36,
                342.38,
                339.92,
                337.47,
                335.03,
                332.61,
                330.2,
                327.8,
                325.42,
                323.05,
                320.69,
                318.34,
                316.01,
                313.69,
                311.39,
                309.1,
                306.82,
                304.55,
                302.3,
                300.06,
                298.0,
                298.03,
                298.06,
                298.09,
                298.12,
                298.14,
                298.16,
                298.18,
                298.2,
                298.21,
                298.23,
                298.24,
                298.25,
                298.26,
                298.26,
                298.27,
                298.27,
                298.27,
                298.26,
                298.26,
                301.54,
                305.81,
                310.11,
                314.46,
                318.84,
                323.26,
                327.71,
                332.2,
                336.73,
                341.3,
                345.9,
                350.54,
                355.21,
                359.93,
                364.68,
                369.47,
                374.29,
                379.15,
                384.05,
                393.06,
                406.81,
                420.64,
                434.55,
                448.54,
                462.61,
                476.77,
                491.01,
                505.33,
                519.73,
                534.22,
                548.79,
                563.44,
                578.17,
                592.99,
                607.89,
                622.87,
                637.94,
                653.09,
                667.06,
                674.05,
                680.83,
                687.41,
                693.78,
                699.95,
                705.91,
                711.66,
                717.21,
                722.55,
                727.69,
                732.62,
                737.34,
                741.85,
                746.16,
                750.26,
                399.82,
                399.42,
                399.01,
                398.6,
                398.19,
                397.78,
                397.36,
                396.95,
                396.54,
                396.13,
                395.72,
                395.31,
                394.89,
                394.48,
                395.36,
                396.35,
                397.32,
                398.28,
                399.24,
                400.18,
                401.11,
                402.03,
                402.93,
                403.83,
                404.71,
                405.59,
                406.45,
                407.3,
                408.14,
                408.97,
                409.79,
                410.59,
                411.39,
                410.56,
                408.9,
                407.27,
                405.66,
                404.07,
                402.5,
                400.96,
                399.45,
                397.96,
                396.49,
                395.05,
                393.63,
                392.23,
                390.86,
                389.52,
                388.2,
                386.9,
                385.63,
                384.38,
                383.46,
                383.04,
                382.63,
                382.23,
                381.85,
                381.48,
                381.13,
                380.78,
                380.46,
                380.14,
                379.84,
                379.55,
                379.28,
                379.02,
                378.77,
                378.54,
                378.32,
                378.11,
                377.92,
                378.08,
                381.04,
                383.98,
                386.92,
                389.85,
                392.76,
                395.67,
                398.56,
                401.45,
                404.33,
                407.2,
                410.05,
                412.9,
                415.74,
                418.57,
                421.39,
                424.19,
                426.99,
                429.78,
                432.56,
                432.43,
                431.71,
                430.98,
                430.24,
                429.48,
                428.71,
                427.92,
                427.12,
                426.3,
                425.47,
                424.63,
                423.77,
                422.89,
                422.01,
                421.11,
                420.19,
                419.26,
                418.32,
                417.36,
                417.61,
                418.85,
                420.1,
                421.35,
                422.61,
                423.88,
                425.15,
                426.51,
                428.05,
                429.54,
                431.01,
                432.43,
                433.82,
                435.17,
                436.49,
                437.77,
                439.01,
                440.22,
                441.38,
                441.57,
                439.23,
                436.89,
                434.56,
                432.23,
                429.92,
                427.6,
                425.3,
                423.0,
                420.71,
                418.42,
                416.14,
                413.87,
                411.61,
                409.35,
                407.1,
                404.85,
                402.62,
                400.38,
                398.16,
                396.75,
                395.36,
                393.97,
                392.59,
                391.22,
                389.85,
                388.49,
                387.13,
                385.78,
                384.44,
                383.1,
                381.77,
                380.45,
                379.13,
                377.82,
                376.51,
                375.21,
                373.92,
                372.63,
                373.31,
                374.85,
                376.36,
                377.86,
                379.34,
                380.81,
                382.25,
                383.68,
                385.09,
                386.49,
                387.86,
                389.22,
                390.57,
                391.89,
                393.2,
                394.48,
                395.76,
                397.01,
                398.25,
                397.65,
                394.32,
                391.03,
                387.76,
                384.51,
                381.29,
                378.09,
                374.92,
                371.77,
                368.65,
                365.55,
                362.48,
                359.44,
                356.41,
                353.42,
                350.45,
                347.5,
                344.58,
                341.69,
                338.88,
                336.63,
                334.4,
                332.18,
                329.97,
                327.77,
                325.59,
                323.42,
                321.26,
                319.12,
                316.99,
                314.87,
                312.77,
                310.67,
                308.6,
                306.53,
                304.48,
                302.44,
                300.42,
                298.4,
                298.0,
                298.0,
                297.99,
                297.99,
                297.98,
                297.97,
                297.95,
                297.94,
                297.92,
                297.9,
                297.88,
                297.86,
                297.84,
                297.81,
                297.78,
                297.75,
                297.72,
                297.68,
                297.65,
                300.0,
                304.86,
                309.76,
                314.69,
                319.66,
                324.66,
                329.7,
                334.78,
                339.9,
                345.05,
                350.24,
                355.47,
                360.73,
                366.03,
                371.37,
                376.74,
                382.15,
                387.6,
                393.09,
                400.32,
                415.36,
                430.48,
                445.68,
                460.97,
                476.33,
                491.78,
                507.31,
                522.92,
                538.61,
                554.39,
                570.25,
                586.19,
                602.22,
                618.32,
                634.51,
                650.79,
                667.14,
                683.58,
                700.1,
                705.57,
                709.14,
                712.5,
                715.66,
                718.61,
                721.35,
                723.89,
                726.23,
                728.35,
                730.27,
                731.98,
                733.49,
                734.79,
                735.88,
                736.76,
                384.18,
                383.77,
                383.36,
                382.95,
                382.54,
                382.13,
                381.72,
                381.31,
                380.9,
                380.49,
                380.08,
                379.66,
                379.25,
                378.84,
                379.24,
                380.02,
                380.79,
                381.55,
                382.3,
                383.04,
                383.76,
                384.48,
                385.18,
                385.87,
                386.55,
                387.22,
                387.88,
                388.52,
                389.16,
                389.78,
                390.39,
                391.0,
                391.59,
                391.43,
                390.22,
                389.04,
                387.88,
                386.74,
                385.63,
                384.54,
                383.48,
                382.44,
                381.42,
                380.43,
                379.46,
                378.52,
                377.6,
                376.71,
                375.84,
                374.99,
                374.17,
                373.37,
                372.68,
                372.51,
                372.35,
                372.2,
                372.06,
                371.94,
                371.83,
                371.74,
                371.66,
                371.59,
                371.54,
                371.5,
                371.47,
                371.46,
                371.46,
                371.47,
                371.5,
                371.54,
                371.59,
                371.66,
                374.06,
                376.83,
                379.6,
                382.35,
                385.09,
                387.83,
                390.55,
                393.27,
                395.97,
                398.67,
                402.26,
                405.91,
                409.51,
                413.04,
                416.52,
                419.94,
                423.3,
                426.6,
                429.84,
                430.99,
                430.61,
                430.19,
                429.7,
                429.17,
                428.58,
                427.93,
                427.23,
                426.48,
                425.67,
                424.81,
                423.9,
                422.92,
                421.9,
                420.82,
                419.68,
                418.49,
                417.25,
                415.95,
                415.35,
                416.51,
                417.63,
                418.71,
                419.76,
                420.77,
                421.74,
                422.68,
                423.58,
                424.45,
                425.27,
                426.07,
                426.82,
                427.54,
                428.22,
                428.87,
                429.48,
                430.05,
                430.59,
                431.06,
                428.83,
                426.61,
                424.4,
                422.19,
                419.99,
                417.8,
                415.61,
                413.43,
                411.26,
                409.09,
                406.93,
                404.78,
                402.63,
                400.49,
                398.36,
                396.23,
                394.11,
                392.0,
                389.89,
                388.38,
                387.09,
                385.81,
                384.54,
                383.27,
                382.01,
                380.76,
                379.51,
                378.27,
                377.03,
                375.8,
                374.58,
                373.36,
                372.15,
                370.95,
                369.75,
                368.56,
                367.37,
                366.19,
                366.05,
                367.3,
                368.52,
                369.73,
                370.93,
                372.1,
                373.26,
                374.4,
                375.52,
                376.62,
                377.71,
                378.78,
                379.83,
                380.87,
                381.88,
                382.88,
                383.86,
                384.83,
                385.78,
                386.21,
                383.31,
                380.43,
                377.58,
                374.75,
                371.94,
                369.17,
                366.41,
                363.68,
                360.98,
                358.3,
                355.65,
                353.02,
                350.42,
                347.84,
                345.29,
                342.76,
                340.26,
                337.78,
                335.33,
                333.23,
                331.21,
                329.2,
                327.2,
                325.22,
                323.25,
                321.3,
                319.36,
                317.43,
                315.51,
                313.61,
                311.72,
                309.84,
                307.98,
                306.13,
                304.29,
                302.47,
                300.66,
                298.86,
                297.98,
                297.95,
                297.91,
                297.87,
                297.83,
                297.79,
                297.74,
                297.7,
                297.65,
                297.6,
                297.55,
                297.49,
                297.44,
                297.38,
                297.32,
                297.25,
                297.19,
                297.12,
                297.05,
                298.14,
                303.58,
                309.07,
                314.58,
                320.14,
                325.73,
                331.36,
                337.03,
                342.73,
                348.47,
                354.25,
                360.06,
                365.91,
                371.8,
                377.72,
                383.68,
                389.68,
                395.72,
                401.79,
                407.9,
                423.17,
                439.58,
                456.07,
                472.65,
                489.3,
                506.04,
                522.86,
                539.76,
                556.75,
                573.81,
                590.96,
                608.19,
                625.51,
                642.91,
                660.39,
                677.95,
                695.59,
                713.32,
                731.13,
                738.97,
                739.32,
                739.47,
                739.41,
                739.15,
                738.68,
                738.0,
                737.12,
                736.03,
                734.74,
                733.23,
                731.52,
                729.61,
                727.49,
                725.16,
                368.54,
                368.13,
                367.72,
                367.31,
                366.9,
                366.49,
                366.08,
                365.67,
                365.26,
                364.85,
                364.43,
                364.02,
                363.61,
                363.2,
                363.22,
                363.8,
                364.36,
                364.92,
                365.46,
                365.99,
                366.51,
                367.02,
                367.52,
                368.01,
                368.49,
                368.95,
                369.4,
                369.85,
                370.28,
                370.7,
                371.1,
                371.5,
                371.89,
                372.07,
                371.32,
                370.58,
                369.87,
                369.19,
                368.53,
                367.89,
                367.28,
                366.69,
                366.13,
                365.59,
                365.07,
                364.86,
                365.08,
                365.33,
                365.61,
                365.92,
                366.27,
                366.64,
                367.05,
                368.01,
                369.11,
                370.27,
                371.51,
                372.82,
                374.2,
                375.66,
                377.19,
                378.79,
                380.46,
                382.2,
                384.02,
                385.91,
                387.88,
                389.92,
                392.03,
                394.21,
                396.47,
                398.8,
                401.69,
                404.87,
                407.99,
                411.05,
                414.05,
                416.99,
                419.88,
                422.7,
                425.47,
                428.18,
                430.83,
                433.42,
                435.95,
                438.42,
                440.84,
                443.19,
                445.49,
                447.73,
                449.9,
                450.91,
                449.57,
                448.18,
                446.73,
                445.23,
                443.67,
                442.06,
                440.4,
                438.68,
                436.9,
                435.08,
                433.19,
                431.26,
                429.27,
                427.22,
                425.12,
                422.96,
                420.75,
                418.49,
                416.28,
                416.81,
                417.29,
                417.74,
                418.15,
                418.53,
                418.87,
                419.18,
                419.45,
                419.68,
                419.87,
                420.03,
                420.16,
                420.24,
                420.29,
                420.31,
                420.28,
                420.22,
                420.13,
                419.99,
                418.37,
                416.27,
                414.18,
                412.09,
                410.01,
                407.93,
                405.86,
                403.8,
                401.75,
                399.7,
                397.66,
                395.62,
                393.59,
                391.57,
                389.56,
                387.55,
                385.55,
                383.55,
                381.56,
                379.95,
                378.77,
                377.6,
                376.43,
                375.27,
                374.12,
                372.97,
                371.83,
                370.7,
                369.57,
                368.45,
                367.33,
                366.22,
                365.12,
                364.02,
                362.93,
                361.85,
                360.77,
                359.7,
                358.95,
                359.91,
                360.84,
                361.76,
                362.66,
                363.55,
                364.42,
                365.27,
                366.1,
                366.91,
                367.71,
                368.49,
                369.25,
                370.0,
                370.73,
                371.44,
                372.13,
                372.81,
                373.46,
                374.1,
                372.07,
                369.6,
                367.17,
                364.76,
                362.37,
                360.01,
                357.68,
                355.37,
                353.08,
                350.82,
                348.58,
                346.37,
                344.19,
                342.03,
                339.89,
                337.78,
                335.7,
                333.64,
                331.61,
                329.7,
                327.9,
                326.1,
                324.32,
                322.56,
                320.8,
                319.06,
                317.33,
                315.62,
                313.92,
                312.23,
                310.55,
                308.89,
                307.24,
                305.6,
                303.98,
                302.37,
                300.77,
                299.19,
                297.98,
                297.92,
                297.85,
                297.78,
                297.71,
                297.63,
                297.55,
                297.48,
                297.39,
                297.31,
                297.23,
                297.14,
                297.05,
                296.96,
                296.87,
                296.77,
                296.68,
                296.58,
                296.48,
                296.38,
                301.97,
                308.04,
                314.15,
                320.29,
                326.47,
                332.68,
                338.94,
                345.23,
                351.55,
                357.92,
                364.32,
                370.75,
                377.23,
                383.74,
                390.29,
                396.87,
                403.49,
                410.15,
                416.85,
                430.24,
                447.94,
                465.72,
                483.59,
                501.53,
                519.56,
                537.67,
                555.86,
                574.13,
                592.49,
                610.93,
                629.45,
                648.05,
                666.74,
                685.51,
                704.36,
                723.3,
                742.31,
                761.41,
                774.24,
                771.38,
                768.31,
                765.04,
                761.56,
                757.88,
                753.99,
                749.89,
                745.59,
                741.08,
                736.37,
                731.44,
                726.32,
                720.98,
                715.44,
                352.9,
                352.49,
                352.08,
                351.67,
                351.26,
                350.85,
                350.44,
                350.03,
                350.08,
                350.13,
                350.16,
                350.16,
                350.13,
                350.07,
                350.17,
                351.04,
                351.93,
                352.83,
                353.73,
                354.65,
                355.58,
                356.52,
                357.47,
                358.43,
                359.4,
                360.39,
                361.38,
                362.39,
                363.4,
                364.43,
                365.47,
                366.52,
                367.58,
                368.65,
                369.12,
                369.57,
                370.05,
                370.57,
                371.11,
                371.69,
                372.3,
                372.93,
                373.61,
                374.31,
                375.04,
                375.81,
                376.61,
                377.44,
                378.3,
                379.19,
                380.12,
                381.07,
                382.06,
                383.93,
                386.33,
                388.81,
                391.36,
                393.98,
                396.67,
                399.44,
                402.27,
                405.18,
                408.17,
                411.22,
                414.35,
                417.55,
                420.83,
                424.17,
                427.59,
                431.09,
                434.65,
                438.29,
                441.44,
                443.55,
                445.6,
                447.6,
                449.54,
                451.42,
                453.24,
                455.0,
                456.71,
                458.35,
                459.94,
                461.47,
                462.94,
                464.35,
                465.7,
                466.99,
                468.22,
                469.4,
                470.51,
                471.33,
                469.02,
                466.66,
                464.25,
                461.78,
                459.26,
                456.68,
                454.05,
                451.37,
                448.63,
                445.84,
                442.99,
                440.09,
                437.13,
                434.12,
                431.05,
                427.93,
                424.76,
                421.53,
                418.24,
                417.43,
                417.28,
                417.1,
                416.88,
                416.63,
                416.33,
                416.01,
                415.64,
                415.24,
                414.8,
                414.33,
                413.82,
                413.28,
                412.69,
                412.07,
                411.42,
                410.72,
                410.0,
                409.23,
                407.85,
                405.87,
                403.89,
                401.92,
                399.96,
                398.0,
                396.05,
                394.11,
                392.17,
                390.24,
                388.32,
                386.4,
                384.49,
                382.59,
                380.69,
                378.8,
                376.92,
                375.04,
                373.17,
                371.46,
                370.39,
                369.32,
                368.27,
                367.22,
                366.17,
                365.13,
                364.1,
                363.07,
                362.05,
                361.03,
                360.03,
                359.02,
                358.03,
                357.04,
                356.05,
                355.08,
                354.11,
                353.14,
                352.18,
                352.67,
                353.32,
                353.95,
                354.56,
                355.15,
                355.73,
                356.29,
                356.84,
                357.36,
                357.87,
                358.36,
                358.84,
                359.29,
                359.73,
                360.15,
                360.56,
                360.94,
                361.31,
                361.66,
                360.59,
                358.55,
                356.53,
                354.54,
                352.57,
                350.63,
                348.71,
                346.82,
                344.95,
                343.11,
                341.29,
                339.5,
                337.73,
                335.99,
                334.27,
                332.58,
                330.91,
                329.27,
                327.65,
                326.06,
                324.47,
                322.89,
                321.32,
                319.77,
                318.23,
                316.7,
                315.19,
                313.69,
                312.2,
                310.73,
                309.27,
                307.82,
                306.38,
                304.96,
                303.55,
                302.16,
                300.77,
                299.4,
                298.05,
                297.9,
                297.8,
                297.7,
                297.6,
                297.49,
                297.38,
                297.27,
                297.16,
                297.04,
                296.93,
                296.81,
                296.69,
                296.56,
                296.44,
                296.31,
                296.18,
                296.05,
                295.92,
                295.79,
                300.03,
                306.68,
                313.37,
                320.1,
                326.87,
                333.67,
                340.51,
                347.39,
                354.3,
                361.25,
                368.24,
                375.26,
                382.32,
                389.42,
                396.55,
                403.73,
                410.94,
                418.18,
                425.46,
                436.57,
                455.56,
                474.63,
                493.78,
                513.02,
                532.33,
                551.73,
                571.21,
                590.78,
                610.42,
                630.15,
                649.96,
                669.85,
                689.83,
                709.89,
                730.03,
                750.25,
                770.56,
                790.94,
                811.38,
                805.31,
                799.03,
                792.54,
                785.85,
                778.95,
                771.85,
                764.54,
                757.03,
                749.31,
                741.38,
                733.25,
                724.91,
                716.36,
                707.6,
                356.19,
                355.93,
                355.65,
                355.34,
                355.0,
                354.63,
                354.23,
                353.8,
                353.35,
                352.87,
                352.36,
                351.82,
                351.25,
                350.65,
                350.03,
                351.03,
                352.12,
                353.22,
                354.33,
                355.45,
                356.59,
                357.73,
                358.88,
                360.05,
                361.23,
                362.41,
                363.61,
                364.82,
                366.04,
                367.27,
                368.51,
                369.77,
                371.03,
                372.31,
                373.4,
                374.43,
                375.49,
                376.59,
                377.71,
                378.87,
                380.06,
                381.28,
                382.53,
                383.81,
                385.12,
                386.47,
                387.85,
                389.26,
                390.7,
                392.17,
                393.68,
                395.21,
                396.78,
                399.19,
                402.91,
                406.69,
                410.55,
                414.48,
                418.48,
                422.56,
                426.71,
                430.93,
                435.22,
                439.58,
                444.02,
                448.53,
                453.11,
                457.77,
                462.5,
                467.3,
                472.18,
                477.13,
                481.72,
                482.77,
                483.76,
                484.69,
                485.57,
                486.39,
                487.14,
                487.84,
                488.49,
                489.07,
                489.59,
                490.06,
                490.46,
                490.81,
                491.1,
                491.33,
                491.5,
                491.61,
                491.66,
                491.65,
                488.97,
                485.64,
                482.26,
                478.83,
                475.34,
                471.8,
                468.21,
                464.56,
                460.85,
                457.1,
                453.28,
                449.42,
                445.49,
                441.52,
                437.49,
                433.4,
                429.26,
                425.07,
                420.82,
                418.39,
                417.61,
                416.79,
                415.94,
                415.05,
                414.13,
                413.17,
                412.17,
                411.14,
                410.07,
                408.96,
                407.82,
                406.64,
                405.42,
                404.17,
                402.88,
                401.56,
                400.2,
                398.8,
                397.27,
                395.4,
                393.54,
                391.69,
                389.85,
                388.01,
                386.18,
                384.35,
                382.53,
                380.72,
                378.92,
                377.12,
                375.33,
                373.54,
                371.76,
                369.99,
                368.22,
                366.47,
                364.72,
                362.97,
                361.95,
                360.99,
                360.04,
                359.1,
                358.16,
                357.23,
                356.3,
                355.38,
                354.47,
                353.56,
                352.66,
                351.77,
                350.88,
                350.0,
                349.12,
                348.25,
                347.38,
                346.53,
                345.68,
                345.59,
                345.95,
                346.29,
                346.61,
                346.92,
                347.21,
                347.48,
                347.73,
                347.97,
                348.19,
                348.39,
                348.57,
                348.74,
                348.89,
                349.02,
                349.14,
                349.23,
                349.31,
                349.38,
                348.89,
                347.27,
                345.67,
                344.09,
                342.54,
                341.02,
                339.52,
                338.04,
                336.59,
                335.17,
                333.77,
                332.39,
                331.04,
                329.72,
                328.42,
                327.14,
                325.89,
                324.67,
                323.47,
                322.3,
                320.92,
                319.56,
                318.2,
                316.87,
                315.54,
                314.23,
                312.93,
                311.64,
                310.37,
                309.11,
                307.86,
                306.63,
                305.41,
                304.2,
                303.0,
                301.82,
                300.65,
                299.5,
                298.36,
                297.91,
                297.78,
                297.64,
                297.51,
                297.37,
                297.23,
                297.08,
                296.94,
                296.79,
                296.64,
                296.49,
                296.34,
                296.18,
                296.03,
                295.87,
                295.71,
                295.55,
                295.38,
                295.21,
                297.75,
                304.99,
                312.27,
                319.58,
                326.94,
                334.33,
                341.75,
                349.21,
                356.71,
                364.25,
                371.82,
                379.43,
                387.08,
                394.76,
                402.49,
                410.24,
                418.04,
                425.87,
                433.74,
                442.16,
                462.44,
                482.8,
                503.24,
                523.76,
                544.37,
                565.05,
                585.82,
                606.67,
                627.61,
                648.62,
                669.72,
                690.9,
                712.17,
                733.51,
                754.94,
                776.45,
                798.05,
                819.73,
                841.49,
                841.11,
                831.61,
                821.92,
                812.01,
                801.9,
                791.59,
                781.07,
                770.35,
                759.41,
                748.28,
                736.93,
                725.38,
                713.62,
                701.66,
                364.02,
                363.23,
                362.41,
                361.56,
                360.68,
                359.77,
                358.84,
                357.87,
                356.88,
                355.86,
                354.81,
                353.74,
                352.63,
                351.5,
                350.34,
                350.92,
                352.21,
                353.52,
                354.83,
                356.16,
                357.49,
                358.84,
                360.2,
                361.57,
                362.95,
                364.34,
                365.74,
                367.16,
                368.58,
                370.02,
                371.46,
                372.92,
                374.39,
                375.87,
                377.4,
                379.01,
                380.65,
                382.32,
                384.03,
                385.76,
                387.53,
                389.33,
                391.16,
                393.02,
                394.92,
                396.84,
                398.8,
                400.79,
                402.81,
                404.86,
                406.95,
                409.07,
                411.21,
                413.81,
                418.83,
                423.92,
                429.09,
                434.33,
                439.64,
                445.03,
                450.48,
                456.01,
                461.61,
                467.29,
                473.03,
                478.85,
                484.74,
                490.71,
                496.75,
                502.86,
                509.04,
                515.3,
                521.63,
                522.52,
                522.45,
                522.32,
                522.14,
                521.89,
                521.59,
                521.22,
                520.8,
                520.32,
                519.78,
                519.19,
                518.53,
                517.82,
                517.04,
                516.21,
                515.32,
                514.37,
                513.36,
                512.29,
                509.4,
                505.11,
                500.77,
                496.37,
                491.92,
                487.42,
                482.86,
                478.24,
                473.57,
                468.85,
                464.07,
                459.24,
                454.36,
                449.42,
                444.42,
                439.37,
                434.27,
                429.11,
                423.89,
                419.67,
                418.26,
                416.81,
                415.32,
                413.8,
                412.25,
                410.65,
                409.02,
                407.36,
                405.66,
                403.92,
                402.14,
                400.33,
                398.49,
                396.6,
                394.68,
                392.72,
                390.73,
                388.7,
                386.63,
                384.87,
                383.13,
                381.4,
                379.67,
                377.95,
                376.24,
                374.53,
                372.83,
                371.14,
                369.45,
                367.77,
                366.1,
                364.43,
                362.77,
                361.12,
                359.47,
                357.83,
                356.2,
                354.57,
                353.46,
                352.61,
                351.76,
                350.93,
                350.1,
                349.27,
                348.45,
                347.64,
                346.83,
                346.03,
                345.24,
                344.45,
                343.67,
                342.9,
                342.13,
                341.36,
                340.61,
                339.86,
                339.11,
                338.66,
                338.73,
                338.78,
                338.82,
                338.84,
                338.84,
                338.82,
                338.78,
                338.73,
                338.66,
                338.57,
                338.47,
                338.35,
                338.21,
                338.05,
                337.88,
                337.69,
                337.48,
                337.25,
                336.96,
                335.75,
                334.57,
                333.41,
                332.28,
                331.17,
                330.09,
                329.03,
                328.0,
                326.99,
                326.01,
                325.05,
                324.12,
                323.21,
                322.33,
                321.48,
                320.64,
                319.84,
                319.06,
                318.3,
                317.25,
                316.1,
                314.97,
                313.84,
                312.73,
                311.63,
                310.55,
                309.47,
                308.42,
                307.37,
                306.34,
                305.32,
                304.31,
                303.32,
                302.33,
                301.37,
                300.41,
                299.47,
                298.54,
                297.93,
                297.77,
                297.6,
                297.43,
                297.26,
                297.09,
                296.91,
                296.74,
                296.56,
                296.38,
                296.19,
                296.01,
                295.82,
                295.63,
                295.44,
                295.25,
                295.06,
                294.86,
                294.66,
                295.14,
                302.96,
                310.83,
                318.73,
                326.67,
                334.65,
                342.66,
                350.71,
                358.79,
                366.91,
                375.07,
                383.27,
                391.5,
                399.77,
                408.08,
                416.42,
                424.81,
                433.22,
                441.68,
                450.17,
                468.57,
                490.22,
                511.95,
                533.76,
                555.65,
                577.13,
                598.21,
                619.19,
                640.07,
                660.85,
                681.53,
                702.12,
                722.6,
                742.98,
                763.26,
                783.44,
                803.52,
                823.51,
                843.39,
                849.14,
                835.83,
                822.5,
                809.15,
                795.78,
                782.39,
                768.98,
                755.54,
                742.09,
                728.62,
                715.12,
                701.6,
                688.06,
                674.5,
                372.11,
                370.78,
                369.42,
                368.03,
                366.62,
                365.17,
                363.7,
                362.2,
                360.67,
                359.12,
                357.53,
                355.92,
                354.27,
                352.6,
                350.9,
                350.71,
                352.2,
                353.71,
                355.23,
                356.76,
                358.3,
                359.85,
                361.42,
                362.99,
                364.57,
                366.17,
                367.77,
                369.39,
                371.02,
                372.66,
                374.31,
                375.97,
                377.64,
                379.33,
                381.11,
                383.3,
                385.52,
                387.77,
                390.06,
                392.37,
                394.72,
                397.1,
                399.51,
                401.95,
                404.43,
                406.93,
                409.47,
                412.04,
                414.64,
                417.27,
                419.93,
                422.63,
                425.36,
                428.12,
                434.1,
                440.5,
                446.98,
                453.53,
                460.15,
                466.84,
                473.6,
                480.44,
                487.35,
                494.33,
                501.39,
                508.52,
                515.72,
                522.99,
                530.34,
                537.75,
                545.25,
                552.81,
                560.45,
                562.81,
                561.68,
                560.49,
                559.24,
                557.93,
                556.57,
                555.14,
                553.66,
                552.12,
                550.52,
                548.86,
                547.14,
                545.36,
                543.53,
                541.63,
                539.68,
                537.66,
                535.59,
                533.46,
                530.34,
                525.08,
                519.77,
                514.41,
                509.0,
                503.53,
                498.0,
                492.42,
                486.79,
                481.1,
                475.36,
                469.57,
                463.72,
                457.81,
                451.85,
                445.84,
                439.77,
                433.65,
                427.47,
                421.28,
                419.24,
                417.15,
                415.04,
                412.89,
                410.7,
                408.47,
                406.21,
                403.91,
                401.58,
                399.21,
                396.8,
                394.36,
                391.88,
                389.36,
                386.81,
                384.22,
                381.6,
                378.93,
                376.23,
                374.29,
                372.66,
                371.05,
                369.44,
                367.84,
                366.24,
                364.65,
                363.07,
                361.49,
                359.93,
                358.36,
                356.81,
                355.26,
                353.72,
                352.18,
                350.65,
                349.13,
                347.62,
                346.11,
                344.91,
                344.16,
                343.43,
                342.7,
                341.97,
                341.26,
                340.55,
                339.84,
                339.14,
                338.45,
                337.76,
                337.08,
                336.41,
                335.74,
                335.08,
                334.42,
                333.77,
                333.13,
                332.49,
                331.9,
                331.68,
                331.44,
                331.18,
                330.91,
                330.62,
                330.32,
                329.99,
                329.65,
                329.29,
                328.92,
                328.52,
                328.11,
                327.68,
                327.24,
                326.77,
                326.29,
                325.8,
                325.28,
                324.75,
                324.01,
                323.25,
                322.51,
                321.79,
                321.1,
                320.44,
                319.8,
                319.18,
                318.59,
                318.03,
                317.49,
                316.97,
                316.48,
                316.02,
                315.58,
                315.16,
                314.78,
                314.41,
                314.07,
                313.47,
                312.53,
                311.61,
                310.7,
                309.8,
                308.92,
                308.05,
                307.19,
                306.34,
                305.51,
                304.69,
                303.89,
                303.09,
                302.31,
                301.55,
                300.79,
                300.05,
                299.32,
                298.61,
                297.97,
                297.78,
                297.58,
                297.38,
                297.17,
                296.97,
                296.76,
                296.55,
                296.34,
                296.13,
                295.92,
                295.7,
                295.51,
                295.34,
                295.18,
                295.02,
                294.88,
                294.74,
                294.61,
                294.48,
                301.14,
                309.46,
                317.77,
                326.08,
                334.39,
                342.7,
                351.0,
                359.3,
                367.59,
                375.88,
                384.17,
                392.45,
                400.73,
                409.01,
                417.28,
                425.55,
                433.82,
                442.08,
                450.34,
                464.18,
                484.19,
                504.1,
                523.92,
                543.63,
                563.25,
                582.76,
                602.18,
                621.5,
                640.72,
                659.84,
                678.86,
                697.78,
                716.6,
                735.32,
                753.94,
                772.46,
                790.88,
                809.2,
                823.08,
                809.46,
                795.81,
                782.14,
                768.46,
                754.75,
                741.02,
                727.27,
                713.5,
                699.7,
                685.89,
                672.05,
                658.2,
                644.32,
                380.46,
                378.59,
                376.69,
                374.77,
                372.82,
                370.83,
                368.82,
                366.79,
                364.72,
                362.63,
                360.51,
                358.35,
                356.18,
                353.97,
                351.73,
                350.4,
                352.1,
                353.81,
                355.53,
                357.27,
                359.01,
                360.77,
                362.53,
                364.31,
                366.1,
                367.9,
                369.71,
                371.53,
                373.36,
                375.2,
                377.06,
                378.92,
                380.8,
                382.68,
                384.58,
                387.31,
                390.11,
                392.94,
                395.8,
                398.7,
                401.62,
                404.58,
                407.57,
                410.59,
                413.65,
                416.73,
                419.85,
                423.0,
                426.18,
                429.39,
                432.63,
                435.91,
                439.21,
                442.55,
                448.71,
                456.43,
                464.21,
                472.07,
                480.0,
                488.0,
                496.07,
                504.22,
                512.43,
                520.72,
                529.09,
                537.52,
                546.03,
                554.61,
                563.27,
                571.99,
                580.79,
                589.67,
                598.61,
                603.64,
                601.45,
                599.19,
                596.88,
                594.51,
                592.08,
                589.6,
                587.05,
                584.45,
                581.79,
                579.07,
                576.29,
                573.45,
                570.55,
                567.59,
                564.58,
                561.5,
                558.37,
                555.18,
                551.76,
                545.54,
                539.27,
                532.95,
                526.57,
                520.13,
                513.64,
                507.1,
                500.5,
                493.85,
                487.15,
                480.39,
                473.57,
                466.71,
                459.78,
                452.8,
                445.77,
                438.69,
                431.54,
                424.35,
                420.54,
                417.83,
                415.08,
                412.3,
                409.47,
                406.62,
                403.72,
                400.8,
                397.83,
                394.83,
                391.79,
                388.71,
                385.6,
                382.45,
                379.27,
                376.05,
                372.79,
                369.5,
                366.17,
                363.63,
                362.13,
                360.63,
                359.14,
                357.66,
                356.18,
                354.71,
                353.25,
                351.79,
                350.34,
                348.89,
                347.46,
                346.02,
                344.6,
                343.18,
                341.77,
                340.37,
                338.97,
                337.58,
                336.3,
                335.66,
                335.03,
                334.41,
                333.79,
                333.18,
                332.58,
                331.98,
                331.39,
                330.8,
                330.22,
                329.65,
                329.08,
                328.52,
                327.97,
                327.42,
                326.88,
                326.34,
                325.81,
                325.29,
                324.77,
                324.25,
                323.71,
                323.14,
                322.57,
                321.97,
                321.36,
                320.73,
                320.08,
                319.42,
                318.73,
                318.03,
                317.32,
                316.58,
                315.83,
                315.06,
                314.27,
                313.47,
                312.65,
                312.05,
                311.7,
                311.37,
                311.08,
                310.8,
                310.56,
                310.33,
                310.14,
                309.96,
                309.81,
                309.69,
                309.59,
                309.52,
                309.47,
                309.45,
                309.45,
                309.48,
                309.59,
                309.73,
                309.73,
                309.05,
                308.38,
                307.72,
                307.05,
                306.4,
                305.74,
                305.1,
                304.45,
                303.81,
                303.18,
                302.55,
                301.92,
                301.3,
                300.69,
                300.08,
                299.47,
                298.87,
                298.27,
                297.68,
                297.47,
                297.34,
                297.22,
                297.1,
                297.0,
                296.9,
                296.81,
                296.73,
                296.65,
                296.59,
                296.53,
                296.48,
                296.43,
                296.4,
                296.37,
                296.35,
                296.34,
                296.34,
                296.35,
                300.66,
                308.92,
                317.18,
                325.44,
                333.69,
                341.94,
                350.19,
                358.43,
                366.67,
                374.91,
                383.14,
                391.37,
                399.59,
                407.82,
                416.03,
                424.25,
                432.46,
                440.67,
                448.88,
                459.0,
                477.45,
                495.8,
                514.05,
                532.21,
                550.26,
                568.22,
                586.07,
                603.83,
                621.49,
                639.04,
                656.5,
                673.86,
                691.12,
                708.28,
                725.34,
                742.3,
                759.15,
                775.91,
                792.57,
                783.27,
                769.31,
                755.32,
                741.32,
                727.29,
                713.24,
                699.18,
                685.09,
                670.98,
                656.84,
                642.69,
                628.52,
                614.32,
                389.06,
                386.65,
                384.22,
                381.76,
                379.27,
                376.75,
                374.21,
                371.63,
                369.03,
                366.4,
                363.74,
                361.05,
                358.34,
                355.59,
                352.82,
                350.02,
                351.89,
                353.81,
                355.74,
                357.67,
                359.62,
                361.58,
                363.55,
                365.53,
                367.52,
                369.53,
                371.54,
                373.56,
                375.6,
                377.65,
                379.7,
                381.77,
                383.85,
                385.94,
                388.05,
                391.03,
                394.41,
                397.82,
                401.26,
                404.74,
                408.24,
                411.78,
                415.35,
                418.95,
                422.58,
                426.25,
                429.94,
                433.67,
                437.43,
                441.22,
                445.04,
                448.89,
                452.78,
                456.7,
                462.68,
                471.7,
                480.79,
                489.96,
                499.19,
                508.5,
                517.88,
                527.34,
                536.86,
                546.46,
                556.13,
                565.87,
                575.69,
                585.58,
                595.54,
                605.57,
                615.68,
                625.86,
                636.11,
                645.0,
                641.75,
                638.43,
                635.06,
                631.63,
                628.14,
                624.59,
                620.99,
                617.32,
                613.6,
                609.81,
                605.97,
                602.07,
                598.12,
                594.1,
                590.02,
                585.89,
                581.69,
                577.44,
                573.13,
                566.5,
                559.26,
                551.97,
                544.63,
                537.23,
                529.78,
                522.27,
                514.71,
                507.1,
                499.43,
                491.71,
                483.93,
                476.1,
                468.21,
                460.27,
                452.27,
                444.22,
                436.12,
                427.96,
                422.18,
                418.83,
                415.45,
                412.04,
                408.58,
                405.09,
                401.57,
                398.01,
                394.41,
                390.78,
                387.11,
                383.4,
                379.66,
                375.88,
                372.06,
                368.21,
                364.32,
                360.4,
                356.43,
                352.92,
                351.53,
                350.15,
                348.78,
                347.42,
                346.06,
                344.7,
                343.36,
                342.02,
                340.69,
                339.36,
                338.04,
                336.73,
                335.42,
                334.12,
                332.83,
                331.54,
                330.26,
                328.99,
                327.73,
                327.1,
                326.58,
                326.07,
                325.56,
                325.05,
                324.56,
                324.07,
                323.58,
                323.1,
                322.63,
                322.16,
                321.7,
                321.25,
                320.8,
                320.36,
                319.93,
                319.5,
                319.07,
                318.66,
                318.03,
                317.21,
                316.41,
                315.75,
                315.09,
                314.42,
                313.76,
                313.09,
                312.43,
                311.76,
                311.09,
                310.42,
                309.75,
                309.07,
                308.4,
                307.72,
                307.05,
                306.37,
                305.69,
                305.2,
                305.34,
                305.49,
                305.63,
                305.77,
                305.91,
                306.05,
                306.19,
                306.33,
                306.46,
                306.59,
                306.73,
                306.86,
                306.99,
                307.11,
                307.24,
                307.37,
                307.49,
                307.61,
                307.74,
                307.21,
                306.61,
                306.02,
                305.43,
                304.85,
                304.27,
                303.7,
                303.13,
                302.57,
                302.01,
                301.45,
                300.9,
                300.36,
                299.81,
                299.28,
                298.75,
                298.22,
                297.7,
                297.18,
                296.95,
                296.95,
                296.96,
                296.97,
                296.99,
                297.02,
                297.06,
                297.11,
                297.16,
                297.22,
                297.29,
                297.37,
                297.45,
                297.55,
                297.65,
                297.76,
                297.88,
                298.0,
                298.14,
                300.21,
                308.42,
                316.62,
                324.83,
                333.02,
                341.22,
                349.41,
                357.6,
                365.78,
                373.96,
                382.14,
                390.32,
                398.49,
                406.65,
                414.82,
                422.98,
                431.14,
                439.29,
                447.44,
                455.59,
                471.61,
                488.4,
                505.09,
                521.68,
                538.18,
                554.57,
                570.87,
                587.06,
                603.16,
                619.15,
                635.05,
                650.85,
                666.55,
                682.15,
                697.64,
                713.04,
                728.34,
                743.54,
                758.64,
                757.27,
                742.99,
                728.69,
                714.36,
                700.02,
                685.66,
                671.27,
                656.86,
                642.43,
                627.99,
                613.52,
                599.02,
                584.51,
                397.92,
                394.98,
                392.01,
                389.01,
                385.98,
                382.93,
                379.85,
                376.73,
                373.59,
                370.43,
                367.23,
                364.01,
                360.75,
                357.47,
                354.16,
                350.83,
                351.59,
                353.71,
                355.84,
                357.98,
                360.13,
                362.3,
                364.47,
                366.65,
                368.85,
                371.06,
                373.27,
                375.5,
                377.74,
                379.99,
                382.25,
                384.53,
                386.81,
                389.1,
                391.41,
                394.46,
                398.42,
                402.41,
                406.44,
                410.49,
                414.58,
                418.69,
                422.84,
                427.02,
                431.23,
                435.47,
                439.75,
                444.05,
                448.39,
                452.76,
                457.16,
                461.59,
                466.06,
                470.56,
                475.99,
                486.32,
                496.72,
                507.19,
                517.74,
                528.35,
                539.04,
                549.8,
                560.63,
                571.54,
                582.52,
                593.57,
                604.69,
                615.89,
                627.15,
                638.5,
                649.91,
                661.4,
                672.96,
                684.59,
                682.59,
                678.21,
                673.78,
                669.28,
                664.73,
                660.12,
                655.46,
                650.73,
                645.95,
                641.1,
                636.2,
                631.24,
                626.22,
                621.14,
                616.01,
                610.81,
                605.56,
                600.24,
                594.87,
                587.95,
                579.75,
                571.5,
                563.19,
                554.83,
                546.41,
                537.94,
                529.42,
                520.84,
                512.21,
                503.52,
                494.78,
                485.99,
                477.14,
                468.23,
                459.27,
                450.26,
                441.19,
                432.07,
                424.14,
                420.16,
                416.15,
                412.1,
                408.02,
                403.9,
                399.74,
                395.55,
                391.32,
                387.06,
                382.75,
                378.42,
                374.04,
                369.63,
                365.18,
                360.7,
                356.18,
                351.62,
                347.03,
                342.4,
                340.88,
                339.62,
                338.36,
                337.11,
                335.87,
                334.64,
                333.52,
                332.53,
                331.54,
                330.56,
                329.58,
                328.6,
                327.63,
                326.67,
                325.71,
                324.75,
                323.8,
                322.85,
                321.91,
                321.39,
                321.14,
                320.9,
                320.66,
                320.41,
                320.17,
                319.93,
                319.68,
                319.44,
                319.2,
                318.95,
                318.71,
                318.47,
                318.22,
                317.98,
                317.73,
                317.49,
                317.25,
                317.0,
                316.64,
                315.96,
                315.28,
                314.59,
                313.91,
                313.23,
                312.54,
                311.86,
                311.17,
                310.48,
                309.79,
                309.1,
                308.4,
                307.71,
                307.01,
                306.31,
                305.62,
                304.92,
                304.22,
                303.51,
                303.6,
                303.73,
                303.85,
                303.97,
                304.09,
                304.2,
                304.32,
                304.43,
                304.55,
                304.66,
                304.77,
                304.88,
                304.99,
                305.1,
                305.2,
                305.31,
                305.41,
                305.51,
                305.61,
                305.32,
                304.8,
                304.28,
                303.77,
                303.26,
                302.76,
                302.26,
                301.77,
                301.28,
                300.79,
                300.31,
                299.84,
                299.37,
                298.9,
                298.44,
                297.98,
                297.53,
                297.08,
                296.64,
                296.36,
                296.49,
                296.62,
                296.77,
                296.92,
                297.07,
                297.24,
                297.41,
                297.6,
                297.79,
                297.98,
                298.19,
                298.4,
                298.62,
                298.85,
                299.09,
                299.34,
                299.59,
                299.85,
                300.12,
                307.95,
                316.1,
                324.24,
                332.39,
                340.53,
                348.66,
                356.8,
                364.93,
                373.05,
                381.18,
                389.3,
                397.41,
                405.52,
                413.63,
                421.74,
                429.84,
                437.94,
                446.04,
                454.13,
                466.67,
                481.9,
                497.03,
                512.06,
                526.99,
                541.83,
                556.56,
                571.2,
                585.73,
                600.17,
                614.5,
                628.74,
                642.88,
                656.92,
                670.86,
                684.69,
                698.43,
                712.07,
                725.61,
                731.46,
                716.86,
                702.24,
                687.6,
                672.94,
                658.25,
                643.55,
                628.82,
                614.08,
                599.31,
                584.53,
                569.72,
                554.89,
                407.04,
                403.56,
                400.05,
                396.52,
                392.95,
                389.36,
                385.74,
                382.09,
                378.42,
                374.71,
                370.98,
                367.22,
                363.43,
                359.62,
                355.77,
                351.9,
                351.19,
                353.51,
                355.85,
                358.19,
                360.55,
                362.91,
                365.29,
                367.68,
                370.08,
                372.49,
                374.91,
                377.34,
                379.78,
                382.24,
                384.7,
                387.18,
                389.66,
                392.16,
                394.67,
                397.62,
                402.16,
                406.73,
                411.33,
                415.96,
                420.62,
                425.32,
                430.04,
                434.8,
                439.59,
                444.41,
                449.27,
                454.15,
                459.07,
                464.02,
                469.0,
                474.01,
                479.05,
                484.13,
                489.23,
                500.29,
                512.0,
                523.78,
                535.63,
                547.55,
                559.55,
                571.61,
                583.75,
                595.96,
                608.25,
                620.61,
                633.04,
                645.54,
                658.11,
                670.76,
                683.48,
                696.27,
                709.14,
                722.08,
                723.96,
                718.52,
                713.03,
                707.47,
                701.86,
                696.19,
                690.46,
                684.68,
                678.83,
                672.93,
                666.97,
                660.95,
                654.87,
                648.73,
                642.53,
                636.28,
                629.96,
                623.59,
                617.15,
                609.89,
                600.73,
                591.51,
                582.24,
                572.92,
                563.54,
                554.1,
                544.62,
                535.08,
                525.48,
                515.52,
                505.35,
                495.19,
                485.04,
                474.9,
                464.76,
                454.63,
                444.5,
                434.38,
                424.27,
                419.58,
                415.0,
                410.45,
                405.9,
                401.36,
                396.84,
                392.33,
                387.84,
                383.36,
                378.89,
                374.43,
                369.98,
                365.55,
                361.13,
                356.73,
                352.34,
                347.96,
                343.59,
                339.24,
                337.15,
                336.2,
                335.26,
                334.33,
                333.4,
                332.48,
                331.56,
                330.64,
                329.73,
                328.82,
                327.92,
                327.02,
                326.12,
                325.23,
                324.35,
                323.46,
                322.59,
                321.71,
                320.84,
                320.19,
                319.95,
                319.7,
                319.46,
                319.22,
                318.97,
                318.73,
                318.49,
                318.24,
                318.0,
                317.76,
                317.51,
                317.27,
                317.03,
                316.78,
                316.54,
                316.3,
                316.05,
                315.81,
                315.56,
                314.86,
                314.15,
                313.45,
                312.75,
                312.04,
                311.34,
                310.63,
                309.92,
                309.21,
                308.5,
                307.78,
                307.07,
                306.35,
                305.63,
                304.92,
                304.2,
                303.48,
                302.75,
                302.03,
                301.87,
                301.98,
                302.08,
                302.17,
                302.27,
                302.37,
                302.46,
                302.56,
                302.65,
                302.74,
                302.83,
                302.92,
                303.0,
                303.09,
                303.17,
                303.26,
                303.34,
                303.42,
                303.5,
                303.39,
                302.94,
                302.5,
                302.06,
                301.63,
                301.2,
                300.78,
                300.36,
                299.95,
                299.54,
                299.13,
                298.73,
                298.33,
                297.94,
                297.56,
                297.17,
                296.8,
                296.42,
                296.05,
                295.7,
                295.96,
                296.22,
                296.49,
                296.77,
                297.05,
                297.35,
                297.65,
                297.96,
                298.28,
                298.6,
                298.94,
                299.28,
                299.63,
                299.99,
                300.35,
                300.72,
                301.11,
                301.5,
                301.89,
                307.51,
                315.6,
                323.69,
                331.78,
                339.87,
                347.95,
                356.03,
                364.1,
                372.17,
                380.24,
                388.31,
                396.37,
                404.43,
                412.48,
                420.53,
                428.58,
                436.62,
                444.66,
                452.7,
                462.63,
                476.3,
                489.87,
                503.34,
                516.71,
                529.98,
                543.16,
                556.23,
                569.21,
                582.08,
                594.86,
                607.54,
                620.12,
                632.6,
                644.97,
                657.25,
                669.43,
                681.51,
                693.49,
                705.37,
                690.91,
                675.97,
                661.01,
                646.04,
                631.04,
                616.01,
                600.97,
                585.91,
                570.83,
                555.72,
                540.6,
                525.45,
                416.41,
                412.39,
                408.35,
                404.28,
                400.18,
                396.05,
                391.9,
                387.71,
                383.5,
                379.26,
                374.99,
                370.7,
                366.37,
                362.02,
                357.63,
                353.22,
                350.69,
                353.21,
                355.75,
                358.3,
                360.86,
                363.43,
                366.01,
                368.6,
                371.2,
                373.82,
                376.44,
                379.08,
                381.72,
                384.38,
                387.05,
                389.73,
                392.42,
                395.12,
                397.83,
                400.56,
                405.6,
                410.75,
                415.93,
                421.14,
                426.38,
                431.66,
                436.96,
                442.3,
                447.67,
                453.07,
                458.5,
                463.96,
                469.46,
                474.98,
                480.54,
                486.13,
                491.76,
                497.41,
                503.09,
                513.61,
                526.63,
                539.71,
                552.87,
                566.1,
                579.4,
                592.77,
                606.22,
                619.74,
                633.33,
                646.99,
                660.72,
                674.0,
                686.91,
                699.73,
                712.44,
                725.05,
                737.55,
                749.95,
                755.21,
                747.58,
                739.96,
                732.34,
                724.72,
                717.1,
                709.49,
                701.88,
                694.27,
                686.67,
                679.07,
                671.47,
                663.88,
                656.28,
                648.69,
                641.11,
                633.52,
                625.94,
                618.36,
                610.69,
                600.58,
                590.47,
                580.37,
                570.28,
                560.2,
                550.12,
                540.05,
                529.98,
                519.92,
                509.87,
                499.82,
                489.79,
                479.75,
                469.73,
                459.71,
                449.7,
                439.69,
                429.69,
                419.7,
                413.78,
                409.44,
                405.1,
                400.78,
                396.47,
                392.18,
                387.89,
                383.62,
                379.37,
                375.12,
                370.89,
                366.67,
                362.47,
                358.27,
                354.09,
                349.93,
                345.77,
                341.63,
                337.5,
                334.69,
                333.83,
                332.96,
                332.1,
                331.25,
                330.4,
                329.55,
                328.71,
                327.88,
                327.04,
                326.21,
                325.39,
                324.57,
                323.75,
                322.94,
                322.14,
                321.33,
                320.54,
                319.74,
                319.0,
                318.75,
                318.51,
                318.27,
                318.02,
                317.78,
                317.54,
                317.29,
                317.05,
                316.81,
                316.56,
                316.32,
                316.07,
                315.83,
                315.59,
                315.34,
                315.1,
                314.86,
                314.61,
                314.37,
                313.77,
                313.04,
                312.32,
                311.6,
                310.87,
                310.14,
                309.41,
                308.68,
                307.95,
                307.22,
                306.48,
                305.75,
                305.01,
                304.27,
                303.53,
                302.79,
                302.05,
                301.3,
                300.56,
                300.16,
                300.24,
                300.32,
                300.39,
                300.47,
                300.54,
                300.62,
                300.69,
                300.76,
                300.83,
                300.9,
                300.96,
                301.03,
                301.09,
                301.16,
                301.22,
                301.28,
                301.34,
                301.39,
                301.42,
                301.04,
                300.68,
                300.32,
                299.96,
                299.61,
                299.26,
                298.91,
                298.57,
                298.24,
                297.91,
                297.58,
                297.26,
                296.94,
                296.63,
                296.33,
                296.02,
                295.72,
                295.43,
                295.14,
                295.35,
                295.74,
                296.14,
                296.55,
                296.96,
                297.38,
                297.81,
                298.25,
                298.7,
                299.15,
                299.61,
                300.08,
                300.56,
                301.04,
                301.54,
                302.04,
                302.55,
                303.07,
                303.59,
                307.1,
                315.14,
                323.17,
                331.21,
                339.24,
                347.26,
                355.29,
                363.31,
                371.33,
                379.34,
                387.35,
                395.35,
                403.36,
                411.36,
                419.35,
                427.35,
                435.34,
                443.32,
                451.3,
                459.48,
                471.59,
                483.6,
                495.51,
                507.32,
                519.04,
                530.65,
                542.17,
                553.58,
                564.9,
                576.12,
                587.24,
                598.26,
                609.18,
                620.0,
                630.72,
                641.34,
                651.86,
                662.28,
                672.6,
                665.15,
                649.89,
                634.62,
                619.32,
                604.0,
                588.67,
                573.31,
                557.93,
                542.53,
                527.1,
                511.66,
                496.2,
                426.04,
                421.49,
                416.91,
                412.3,
                407.66,
                403.0,
                398.31,
                393.59,
                388.84,
                384.06,
                379.26,
                374.43,
                369.57,
                364.68,
                359.76,
                354.81,
                350.09,
                352.82,
                355.56,
                358.31,
                361.07,
                363.85,
                366.63,
                369.43,
                372.23,
                375.05,
                377.88,
                380.72,
                383.57,
                386.43,
                389.72,
                393.26,
                396.81,
                400.36,
                403.91,
                407.48,
                412.97,
                419.19,
                425.4,
                431.61,
                437.81,
                444.0,
                450.19,
                456.38,
                462.55,
                468.72,
                474.89,
                481.05,
                487.2,
                493.35,
                499.49,
                505.62,
                511.75,
                517.87,
                523.99,
                532.7,
                544.98,
                557.16,
                569.23,
                581.21,
                593.08,
                604.85,
                616.51,
                628.08,
                639.54,
                650.9,
                662.15,
                673.31,
                684.36,
                695.3,
                706.15,
                716.89,
                727.52,
                738.06,
                746.63,
                739.06,
                731.49,
                723.92,
                716.36,
                708.8,
                701.24,
                693.69,
                686.14,
                678.59,
                671.04,
                663.5,
                655.96,
                648.42,
                640.88,
                633.35,
                625.82,
                618.3,
                610.77,
                603.25,
                593.79,
                583.81,
                573.83,
                563.86,
                553.89,
                543.93,
                533.98,
                524.03,
                514.09,
                504.16,
                494.24,
                484.32,
                474.4,
                464.5,
                454.6,
                444.71,
                434.82,
                424.94,
                415.07,
                407.87,
                403.75,
                399.64,
                395.55,
                391.46,
                387.39,
                383.34,
                379.29,
                375.26,
                371.24,
                367.24,
                363.24,
                359.26,
                355.3,
                351.34,
                347.4,
                343.47,
                339.56,
                335.65,
                332.2,
                331.41,
                330.62,
                329.84,
                329.06,
                328.28,
                327.51,
                326.75,
                325.98,
                325.23,
                324.47,
                323.72,
                322.98,
                322.24,
                321.5,
                320.77,
                320.04,
                319.32,
                318.6,
                317.89,
                317.56,
                317.31,
                317.07,
                316.83,
                316.58,
                316.34,
                316.1,
                315.85,
                315.61,
                315.37,
                315.12,
                314.88,
                314.64,
                314.39,
                314.15,
                313.91,
                313.66,
                313.42,
                313.17,
                312.69,
                311.95,
                311.2,
                310.45,
                309.71,
                308.96,
                308.21,
                307.45,
                306.7,
                305.95,
                305.19,
                304.43,
                303.68,
                302.92,
                302.16,
                301.39,
                300.63,
                299.86,
                299.1,
                298.45,
                298.51,
                298.57,
                298.62,
                298.68,
                298.73,
                298.78,
                298.83,
                298.88,
                298.93,
                298.98,
                299.02,
                299.07,
                299.11,
                299.15,
                299.19,
                299.23,
                299.27,
                299.3,
                299.34,
                299.11,
                298.81,
                298.53,
                298.24,
                297.97,
                297.69,
                297.42,
                297.16,
                296.9,
                296.64,
                296.39,
                296.15,
                295.9,
                295.67,
                295.43,
                295.21,
                294.98,
                294.76,
                294.55,
                294.67,
                295.19,
                295.72,
                296.25,
                296.79,
                297.34,
                297.9,
                298.47,
                299.04,
                299.62,
                300.21,
                300.81,
                301.42,
                302.03,
                302.65,
                303.28,
                303.92,
                304.57,
                305.22,
                306.72,
                314.7,
                322.68,
                330.66,
                338.64,
                346.61,
                354.58,
                362.55,
                370.51,
                378.47,
                386.42,
                394.37,
                402.32,
                410.27,
                418.21,
                426.15,
                434.08,
                442.01,
                449.94,
                457.86,
                467.78,
                478.23,
                488.58,
                498.84,
                508.99,
                519.05,
                529.0,
                538.86,
                548.62,
                558.28,
                567.84,
                577.3,
                586.66,
                595.92,
                605.08,
                614.15,
                623.11,
                631.97,
                640.73,
                639.57,
                624.0,
                608.4,
                592.79,
                577.16,
                561.5,
                545.83,
                530.13,
                514.41,
                498.67,
                482.91,
                467.13,
                437.85,
                433.26,
                428.7,
                424.16,
                419.66,
                415.18,
                410.73,
                406.31,
                401.92,
                397.56,
                393.22,
                388.92,
                384.64,
                380.39,
                376.18,
                371.99,
                367.83,
                369.79,
                373.35,
                376.92,
                380.5,
                384.08,
                387.67,
                391.26,
                394.86,
                398.47,
                402.08,
                405.7,
                409.33,
                412.96,
                416.6,
                420.24,
                423.9,
                427.55,
                431.22,
                434.89,
                439.73,
                445.83,
                451.92,
                458.01,
                464.09,
                470.17,
                476.24,
                482.3,
                488.36,
                494.41,
                500.46,
                506.5,
                512.53,
                518.56,
                524.58,
                530.6,
                536.61,
                542.61,
                548.61,
                555.36,
                565.77,
                576.08,
                586.29,
                596.39,
                606.4,
                616.3,
                626.1,
                635.79,
                645.39,
                654.88,
                664.27,
                673.55,
                682.74,
                691.82,
                700.79,
                709.67,
                718.43,
                727.1,
                735.66,
                730.51,
                722.99,
                715.48,
                707.97,
                700.47,
                692.97,
                685.47,
                677.97,
                670.47,
                662.98,
                655.49,
                648.01,
                640.53,
                633.05,
                625.57,
                618.1,
                610.62,
                603.16,
                595.69,
                586.95,
                577.08,
                567.22,
                557.37,
                547.52,
                537.68,
                527.85,
                518.03,
                508.21,
                498.39,
                488.59,
                478.79,
                468.99,
                459.21,
                449.43,
                439.65,
                429.89,
                420.13,
                410.38,
                401.84,
                397.95,
                394.07,
                390.2,
                386.34,
                382.49,
                378.66,
                374.84,
                371.04,
                367.24,
                363.46,
                359.7,
                355.94,
                352.2,
                348.47,
                344.76,
                341.05,
                337.36,
                333.68,
                330.02,
                328.95,
                328.24,
                327.53,
                326.83,
                326.13,
                325.43,
                324.74,
                324.05,
                323.37,
                322.69,
                322.02,
                321.35,
                320.68,
                320.02,
                319.36,
                318.71,
                318.06,
                317.42,
                316.78,
                316.36,
                316.12,
                315.87,
                315.63,
                315.39,
                315.14,
                314.9,
                314.66,
                314.41,
                314.17,
                313.93,
                313.68,
                313.44,
                313.2,
                312.95,
                312.71,
                312.47,
                312.22,
                311.98,
                311.62,
                310.86,
                310.09,
                309.32,
                308.56,
                307.79,
                307.01,
                306.24,
                305.47,
                304.69,
                303.91,
                303.13,
                302.36,
                301.57,
                300.79,
                300.01,
                299.22,
                298.44,
                297.65,
                296.86,
                296.79,
                296.83,
                296.87,
                296.9,
                296.93,
                296.96,
                296.99,
                297.02,
                297.05,
                297.07,
                297.09,
                297.12,
                297.14,
                297.16,
                297.18,
                297.19,
                297.21,
                297.23,
                297.24,
                297.13,
                296.91,
                296.7,
                296.49,
                296.29,
                296.09,
                295.89,
                295.7,
                295.52,
                295.34,
                295.16,
                294.99,
                294.82,
                294.66,
                294.5,
                294.35,
                294.2,
                294.06,
                293.92,
                293.92,
                294.57,
                295.23,
                295.89,
                296.56,
                297.24,
                297.92,
                298.62,
                299.32,
                300.03,
                300.75,
                301.47,
                302.2,
                302.95,
                303.7,
                304.45,
                305.22,
                305.99,
                306.77,
                307.56,
                314.3,
                322.23,
                330.15,
                338.07,
                345.99,
                353.9,
                361.81,
                369.72,
                377.63,
                385.53,
                393.42,
                401.32,
                409.21,
                417.09,
                424.98,
                432.86,
                440.73,
                448.61,
                456.48,
                464.87,
                473.76,
                482.55,
                491.25,
                499.85,
                508.34,
                516.74,
                525.04,
                533.24,
                541.34,
                549.34,
                557.25,
                565.05,
                572.75,
                580.36,
                587.86,
                595.27,
                602.57,
                609.77,
                614.17,
                598.29,
                582.38,
                566.45,
                550.5,
                534.52,
                518.53,
                502.52,
                486.48,
                470.43,
                454.35,
                438.25,
                456.24,
                452.19,
                448.16,
                444.16,
                440.19,
                436.25,
                432.34,
                428.46,
                424.61,
                420.78,
                416.98,
                413.22,
                409.48,
                405.77,
                402.09,
                398.44,
                394.82,
                395.23,
                398.9,
                402.57,
                406.25,
                409.94,
                413.64,
                417.34,
                421.05,
                424.76,
                428.48,
                432.21,
                435.94,
                439.68,
                443.42,
                447.17,
                450.93,
                454.7,
                458.47,
                462.24,
                466.55,
                472.53,
                478.5,
                484.47,
                490.43,
                496.39,
                502.34,
                508.29,
                514.22,
                520.16,
                526.08,
                532.01,
                537.92,
                543.83,
                549.73,
                555.63,
                561.52,
                567.41,
                573.28,
                579.16,
                587.49,
                595.93,
                604.27,
                612.51,
                620.65,
                628.68,
                636.61,
                644.44,
                652.17,
                659.8,
                667.32,
                674.74,
                682.05,
                689.27,
                696.38,
                703.38,
                710.29,
                717.09,
                723.78,
                721.93,
                714.47,
                707.01,
                699.56,
                692.11,
                684.66,
                677.22,
                669.77,
                662.33,
                654.9,
                647.46,
                640.03,
                632.61,
                625.18,
                617.76,
                610.34,
                602.92,
                595.51,
                588.1,
                580.04,
                570.29,
                560.55,
                550.82,
                541.1,
                531.38,
                521.66,
                511.96,
                502.26,
                492.56,
                482.88,
                473.2,
                463.52,
                453.86,
                444.19,
                434.54,
                424.89,
                415.25,
                405.62,
                395.99,
                392.03,
                388.37,
                384.73,
                381.1,
                377.48,
                373.87,
                370.28,
                366.7,
                363.13,
                359.57,
                356.03,
                352.5,
                348.99,
                345.48,
                341.99,
                338.51,
                335.05,
                331.6,
                328.16,
                326.46,
                325.82,
                325.18,
                324.56,
                323.93,
                323.31,
                322.69,
                322.08,
                321.47,
                320.87,
                320.27,
                319.68,
                319.09,
                318.5,
                317.92,
                317.34,
                316.77,
                316.2,
                315.64,
                315.17,
                314.92,
                314.68,
                314.44,
                314.19,
                313.95,
                313.71,
                313.46,
                313.22,
                312.98,
                312.73,
                312.49,
                312.25,
                312.0,
                311.76,
                311.52,
                311.27,
                311.03,
                310.78,
                310.54,
                309.78,
                309.0,
                308.21,
                307.42,
                306.62,
                305.83,
                305.04,
                304.24,
                303.44,
                302.65,
                301.85,
                301.05,
                300.24,
                299.44,
                298.64,
                297.83,
                297.02,
                296.21,
                295.4,
                295.09,
                295.11,
                295.12,
                295.13,
                295.14,
                295.15,
                295.16,
                295.17,
                295.17,
                295.17,
                295.18,
                295.18,
                295.18,
                295.18,
                295.17,
                295.17,
                295.17,
                295.16,
                295.15,
                295.11,
                294.96,
                294.83,
                294.69,
                294.56,
                294.44,
                294.32,
                294.21,
                294.1,
                293.99,
                293.89,
                293.79,
                293.7,
                293.61,
                293.53,
                293.45,
                293.38,
                293.31,
                293.24,
                293.18,
                293.88,
                294.66,
                295.45,
                296.25,
                297.05,
                297.87,
                298.69,
                299.52,
                300.36,
                301.2,
                302.06,
                302.92,
                303.79,
                304.67,
                305.55,
                306.44,
                307.35,
                308.26,
                309.17,
                313.93,
                321.8,
                329.67,
                337.54,
                345.4,
                353.26,
                361.11,
                368.97,
                376.82,
                384.66,
                392.5,
                400.34,
                408.18,
                416.01,
                423.84,
                431.67,
                439.49,
                447.31,
                455.12,
                462.85,
                470.19,
                477.42,
                484.56,
                491.6,
                498.54,
                505.38,
                512.12,
                518.76,
                525.31,
                531.75,
                538.1,
                544.34,
                550.49,
                556.53,
                562.48,
                568.33,
                574.07,
                579.72,
                585.27,
                572.76,
                556.53,
                540.33,
                524.22,
                508.22,
                492.31,
                476.49,
                460.77,
                445.15,
                429.63,
                414.2,
                474.37,
                470.85,
                467.37,
                463.91,
                460.47,
                457.07,
                453.7,
                450.35,
                447.04,
                443.75,
                440.49,
                437.26,
                434.06,
                430.89,
                427.74,
                424.63,
                421.54,
                420.61,
                424.39,
                428.17,
                431.96,
                435.76,
                439.56,
                443.37,
                447.18,
                451.0,
                454.83,
                458.66,
                462.5,
                466.34,
                470.2,
                474.05,
                477.92,
                481.79,
                485.66,
                489.55,
                493.44,
                499.29,
                505.14,
                510.99,
                516.83,
                522.67,
                528.5,
                534.33,
                540.15,
                545.96,
                551.77,
                557.57,
                563.37,
                569.16,
                574.94,
                580.72,
                586.49,
                592.26,
                598.02,
                603.77,
                610.14,
                616.71,
                623.19,
                629.56,
                635.83,
                642.0,
                648.06,
                654.03,
                659.89,
                665.65,
                671.3,
                676.86,
                682.31,
                687.66,
                692.9,
                698.04,
                703.08,
                708.02,
                712.85,
                713.32,
                705.92,
                698.52,
                691.12,
                683.72,
                676.33,
                668.94,
                661.55,
                654.17,
                646.78,
                639.41,
                632.03,
                624.66,
                617.29,
                609.92,
                602.56,
                595.19,
                587.84,
                580.48,
                573.07,
                563.45,
                553.83,
                544.21,
                534.61,
                525.01,
                515.41,
                505.82,
                496.24,
                486.67,
                477.1,
                467.54,
                457.99,
                448.44,
                438.9,
                429.37,
                419.84,
                410.32,
                400.8,
                391.3,
                385.99,
                382.56,
                379.14,
                375.74,
                372.34,
                368.96,
                365.59,
                362.24,
                358.9,
                355.57,
                352.25,
                348.95,
                345.65,
                342.38,
                339.11,
                335.86,
                332.62,
                329.39,
                326.18,
                323.92,
                323.36,
                322.8,
                322.25,
                321.7,
                321.15,
                320.61,
                320.07,
                319.54,
                319.01,
                318.49,
                317.97,
                317.45,
                316.94,
                316.43,
                315.93,
                315.43,
                314.94,
                314.45,
                313.97,
                313.73,
                313.48,
                313.24,
                313.0,
                312.75,
                312.51,
                312.27,
                312.02,
                311.78,
                311.54,
                311.29,
                311.05,
                310.81,
                310.56,
                310.32,
                310.08,
                309.83,
                309.59,
                309.35,
                308.72,
                307.91,
                307.1,
                306.29,
                305.48,
                304.66,
                303.85,
                303.03,
                302.21,
                301.39,
                300.57,
                299.75,
                298.93,
                298.1,
                297.27,
                296.45,
                295.62,
                294.79,
                293.96,
                293.4,
                293.39,
                293.38,
                293.38,
                293.36,
                293.35,
                293.34,
                293.32,
                293.31,
                293.29,
                293.27,
                293.25,
                293.23,
                293.21,
                293.18,
                293.16,
                293.13,
                293.1,
                293.07,
                293.04,
                292.98,
                292.91,
                292.86,
                292.8,
                292.75,
                292.71,
                292.67,
                292.63,
                292.6,
                292.57,
                292.55,
                292.53,
                292.52,
                292.51,
                292.51,
                292.51,
                292.51,
                292.52,
                292.54,
                293.11,
                294.02,
                294.94,
                295.87,
                296.8,
                297.74,
                298.69,
                299.65,
                300.62,
                301.59,
                302.57,
                303.56,
                304.56,
                305.56,
                306.58,
                307.6,
                308.63,
                309.66,
                310.71,
                313.59,
                321.41,
                329.22,
                337.03,
                344.84,
                352.64,
                360.45,
                368.24,
                376.04,
                383.79,
                391.4,
                398.94,
                406.41,
                413.81,
                421.14,
                428.41,
                435.6,
                442.72,
                449.78,
                456.76,
                462.42,
                467.89,
                473.3,
                478.65,
                483.95,
                489.18,
                494.35,
                499.47,
                504.53,
                509.52,
                514.46,
                519.34,
                524.15,
                528.91,
                533.61,
                538.25,
                542.83,
                547.35,
                551.81,
                545.79,
                531.0,
                516.31,
                501.71,
                487.21,
                472.8,
                458.49,
                444.28,
                430.16,
                416.14,
                402.22,
                492.25,
                489.27,
                486.31,
                483.39,
                480.5,
                477.63,
                474.79,
                471.99,
                469.21,
                466.46,
                463.73,
                461.04,
                458.38,
                455.74,
                453.13,
                450.56,
                448.01,
                445.95,
                449.83,
                453.72,
                457.61,
                461.52,
                465.42,
                469.34,
                473.26,
                477.19,
                481.12,
                485.06,
                489.0,
                492.96,
                496.91,
                500.88,
                504.85,
                508.83,
                512.81,
                516.8,
                520.79,
                526.1,
                531.84,
                537.57,
                543.29,
                549.01,
                554.72,
                560.43,
                566.13,
                571.83,
                577.52,
                583.2,
                588.88,
                594.55,
                600.21,
                605.87,
                611.52,
                617.17,
                622.81,
                628.45,
                633.72,
                638.43,
                643.03,
                647.54,
                651.94,
                656.25,
                660.45,
                664.55,
                668.54,
                672.44,
                676.23,
                679.91,
                683.5,
                686.98,
                690.36,
                693.64,
                696.81,
                699.88,
                702.85,
                704.69,
                697.34,
                689.99,
                682.65,
                675.31,
                667.97,
                660.63,
                653.3,
                645.97,
                638.64,
                631.32,
                624.0,
                616.68,
                609.37,
                602.05,
                594.74,
                587.44,
                580.13,
                572.83,
                565.53,
                556.54,
                547.04,
                537.54,
                528.05,
                518.57,
                509.1,
                499.63,
                490.17,
                480.72,
                471.27,
                461.83,
                452.39,
                442.96,
                433.54,
                424.13,
                414.72,
                405.32,
                395.92,
                386.54,
                379.84,
                376.63,
                373.44,
                370.26,
                367.09,
                363.93,
                360.79,
                357.66,
                354.54,
                351.44,
                348.35,
                345.27,
                342.2,
                339.15,
                336.11,
                333.08,
                330.07,
                327.07,
                324.08,
                321.35,
                320.86,
                320.37,
                319.9,
                319.42,
                318.95,
                318.48,
                318.02,
                317.56,
                317.11,
                316.66,
                316.22,
                315.78,
                315.34,
                314.91,
                314.48,
                314.06,
                313.64,
                313.22,
                312.81,
                312.53,
                312.29,
                312.04,
                311.8,
                311.56,
                311.31,
                311.07,
                310.83,
                310.59,
                310.34,
                310.1,
                309.86,
                309.61,
                309.37,
                309.12,
                308.88,
                308.64,
                308.39,
                308.15,
                307.67,
                306.84,
                306.01,
                305.17,
                304.34,
                303.5,
                302.67,
                301.83,
                300.99,
                300.15,
                299.31,
                298.46,
                297.62,
                296.77,
                295.92,
                295.08,
                294.23,
                293.38,
                292.52,
                291.72,
                291.69,
                291.66,
                291.63,
                291.6,
                291.57,
                291.53,
                291.49,
                291.46,
                291.42,
                291.38,
                291.34,
                291.29,
                291.25,
                291.21,
                291.16,
                291.11,
                291.06,
                291.01,
                290.96,
                290.95,
                290.96,
                290.98,
                291.0,
                291.02,
                291.05,
                291.09,
                291.13,
                291.17,
                291.22,
                291.27,
                291.33,
                291.39,
                291.46,
                291.53,
                291.67,
                291.84,
                292.01,
                292.17,
                292.68,
                293.83,
                294.99,
                296.17,
                297.37,
                298.58,
                299.81,
                301.06,
                302.32,
                303.6,
                304.89,
                306.21,
                307.54,
                308.88,
                310.25,
                311.62,
                313.02,
                314.43,
                315.86,
                317.31,
                324.37,
                331.43,
                338.42,
                345.35,
                352.2,
                358.99,
                365.7,
                372.35,
                378.93,
                385.44,
                391.88,
                398.25,
                404.56,
                410.79,
                416.95,
                423.05,
                429.07,
                435.03,
                440.91,
                445.97,
                450.52,
                455.0,
                459.42,
                463.79,
                468.09,
                472.34,
                476.53,
                480.66,
                484.72,
                488.73,
                492.68,
                496.57,
                500.4,
                504.18,
                507.89,
                511.54,
                515.13,
                518.66,
                517.89,
                504.6,
                491.41,
                478.32,
                465.32,
                452.42,
                439.62,
                426.91,
                414.29,
                401.78,
                389.36,
                509.86,
                507.42,
                505.01,
                502.62,
                500.26,
                497.93,
                495.63,
                493.36,
                491.12,
                488.91,
                486.72,
                484.56,
                482.44,
                480.34,
                478.27,
                476.23,
                474.22,
                472.23,
                475.22,
                479.21,
                483.22,
                487.22,
                491.24,
                495.26,
                499.29,
                503.32,
                507.36,
                511.41,
                515.46,
                519.52,
                523.58,
                527.65,
                531.73,
                535.81,
                539.9,
                544.0,
                548.1,
                552.98,
                558.59,
                564.21,
                569.81,
                575.41,
                581.0,
                586.59,
                592.17,
                597.75,
                603.32,
                608.88,
                614.44,
                619.99,
                625.54,
                631.08,
                636.61,
                642.14,
                647.67,
                653.18,
                658.22,
                661.07,
                663.81,
                666.45,
                668.99,
                671.43,
                673.76,
                676.0,
                678.13,
                680.16,
                682.08,
                683.91,
                685.63,
                687.25,
                688.76,
                690.18,
                691.48,
                692.69,
                693.79,
                694.79,
                688.73,
                681.44,
                674.15,
                666.86,
                659.58,
                652.3,
                645.02,
                637.75,
                630.48,
                623.21,
                615.94,
                608.68,
                601.42,
                594.16,
                586.9,
                579.65,
                572.4,
                565.16,
                557.91,
                549.57,
                540.19,
                530.81,
                521.44,
                512.08,
                502.73,
                493.38,
                484.04,
                474.7,
                465.37,
                456.05,
                446.74,
                437.43,
                428.13,
                418.83,
                409.54,
                400.26,
                390.98,
                381.71,
                373.57,
                370.59,
                367.62,
                364.66,
                361.72,
                358.79,
                355.87,
                352.97,
                350.08,
                347.2,
                344.33,
                341.48,
                338.64,
                335.81,
                332.99,
                330.19,
                327.4,
                324.63,
                321.86,
                319.11,
                318.32,
                317.91,
                317.51,
                317.11,
                316.71,
                316.32,
                315.93,
                315.55,
                315.17,
                314.8,
                314.43,
                314.06,
                313.7,
                313.34,
                312.99,
                312.64,
                312.3,
                311.96,
                311.62,
                311.34,
                311.09,
                310.85,
                310.61,
                310.36,
                310.12,
                309.88,
                309.63,
                309.39,
                309.15,
                308.9,
                308.66,
                308.42,
                308.17,
                307.93,
                307.69,
                307.44,
                307.2,
                306.96,
                306.63,
                305.77,
                304.92,
                304.07,
                303.21,
                302.36,
                301.5,
                300.64,
                299.78,
                298.92,
                298.05,
                297.19,
                296.32,
                295.46,
                294.59,
                293.72,
                292.85,
                291.97,
                291.1,
                290.23,
                290.0,
                290.08,
                290.16,
                290.24,
                290.32,
                290.4,
                290.47,
                290.54,
                290.6,
                290.67,
                290.73,
                290.79,
                290.85,
                290.9,
                290.95,
                291.0,
                291.05,
                291.09,
                291.14,
                291.23,
                291.4,
                291.56,
                291.73,
                291.9,
                292.06,
                292.23,
                292.39,
                292.56,
                292.73,
                292.89,
                293.06,
                293.23,
                293.39,
                293.56,
                293.73,
                293.89,
                294.06,
                294.23,
                294.48,
                295.9,
                297.33,
                298.77,
                300.24,
                301.72,
                303.21,
                304.73,
                306.25,
                307.8,
                309.36,
                310.94,
                312.54,
                314.15,
                315.78,
                317.42,
                319.09,
                320.77,
                322.46,
                324.17,
                328.93,
                334.9,
                340.79,
                346.61,
                352.37,
                358.05,
                363.67,
                369.22,
                374.7,
                380.11,
                385.45,
                390.73,
                395.93,
                401.06,
                406.13,
                411.12,
                416.05,
                420.91,
                425.69,
                430.06,
                433.68,
                437.23,
                440.73,
                444.16,
                447.54,
                450.86,
                454.12,
                457.32,
                460.46,
                463.54,
                466.57,
                469.53,
                472.43,
                475.28,
                478.06,
                480.79,
                483.45,
                486.05,
                488.6,
                477.33,
                465.64,
                454.05,
                442.56,
                431.16,
                419.86,
                408.65,
                397.54,
                386.53,
                375.61,
                527.23,
                525.32,
                523.44,
                521.59,
                519.77,
                517.98,
                516.22,
                514.48,
                512.78,
                511.1,
                509.45,
                507.83,
                506.24,
                504.68,
                503.14,
                501.64,
                500.16,
                498.72,
                500.55,
                504.66,
                508.77,
                512.88,
                517.0,
                521.13,
                525.26,
                529.4,
                533.55,
                537.7,
                541.86,
                546.03,
                550.2,
                554.37,
                558.56,
                562.75,
                566.94,
                571.15,
                575.35,
                579.91,
                585.41,
                590.9,
                596.39,
                601.87,
                607.34,
                612.81,
                618.27,
                623.73,
                629.18,
                634.63,
                640.07,
                645.5,
                650.93,
                656.35,
                661.76,
                667.17,
                672.58,
                677.97,
                683.37,
                684.64,
                685.52,
                686.29,
                686.97,
                687.54,
                688.01,
                688.38,
                688.65,
                688.81,
                688.88,
                688.84,
                688.69,
                688.45,
                688.1,
                687.65,
                687.09,
                686.44,
                685.68,
                684.81,
                680.1,
                672.86,
                665.63,
                658.39,
                651.17,
                643.94,
                636.72,
                629.5,
                622.28,
                615.07,
                607.85,
                600.65,
                593.44,
                586.24,
                579.04,
                571.84,
                564.65,
                557.45,
                550.27,
                542.54,
                533.27,
                524.02,
                514.77,
                505.53,
                496.29,
                487.06,
                477.84,
                468.63,
                459.42,
                450.21,
                441.02,
                431.83,
                422.65,
                413.47,
                404.3,
                395.14,
                385.98,
                376.83,
                367.69,
                364.43,
                361.68,
                358.95,
                356.23,
                353.53,
                350.84,
                348.16,
                345.49,
                342.84,
                340.19,
                337.57,
                334.95,
                332.35,
                329.76,
                327.18,
                324.62,
                322.07,
                319.53,
                317.0,
                315.74,
                315.41,
                315.08,
                314.75,
                314.43,
                314.11,
                313.8,
                313.49,
                313.19,
                312.89,
                312.6,
                312.31,
                312.02,
                311.74,
                311.46,
                311.19,
                310.92,
                310.65,
                310.39,
                310.14,
                309.9,
                309.65,
                309.41,
                309.17,
                308.92,
                308.77,
                308.63,
                308.48,
                308.32,
                308.16,
                307.99,
                307.82,
                307.64,
                307.46,
                307.27,
                307.07,
                306.87,
                306.67,
                306.45,
                305.77,
                305.02,
                304.28,
                303.54,
                302.81,
                302.08,
                301.36,
                300.65,
                299.94,
                299.24,
                298.54,
                297.85,
                297.17,
                296.49,
                295.81,
                295.15,
                294.49,
                293.83,
                293.18,
                292.89,
                292.93,
                292.97,
                293.01,
                293.04,
                293.08,
                293.11,
                293.13,
                293.16,
                293.18,
                293.2,
                293.22,
                293.23,
                293.24,
                293.25,
                293.26,
                293.26,
                293.26,
                293.26,
                293.28,
                293.45,
                293.62,
                293.78,
                293.95,
                294.12,
                294.28,
                294.45,
                294.62,
                294.78,
                294.95,
                295.12,
                295.28,
                295.45,
                295.62,
                295.78,
                295.95,
                296.12,
                296.28,
                296.45,
                297.82,
                299.51,
                301.23,
                302.96,
                304.7,
                306.47,
                308.24,
                310.04,
                311.85,
                313.68,
                315.53,
                317.39,
                319.27,
                321.16,
                323.07,
                325.0,
                326.95,
                328.91,
                330.89,
                334.12,
                338.98,
                343.78,
                348.5,
                353.16,
                357.75,
                362.27,
                366.72,
                371.1,
                375.41,
                379.65,
                383.83,
                387.93,
                391.97,
                395.94,
                399.83,
                403.66,
                407.42,
                411.11,
                414.68,
                417.37,
                420.0,
                422.57,
                425.08,
                427.53,
                429.92,
                432.25,
                434.52,
                436.74,
                438.89,
                440.99,
                443.02,
                445.0,
                446.92,
                448.77,
                450.57,
                452.31,
                453.99,
                455.61,
                449.18,
                439.0,
                428.91,
                418.92,
                409.02,
                399.23,
                389.52,
                379.92,
                370.41,
                360.99,
                544.33,
                542.96,
                541.62,
                540.31,
                539.02,
                537.77,
                536.54,
                535.34,
                534.17,
                533.03,
                531.92,
                530.84,
                529.78,
                528.76,
                527.76,
                526.79,
                525.85,
                524.94,
                525.84,
                530.05,
                534.26,
                538.49,
                542.71,
                546.95,
                551.19,
                555.43,
                559.69,
                563.95,
                568.21,
                572.48,
                576.76,
                581.04,
                585.33,
                589.63,
                593.93,
                598.24,
                602.56,
                606.9,
                612.28,
                617.65,
                623.02,
                628.38,
                633.74,
                639.09,
                644.43,
                649.77,
                655.1,
                660.43,
                665.75,
                671.06,
                676.37,
                681.67,
                686.97,
                692.26,
                697.55,
                702.83,
                708.1,
                709.13,
                708.15,
                707.06,
                705.87,
                704.58,
                703.19,
                701.7,
                700.1,
                698.4,
                696.6,
                694.7,
                692.69,
                690.59,
                688.37,
                686.06,
                683.64,
                681.12,
                678.5,
                675.77,
                671.43,
                664.25,
                657.07,
                649.9,
                642.72,
                635.55,
                628.38,
                621.22,
                614.06,
                606.9,
                599.74,
                592.59,
                585.43,
                578.29,
                571.14,
                564.0,
                556.86,
                549.72,
                542.59,
                535.44,
                526.3,
                517.16,
                508.04,
                498.91,
                489.8,
                480.69,
                471.58,
                462.49,
                453.4,
                444.31,
                435.24,
                426.17,
                417.1,
                408.05,
                399.0,
                389.95,
                380.92,
                371.89,
                362.86,
                358.15,
                355.63,
                353.12,
                350.63,
                348.15,
                345.68,
                343.23,
                340.79,
                338.36,
                336.11,
                333.98,
                331.85,
                329.73,
                327.62,
                325.5,
                323.39,
                321.29,
                319.18,
                317.09,
                315.62,
                315.53,
                315.43,
                315.32,
                315.2,
                315.08,
                314.95,
                314.81,
                314.66,
                314.5,
                314.34,
                314.16,
                313.98,
                313.8,
                313.6,
                313.39,
                313.18,
                312.96,
                312.73,
                312.49,
                312.29,
                312.07,
                311.86,
                311.63,
                311.4,
                311.17,
                310.93,
                310.68,
                310.43,
                310.17,
                309.91,
                309.64,
                309.36,
                309.08,
                308.8,
                308.51,
                308.21,
                307.91,
                307.6,
                307.08,
                306.43,
                305.78,
                305.14,
                304.5,
                303.88,
                303.25,
                302.63,
                302.02,
                301.42,
                300.82,
                300.22,
                299.63,
                299.05,
                298.47,
                297.9,
                297.34,
                296.78,
                296.22,
                295.81,
                295.81,
                295.81,
                295.8,
                295.79,
                295.78,
                295.77,
                295.75,
                295.73,
                295.71,
                295.69,
                295.66,
                295.64,
                295.6,
                295.57,
                295.53,
                295.5,
                295.45,
                295.41,
                295.36,
                295.51,
                295.67,
                295.84,
                296.0,
                296.17,
                296.34,
                296.5,
                296.67,
                296.84,
                297.0,
                297.17,
                297.34,
                297.5,
                297.67,
                297.84,
                298.0,
                298.17,
                298.34,
                298.5,
                299.59,
                301.55,
                303.53,
                305.53,
                307.54,
                309.57,
                311.61,
                313.68,
                315.75,
                317.85,
                319.96,
                322.09,
                324.23,
                326.39,
                328.57,
                330.77,
                332.98,
                335.21,
                337.45,
                339.93,
                343.7,
                347.39,
                351.02,
                354.58,
                358.07,
                361.49,
                364.84,
                368.12,
                371.34,
                374.48,
                377.56,
                380.57,
                383.5,
                386.37,
                389.17,
                391.9,
                394.56,
                397.15,
                399.68,
                401.6,
                403.3,
                404.94,
                406.52,
                408.05,
                409.51,
                410.92,
                412.26,
                413.55,
                414.78,
                415.95,
                417.06,
                418.11,
                419.1,
                420.03,
                420.9,
                421.71,
                422.46,
                423.15,
                420.15,
                411.47,
                402.89,
                394.4,
                386.01,
                377.71,
                369.51,
                361.41,
                353.4,
                345.49,
                561.18,
                560.35,
                559.54,
                558.77,
                558.02,
                557.3,
                556.61,
                555.95,
                555.31,
                554.71,
                554.13,
                553.58,
                553.07,
                552.58,
                552.12,
                551.68,
                551.28,
                550.91,
                551.07,
                555.39,
                559.71,
                564.04,
                568.37,
                572.71,
                577.06,
                581.41,
                585.77,
                590.14,
                594.51,
                598.89,
                603.27,
                607.66,
                612.06,
                616.46,
                620.87,
                625.28,
                629.71,
                634.13,
                639.21,
                644.46,
                649.71,
                654.95,
                660.19,
                665.42,
                670.65,
                675.87,
                681.08,
                686.29,
                691.49,
                696.69,
                701.88,
                707.06,
                712.24,
                717.41,
                722.58,
                727.74,
                732.89,
                734.56,
                731.71,
                728.76,
                725.71,
                722.56,
                719.3,
                715.95,
                712.49,
                708.93,
                705.26,
                701.5,
                697.63,
                693.66,
                689.58,
                685.41,
                681.13,
                676.75,
                672.26,
                667.67,
                662.74,
                655.62,
                648.49,
                641.37,
                634.25,
                627.14,
                620.02,
                612.91,
                605.8,
                598.7,
                591.6,
                584.5,
                577.45,
                570.63,
                563.76,
                556.84,
                549.86,
                542.83,
                535.75,
                528.61,
                520.19,
                511.38,
                502.61,
                493.87,
                485.15,
                476.47,
                467.81,
                459.18,
                450.58,
                442.01,
                433.47,
                424.96,
                416.48,
                408.03,
                399.6,
                391.21,
                382.84,
                374.51,
                366.2,
                360.43,
                358.33,
                356.24,
                354.15,
                352.06,
                349.98,
                347.9,
                345.82,
                343.75,
                341.68,
                339.62,
                337.56,
                335.5,
                333.45,
                331.4,
                329.35,
                327.31,
                325.27,
                323.24,
                321.31,
                321.08,
                320.84,
                320.59,
                320.33,
                320.07,
                319.8,
                319.52,
                319.23,
                318.94,
                318.63,
                318.32,
                318.0,
                317.67,
                317.34,
                317.0,
                316.64,
                316.28,
                315.92,
                315.54,
                315.21,
                314.9,
                314.59,
                314.27,
                313.94,
                313.61,
                313.28,
                312.93,
                312.59,
                312.23,
                311.87,
                311.51,
                311.14,
                310.76,
                310.38,
                309.99,
                309.6,
                309.2,
                308.8,
                308.34,
                307.78,
                307.23,
                306.69,
                306.15,
                305.62,
                305.09,
                304.57,
                304.05,
                303.54,
                303.04,
                302.54,
                302.05,
                301.56,
                301.08,
                300.6,
                300.13,
                299.67,
                299.21,
                298.76,
                298.71,
                298.66,
                298.61,
                298.56,
                298.51,
                298.45,
                298.39,
                298.33,
                298.27,
                298.2,
                298.14,
                298.06,
                297.99,
                297.91,
                297.84,
                297.75,
                297.67,
                297.58,
                297.5,
                297.56,
                297.73,
                297.89,
                298.06,
                298.23,
                298.39,
                298.56,
                298.72,
                298.89,
                299.06,
                299.22,
                299.39,
                299.56,
                299.72,
                299.89,
                300.06,
                300.22,
                300.39,
                300.56,
                301.21,
                303.44,
                305.69,
                307.95,
                310.23,
                312.52,
                314.83,
                317.16,
                319.5,
                321.87,
                324.24,
                326.64,
                329.05,
                331.48,
                333.92,
                336.38,
                338.86,
                341.35,
                343.86,
                346.39,
                349.03,
                351.63,
                354.16,
                356.62,
                359.01,
                361.34,
                363.59,
                365.77,
                367.89,
                369.94,
                371.92,
                373.83,
                375.67,
                377.44,
                379.14,
                380.77,
                382.34,
                383.83,
                385.25,
                386.36,
                387.13,
                387.85,
                388.5,
                389.1,
                389.64,
                390.12,
                390.54,
                390.9,
                391.2,
                391.44,
                391.62,
                391.75,
                391.81,
                391.81,
                391.76,
                391.64,
                391.47,
                391.23,
                390.25,
                383.08,
                376.0,
                369.01,
                362.12,
                355.33,
                348.63,
                342.03,
                335.52,
                329.11,
                577.77,
                577.48,
                577.21,
                576.97,
                576.76,
                576.57,
                576.42,
                576.29,
                576.19,
                576.13,
                576.09,
                576.08,
                576.09,
                576.14,
                576.21,
                576.32,
                576.45,
                576.61,
                576.8,
                580.68,
                585.11,
                589.54,
                593.98,
                598.43,
                602.88,
                607.34,
                611.8,
                616.28,
                620.75,
                625.24,
                629.73,
                634.23,
                638.73,
                643.24,
                647.75,
                652.27,
                656.8,
                661.34,
                666.2,
                671.33,
                676.46,
                681.59,
                686.7,
                691.82,
                696.92,
                702.02,
                707.12,
                712.21,
                717.29,
                722.37,
                727.44,
                732.5,
                737.4,
                742.17,
                746.88,
                751.54,
                756.15,
                758.94,
                754.21,
                749.5,
                744.78,
                740.07,
                735.35,
                730.64,
                725.93,
                721.23,
                716.52,
                711.82,
                707.12,
                702.42,
                697.73,
                693.03,
                688.34,
                683.65,
                678.97,
                674.28,
                669.6,
                662.83,
                655.64,
                648.4,
                641.1,
                633.76,
                626.35,
                618.9,
                611.39,
                603.83,
                596.21,
                588.54,
                580.82,
                573.04,
                565.21,
                557.32,
                549.39,
                541.39,
                533.35,
                525.24,
                517.02,
                508.73,
                500.48,
                492.25,
                484.05,
                475.88,
                467.73,
                459.62,
                451.54,
                443.48,
                435.46,
                427.46,
                419.49,
                411.55,
                403.65,
                395.77,
                387.92,
                380.09,
                372.3,
                365.39,
                363.35,
                361.32,
                359.3,
                357.28,
                355.26,
                353.24,
                351.23,
                349.22,
                347.22,
                345.22,
                343.23,
                341.23,
                339.25,
                337.26,
                335.28,
                333.3,
                331.33,
                329.36,
                327.4,
                326.7,
                326.32,
                325.93,
                325.54,
                325.13,
                324.72,
                324.3,
                323.88,
                323.44,
                323.0,
                322.55,
                322.09,
                321.63,
                321.15,
                320.67,
                320.18,
                319.68,
                319.17,
                318.66,
                318.19,
                317.79,
                317.38,
                316.96,
                316.54,
                316.11,
                315.68,
                315.24,
                314.8,
                314.35,
                313.89,
                313.43,
                312.96,
                312.49,
                312.01,
                311.53,
                311.04,
                310.55,
                310.05,
                309.54,
                309.08,
                308.63,
                308.18,
                307.74,
                307.3,
                306.87,
                306.45,
                306.03,
                305.61,
                305.21,
                304.8,
                304.41,
                304.02,
                303.63,
                303.25,
                302.88,
                302.51,
                302.15,
                301.79,
                301.63,
                301.54,
                301.45,
                301.36,
                301.26,
                301.16,
                301.06,
                300.96,
                300.85,
                300.74,
                300.63,
                300.52,
                300.4,
                300.28,
                300.16,
                300.04,
                299.91,
                299.78,
                299.65,
                299.62,
                299.78,
                299.95,
                300.11,
                300.28,
                300.45,
                300.61,
                300.78,
                300.95,
                301.11,
                301.28,
                301.45,
                301.61,
                301.78,
                301.95,
                302.11,
                302.28,
                302.45,
                302.61,
                302.78,
                305.18,
                307.69,
                310.22,
                312.76,
                315.32,
                317.9,
                320.5,
                323.11,
                325.73,
                328.38,
                331.04,
                333.71,
                336.41,
                339.12,
                341.84,
                344.59,
                347.35,
                350.12,
                352.92,
                355.0,
                356.5,
                357.93,
                359.29,
                360.58,
                361.81,
                362.97,
                364.05,
                365.07,
                366.02,
                366.9,
                367.72,
                368.46,
                369.13,
                369.74,
                370.27,
                370.74,
                371.14,
                371.46,
                371.65,
                371.5,
                371.29,
                371.02,
                370.69,
                370.3,
                369.85,
                369.35,
                368.78,
                368.16,
                367.47,
                366.73,
                365.93,
                365.06,
                364.14,
                363.16,
                362.12,
                361.02,
                359.86,
                358.64,
                353.8,
                348.23,
                342.74,
                337.35,
                332.06,
                326.86,
                321.76,
                316.76,
                311.85,
                594.11,
                594.35,
                594.62,
                594.91,
                595.24,
                595.59,
                595.97,
                596.38,
                596.82,
                597.29,
                597.78,
                598.31,
                598.86,
                599.44,
                600.1,
                601.25,
                602.36,
                603.45,
                604.51,
                607.95,
                612.78,
                617.58,
                622.34,
                627.08,
                631.78,
                636.46,
                641.1,
                645.72,
                650.3,
                654.86,
                659.38,
                663.87,
                668.34,
                672.77,
                677.17,
                681.54,
                685.88,
                690.19,
                694.56,
                699.09,
                703.58,
                708.01,
                712.4,
                716.73,
                721.02,
                725.26,
                729.45,
                733.59,
                737.68,
                741.71,
                745.7,
                749.64,
                753.53,
                757.37,
                761.16,
                764.91,
                768.6,
                772.24,
                768.01,
                763.32,
                758.64,
                753.96,
                749.28,
                744.6,
                739.92,
                735.25,
                730.58,
                725.91,
                721.24,
                716.58,
                711.92,
                707.25,
                702.6,
                697.94,
                693.28,
                688.63,
                683.98,
                677.27,
                669.12,
                660.91,
                652.66,
                644.34,
                635.98,
                627.56,
                619.09,
                610.57,
                601.99,
                593.36,
                584.68,
                575.94,
                567.15,
                558.3,
                549.4,
                540.44,
                531.44,
                522.37,
                513.59,
                505.82,
                498.07,
                490.36,
                482.68,
                475.02,
                467.39,
                459.79,
                452.22,
                444.68,
                437.17,
                429.69,
                422.24,
                414.81,
                407.42,
                400.06,
                392.72,
                385.41,
                378.14,
                370.89,
                368.35,
                366.38,
                364.42,
                362.46,
                360.51,
                358.56,
                356.61,
                354.67,
                352.73,
                350.79,
                348.86,
                346.93,
                345.01,
                343.09,
                341.17,
                339.26,
                337.35,
                335.45,
                333.55,
                332.39,
                331.87,
                331.34,
                330.81,
                330.27,
                329.72,
                329.16,
                328.6,
                328.03,
                327.44,
                326.85,
                326.26,
                325.65,
                325.04,
                324.42,
                323.79,
                323.15,
                322.51,
                321.85,
                321.22,
                320.72,
                320.21,
                319.7,
                319.18,
                318.66,
                318.13,
                317.6,
                317.06,
                316.51,
                315.96,
                315.4,
                314.84,
                314.27,
                313.7,
                313.12,
                312.54,
                311.94,
                311.35,
                310.75,
                310.34,
                309.98,
                309.63,
                309.28,
                308.94,
                308.6,
                308.27,
                307.95,
                307.63,
                307.32,
                307.02,
                306.72,
                306.42,
                306.13,
                305.85,
                305.57,
                305.3,
                305.03,
                304.78,
                304.57,
                304.44,
                304.31,
                304.17,
                304.03,
                303.89,
                303.75,
                303.6,
                303.45,
                303.3,
                303.15,
                302.99,
                302.83,
                302.67,
                302.51,
                302.34,
                302.17,
                302.0,
                301.83,
                301.67,
                301.84,
                302.0,
                302.17,
                302.34,
                302.5,
                302.67,
                302.83,
                303.0,
                303.17,
                303.33,
                303.5,
                303.67,
                303.83,
                304.0,
                304.17,
                304.33,
                304.5,
                304.67,
                304.83,
                306.77,
                309.55,
                312.34,
                315.15,
                317.98,
                320.82,
                323.68,
                326.56,
                329.45,
                332.36,
                335.29,
                338.23,
                341.19,
                344.16,
                347.16,
                350.17,
                353.19,
                356.23,
                359.29,
                361.58,
                361.98,
                362.32,
                362.58,
                362.78,
                362.91,
                362.97,
                362.96,
                362.88,
                362.73,
                362.52,
                362.23,
                361.88,
                361.45,
                360.96,
                360.4,
                359.77,
                359.07,
                358.3,
                357.46,
                356.4,
                355.26,
                354.07,
                352.81,
                351.5,
                350.12,
                348.69,
                347.2,
                345.65,
                344.04,
                342.37,
                340.64,
                338.85,
                337.01,
                335.1,
                333.13,
                331.11,
                329.02,
                326.87,
                323.66,
                319.58,
                315.6,
                311.71,
                307.92,
                304.22,
                300.62,
                297.12,
                293.71,
                632.89,
                633.88,
                634.85,
                635.78,
                636.69,
                637.58,
                638.43,
                639.26,
                640.06,
                640.84,
                641.58,
                642.3,
                642.99,
                643.66,
                644.29,
                644.9,
                645.48,
                646.04,
                646.56,
                648.53,
                652.78,
                657.0,
                661.18,
                665.34,
                669.46,
                673.56,
                677.63,
                681.66,
                685.67,
                689.64,
                693.59,
                697.5,
                701.38,
                705.24,
                709.06,
                712.85,
                716.62,
                720.35,
                724.05,
                727.66,
                731.22,
                734.73,
                738.2,
                741.61,
                744.98,
                748.3,
                751.56,
                754.78,
                757.95,
                761.07,
                764.13,
                767.15,
                770.12,
                773.04,
                775.91,
                778.73,
                781.5,
                784.22,
                781.78,
                777.13,
                772.48,
                767.83,
                763.18,
                758.54,
                753.9,
                749.26,
                744.62,
                739.98,
                735.35,
                730.72,
                726.09,
                721.46,
                716.83,
                712.21,
                707.59,
                702.97,
                698.35,
                692.19,
                683.08,
                673.91,
                664.69,
                655.42,
                646.1,
                636.72,
                627.29,
                617.8,
                608.26,
                598.67,
                589.02,
                579.32,
                569.57,
                559.76,
                549.9,
                539.99,
                530.02,
                520.0,
                509.92,
                502.64,
                495.41,
                488.21,
                481.04,
                473.9,
                466.79,
                459.7,
                452.65,
                445.62,
                438.62,
                431.66,
                424.72,
                417.81,
                410.93,
                404.08,
                397.26,
                390.47,
                383.7,
                376.97,
                373.3,
                371.4,
                369.5,
                367.61,
                365.72,
                363.84,
                361.95,
                360.07,
                358.2,
                356.33,
                354.46,
                352.6,
                350.74,
                348.88,
                347.03,
                345.19,
                343.34,
                341.5,
                339.66,
                338.15,
                337.5,
                336.83,
                336.16,
                335.48,
                334.79,
                334.1,
                333.39,
                332.68,
                331.96,
                331.23,
                330.5,
                329.75,
                329.0,
                328.24,
                327.47,
                326.7,
                325.91,
                325.12,
                324.32,
                323.7,
                323.1,
                322.49,
                321.88,
                321.26,
                320.64,
                320.01,
                319.37,
                318.73,
                318.08,
                317.43,
                316.77,
                316.11,
                315.44,
                314.76,
                314.08,
                313.39,
                312.7,
                312.0,
                311.54,
                311.27,
                311.02,
                310.77,
                310.52,
                310.28,
                310.05,
                309.82,
                309.6,
                309.39,
                309.18,
                308.97,
                308.77,
                308.58,
                308.39,
                308.21,
                308.04,
                307.87,
                307.7,
                307.54,
                307.37,
                307.19,
                307.01,
                306.83,
                306.65,
                306.46,
                306.27,
                306.08,
                305.89,
                305.69,
                305.49,
                305.29,
                305.09,
                304.88,
                304.67,
                304.46,
                304.25,
                304.03,
                303.81,
                303.89,
                304.06,
                304.22,
                304.39,
                304.56,
                304.72,
                304.89,
                305.06,
                305.22,
                305.39,
                305.55,
                305.72,
                305.89,
                306.05,
                306.22,
                306.39,
                306.55,
                306.72,
                306.89,
                308.21,
                311.25,
                314.31,
                317.39,
                320.48,
                323.59,
                326.72,
                329.86,
                333.02,
                336.19,
                339.39,
                342.59,
                345.82,
                349.06,
                352.32,
                355.59,
                358.89,
                362.19,
                365.52,
                368.79,
                368.09,
                367.33,
                366.5,
                365.6,
                364.63,
                363.59,
                362.49,
                361.31,
                360.07,
                358.76,
                357.38,
                355.93,
                354.41,
                352.82,
                351.16,
                349.43,
                347.64,
                345.77,
                343.84,
                341.83,
                339.77,
                337.65,
                335.47,
                333.23,
                330.93,
                328.57,
                326.15,
                323.68,
                321.14,
                318.55,
                315.89,
                313.18,
                310.41,
                307.57,
                304.68,
                301.73,
                298.72,
                295.65,
                292.64,
                290.06,
                287.58,
                285.19,
                282.9,
                280.71,
                278.61,
                276.6,
                274.7,
                684.84,
                685.3,
                685.73,
                686.13,
                686.5,
                686.85,
                687.17,
                687.46,
                687.73,
                687.97,
                688.18,
                688.36,
                688.52,
                688.64,
                688.75,
                688.82,
                688.86,
                688.88,
                688.87,
                689.39,
                693.06,
                696.7,
                700.3,
                703.88,
                707.43,
                710.94,
                714.43,
                717.89,
                721.31,
                724.71,
                728.08,
                731.41,
                734.72,
                737.99,
                741.24,
                744.45,
                747.63,
                750.78,
                753.91,
                756.68,
                759.32,
                761.91,
                764.45,
                766.95,
                769.39,
                771.79,
                774.13,
                776.43,
                778.67,
                780.87,
                783.02,
                785.12,
                787.16,
                789.16,
                791.11,
                793.01,
                794.86,
                796.66,
                795.54,
                790.92,
                786.3,
                781.69,
                777.07,
                772.46,
                767.85,
                763.25,
                758.64,
                754.04,
                749.44,
                744.84,
                740.24,
                735.64,
                731.05,
                726.46,
                721.87,
                717.29,
                712.7,
                707.59,
                697.52,
                687.39,
                677.22,
                666.98,
                656.7,
                646.36,
                635.97,
                625.52,
                615.02,
                604.47,
                593.86,
                583.2,
                572.49,
                561.72,
                550.9,
                540.02,
                529.09,
                518.11,
                507.07,
                499.2,
                492.48,
                485.8,
                479.14,
                472.51,
                465.92,
                459.35,
                452.81,
                446.29,
                439.81,
                433.36,
                426.93,
                420.54,
                414.17,
                407.84,
                401.53,
                395.25,
                389.0,
                382.78,
                378.23,
                376.39,
                374.56,
                372.73,
                370.9,
                369.08,
                367.26,
                365.45,
                363.64,
                361.83,
                360.03,
                358.23,
                356.44,
                354.65,
                352.86,
                351.08,
                349.3,
                347.52,
                345.75,
                343.99,
                343.19,
                342.39,
                341.58,
                340.76,
                339.94,
                339.1,
                338.26,
                337.41,
                336.55,
                335.68,
                334.81,
                333.93,
                333.04,
                332.14,
                331.23,
                330.32,
                329.39,
                328.46,
                327.52,
                326.74,
                326.04,
                325.34,
                324.63,
                323.91,
                323.19,
                322.47,
                321.73,
                321.0,
                320.25,
                319.51,
                318.75,
                317.99,
                317.23,
                316.45,
                315.68,
                314.89,
                314.11,
                313.31,
                312.68,
                312.52,
                312.36,
                312.2,
                312.05,
                311.91,
                311.77,
                311.64,
                311.52,
                311.4,
                311.28,
                311.18,
                311.07,
                310.98,
                310.89,
                310.8,
                310.72,
                310.65,
                310.58,
                310.52,
                310.32,
                310.1,
                309.88,
                309.65,
                309.43,
                309.2,
                308.97,
                308.73,
                308.5,
                308.26,
                308.02,
                307.77,
                307.52,
                307.28,
                307.02,
                306.77,
                306.51,
                306.25,
                305.99,
                305.95,
                306.11,
                306.28,
                306.45,
                306.61,
                306.78,
                306.94,
                307.11,
                307.28,
                307.44,
                307.61,
                307.78,
                307.94,
                308.11,
                308.28,
                308.44,
                308.61,
                308.78,
                308.94,
                309.5,
                312.81,
                316.14,
                319.48,
                322.84,
                326.21,
                329.6,
                333.01,
                336.44,
                339.88,
                343.33,
                346.81,
                350.3,
                353.81,
                357.33,
                360.87,
                364.43,
                368.0,
                371.59,
                375.2,
                374.83,
                372.97,
                371.04,
                369.04,
                366.98,
                364.85,
                362.64,
                360.37,
                358.03,
                355.62,
                353.15,
                350.6,
                347.98,
                345.3,
                342.55,
                339.72,
                336.83,
                333.87,
                330.84,
                327.8,
                324.81,
                321.76,
                318.66,
                315.49,
                312.27,
                308.99,
                306.18,
                303.37,
                300.58,
                297.8,
                295.03,
                292.27,
                289.52,
                286.79,
                284.06,
                281.35,
                278.65,
                275.96,
                273.28,
                272.42,
                271.66,
                270.9,
                270.15,
                269.4,
                268.65,
                267.91,
                267.16,
                737.05,
                736.97,
                736.86,
                736.73,
                736.57,
                736.38,
                736.16,
                735.92,
                735.65,
                735.35,
                735.03,
                734.68,
                734.3,
                733.89,
                733.45,
                732.99,
                732.5,
                731.99,
                731.44,
                730.87,
                733.62,
                736.68,
                739.71,
                742.7,
                745.67,
                748.61,
                751.52,
                754.4,
                757.24,
                760.06,
                762.85,
                765.6,
                768.33,
                771.03,
                773.69,
                776.33,
                778.93,
                781.51,
                784.05,
                786.15,
                787.87,
                789.54,
                791.16,
                792.73,
                794.26,
                795.73,
                797.16,
                798.53,
                799.86,
                801.13,
                802.36,
                803.54,
                804.66,
                805.74,
                806.77,
                807.75,
                808.67,
                809.55,
                809.28,
                804.7,
                800.11,
                795.53,
                790.95,
                786.37,
                781.79,
                777.22,
                772.65,
                768.08,
                763.51,
                758.94,
                754.38,
                749.81,
                745.25,
                740.7,
                736.14,
                731.59,
                727.04,
                722.49,
                712.45,
                701.36,
                690.22,
                679.03,
                667.79,
                656.49,
                645.13,
                633.73,
                622.27,
                610.75,
                599.19,
                587.57,
                575.89,
                564.16,
                552.38,
                540.55,
                528.66,
                516.71,
                504.72,
                495.49,
                489.29,
                483.12,
                476.98,
                470.87,
                464.78,
                458.73,
                452.7,
                446.7,
                440.73,
                434.79,
                428.88,
                423.0,
                417.15,
                411.33,
                405.54,
                399.77,
                394.04,
                388.33,
                383.12,
                381.34,
                379.58,
                377.81,
                376.05,
                374.29,
                372.54,
                370.79,
                369.04,
                367.3,
                365.56,
                363.83,
                362.1,
                360.37,
                358.65,
                356.93,
                355.22,
                353.5,
                351.8,
                350.09,
                348.97,
                348.02,
                347.08,
                346.12,
                345.15,
                344.18,
                343.2,
                342.21,
                341.21,
                340.21,
                339.19,
                338.17,
                337.14,
                336.11,
                335.06,
                334.01,
                332.95,
                331.88,
                330.8,
                329.83,
                329.03,
                328.23,
                327.43,
                326.62,
                325.8,
                324.98,
                324.15,
                323.32,
                322.48,
                321.63,
                320.78,
                319.93,
                319.07,
                318.2,
                317.33,
                316.45,
                315.56,
                314.67,
                313.78,
                313.71,
                313.64,
                313.59,
                313.53,
                313.49,
                313.45,
                313.41,
                313.38,
                313.36,
                313.34,
                313.33,
                313.32,
                313.32,
                313.32,
                313.34,
                313.35,
                313.37,
                313.4,
                313.44,
                313.29,
                313.03,
                312.77,
                312.5,
                312.23,
                311.96,
                311.68,
                311.41,
                311.13,
                310.85,
                310.56,
                310.28,
                309.99,
                309.69,
                309.4,
                309.1,
                308.8,
                308.5,
                308.2,
                308.0,
                308.17,
                308.33,
                308.5,
                308.67,
                308.83,
                309.0,
                309.16,
                309.33,
                309.5,
                309.66,
                309.83,
                310.0,
                310.16,
                310.33,
                310.5,
                310.66,
                310.83,
                311.0,
                311.16,
                314.22,
                317.81,
                321.42,
                325.04,
                328.68,
                332.34,
                336.01,
                339.7,
                343.41,
                347.13,
                350.87,
                354.63,
                358.36,
                361.86,
                365.33,
                368.78,
                372.2,
                375.6,
                378.97,
                379.4,
                376.33,
                373.27,
                370.21,
                367.14,
                364.08,
                361.02,
                357.96,
                354.9,
                351.84,
                348.78,
                345.72,
                342.66,
                339.6,
                336.54,
                333.49,
                330.43,
                327.37,
                324.32,
                321.28,
                318.58,
                315.89,
                313.21,
                310.54,
                307.89,
                305.24,
                302.61,
                299.98,
                297.37,
                294.77,
                292.18,
                289.61,
                287.04,
                284.49,
                281.95,
                279.41,
                276.9,
                274.39,
                271.89,
                270.57,
                269.87,
                269.16,
                268.46,
                267.76,
                267.07,
                266.38,
                265.69,
                789.51,
                788.9,
                788.26,
                787.59,
                786.89,
                786.17,
                785.41,
                784.64,
                783.83,
                783.0,
                782.14,
                781.25,
                780.33,
                779.39,
                778.42,
                777.42,
                776.4,
                775.35,
                774.27,
                773.16,
                774.46,
                776.94,
                779.39,
                781.81,
                784.2,
                786.56,
                788.89,
                791.19,
                793.45,
                795.69,
                797.9,
                800.08,
                802.23,
                804.34,
                806.43,
                808.49,
                810.51,
                812.51,
                814.48,
                816.07,
                816.87,
                817.62,
                818.32,
                818.97,
                819.58,
                820.13,
                820.63,
                821.09,
                821.49,
                821.85,
                822.16,
                822.41,
                822.62,
                822.78,
                822.88,
                822.94,
                822.95,
                822.91,
                822.81,
                818.46,
                813.9,
                809.35,
                804.81,
                800.26,
                795.72,
                791.18,
                786.64,
                782.1,
                777.56,
                773.03,
                768.5,
                763.97,
                759.44,
                754.92,
                750.39,
                745.87,
                741.35,
                736.84,
                727.87,
                715.82,
                703.72,
                691.57,
                679.36,
                667.1,
                654.79,
                642.42,
                630.0,
                617.53,
                605.0,
                592.42,
                579.79,
                567.1,
                554.36,
                541.56,
                528.71,
                515.81,
                502.85,
                491.52,
                485.84,
                480.18,
                474.55,
                468.95,
                463.38,
                457.84,
                452.33,
                446.84,
                441.39,
                435.97,
                430.57,
                425.2,
                419.86,
                414.56,
                409.28,
                404.03,
                398.8,
                393.61,
                388.45,
                386.27,
                384.56,
                382.86,
                381.16,
                379.47,
                377.78,
                376.1,
                374.42,
                372.74,
                371.07,
                369.4,
                367.73,
                366.07,
                364.41,
                362.75,
                361.1,
                359.46,
                357.81,
                356.17,
                354.81,
                353.73,
                352.64,
                351.55,
                350.44,
                349.33,
                348.21,
                347.08,
                345.95,
                344.8,
                343.65,
                342.49,
                341.32,
                340.15,
                338.97,
                337.77,
                336.57,
                335.37,
                334.15,
                332.97,
                332.07,
                331.18,
                330.28,
                329.37,
                328.46,
                327.54,
                326.62,
                325.69,
                324.75,
                323.81,
                322.87,
                321.91,
                320.96,
                319.99,
                319.02,
                318.05,
                317.07,
                316.09,
                315.09,
                314.85,
                314.88,
                314.92,
                314.96,
                315.01,
                315.07,
                315.13,
                315.19,
                315.26,
                315.34,
                315.43,
                315.52,
                315.61,
                315.71,
                315.82,
                315.93,
                316.05,
                316.17,
                316.3,
                316.29,
                315.98,
                315.68,
                315.37,
                315.05,
                314.74,
                314.42,
                314.1,
                313.78,
                313.46,
                313.13,
                312.8,
                312.47,
                312.14,
                311.8,
                311.46,
                311.12,
                310.77,
                310.43,
                310.13,
                310.32,
                310.52,
                310.73,
                310.93,
                311.12,
                311.31,
                311.49,
                311.68,
                311.85,
                312.02,
                312.19,
                312.35,
                312.51,
                312.66,
                312.81,
                312.96,
                313.1,
                313.23,
                313.36,
                315.36,
                318.75,
                322.12,
                325.46,
                328.78,
                332.07,
                335.33,
                338.57,
                341.78,
                344.97,
                348.13,
                351.27,
                354.38,
                357.46,
                360.52,
                363.55,
                366.56,
                369.54,
                372.49,
                374.38,
                371.33,
                368.28,
                365.22,
                362.17,
                359.12,
                356.07,
                353.02,
                349.97,
                346.92,
                343.87,
                340.82,
                337.77,
                334.73,
                331.68,
                328.64,
                325.59,
                322.55,
                319.5,
                316.46,
                313.82,
                311.31,
                308.81,
                306.33,
                303.85,
                301.39,
                298.93,
                296.49,
                294.06,
                291.64,
                289.24,
                286.84,
                284.46,
                282.08,
                279.72,
                277.37,
                275.03,
                272.71,
                270.39,
                268.7,
                268.04,
                267.39,
                266.74,
                266.1,
                265.46,
                264.82,
                264.19,
                840.89,
                841.08,
                839.9,
                838.7,
                837.47,
                836.21,
                834.92,
                833.61,
                832.27,
                830.9,
                829.5,
                828.08,
                826.63,
                825.15,
                823.65,
                822.11,
                820.55,
                818.97,
                817.35,
                815.71,
                815.58,
                817.48,
                819.35,
                821.19,
                823.0,
                824.78,
                826.53,
                828.26,
                829.95,
                831.61,
                833.24,
                834.84,
                836.41,
                837.95,
                839.45,
                840.93,
                842.38,
                843.8,
                845.19,
                846.44,
                846.32,
                846.15,
                845.93,
                845.66,
                845.35,
                844.98,
                844.57,
                844.1,
                843.59,
                843.02,
                842.41,
                841.74,
                841.03,
                840.27,
                839.46,
                838.59,
                837.68,
                836.72,
                835.71,
                832.2,
                827.68,
                823.16,
                818.65,
                814.13,
                809.62,
                805.12,
                800.61,
                796.1,
                791.6,
                787.1,
                782.6,
                778.11,
                773.61,
                769.12,
                764.63,
                760.14,
                755.65,
                751.17,
                743.77,
                730.76,
                717.7,
                704.59,
                691.42,
                678.2,
                664.93,
                651.6,
                638.22,
                624.79,
                611.3,
                597.76,
                584.17,
                570.52,
                556.82,
                543.06,
                529.26,
                515.39,
                501.48,
                487.51,
                482.12,
                476.98,
                471.86,
                466.78,
                461.72,
                456.69,
                451.69,
                446.72,
                441.78,
                436.87,
                431.99,
                427.14,
                422.31,
                417.52,
                412.75,
                408.01,
                403.31,
                398.63,
                393.98,
                391.15,
                389.51,
                387.88,
                386.25,
                384.62,
                382.99,
                381.37,
                379.75,
                378.14,
                376.53,
                374.93,
                373.33,
                371.73,
                370.13,
                368.54,
                366.96,
                365.37,
                363.79,
                362.22,
                360.73,
                359.51,
                358.28,
                357.05,
                355.81,
                354.56,
                353.3,
                352.03,
                350.76,
                349.47,
                348.18,
                346.89,
                345.58,
                344.27,
                342.94,
                341.61,
                340.28,
                338.93,
                337.57,
                336.21,
                335.17,
                334.18,
                333.18,
                332.18,
                331.17,
                330.15,
                329.13,
                328.11,
                327.08,
                326.04,
                325.0,
                323.95,
                322.9,
                321.84,
                320.78,
                319.71,
                318.63,
                317.55,
                316.46,
                315.94,
                316.06,
                316.2,
                316.34,
                316.54,
                316.81,
                317.11,
                317.41,
                317.73,
                318.07,
                318.41,
                318.78,
                319.16,
                319.55,
                319.96,
                320.38,
                320.82,
                321.28,
                321.74,
                322.17,
                321.89,
                321.6,
                321.29,
                320.96,
                320.62,
                320.26,
                319.88,
                319.49,
                319.08,
                318.66,
                318.22,
                317.76,
                317.28,
                316.79,
                316.28,
                315.76,
                315.21,
                314.65,
                314.08,
                313.99,
                314.12,
                314.25,
                314.37,
                314.49,
                314.61,
                314.72,
                314.82,
                314.93,
                315.02,
                315.12,
                315.2,
                315.29,
                315.37,
                315.44,
                315.51,
                315.58,
                315.64,
                315.69,
                316.6,
                319.58,
                322.53,
                325.45,
                328.35,
                331.23,
                334.08,
                336.9,
                339.7,
                342.47,
                345.21,
                347.93,
                350.63,
                353.3,
                355.94,
                358.55,
                361.14,
                363.71,
                366.25,
                368.76,
                366.32,
                363.27,
                360.23,
                357.19,
                354.15,
                351.11,
                348.07,
                345.03,
                341.99,
                338.96,
                335.92,
                332.88,
                329.85,
                326.81,
                323.78,
                320.75,
                317.71,
                314.68,
                311.65,
                308.96,
                306.63,
                304.31,
                302.0,
                299.71,
                297.43,
                295.15,
                292.89,
                290.65,
                288.41,
                286.18,
                283.97,
                281.76,
                279.57,
                277.39,
                275.22,
                273.07,
                270.92,
                268.79,
                266.79,
                266.19,
                265.59,
                265.0,
                264.41,
                263.82,
                263.23,
                262.65,
                891.55,
                893.52,
                891.81,
                890.07,
                888.3,
                886.51,
                884.68,
                882.83,
                880.96,
                879.06,
                877.12,
                875.17,
                873.18,
                871.17,
                869.13,
                867.06,
                864.97,
                862.84,
                860.69,
                858.52,
                856.98,
                858.3,
                859.59,
                860.86,
                862.09,
                863.29,
                864.46,
                865.61,
                866.72,
                867.8,
                868.85,
                869.88,
                870.87,
                871.83,
                872.76,
                873.66,
                874.53,
                875.37,
                876.18,
                876.96,
                876.23,
                875.14,
                874.0,
                872.81,
                871.57,
                870.29,
                868.95,
                867.57,
                866.13,
                864.65,
                863.11,
                861.53,
                859.9,
                858.22,
                856.48,
                854.7,
                852.87,
                850.99,
                849.06,
                845.92,
                841.44,
                836.95,
                832.47,
                827.99,
                823.51,
                819.04,
                814.56,
                810.09,
                805.62,
                801.16,
                796.69,
                792.23,
                787.76,
                783.3,
                778.85,
                774.39,
                769.94,
                765.49,
                760.15,
                746.19,
                732.17,
                718.09,
                703.97,
                689.79,
                675.56,
                661.27,
                646.93,
                632.54,
                618.09,
                603.59,
                589.04,
                574.43,
                559.77,
                545.06,
                530.29,
                515.47,
                500.59,
                485.66,
                478.14,
                473.51,
                468.91,
                464.34,
                459.79,
                455.28,
                450.79,
                446.34,
                441.91,
                437.51,
                433.14,
                428.8,
                424.49,
                420.21,
                415.96,
                411.74,
                407.54,
                403.38,
                399.24,
                396.01,
                394.43,
                392.86,
                391.29,
                389.73,
                388.17,
                386.61,
                385.06,
                383.51,
                381.97,
                380.42,
                378.89,
                377.35,
                375.82,
                374.3,
                372.78,
                371.26,
                369.74,
                368.23,
                366.72,
                365.36,
                364.0,
                362.62,
                361.24,
                359.85,
                358.46,
                357.05,
                355.64,
                354.19,
                352.74,
                351.31,
                349.89,
                348.49,
                347.1,
                345.72,
                344.36,
                343.01,
                341.68,
                340.37,
                339.24,
                338.25,
                337.25,
                336.24,
                335.23,
                334.22,
                333.21,
                332.19,
                331.17,
                330.14,
                329.11,
                328.08,
                327.05,
                326.01,
                324.96,
                323.91,
                322.86,
                321.81,
                320.75,
                319.99,
                320.45,
                320.93,
                321.42,
                321.92,
                322.44,
                322.98,
                323.53,
                324.1,
                324.67,
                325.27,
                325.88,
                326.5,
                327.14,
                327.8,
                328.46,
                329.15,
                329.85,
                330.56,
                331.29,
                330.98,
                330.41,
                329.83,
                329.22,
                328.6,
                327.97,
                327.31,
                326.64,
                325.96,
                325.26,
                324.54,
                323.8,
                323.05,
                322.28,
                321.49,
                320.69,
                319.87,
                319.04,
                318.18,
                317.7,
                317.76,
                317.81,
                317.86,
                317.91,
                317.95,
                317.98,
                318.01,
                318.04,
                318.06,
                318.08,
                318.1,
                318.1,
                318.11,
                318.11,
                318.1,
                318.09,
                318.08,
                318.06,
                318.07,
                320.63,
                323.17,
                325.68,
                328.16,
                330.62,
                333.06,
                335.46,
                337.85,
                340.2,
                342.53,
                344.84,
                347.11,
                349.37,
                351.59,
                353.79,
                355.97,
                358.12,
                360.24,
                362.34,
                361.3,
                358.27,
                355.23,
                352.21,
                349.18,
                346.15,
                343.12,
                340.09,
                337.07,
                334.04,
                331.01,
                327.99,
                324.96,
                321.94,
                318.92,
                315.89,
                312.87,
                309.85,
                306.83,
                303.99,
                301.84,
                299.7,
                297.58,
                295.46,
                293.36,
                291.27,
                289.19,
                287.12,
                285.07,
                283.02,
                280.99,
                278.97,
                276.96,
                274.96,
                272.97,
                271.0,
                269.03,
                267.08,
                265.14,
                264.3,
                263.76,
                263.22,
                262.68,
                262.15,
                261.61,
                261.09,
                941.56,
                946.21,
                943.97,
                941.69,
                939.39,
                937.06,
                934.7,
                932.32,
                929.91,
                927.47,
                925.0,
                922.51,
                919.99,
                917.44,
                914.87,
                912.27,
                909.64,
                906.98,
                904.29,
                901.58,
                898.84,
                899.4,
                900.12,
                900.8,
                901.46,
                902.08,
                902.67,
                903.24,
                903.77,
                904.28,
                904.75,
                905.2,
                905.61,
                905.99,
                906.35,
                906.67,
                906.96,
                907.22,
                907.45,
                907.65,
                906.58,
                904.57,
                902.51,
                900.41,
                898.25,
                896.04,
                893.79,
                891.48,
                889.13,
                886.73,
                884.27,
                881.77,
                879.22,
                876.62,
                873.97,
                871.27,
                868.52,
                865.71,
                862.86,
                859.63,
                855.18,
                850.73,
                846.28,
                841.83,
                837.39,
                832.95,
                828.5,
                824.07,
                819.63,
                815.19,
                810.76,
                806.33,
                801.9,
                797.47,
                793.05,
                788.63,
                784.21,
                779.79,
                775.37,
                762.1,
                747.12,
                732.08,
                717.0,
                701.86,
                686.67,
                671.42,
                656.13,
                640.77,
                625.37,
                609.91,
                594.4,
                578.83,
                563.21,
                547.54,
                531.81,
                516.03,
                500.2,
                484.31,
                473.89,
                469.78,
                465.69,
                461.63,
                457.6,
                453.6,
                449.63,
                445.69,
                441.77,
                437.89,
                434.03,
                430.21,
                426.36,
                422.48,
                418.58,
                414.65,
                410.7,
                406.72,
                402.71,
                398.89,
                397.18,
                395.49,
                393.81,
                392.15,
                390.5,
                388.87,
                387.26,
                385.66,
                384.07,
                382.5,
                380.95,
                379.41,
                377.88,
                376.37,
                374.88,
                373.4,
                371.94,
                370.49,
                369.06,
                367.71,
                366.39,
                365.09,
                363.8,
                362.53,
                361.27,
                360.03,
                358.8,
                357.58,
                356.38,
                355.19,
                354.02,
                352.86,
                351.72,
                350.59,
                349.47,
                348.37,
                347.28,
                346.21,
                345.15,
                344.09,
                343.03,
                341.96,
                340.89,
                339.82,
                338.74,
                337.66,
                336.57,
                335.48,
                334.39,
                333.29,
                332.19,
                331.09,
                329.98,
                328.87,
                327.75,
                326.64,
                325.51,
                324.39,
                324.96,
                325.69,
                326.42,
                327.17,
                327.94,
                328.72,
                329.52,
                330.33,
                331.15,
                331.99,
                332.85,
                333.71,
                334.6,
                335.5,
                336.41,
                337.34,
                338.28,
                339.24,
                340.21,
                340.22,
                339.38,
                338.51,
                337.63,
                336.74,
                335.82,
                334.89,
                333.95,
                332.99,
                332.01,
                331.01,
                330.0,
                328.97,
                327.92,
                326.86,
                325.78,
                324.69,
                323.57,
                322.44,
                321.45,
                321.44,
                321.42,
                321.39,
                321.36,
                321.33,
                321.29,
                321.25,
                321.2,
                321.15,
                321.09,
                321.03,
                320.96,
                320.89,
                320.82,
                320.74,
                320.66,
                320.57,
                320.47,
                320.38,
                321.93,
                324.05,
                326.14,
                328.21,
                330.25,
                332.27,
                334.26,
                336.23,
                338.17,
                340.08,
                341.97,
                343.83,
                345.67,
                347.48,
                349.27,
                351.03,
                352.76,
                354.47,
                356.15,
                356.27,
                353.25,
                350.23,
                347.21,
                344.2,
                341.18,
                338.16,
                335.14,
                332.13,
                329.11,
                326.1,
                323.09,
                320.07,
                317.06,
                314.05,
                311.04,
                308.02,
                305.01,
                302.0,
                299.0,
                296.94,
                294.99,
                293.05,
                291.11,
                289.19,
                287.28,
                285.39,
                283.5,
                281.62,
                279.76,
                277.91,
                276.07,
                274.24,
                272.42,
                270.61,
                268.82,
                267.04,
                265.26,
                263.5,
                262.39,
                261.89,
                261.41,
                260.92,
                260.44,
                259.96,
                259.49,
                990.95,
                999.16,
                996.38,
                993.57,
                990.73,
                987.87,
                984.98,
                982.06,
                979.11,
                976.14,
                973.14,
                970.11,
                967.06,
                963.97,
                960.87,
                957.73,
                954.56,
                951.37,
                948.15,
                944.91,
                941.63,
                940.78,
                940.92,
                941.02,
                941.1,
                941.15,
                941.17,
                941.15,
                941.11,
                941.04,
                940.93,
                940.8,
                940.63,
                940.44,
                940.22,
                939.96,
                939.67,
                939.36,
                939.01,
                938.64,
                937.39,
                934.46,
                931.48,
                928.46,
                925.38,
                922.26,
                919.08,
                915.86,
                912.58,
                909.26,
                905.89,
                902.47,
                899.0,
                895.48,
                891.91,
                888.29,
                884.62,
                880.9,
                877.13,
                873.31,
                868.91,
                864.49,
                860.07,
                855.66,
                851.25,
                846.84,
                842.43,
                838.02,
                833.62,
                829.21,
                824.81,
                820.42,
                816.02,
                811.63,
                806.91,
                802.17,
                797.45,
                792.75,
                788.08,
                776.68,
                760.65,
                744.63,
                728.62,
                712.61,
                696.62,
                680.64,
                664.66,
                648.7,
                632.75,
                616.8,
                600.87,
                584.94,
                569.03,
                553.12,
                537.23,
                521.34,
                505.47,
                489.6,
                476.27,
                472.25,
                468.2,
                464.12,
                460.02,
                455.89,
                451.74,
                447.56,
                443.36,
                439.13,
                434.87,
                430.59,
                426.28,
                421.95,
                417.58,
                413.2,
                408.79,
                404.35,
                399.88,
                395.39,
                393.43,
                392.01,
                390.6,
                389.21,
                387.83,
                386.46,
                385.11,
                383.78,
                382.46,
                381.16,
                379.87,
                378.6,
                377.34,
                376.1,
                374.87,
                373.66,
                372.47,
                371.29,
                370.12,
                369.0,
                367.93,
                366.88,
                365.83,
                364.81,
                363.79,
                362.8,
                361.81,
                360.84,
                359.88,
                358.94,
                358.02,
                357.1,
                356.2,
                355.32,
                354.45,
                353.6,
                352.75,
                351.93,
                351.1,
                349.97,
                348.85,
                347.72,
                346.58,
                345.44,
                344.3,
                343.16,
                342.01,
                340.85,
                339.7,
                338.54,
                337.37,
                336.2,
                335.03,
                333.86,
                332.68,
                331.5,
                330.31,
                329.12,
                329.35,
                330.32,
                331.3,
                332.29,
                333.3,
                334.33,
                335.37,
                336.43,
                337.5,
                338.58,
                339.68,
                340.79,
                341.92,
                343.07,
                344.22,
                345.4,
                346.59,
                347.79,
                349.01,
                349.62,
                348.49,
                347.35,
                346.2,
                345.02,
                343.83,
                342.63,
                341.4,
                340.17,
                338.91,
                337.64,
                336.35,
                335.04,
                333.72,
                332.38,
                331.02,
                329.65,
                328.26,
                326.86,
                325.43,
                325.16,
                325.06,
                324.96,
                324.86,
                324.75,
                324.64,
                324.52,
                324.4,
                324.27,
                324.14,
                324.0,
                323.86,
                323.72,
                323.57,
                323.42,
                323.26,
                323.1,
                322.93,
                322.76,
                323.45,
                325.15,
                326.83,
                328.49,
                330.11,
                331.72,
                333.29,
                334.84,
                336.37,
                337.87,
                339.34,
                340.79,
                342.21,
                343.61,
                344.98,
                346.32,
                347.64,
                348.93,
                350.2,
                351.24,
                348.23,
                345.22,
                342.21,
                339.21,
                336.2,
                333.2,
                330.19,
                327.19,
                324.18,
                321.18,
                318.18,
                315.17,
                312.17,
                309.17,
                306.17,
                303.17,
                300.17,
                297.17,
                294.18,
                291.95,
                290.17,
                288.41,
                286.66,
                284.92,
                283.19,
                281.48,
                279.77,
                278.08,
                276.39,
                274.72,
                273.06,
                271.41,
                269.78,
                268.15,
                266.54,
                264.93,
                263.34,
                261.76,
                260.44,
                260.0,
                259.57,
                259.13,
                258.71,
                258.28,
                257.86,
                1039.69,
                1050.08,
                1049.05,
                1045.7,
                1042.33,
                1038.93,
                1035.51,
                1032.05,
                1028.57,
                1025.06,
                1021.53,
                1017.97,
                1014.38,
                1010.76,
                1007.12,
                1003.45,
                999.75,
                996.02,
                992.27,
                988.49,
                984.68,
                982.44,
                982.0,
                981.53,
                981.03,
                980.5,
                979.94,
                979.35,
                978.73,
                978.07,
                977.39,
                976.68,
                975.94,
                975.17,
                974.37,
                973.53,
                972.67,
                970.95,
                968.98,
                967.0,
                964.72,
                959.96,
                955.21,
                950.46,
                945.71,
                940.96,
                936.21,
                931.47,
                926.73,
                921.99,
                917.26,
                912.53,
                907.8,
                903.08,
                898.35,
                893.63,
                888.92,
                884.2,
                879.49,
                874.78,
                870.11,
                865.46,
                860.84,
                856.25,
                851.68,
                847.13,
                842.61,
                838.11,
                833.63,
                829.18,
                824.76,
                820.35,
                815.98,
                811.62,
                807.29,
                802.99,
                798.7,
                794.45,
                790.21,
                782.02,
                766.16,
                750.32,
                734.48,
                718.65,
                702.83,
                687.02,
                671.22,
                655.43,
                639.65,
                623.88,
                608.12,
                592.37,
                576.62,
                560.89,
                545.17,
                529.46,
                513.76,
                498.07,
                482.38,
                477.45,
                472.94,
                468.4,
                463.84,
                459.26,
                454.65,
                450.01,
                445.35,
                440.66,
                435.94,
                431.2,
                426.44,
                421.64,
                416.82,
                411.98,
                407.11,
                402.21,
                397.29,
                392.34,
                389.55,
                388.39,
                387.25,
                386.12,
                385.01,
                383.91,
                382.83,
                381.77,
                380.71,
                379.68,
                378.66,
                377.65,
                376.66,
                375.69,
                374.73,
                373.78,
                372.86,
                371.94,
                371.04,
                370.17,
                369.35,
                368.53,
                367.74,
                366.96,
                366.19,
                365.44,
                364.7,
                363.97,
                363.26,
                362.57,
                361.88,
                361.22,
                360.56,
                359.92,
                359.3,
                358.69,
                358.09,
                357.51,
                356.95,
                355.89,
                354.7,
                353.5,
                352.31,
                351.1,
                349.9,
                348.69,
                347.48,
                346.26,
                345.04,
                343.81,
                342.59,
                341.35,
                340.12,
                338.88,
                337.64,
                336.39,
                335.14,
                333.89,
                333.6,
                334.81,
                336.04,
                337.28,
                338.54,
                339.81,
                341.09,
                342.39,
                343.71,
                345.04,
                346.38,
                347.74,
                349.11,
                350.5,
                351.9,
                353.32,
                354.75,
                356.2,
                357.66,
                359.14,
                357.76,
                356.34,
                354.91,
                353.46,
                352.0,
                350.51,
                349.01,
                347.5,
                345.96,
                344.41,
                342.85,
                341.27,
                339.67,
                338.05,
                336.42,
                334.77,
                333.1,
                331.42,
                329.72,
                328.92,
                328.75,
                328.58,
                328.4,
                328.21,
                328.03,
                327.83,
                327.64,
                327.44,
                327.23,
                327.02,
                326.81,
                326.59,
                326.36,
                326.14,
                325.9,
                325.67,
                325.42,
                325.18,
                325.21,
                326.5,
                327.76,
                329.0,
                330.21,
                331.4,
                332.56,
                333.69,
                334.8,
                335.89,
                336.95,
                337.98,
                338.99,
                339.97,
                340.92,
                341.85,
                342.76,
                343.63,
                344.48,
                345.31,
                343.2,
                340.21,
                337.21,
                334.21,
                331.22,
                328.22,
                325.23,
                322.24,
                319.24,
                316.25,
                313.26,
                310.27,
                307.28,
                304.29,
                301.3,
                298.31,
                295.32,
                292.34,
                289.35,
                286.85,
                285.25,
                283.67,
                282.1,
                280.54,
                279.0,
                277.46,
                275.94,
                274.42,
                272.92,
                271.43,
                269.95,
                268.48,
                267.03,
                265.58,
                264.15,
                262.73,
                261.32,
                259.92,
                258.53,
                258.08,
                257.69,
                257.32,
                256.94,
                256.57,
                256.2,
                1086.09,
                1096.63,
                1097.9,
                1092.94,
                1087.94,
                1082.92,
                1077.87,
                1072.79,
                1067.69,
                1062.56,
                1057.39,
                1052.21,
                1046.99,
                1041.74,
                1036.47,
                1031.17,
                1025.84,
                1020.48,
                1015.09,
                1009.68,
                1004.23,
                999.55,
                997.52,
                995.47,
                993.4,
                991.31,
                989.19,
                987.05,
                984.89,
                982.7,
                980.49,
                978.26,
                976.01,
                973.73,
                971.43,
                969.11,
                966.76,
                964.39,
                962.0,
                959.58,
                957.15,
                952.78,
                948.08,
                943.38,
                938.68,
                933.99,
                929.3,
                924.61,
                919.92,
                915.24,
                910.56,
                905.88,
                901.21,
                896.54,
                891.87,
                887.21,
                882.54,
                877.88,
                873.23,
                868.57,
                864.12,
                859.92,
                855.74,
                851.58,
                847.45,
                843.34,
                839.26,
                835.2,
                831.16,
                827.15,
                823.16,
                819.2,
                815.26,
                811.34,
                807.45,
                803.58,
                799.74,
                795.92,
                792.13,
                787.27,
                771.59,
                755.91,
                740.25,
                724.59,
                708.95,
                693.31,
                677.69,
                662.07,
                646.46,
                630.87,
                615.28,
                599.7,
                584.13,
                568.58,
                553.03,
                537.49,
                521.96,
                506.44,
                490.93,
                482.88,
                477.91,
                472.92,
                467.9,
                462.86,
                457.79,
                452.69,
                447.57,
                442.42,
                437.25,
                432.05,
                426.83,
                421.58,
                416.3,
                411.0,
                405.67,
                400.31,
                394.93,
                389.52,
                385.53,
                384.64,
                383.76,
                382.9,
                382.06,
                381.23,
                380.41,
                379.61,
                378.83,
                378.06,
                377.3,
                376.57,
                375.84,
                375.13,
                374.44,
                373.76,
                373.1,
                372.46,
                371.83,
                371.21,
                370.63,
                370.06,
                369.51,
                368.98,
                368.45,
                367.95,
                367.45,
                366.97,
                366.51,
                366.06,
                365.62,
                365.2,
                364.79,
                364.4,
                364.02,
                363.65,
                363.3,
                362.97,
                362.65,
                361.84,
                360.59,
                359.33,
                358.06,
                356.8,
                355.53,
                354.26,
                352.98,
                351.7,
                350.41,
                349.13,
                347.83,
                346.54,
                345.24,
                343.94,
                342.63,
                341.32,
                340.01,
                338.69,
                337.72,
                339.18,
                340.65,
                342.13,
                343.63,
                345.15,
                346.68,
                348.22,
                349.78,
                351.36,
                352.95,
                354.55,
                356.17,
                357.8,
                359.45,
                361.11,
                362.79,
                364.48,
                366.19,
                367.91,
                367.18,
                365.49,
                363.78,
                362.05,
                360.31,
                358.55,
                356.77,
                354.98,
                353.17,
                351.35,
                349.5,
                347.64,
                345.77,
                343.88,
                341.97,
                340.04,
                338.1,
                336.14,
                334.16,
                332.72,
                332.48,
                332.23,
                331.98,
                331.72,
                331.46,
                331.19,
                330.92,
                330.64,
                330.36,
                330.08,
                329.79,
                329.5,
                329.2,
                328.9,
                328.59,
                328.28,
                327.96,
                327.64,
                327.32,
                328.07,
                328.92,
                329.74,
                330.54,
                331.31,
                332.06,
                332.78,
                333.47,
                334.14,
                334.79,
                335.4,
                336.0,
                336.56,
                337.1,
                337.62,
                338.11,
                338.57,
                339.0,
                339.42,
                338.17,
                335.18,
                332.2,
                329.21,
                326.23,
                323.25,
                320.26,
                317.28,
                314.3,
                311.32,
                308.34,
                305.36,
                302.38,
                299.4,
                296.42,
                293.45,
                290.47,
                287.49,
                284.52,
                281.64,
                280.23,
                278.83,
                277.44,
                276.06,
                274.7,
                273.34,
                272.0,
                270.66,
                269.34,
                268.03,
                266.74,
                265.45,
                264.17,
                262.91,
                261.66,
                260.42,
                259.19,
                257.97,
                256.76,
                256.12,
                255.79,
                255.47,
                255.14,
                254.83,
                254.51,
                1096.74,
                1105.67,
                1109.37,
                1103.87,
                1098.34,
                1092.79,
                1087.2,
                1081.59,
                1075.95,
                1070.29,
                1064.59,
                1058.87,
                1053.12,
                1047.34,
                1041.53,
                1035.69,
                1029.83,
                1023.94,
                1018.02,
                1012.07,
                1006.09,
                1000.09,
                997.58,
                995.11,
                992.61,
                990.09,
                987.54,
                984.98,
                982.39,
                979.77,
                977.14,
                974.48,
                971.8,
                969.09,
                966.37,
                963.62,
                960.84,
                958.05,
                955.23,
                952.38,
                949.52,
                945.56,
                940.91,
                936.27,
                931.63,
                926.99,
                922.35,
                917.72,
                913.09,
                908.46,
                903.83,
                899.21,
                894.59,
                889.98,
                885.36,
                880.75,
                876.14,
                871.54,
                866.93,
                862.33,
                857.92,
                854.15,
                850.41,
                846.7,
                843.0,
                839.33,
                835.69,
                832.07,
                828.47,
                824.89,
                821.35,
                817.82,
                814.32,
                810.84,
                807.39,
                803.96,
                800.56,
                797.18,
                793.82,
                790.49,
                776.93,
                761.43,
                745.93,
                730.45,
                714.98,
                699.52,
                684.06,
                668.62,
                653.19,
                637.76,
                622.35,
                606.95,
                591.55,
                576.17,
                560.79,
                545.43,
                530.07,
                514.73,
                499.39,
                488.54,
                483.12,
                477.67,
                472.19,
                466.69,
                461.16,
                455.61,
                450.03,
                444.43,
                438.79,
                433.14,
                427.46,
                421.75,
                416.01,
                410.25,
                404.46,
                398.65,
                392.81,
                386.95,
                381.37,
                380.74,
                380.13,
                379.54,
                378.96,
                378.4,
                377.85,
                377.32,
                376.8,
                376.3,
                375.81,
                375.34,
                374.88,
                374.44,
                374.02,
                373.61,
                373.21,
                372.83,
                372.47,
                372.12,
                371.78,
                371.46,
                371.16,
                370.87,
                370.59,
                370.33,
                370.08,
                369.84,
                369.63,
                369.42,
                369.23,
                369.05,
                368.89,
                368.74,
                368.61,
                368.49,
                368.38,
                368.29,
                368.22,
                367.82,
                366.51,
                365.18,
                363.86,
                362.53,
                361.19,
                359.86,
                358.52,
                357.17,
                355.82,
                354.47,
                353.12,
                351.76,
                350.4,
                349.03,
                347.66,
                346.29,
                344.91,
                343.53,
                342.14,
                343.41,
                345.12,
                346.86,
                348.6,
                350.36,
                352.14,
                353.92,
                355.73,
                357.55,
                359.38,
                361.23,
                363.09,
                364.97,
                366.86,
                368.77,
                370.69,
                372.63,
                374.58,
                376.55,
                376.75,
                374.78,
                372.8,
                370.79,
                368.77,
                366.74,
                364.68,
                362.62,
                360.53,
                358.43,
                356.31,
                354.17,
                352.02,
                349.85,
                347.67,
                345.47,
                343.25,
                341.01,
                338.76,
                336.57,
                336.25,
                335.92,
                335.6,
                335.27,
                334.93,
                334.59,
                334.24,
                333.89,
                333.54,
                333.18,
                332.82,
                332.45,
                332.08,
                331.7,
                331.32,
                330.93,
                330.54,
                330.14,
                329.75,
                329.88,
                330.31,
                330.72,
                331.1,
                331.46,
                331.79,
                332.1,
                332.38,
                332.63,
                332.86,
                333.06,
                333.24,
                333.39,
                333.52,
                333.62,
                333.69,
                333.74,
                333.76,
                333.76,
                333.13,
                330.16,
                327.18,
                324.21,
                321.24,
                318.26,
                315.29,
                312.32,
                309.35,
                306.38,
                303.41,
                300.44,
                297.47,
                294.51,
                291.54,
                288.57,
                285.61,
                282.64,
                279.68,
                276.71,
                275.1,
                273.88,
                272.67,
                271.48,
                270.29,
                269.12,
                267.95,
                266.8,
                265.66,
                264.53,
                263.42,
                262.31,
                261.21,
                260.13,
                259.06,
                258.0,
                256.95,
                255.91,
                254.89,
                254.13,
                253.86,
                253.59,
                253.32,
                253.13,
                252.95,
                1108.15,
                1115.48,
                1121.1,
                1115.06,
                1109.0,
                1102.91,
                1096.79,
                1090.65,
                1084.48,
                1078.27,
                1072.05,
                1065.79,
                1059.5,
                1053.19,
                1046.85,
                1040.48,
                1034.08,
                1027.66,
                1021.2,
                1014.72,
                1008.21,
                1001.67,
                997.85,
                994.95,
                992.02,
                989.07,
                986.1,
                983.11,
                980.09,
                977.05,
                973.99,
                970.9,
                967.8,
                964.67,
                961.51,
                958.33,
                955.13,
                951.91,
                948.66,
                945.39,
                942.1,
                938.32,
                933.73,
                929.14,
                924.55,
                919.96,
                915.38,
                910.8,
                906.23,
                901.65,
                897.08,
                892.51,
                887.95,
                883.38,
                878.83,
                874.27,
                869.71,
                865.16,
                860.61,
                856.07,
                851.52,
                848.17,
                844.87,
                841.59,
                838.34,
                835.1,
                831.9,
                828.71,
                825.56,
                822.42,
                819.31,
                816.22,
                813.16,
                810.12,
                807.11,
                804.12,
                801.15,
                798.21,
                795.29,
                792.4,
                782.18,
                766.85,
                751.53,
                736.22,
                720.92,
                705.63,
                690.36,
                675.09,
                659.83,
                644.58,
                629.34,
                614.1,
                598.88,
                583.67,
                568.47,
                553.28,
                538.1,
                522.93,
                507.76,
                494.44,
                488.56,
                482.65,
                476.72,
                470.76,
                464.77,
                458.76,
                452.73,
                446.66,
                440.57,
                434.46,
                428.32,
                422.15,
                415.96,
                409.74,
                403.5,
                397.23,
                390.93,
                384.61,
                378.26,
                376.71,
                376.37,
                376.04,
                375.73,
                375.43,
                375.15,
                374.89,
                374.63,
                374.4,
                374.18,
                373.97,
                373.78,
                373.61,
                373.45,
                373.31,
                373.18,
                373.06,
                372.97,
                372.88,
                372.81,
                372.73,
                372.67,
                372.63,
                372.6,
                372.58,
                372.58,
                372.59,
                372.61,
                372.65,
                372.71,
                372.77,
                372.86,
                372.95,
                373.07,
                373.19,
                373.33,
                373.49,
                373.65,
                373.84,
                372.46,
                371.07,
                369.68,
                368.29,
                366.89,
                365.49,
                364.09,
                362.68,
                361.27,
                359.85,
                358.43,
                357.01,
                355.59,
                354.16,
                352.72,
                351.29,
                349.84,
                348.4,
                346.95,
                347.51,
                349.47,
                351.44,
                353.43,
                355.44,
                357.46,
                359.49,
                361.54,
                363.6,
                365.68,
                367.77,
                369.88,
                372.0,
                374.14,
                376.29,
                378.46,
                380.64,
                382.84,
                385.05,
                386.47,
                384.23,
                381.96,
                379.69,
                377.39,
                375.08,
                372.75,
                370.4,
                368.04,
                365.66,
                363.27,
                360.86,
                358.43,
                355.98,
                353.52,
                351.04,
                348.55,
                346.04,
                343.51,
                340.96,
                340.06,
                339.66,
                339.26,
                338.85,
                338.44,
                338.03,
                337.61,
                337.18,
                336.75,
                336.32,
                335.88,
                335.44,
                334.99,
                334.54,
                334.09,
                333.63,
                333.16,
                332.69,
                332.22,
                331.92,
                331.94,
                331.93,
                331.9,
                331.84,
                331.76,
                331.65,
                331.51,
                331.35,
                331.17,
                330.96,
                330.72,
                330.45,
                330.17,
                329.85,
                329.51,
                329.14,
                328.75,
                328.33,
                327.89,
                325.12,
                322.16,
                319.2,
                316.23,
                313.27,
                310.31,
                307.35,
                304.39,
                301.44,
                298.48,
                295.67,
                292.93,
                290.2,
                287.48,
                284.77,
                282.07,
                279.37,
                276.69,
                274.01,
                272.23,
                271.35,
                270.47,
                269.58,
                268.69,
                267.8,
                266.9,
                266.0,
                265.1,
                264.19,
                263.27,
                262.36,
                261.44,
                260.51,
                259.58,
                258.65,
                257.71,
                256.77,
                255.83,
                254.92,
                254.72,
                254.52,
                254.31,
                254.1,
                253.89,
                1120.33,
                1126.05,
                1131.69,
                1126.51,
                1119.91,
                1113.29,
                1106.64,
                1099.96,
                1093.25,
                1086.52,
                1079.75,
                1072.96,
                1066.15,
                1059.3,
                1052.42,
                1045.52,
                1038.59,
                1031.63,
                1024.64,
                1017.63,
                1010.58,
                1003.51,
                998.33,
                995.0,
                991.65,
                988.27,
                984.87,
                981.45,
                978.01,
                974.54,
                971.05,
                967.54,
                964.0,
                960.44,
                956.86,
                953.26,
                949.63,
                945.98,
                942.31,
                938.61,
                934.89,
                931.05,
                926.51,
                921.98,
                917.44,
                912.91,
                908.39,
                903.86,
                899.34,
                894.82,
                890.3,
                885.79,
                881.28,
                876.77,
                872.26,
                867.76,
                863.26,
                858.76,
                854.27,
                849.78,
                845.29,
                841.97,
                839.11,
                836.27,
                833.45,
                830.66,
                827.89,
                825.14,
                822.42,
                819.73,
                817.05,
                814.41,
                811.78,
                809.18,
                806.6,
                804.05,
                801.53,
                799.02,
                796.54,
                794.09,
                787.34,
                772.18,
                757.04,
                741.9,
                726.78,
                711.66,
                696.56,
                681.46,
                666.38,
                651.3,
                636.23,
                621.17,
                606.13,
                591.09,
                576.06,
                561.04,
                546.04,
                531.04,
                516.05,
                501.07,
                494.24,
                487.87,
                481.48,
                475.06,
                468.62,
                462.15,
                455.66,
                449.14,
                442.59,
                436.02,
                429.42,
                422.8,
                416.15,
                409.47,
                402.77,
                396.04,
                389.29,
                382.51,
                375.7,
                372.54,
                372.46,
                372.4,
                372.36,
                372.33,
                372.31,
                372.31,
                372.33,
                372.36,
                372.41,
                372.47,
                372.55,
                372.64,
                372.74,
                372.87,
                373.01,
                373.16,
                373.33,
                373.51,
                373.71,
                373.88,
                374.06,
                374.26,
                374.47,
                374.7,
                374.94,
                375.2,
                375.47,
                375.75,
                376.05,
                376.37,
                376.69,
                377.04,
                377.39,
                377.76,
                378.15,
                378.55,
                378.96,
                379.39,
                378.45,
                377.0,
                375.54,
                374.09,
                372.63,
                371.16,
                369.69,
                368.22,
                366.75,
                365.27,
                363.79,
                362.3,
                360.81,
                359.32,
                357.82,
                356.32,
                354.81,
                353.31,
                351.79,
                351.48,
                353.68,
                355.9,
                358.14,
                360.38,
                362.65,
                364.93,
                367.22,
                369.53,
                371.85,
                374.19,
                376.54,
                378.91,
                381.29,
                383.68,
                386.1,
                388.52,
                390.96,
                393.42,
                395.89,
                393.82,
                391.29,
                388.73,
                386.16,
                383.57,
                380.96,
                378.34,
                375.7,
                373.05,
                370.38,
                367.69,
                364.98,
                362.26,
                359.53,
                356.77,
                354.0,
                351.21,
                348.41,
                345.59,
                343.91,
                343.44,
                342.96,
                342.48,
                342.0,
                341.51,
                341.01,
                340.51,
                340.01,
                339.5,
                338.99,
                338.47,
                337.95,
                337.43,
                336.9,
                336.36,
                335.81,
                335.15,
                334.5,
                333.87,
                333.43,
                333.0,
                332.57,
                332.14,
                331.71,
                331.28,
                330.86,
                330.44,
                330.02,
                329.61,
                329.2,
                328.79,
                328.38,
                327.98,
                327.58,
                327.18,
                326.78,
                326.39,
                326.0,
                324.13,
                321.44,
                318.76,
                316.09,
                313.43,
                310.77,
                308.12,
                305.48,
                302.85,
                300.23,
                297.61,
                295.01,
                292.41,
                289.82,
                287.23,
                284.66,
                282.09,
                279.53,
                276.98,
                274.77,
                273.83,
                272.89,
                271.94,
                270.99,
                270.03,
                269.07,
                268.1,
                267.13,
                266.16,
                265.19,
                264.21,
                263.22,
                262.23,
                261.24,
                260.24,
                259.24,
                258.24,
                257.23,
                256.22,
                255.8,
                255.57,
                255.33,
                255.09,
                254.85,
                1133.27,
                1137.39,
                1141.43,
                1138.2,
                1131.08,
                1123.92,
                1116.74,
                1109.52,
                1102.28,
                1095.02,
                1087.72,
                1080.4,
                1073.04,
                1065.66,
                1058.26,
                1050.82,
                1043.36,
                1035.86,
                1028.34,
                1020.79,
                1013.22,
                1005.61,
                999.01,
                995.25,
                991.47,
                987.67,
                983.85,
                980.0,
                976.13,
                972.23,
                968.32,
                964.38,
                960.42,
                956.43,
                952.42,
                948.39,
                944.34,
                940.26,
                936.16,
                932.04,
                927.89,
                923.72,
                919.27,
                914.79,
                910.31,
                905.84,
                901.36,
                896.89,
                892.42,
                887.96,
                883.5,
                879.04,
                874.58,
                870.12,
                865.67,
                861.22,
                856.78,
                852.33,
                847.89,
                843.46,
                839.02,
                835.55,
                833.13,
                830.72,
                828.35,
                825.99,
                823.66,
                821.35,
                819.07,
                816.81,
                814.58,
                812.37,
                810.18,
                808.02,
                805.88,
                803.77,
                801.68,
                799.61,
                797.57,
                795.55,
                792.41,
                777.43,
                762.46,
                747.5,
                732.55,
                717.6,
                702.67,
                687.75,
                672.84,
                657.93,
                643.04,
                628.16,
                613.28,
                598.42,
                583.56,
                568.72,
                553.88,
                539.06,
                524.24,
                509.44,
                500.15,
                493.33,
                486.48,
                479.6,
                472.7,
                465.78,
                458.82,
                451.85,
                444.84,
                437.81,
                430.76,
                423.67,
                416.57,
                409.43,
                402.27,
                395.09,
                387.88,
                380.64,
                373.38,
                368.23,
                368.42,
                368.63,
                368.85,
                369.08,
                369.34,
                369.6,
                369.89,
                370.18,
                370.5,
                370.82,
                371.17,
                371.53,
                371.9,
                372.29,
                372.69,
                373.11,
                373.55,
                374.0,
                374.47,
                374.89,
                375.32,
                375.76,
                376.22,
                376.69,
                377.18,
                377.68,
                378.2,
                378.73,
                379.27,
                379.83,
                380.4,
                380.99,
                381.59,
                382.21,
                382.84,
                383.48,
                384.14,
                384.81,
                384.47,
                382.95,
                381.44,
                379.92,
                378.39,
                376.86,
                375.33,
                373.8,
                372.26,
                370.72,
                369.17,
                367.62,
                366.07,
                364.51,
                362.95,
                361.38,
                359.82,
                358.24,
                356.67,
                355.31,
                357.76,
                360.23,
                362.7,
                365.2,
                367.71,
                370.23,
                372.77,
                375.32,
                377.89,
                380.47,
                383.06,
                385.68,
                388.3,
                390.94,
                393.6,
                396.27,
                398.95,
                401.65,
                404.37,
                403.57,
                400.76,
                397.89,
                394.93,
                391.93,
                388.9,
                385.85,
                382.77,
                379.65,
                376.51,
                373.33,
                370.13,
                366.9,
                363.64,
                360.35,
                357.03,
                353.68,
                350.29,
                346.88,
                344.04,
                343.39,
                342.75,
                342.11,
                341.48,
                340.86,
                340.24,
                339.63,
                339.04,
                338.44,
                337.86,
                337.28,
                336.72,
                336.16,
                335.6,
                335.06,
                334.52,
                333.99,
                333.47,
                332.96,
                332.54,
                332.15,
                331.76,
                331.37,
                330.98,
                330.6,
                330.22,
                329.84,
                329.47,
                329.1,
                328.73,
                328.36,
                328.0,
                327.64,
                327.28,
                326.92,
                326.57,
                326.22,
                325.87,
                324.73,
                322.17,
                319.61,
                317.07,
                314.53,
                312.01,
                309.49,
                306.97,
                304.47,
                301.98,
                299.49,
                297.01,
                294.54,
                292.07,
                289.62,
                287.17,
                284.73,
                282.3,
                279.88,
                277.46,
                276.35,
                275.34,
                274.33,
                273.31,
                272.29,
                271.27,
                270.24,
                269.21,
                268.17,
                267.13,
                266.09,
                265.04,
                263.99,
                262.93,
                261.87,
                260.81,
                259.74,
                258.67,
                257.59,
                256.91,
                256.64,
                256.37,
                256.1,
                255.82,
                1146.98,
                1149.49,
                1151.92,
                1150.16,
                1142.5,
                1134.81,
                1127.09,
                1119.34,
                1111.57,
                1103.77,
                1095.94,
                1088.08,
                1080.2,
                1072.29,
                1064.34,
                1056.38,
                1048.38,
                1040.35,
                1032.3,
                1024.22,
                1016.11,
                1007.97,
                999.9,
                995.72,
                991.51,
                987.28,
                983.03,
                978.75,
                974.46,
                970.14,
                965.79,
                961.43,
                957.04,
                952.63,
                948.19,
                943.73,
                939.25,
                934.75,
                930.22,
                925.67,
                921.1,
                916.5,
                912.01,
                907.58,
                903.15,
                898.73,
                894.31,
                889.9,
                885.48,
                881.07,
                876.66,
                872.26,
                867.85,
                863.45,
                859.06,
                854.66,
                850.27,
                845.88,
                841.49,
                837.11,
                832.73,
                828.92,
                826.93,
                824.96,
                823.02,
                821.11,
                819.21,
                817.35,
                815.5,
                813.68,
                811.88,
                810.11,
                808.36,
                806.64,
                804.94,
                803.26,
                801.61,
                799.98,
                798.38,
                796.8,
                795.25,
                782.59,
                767.79,
                753.01,
                738.23,
                723.46,
                708.7,
                693.95,
                679.21,
                664.48,
                649.76,
                635.05,
                620.35,
                605.66,
                590.98,
                576.3,
                561.64,
                546.99,
                532.35,
                517.72,
                506.3,
                499.01,
                491.71,
                484.38,
                477.02,
                469.63,
                462.22,
                454.79,
                447.33,
                439.84,
                432.33,
                424.79,
                417.23,
                409.63,
                402.02,
                394.38,
                386.71,
                379.01,
                371.29,
                363.78,
                364.24,
                364.71,
                365.2,
                365.7,
                366.22,
                366.76,
                367.3,
                367.87,
                368.45,
                369.04,
                369.65,
                370.28,
                370.92,
                371.57,
                372.24,
                372.93,
                373.63,
                374.35,
                375.08,
                375.77,
                376.45,
                377.14,
                377.84,
                378.56,
                379.29,
                380.04,
                380.8,
                381.57,
                382.36,
                383.16,
                383.98,
                384.81,
                385.66,
                386.52,
                387.4,
                388.28,
                389.19,
                390.11,
                390.52,
                388.95,
                387.37,
                385.78,
                384.19,
                382.6,
                381.01,
                379.42,
                377.93,
                376.52,
                375.18,
                373.91,
                372.71,
                371.59,
                370.54,
                369.56,
                368.65,
                367.82,
                367.06,
                366.37,
                368.45,
                371.21,
                373.93,
                376.59,
                379.2,
                381.77,
                384.28,
                386.74,
                389.15,
                391.51,
                393.83,
                396.09,
                398.3,
                400.45,
                402.56,
                404.62,
                406.63,
                408.58,
                410.48,
                410.43,
                407.03,
                403.61,
                400.15,
                396.67,
                393.15,
                389.61,
                386.04,
                382.43,
                378.8,
                375.14,
                371.45,
                367.73,
                363.98,
                360.2,
                356.39,
                352.55,
                348.68,
                344.78,
                340.85,
                340.12,
                339.6,
                339.09,
                338.59,
                338.1,
                337.61,
                337.13,
                336.66,
                336.2,
                335.74,
                335.29,
                334.85,
                334.42,
                333.99,
                333.58,
                333.17,
                332.77,
                332.37,
                331.99,
                331.62,
                331.27,
                330.92,
                330.58,
                330.24,
                329.9,
                329.56,
                329.22,
                328.89,
                328.56,
                328.24,
                327.91,
                327.59,
                327.27,
                326.95,
                326.64,
                326.33,
                326.02,
                325.72,
                325.25,
                322.82,
                320.39,
                317.98,
                315.57,
                313.17,
                310.78,
                308.39,
                306.02,
                303.65,
                301.29,
                298.94,
                296.59,
                294.26,
                291.93,
                289.61,
                287.3,
                285.0,
                282.7,
                280.41,
                278.91,
                277.84,
                276.76,
                275.68,
                274.6,
                273.51,
                272.42,
                271.32,
                270.22,
                269.12,
                268.01,
                266.9,
                265.78,
                264.66,
                263.54,
                262.41,
                261.28,
                260.14,
                259.01,
                258.03,
                257.73,
                257.43,
                257.12,
                256.82,
                1161.45,
                1162.36,
                1163.18,
                1162.37,
                1154.17,
                1145.95,
                1137.7,
                1129.42,
                1121.11,
                1112.78,
                1104.42,
                1096.03,
                1087.61,
                1079.16,
                1070.69,
                1062.19,
                1053.66,
                1045.1,
                1036.51,
                1027.9,
                1019.26,
                1010.58,
                1001.88,
                996.38,
                991.75,
                987.1,
                982.42,
                977.72,
                972.99,
                968.25,
                963.48,
                958.68,
                953.87,
                949.03,
                944.17,
                939.29,
                934.38,
                929.45,
                924.49,
                919.52,
                914.52,
                909.5,
                904.71,
                900.34,
                895.97,
                891.6,
                887.24,
                882.87,
                878.51,
                874.16,
                869.8,
                865.45,
                861.1,
                856.76,
                852.41,
                848.07,
                843.73,
                839.4,
                835.07,
                830.74,
                826.41,
                822.09,
                820.51,
                818.98,
                817.48,
                816.0,
                814.55,
                813.12,
                811.71,
                810.33,
                808.97,
                807.64,
                806.33,
                805.04,
                803.78,
                802.54,
                801.32,
                800.13,
                798.97,
                797.83,
                796.71,
                787.66,
                773.04,
                758.42,
                743.82,
                729.22,
                714.64,
                700.06,
                685.5,
                670.94,
                656.39,
                641.85,
                627.33,
                612.81,
                598.3,
                583.8,
                569.31,
                554.84,
                540.37,
                525.91,
                512.68,
                504.94,
                497.17,
                489.38,
                481.57,
                473.73,
                465.86,
                457.97,
                450.05,
                442.11,
                434.14,
                426.14,
                418.12,
                410.07,
                402.0,
                393.9,
                385.77,
                377.62,
                369.44,
                361.24,
                359.92,
                360.66,
                361.41,
                362.18,
                362.97,
                363.77,
                364.58,
                365.41,
                366.26,
                367.12,
                367.99,
                369.0,
                370.01,
                371.0,
                371.99,
                372.95,
                373.91,
                374.84,
                375.77,
                376.65,
                377.48,
                378.29,
                379.1,
                379.9,
                380.7,
                381.48,
                382.26,
                383.03,
                383.79,
                384.54,
                385.29,
                386.02,
                386.75,
                387.47,
                388.19,
                388.89,
                389.59,
                390.28,
                390.96,
                390.28,
                389.59,
                388.96,
                388.41,
                387.94,
                387.53,
                387.2,
                386.94,
                386.75,
                386.63,
                386.59,
                386.61,
                386.71,
                386.89,
                387.13,
                387.45,
                387.84,
                388.31,
                388.84,
                390.14,
                392.05,
                393.91,
                395.73,
                397.49,
                399.2,
                400.86,
                402.47,
                404.03,
                405.54,
                407.0,
                408.41,
                409.77,
                411.07,
                412.33,
                413.54,
                414.69,
                415.79,
                416.85,
                417.39,
                413.51,
                409.59,
                405.65,
                401.67,
                397.67,
                393.64,
                389.57,
                385.48,
                381.36,
                377.21,
                373.03,
                368.82,
                364.58,
                360.32,
                356.02,
                351.69,
                347.33,
                342.95,
                338.53,
                336.78,
                336.39,
                336.01,
                335.63,
                335.27,
                334.91,
                334.56,
                334.21,
                333.88,
                333.55,
                333.23,
                332.91,
                332.61,
                332.31,
                332.02,
                331.74,
                331.47,
                331.2,
                330.94,
                330.68,
                330.37,
                330.07,
                329.76,
                329.46,
                329.17,
                328.87,
                328.58,
                328.29,
                328.0,
                327.72,
                327.44,
                327.16,
                326.88,
                326.61,
                326.34,
                326.07,
                325.8,
                325.54,
                325.28,
                323.39,
                321.1,
                318.81,
                316.53,
                314.26,
                311.99,
                309.74,
                307.49,
                305.25,
                303.02,
                300.79,
                298.58,
                296.37,
                294.17,
                291.98,
                289.79,
                287.62,
                285.45,
                283.29,
                281.5,
                280.36,
                279.23,
                278.08,
                276.94,
                275.78,
                274.63,
                273.47,
                272.31,
                271.14,
                269.97,
                268.79,
                267.61,
                266.43,
                265.24,
                264.05,
                262.86,
                261.66,
                260.46,
                259.25,
                258.84,
                258.5,
                258.17,
                257.83,
                1176.68,
                1175.99,
                1175.21,
                1174.35,
                1166.1,
                1157.34,
                1148.56,
                1139.75,
                1130.91,
                1122.04,
                1113.15,
                1104.23,
                1095.28,
                1086.3,
                1077.29,
                1068.26,
                1059.19,
                1050.1,
                1040.98,
                1031.84,
                1022.66,
                1013.46,
                1004.22,
                997.26,
                992.2,
                987.12,
                982.01,
                976.89,
                971.74,
                966.56,
                961.37,
                956.15,
                950.91,
                945.64,
                940.35,
                935.04,
                929.71,
                924.35,
                918.98,
                913.57,
                908.15,
                902.7,
                897.39,
                893.07,
                888.76,
                884.44,
                880.13,
                875.82,
                871.52,
                867.22,
                862.92,
                858.62,
                854.32,
                850.03,
                845.74,
                841.46,
                837.17,
                832.89,
                828.61,
                824.34,
                820.07,
                815.8,
                813.87,
                812.79,
                811.72,
                810.68,
                809.66,
                808.67,
                807.7,
                806.76,
                805.84,
                804.94,
                804.07,
                803.22,
                802.39,
                801.59,
                800.82,
                800.06,
                799.34,
                798.63,
                797.95,
                792.65,
                778.2,
                763.76,
                749.32,
                734.9,
                720.49,
                706.09,
                691.69,
                677.31,
                662.94,
                648.57,
                634.22,
                619.87,
                605.54,
                591.1,
                576.48,
                561.89,
                547.33,
                532.81,
                518.33,
                509.61,
                501.36,
                493.15,
                484.97,
                476.83,
                468.72,
                460.64,
                452.59,
                444.57,
                436.59,
                428.64,
                420.72,
                412.84,
                404.99,
                397.17,
                389.39,
                381.63,
                373.91,
                366.23,
                362.98,
                363.89,
                364.79,
                365.68,
                366.55,
                367.41,
                368.25,
                369.08,
                369.9,
                370.7,
                371.48,
                372.25,
                373.01,
                373.75,
                374.47,
                375.19,
                375.88,
                376.57,
                377.23,
                377.89,
                378.58,
                379.26,
                379.93,
                380.59,
                381.24,
                381.89,
                382.53,
                383.16,
                383.78,
                384.4,
                385.0,
                385.6,
                386.19,
                386.77,
                387.35,
                387.91,
                388.47,
                389.02,
                389.56,
                390.06,
                390.59,
                391.19,
                391.86,
                392.61,
                393.43,
                394.32,
                395.28,
                396.32,
                397.42,
                398.6,
                399.86,
                401.18,
                402.58,
                404.05,
                405.59,
                407.2,
                408.89,
                410.65,
                412.29,
                413.35,
                414.36,
                415.32,
                416.23,
                417.09,
                417.9,
                418.66,
                419.37,
                420.03,
                420.64,
                421.19,
                421.7,
                422.16,
                422.56,
                422.92,
                423.22,
                423.47,
                423.67,
                423.83,
                420.24,
                415.84,
                411.41,
                406.94,
                402.45,
                397.93,
                393.38,
                388.8,
                384.19,
                379.55,
                374.89,
                370.19,
                365.46,
                360.71,
                355.92,
                351.1,
                346.26,
                341.38,
                336.48,
                333.37,
                333.11,
                332.85,
                332.6,
                332.37,
                332.13,
                331.91,
                331.69,
                331.49,
                331.28,
                331.09,
                330.91,
                330.73,
                330.56,
                330.4,
                330.24,
                330.1,
                329.96,
                329.83,
                329.71,
                329.45,
                329.19,
                328.93,
                328.67,
                328.41,
                328.16,
                327.91,
                327.66,
                327.42,
                327.18,
                326.94,
                326.7,
                326.47,
                326.24,
                326.01,
                325.78,
                325.56,
                325.34,
                325.12,
                323.9,
                321.73,
                319.57,
                317.42,
                315.27,
                313.14,
                311.01,
                308.89,
                306.78,
                304.67,
                302.57,
                300.49,
                298.41,
                296.33,
                294.27,
                292.21,
                290.17,
                288.13,
                286.1,
                284.13,
                282.93,
                281.73,
                280.52,
                279.31,
                278.1,
                276.88,
                275.65,
                274.43,
                273.2,
                271.96,
                270.72,
                269.48,
                268.24,
                266.98,
                265.73,
                264.47,
                263.21,
                261.94,
                260.67,
                259.96,
                259.6,
                259.23,
                258.86,
                1192.68,
                1190.38,
                1188.0,
                1185.53,
                1178.28,
                1168.99,
                1159.68,
                1150.33,
                1140.96,
                1131.56,
                1122.14,
                1112.68,
                1103.2,
                1093.69,
                1084.15,
                1074.58,
                1064.98,
                1055.36,
                1045.71,
                1036.03,
                1026.32,
                1016.59,
                1006.82,
                998.34,
                992.86,
                987.35,
                981.82,
                976.26,
                970.69,
                965.09,
                959.47,
                953.82,
                948.15,
                942.46,
                936.75,
                931.01,
                925.25,
                919.47,
                913.66,
                907.84,
                901.98,
                896.11,
                890.21,
                885.78,
                881.52,
                877.26,
                873.0,
                868.75,
                864.5,
                860.25,
                856.0,
                851.76,
                847.52,
                843.28,
                839.05,
                834.81,
                830.58,
                826.36,
                822.13,
                818.13,
                814.19,
                810.24,
                807.97,
                807.38,
                806.74,
                806.04,
                805.29,
                804.48,
                803.61,
                802.69,
                801.71,
                800.68,
                799.59,
                798.45,
                797.25,
                795.99,
                794.68,
                793.31,
                791.88,
                790.4,
                788.86,
                786.16,
                771.68,
                757.23,
                742.82,
                728.44,
                714.1,
                699.79,
                685.51,
                671.28,
                657.07,
                642.9,
                628.77,
                614.67,
                600.61,
                586.58,
                572.59,
                558.63,
                544.71,
                530.82,
                516.97,
                507.27,
                499.61,
                491.97,
                484.37,
                476.8,
                469.27,
                461.76,
                454.29,
                446.85,
                439.45,
                432.08,
                424.74,
                417.43,
                410.15,
                402.91,
                395.7,
                388.53,
                381.38,
                374.27,
                369.18,
                369.84,
                370.49,
                371.12,
                371.73,
                372.33,
                372.92,
                373.5,
                374.05,
                374.6,
                375.13,
                375.64,
                376.14,
                376.63,
                377.1,
                377.55,
                377.99,
                378.42,
                378.83,
                379.23,
                379.75,
                380.29,
                380.82,
                381.35,
                381.86,
                382.37,
                382.87,
                383.36,
                383.85,
                384.32,
                384.79,
                385.25,
                385.7,
                386.15,
                386.58,
                387.01,
                387.43,
                387.84,
                388.24,
                389.18,
                390.93,
                392.76,
                394.66,
                396.63,
                398.67,
                400.79,
                402.97,
                405.23,
                407.56,
                409.96,
                412.44,
                414.99,
                417.61,
                420.3,
                423.07,
                425.9,
                428.82,
                431.8,
                434.86,
                435.11,
                435.27,
                435.38,
                435.44,
                435.45,
                435.41,
                435.31,
                435.17,
                434.98,
                434.74,
                434.44,
                434.1,
                433.7,
                433.26,
                432.76,
                432.21,
                431.62,
                430.97,
                430.27,
                427.25,
                422.36,
                417.44,
                412.49,
                407.51,
                402.5,
                397.46,
                392.39,
                387.29,
                382.17,
                377.01,
                371.82,
                366.61,
                361.36,
                356.09,
                350.78,
                345.45,
                340.09,
                334.69,
                329.89,
                329.75,
                329.63,
                329.51,
                329.39,
                329.29,
                329.19,
                329.1,
                329.02,
                328.95,
                328.89,
                328.83,
                328.78,
                328.74,
                328.7,
                328.68,
                328.66,
                328.65,
                328.64,
                328.65,
                328.5,
                328.28,
                328.06,
                327.85,
                327.64,
                327.43,
                327.22,
                327.01,
                326.81,
                326.61,
                326.42,
                326.22,
                326.03,
                325.84,
                325.66,
                325.47,
                325.29,
                325.11,
                324.94,
                324.33,
                322.29,
                320.26,
                318.23,
                316.22,
                314.21,
                312.21,
                310.21,
                308.23,
                306.25,
                304.28,
                302.32,
                300.37,
                298.43,
                296.49,
                294.56,
                292.64,
                290.73,
                288.83,
                286.93,
                285.53,
                284.27,
                283.0,
                281.72,
                280.44,
                279.16,
                277.88,
                276.59,
                275.29,
                273.99,
                272.69,
                271.39,
                270.08,
                268.76,
                267.44,
                266.12,
                264.8,
                263.47,
                262.13,
                261.11,
                260.71,
                260.31,
                259.91,
                1209.43,
                1205.53,
                1201.55,
                1197.48,
                1190.72,
                1180.9,
                1171.05,
                1161.17,
                1151.27,
                1141.34,
                1131.38,
                1121.39,
                1111.37,
                1101.33,
                1091.26,
                1081.16,
                1071.03,
                1060.88,
                1050.69,
                1040.48,
                1030.24,
                1020.12,
                1010.69,
                1001.55,
                996.44,
                991.26,
                986.01,
                980.69,
                975.29,
                969.82,
                964.28,
                958.66,
                952.97,
                947.21,
                941.37,
                935.46,
                929.48,
                923.42,
                917.29,
                911.09,
                904.81,
                898.46,
                892.03,
                887.21,
                883.24,
                879.26,
                875.27,
                871.27,
                867.26,
                863.23,
                859.2,
                855.15,
                851.1,
                847.03,
                842.95,
                838.86,
                834.76,
                830.65,
                826.53,
                822.4,
                818.25,
                814.1,
                810.58,
                808.98,
                807.32,
                805.61,
                803.84,
                802.02,
                800.14,
                798.21,
                796.22,
                794.17,
                792.07,
                789.91,
                787.7,
                785.43,
                783.1,
                780.72,
                778.28,
                775.79,
                773.24,
                770.63,
                758.63,
                744.81,
                731.03,
                717.28,
                703.57,
                689.9,
                676.25,
                662.65,
                649.07,
                635.54,
                622.04,
                608.57,
                595.14,
                581.74,
                568.38,
                555.05,
                541.76,
                528.51,
                515.29,
                504.64,
                497.55,
                490.5,
                483.47,
                476.48,
                469.52,
                462.59,
                455.7,
                448.84,
                442.01,
                435.21,
                428.45,
                421.72,
                415.02,
                408.36,
                401.72,
                395.12,
                388.56,
                382.02,
                375.52,
                375.92,
                376.31,
                376.69,
                377.05,
                377.39,
                377.72,
                378.04,
                378.34,
                378.63,
                378.9,
                379.16,
                379.41,
                379.64,
                379.85,
                380.05,
                380.24,
                380.41,
                380.57,
                380.71,
                381.0,
                381.4,
                381.79,
                382.18,
                382.56,
                382.93,
                383.29,
                383.64,
                383.99,
                384.32,
                384.65,
                384.97,
                385.29,
                385.59,
                385.89,
                386.18,
                386.46,
                386.73,
                387.0,
                387.65,
                390.63,
                393.68,
                396.8,
                399.99,
                403.26,
                406.6,
                410.01,
                413.49,
                417.04,
                420.67,
                424.37,
                428.14,
                431.98,
                435.9,
                439.89,
                443.95,
                448.08,
                452.29,
                456.57,
                457.33,
                456.64,
                455.9,
                455.1,
                454.26,
                453.37,
                452.43,
                451.43,
                450.39,
                449.3,
                448.15,
                446.96,
                445.71,
                444.42,
                443.07,
                441.67,
                440.22,
                438.72,
                437.17,
                434.52,
                429.14,
                423.73,
                418.29,
                412.83,
                407.33,
                401.8,
                396.25,
                390.66,
                385.05,
                379.4,
                373.73,
                368.02,
                362.29,
                356.53,
                350.74,
                344.91,
                339.06,
                333.18,
                327.27,
                326.33,
                326.33,
                326.34,
                326.35,
                326.37,
                326.41,
                326.44,
                326.49,
                326.55,
                326.61,
                326.68,
                326.76,
                326.84,
                326.93,
                327.04,
                327.14,
                327.26,
                327.39,
                327.52,
                327.53,
                327.35,
                327.18,
                327.0,
                326.83,
                326.67,
                326.5,
                326.34,
                326.18,
                326.02,
                325.87,
                325.72,
                325.57,
                325.42,
                325.28,
                325.14,
                325.0,
                324.87,
                324.73,
                324.6,
                322.78,
                320.87,
                318.97,
                317.09,
                315.2,
                313.33,
                311.47,
                309.61,
                307.76,
                305.92,
                304.09,
                302.26,
                300.45,
                298.64,
                296.84,
                295.04,
                293.26,
                291.48,
                289.72,
                288.17,
                286.84,
                285.51,
                284.17,
                282.83,
                281.48,
                280.13,
                278.78,
                277.42,
                276.06,
                274.7,
                273.33,
                271.95,
                270.58,
                269.2,
                267.81,
                266.42,
                265.03,
                263.63,
                262.27,
                261.84,
                261.41,
                260.98,
                1233.74,
                1229.33,
                1224.91,
                1220.49,
                1214.92,
                1205.5,
                1196.08,
                1186.68,
                1177.28,
                1167.89,
                1158.51,
                1149.14,
                1139.78,
                1130.42,
                1121.07,
                1111.73,
                1102.4,
                1093.07,
                1083.76,
                1074.45,
                1065.15,
                1055.86,
                1046.57,
                1037.3,
                1030.31,
                1023.76,
                1017.13,
                1010.43,
                1003.66,
                996.81,
                989.89,
                982.9,
                975.83,
                968.69,
                961.48,
                954.2,
                946.84,
                939.41,
                931.9,
                924.32,
                916.67,
                908.94,
                901.14,
                894.81,
                890.64,
                886.45,
                882.26,
                878.06,
                873.84,
                869.62,
                865.38,
                861.13,
                856.88,
                852.61,
                848.33,
                844.04,
                839.73,
                835.42,
                831.1,
                826.76,
                822.42,
                818.06,
                813.7,
                811.08,
                808.41,
                805.69,
                802.91,
                800.07,
                797.18,
                794.23,
                791.23,
                788.17,
                785.06,
                781.89,
                778.66,
                775.38,
                772.04,
                768.65,
                765.2,
                761.69,
                758.13,
                754.51,
                745.26,
                732.08,
                718.93,
                705.81,
                692.73,
                679.68,
                666.67,
                653.7,
                640.76,
                627.85,
                614.98,
                602.14,
                589.34,
                576.58,
                563.85,
                551.15,
                538.49,
                525.87,
                513.28,
                501.72,
                495.2,
                488.72,
                482.28,
                475.86,
                469.48,
                463.13,
                456.81,
                450.53,
                444.27,
                438.05,
                431.87,
                425.71,
                419.59,
                413.5,
                407.45,
                401.42,
                395.43,
                389.47,
                383.55,
                382.13,
                382.27,
                382.39,
                382.49,
                382.58,
                382.66,
                382.72,
                382.77,
                382.8,
                382.82,
                382.82,
                382.81,
                382.78,
                382.74,
                382.69,
                382.62,
                382.53,
                382.43,
                382.32,
                382.32,
                382.58,
                382.83,
                383.08,
                383.32,
                383.55,
                383.78,
                383.99,
                384.2,
                384.4,
                384.59,
                384.77,
                384.95,
                385.11,
                385.27,
                385.42,
                385.56,
                385.7,
                385.83,
                385.94,
                389.67,
                393.95,
                398.29,
                402.71,
                407.2,
                411.76,
                416.39,
                421.09,
                425.87,
                430.72,
                435.64,
                440.63,
                445.7,
                450.84,
                456.05,
                461.33,
                466.69,
                472.12,
                477.62,
                480.01,
                478.46,
                476.87,
                475.23,
                473.54,
                471.79,
                470.0,
                468.16,
                466.26,
                464.32,
                462.33,
                460.28,
                458.18,
                456.04,
                453.84,
                451.59,
                449.29,
                446.95,
                444.54,
                442.06,
                436.19,
                430.3,
                424.37,
                418.41,
                412.43,
                406.41,
                400.37,
                394.3,
                388.19,
                382.06,
                375.9,
                369.71,
                363.49,
                357.24,
                350.96,
                344.65,
                338.31,
                331.94,
                325.54,
                322.83,
                322.96,
                323.09,
                323.24,
                323.39,
                323.55,
                323.71,
                323.89,
                324.07,
                324.26,
                324.46,
                324.66,
                324.87,
                325.1,
                325.32,
                325.56,
                325.8,
                326.06,
                326.32,
                326.53,
                326.4,
                326.27,
                326.14,
                326.01,
                325.88,
                325.76,
                325.64,
                325.53,
                325.41,
                325.3,
                325.19,
                325.08,
                324.98,
                324.88,
                324.78,
                324.69,
                324.59,
                324.5,
                324.41,
                323.19,
                321.41,
                319.64,
                317.88,
                316.13,
                314.38,
                312.65,
                310.92,
                309.2,
                307.48,
                305.78,
                304.08,
                302.39,
                300.71,
                299.04,
                297.37,
                295.72,
                294.07,
                292.43,
                290.85,
                289.45,
                288.06,
                286.66,
                285.25,
                283.84,
                282.43,
                281.01,
                279.59,
                278.17,
                276.74,
                275.31,
                273.87,
                272.43,
                270.98,
                269.53,
                268.08,
                266.62,
                265.16,
                263.7,
                262.99,
                262.53,
                262.06,
                1266.44,
                1261.88,
                1257.31,
                1252.74,
                1248.16,
                1238.92,
                1229.65,
                1220.39,
                1211.13,
                1201.88,
                1192.64,
                1183.41,
                1174.18,
                1164.97,
                1155.76,
                1146.56,
                1137.37,
                1128.18,
                1119.01,
                1109.84,
                1100.68,
                1091.53,
                1082.38,
                1073.25,
                1064.85,
                1056.92,
                1048.91,
                1040.84,
                1032.69,
                1024.47,
                1016.17,
                1007.81,
                999.36,
                990.85,
                982.26,
                973.6,
                964.87,
                956.06,
                947.18,
                938.23,
                929.2,
                920.1,
                910.92,
                902.5,
                898.13,
                893.75,
                889.35,
                884.95,
                880.53,
                876.1,
                871.66,
                867.21,
                862.75,
                858.28,
                853.8,
                849.31,
                844.8,
                840.29,
                835.76,
                831.23,
                826.68,
                822.12,
                817.55,
                813.69,
                810.01,
                806.27,
                802.48,
                798.63,
                794.72,
                790.77,
                786.75,
                782.68,
                778.55,
                774.37,
                770.13,
                765.84,
                761.49,
                757.08,
                752.62,
                748.1,
                743.52,
                738.89,
                731.57,
                719.02,
                706.5,
                694.02,
                681.57,
                669.15,
                656.77,
                644.43,
                632.12,
                619.84,
                607.6,
                595.4,
                583.23,
                571.09,
                558.99,
                546.93,
                534.9,
                522.91,
                510.95,
                499.03,
                492.56,
                486.66,
                480.78,
                474.95,
                469.14,
                463.37,
                457.62,
                451.92,
                446.24,
                440.6,
                434.99,
                429.41,
                423.86,
                418.35,
                412.87,
                407.42,
                402.01,
                396.63,
                391.28,
                388.48,
                388.36,
                388.22,
                388.07,
                387.91,
                387.73,
                387.53,
                387.32,
                387.1,
                386.86,
                386.61,
                386.34,
                386.06,
                385.77,
                385.45,
                385.13,
                384.79,
                384.43,
                384.06,
                383.71,
                383.83,
                383.95,
                384.06,
                384.16,
                384.25,
                384.34,
                384.41,
                384.48,
                384.54,
                384.6,
                384.64,
                384.68,
                384.71,
                384.73,
                384.74,
                384.74,
                384.74,
                384.73,
                384.71,
                388.06,
                393.56,
                399.13,
                404.77,
                410.48,
                416.26,
                422.11,
                428.04,
                434.04,
                440.11,
                446.25,
                452.47,
                458.76,
                465.12,
                471.55,
                478.06,
                484.63,
                491.29,
                498.01,
                503.14,
                500.75,
                498.31,
                495.82,
                493.27,
                490.68,
                488.04,
                485.34,
                482.6,
                479.8,
                476.96,
                474.06,
                471.12,
                468.12,
                465.08,
                461.98,
                458.83,
                455.63,
                452.38,
                449.08,
                443.51,
                437.13,
                430.71,
                424.27,
                417.79,
                411.29,
                404.76,
                398.2,
                391.61,
                384.99,
                378.34,
                371.66,
                364.95,
                358.22,
                351.45,
                344.65,
                337.82,
                330.97,
                324.08,
                319.27,
                319.52,
                319.78,
                320.05,
                320.33,
                320.62,
                320.91,
                321.21,
                321.52,
                321.84,
                322.16,
                322.5,
                322.84,
                323.19,
                323.54,
                323.9,
                324.28,
                324.66,
                325.04,
                325.44,
                325.42,
                325.33,
                325.24,
                325.16,
                325.08,
                325.0,
                324.92,
                324.85,
                324.78,
                324.71,
                324.64,
                324.58,
                324.51,
                324.46,
                324.4,
                324.35,
                324.3,
                324.25,
                324.2,
                323.53,
                321.88,
                320.24,
                318.61,
                316.98,
                315.36,
                313.76,
                312.15,
                310.56,
                308.97,
                307.4,
                305.83,
                304.27,
                302.71,
                301.17,
                299.63,
                298.1,
                296.58,
                295.07,
                293.56,
                292.1,
                290.64,
                289.18,
                287.71,
                286.24,
                284.76,
                283.28,
                281.75,
                280.16,
                278.56,
                276.97,
                275.38,
                273.79,
                272.2,
                270.61,
                269.03,
                267.45,
                265.87,
                264.29,
                263.19,
                262.64,
                262.09,
                1299.21,
                1294.5,
                1289.79,
                1285.06,
                1280.33,
                1272.28,
                1263.15,
                1254.02,
                1244.91,
                1235.8,
                1226.7,
                1217.61,
                1208.52,
                1199.45,
                1190.38,
                1181.32,
                1172.27,
                1163.23,
                1154.19,
                1145.16,
                1136.14,
                1127.13,
                1118.13,
                1109.13,
                1100.05,
                1090.75,
                1081.37,
                1071.92,
                1062.39,
                1052.79,
                1043.12,
                1033.38,
                1023.57,
                1013.68,
                1003.72,
                993.68,
                983.57,
                973.39,
                963.13,
                952.81,
                942.4,
                931.93,
                921.38,
                910.75,
                905.72,
                901.14,
                896.54,
                891.93,
                887.31,
                882.68,
                878.04,
                873.39,
                868.73,
                864.06,
                859.37,
                854.68,
                849.97,
                845.26,
                840.53,
                835.79,
                831.04,
                826.28,
                821.51,
                816.8,
                812.1,
                807.35,
                802.55,
                797.69,
                792.77,
                787.8,
                782.78,
                777.69,
                772.55,
                767.36,
                762.11,
                756.8,
                751.44,
                746.02,
                740.55,
                735.02,
                729.43,
                723.79,
                717.57,
                705.64,
                693.76,
                681.9,
                670.08,
                658.3,
                646.55,
                634.84,
                623.16,
                611.51,
                599.91,
                588.33,
                576.79,
                565.29,
                553.82,
                542.39,
                530.99,
                519.62,
                508.3,
                497.0,
                489.62,
                484.29,
                479.0,
                473.74,
                468.51,
                463.31,
                458.14,
                453.01,
                447.91,
                442.84,
                437.81,
                432.81,
                427.84,
                422.9,
                418.0,
                413.12,
                408.29,
                403.48,
                398.71,
                394.96,
                394.58,
                394.19,
                393.78,
                393.36,
                392.93,
                392.48,
                392.01,
                391.53,
                391.04,
                390.53,
                390.01,
                389.47,
                388.92,
                388.36,
                387.78,
                387.18,
                386.57,
                385.95,
                385.31,
                385.16,
                385.14,
                385.11,
                385.07,
                385.03,
                384.97,
                384.91,
                384.84,
                384.76,
                384.68,
                384.58,
                384.48,
                384.37,
                384.25,
                384.13,
                383.99,
                383.85,
                383.7,
                383.54,
                385.8,
                392.52,
                399.31,
                406.17,
                413.11,
                420.11,
                427.19,
                434.34,
                441.56,
                448.85,
                456.21,
                463.65,
                471.16,
                478.74,
                486.4,
                494.12,
                501.92,
                509.8,
                517.74,
                525.76,
                523.5,
                520.2,
                516.86,
                513.47,
                510.03,
                506.53,
                502.99,
                499.4,
                495.75,
                492.06,
                488.31,
                484.52,
                480.67,
                476.77,
                472.83,
                468.83,
                464.78,
                460.68,
                456.53,
                451.1,
                444.22,
                437.32,
                430.39,
                423.43,
                416.44,
                409.42,
                402.37,
                395.29,
                388.19,
                381.05,
                373.88,
                366.69,
                359.46,
                352.21,
                344.92,
                337.61,
                330.27,
                322.89,
                315.63,
                316.01,
                316.4,
                316.8,
                317.2,
                317.62,
                318.04,
                318.47,
                318.9,
                319.35,
                319.8,
                320.26,
                320.73,
                321.2,
                321.69,
                322.18,
                322.68,
                323.18,
                323.7,
                324.22,
                324.42,
                324.37,
                324.33,
                324.29,
                324.25,
                324.21,
                324.18,
                324.14,
                324.11,
                324.09,
                324.06,
                324.04,
                324.02,
                324.01,
                323.91,
                323.73,
                323.56,
                323.39,
                323.23,
                322.92,
                321.25,
                319.58,
                317.93,
                316.29,
                314.65,
                313.03,
                311.42,
                309.82,
                308.23,
                306.65,
                305.08,
                303.53,
                301.98,
                300.44,
                298.91,
                297.4,
                295.89,
                294.4,
                292.92,
                291.37,
                289.79,
                288.22,
                286.64,
                285.07,
                283.5,
                281.93,
                280.36,
                278.79,
                277.23,
                275.67,
                274.11,
                272.55,
                271.0,
                269.44,
                267.89,
                266.34,
                264.79,
                263.25,
                261.89,
                261.36,
                260.83,
                1332.05,
                1327.19,
                1322.33,
                1317.46,
                1312.58,
                1305.57,
                1296.58,
                1287.6,
                1278.62,
                1269.65,
                1260.69,
                1251.74,
                1242.8,
                1233.86,
                1224.93,
                1216.01,
                1207.1,
                1198.2,
                1189.3,
                1180.42,
                1171.54,
                1162.67,
                1153.8,
                1144.95,
                1135.92,
                1125.24,
                1114.48,
                1103.66,
                1092.76,
                1081.79,
                1070.74,
                1059.63,
                1048.44,
                1037.17,
                1025.84,
                1014.43,
                1002.94,
                991.39,
                979.76,
                968.06,
                956.28,
                944.43,
                932.5,
                920.51,
                913.42,
                908.63,
                903.83,
                899.02,
                894.2,
                889.37,
                884.53,
                879.67,
                874.81,
                869.93,
                865.05,
                860.15,
                855.25,
                850.33,
                845.4,
                840.46,
                835.51,
                830.55,
                825.57,
                820.41,
                814.71,
                808.94,
                803.13,
                797.26,
                791.33,
                785.35,
                779.31,
                773.21,
                767.06,
                760.86,
                754.59,
                748.28,
                741.9,
                735.47,
                728.99,
                722.44,
                715.85,
                709.19,
                702.48,
                691.95,
                680.69,
                669.47,
                658.28,
                647.13,
                636.01,
                624.93,
                613.88,
                602.87,
                591.89,
                580.94,
                570.04,
                559.16,
                548.32,
                537.52,
                526.75,
                516.02,
                505.32,
                494.66,
                486.39,
                481.63,
                476.92,
                472.23,
                467.58,
                462.95,
                458.36,
                453.81,
                449.28,
                444.79,
                440.33,
                435.91,
                431.51,
                427.15,
                422.82,
                418.53,
                414.27,
                410.04,
                405.84,
                401.68,
                400.93,
                400.29,
                399.63,
                398.95,
                398.26,
                397.55,
                396.84,
                396.1,
                395.35,
                394.59,
                393.81,
                393.02,
                392.21,
                391.39,
                390.56,
                389.71,
                388.84,
                387.96,
                387.07,
                386.56,
                386.4,
                386.23,
                386.06,
                385.87,
                385.68,
                385.48,
                385.27,
                385.06,
                384.83,
                384.6,
                384.36,
                384.11,
                383.86,
                383.59,
                383.32,
                383.04,
                382.75,
                382.45,
                382.89,
                390.83,
                398.84,
                406.93,
                415.08,
                423.31,
                431.61,
                439.98,
                448.42,
                456.93,
                465.52,
                474.18,
                482.91,
                491.71,
                500.59,
                509.53,
                518.56,
                527.65,
                536.81,
                546.05,
                546.7,
                542.56,
                538.37,
                534.12,
                529.83,
                525.49,
                521.1,
                516.65,
                512.16,
                507.61,
                503.02,
                498.38,
                493.68,
                488.93,
                484.14,
                479.29,
                474.39,
                469.44,
                464.44,
                458.95,
                451.59,
                444.2,
                436.78,
                429.33,
                421.85,
                414.35,
                406.81,
                399.25,
                391.65,
                384.03,
                376.37,
                368.69,
                360.98,
                353.24,
                345.46,
                337.66,
                329.83,
                321.97,
                314.08,
                312.39,
                312.75,
                313.11,
                313.48,
                313.84,
                314.21,
                314.58,
                314.95,
                315.33,
                315.7,
                316.08,
                316.46,
                316.84,
                317.22,
                317.6,
                317.99,
                318.38,
                318.77,
                319.16,
                319.39,
                319.23,
                319.07,
                318.93,
                318.79,
                318.66,
                318.54,
                318.42,
                318.31,
                318.21,
                318.12,
                318.04,
                317.96,
                317.89,
                317.83,
                317.78,
                317.73,
                317.69,
                317.66,
                317.64,
                316.4,
                314.9,
                313.42,
                311.95,
                310.49,
                309.03,
                307.59,
                306.16,
                304.74,
                303.33,
                301.93,
                300.55,
                299.17,
                297.8,
                296.45,
                295.1,
                293.76,
                292.44,
                291.13,
                289.73,
                288.19,
                286.64,
                285.1,
                283.56,
                282.02,
                280.48,
                278.94,
                277.41,
                275.88,
                274.35,
                272.82,
                271.3,
                269.77,
                268.25,
                266.73,
                265.22,
                263.7,
                262.19,
                260.68,
                260.06,
                259.55,
                1364.96,
                1359.95,
                1354.94,
                1349.92,
                1344.9,
                1338.79,
                1329.94,
                1321.1,
                1312.26,
                1303.44,
                1294.62,
                1285.81,
                1277.0,
                1268.21,
                1259.42,
                1250.64,
                1241.87,
                1233.11,
                1224.35,
                1215.6,
                1206.87,
                1198.13,
                1189.41,
                1180.7,
                1171.99,
                1160.4,
                1148.27,
                1136.07,
                1123.8,
                1111.45,
                1099.03,
                1086.54,
                1073.98,
                1061.34,
                1048.63,
                1035.85,
                1022.99,
                1010.06,
                997.06,
                983.98,
                970.83,
                957.6,
                944.31,
                930.93,
                921.21,
                916.22,
                911.22,
                906.21,
                901.18,
                896.15,
                891.11,
                886.05,
                880.99,
                875.91,
                870.82,
                865.72,
                860.62,
                855.5,
                850.37,
                845.22,
                840.07,
                834.91,
                829.73,
                824.53,
                817.81,
                811.04,
                804.21,
                797.33,
                790.39,
                783.4,
                776.34,
                769.24,
                762.08,
                754.86,
                747.59,
                740.26,
                732.87,
                725.43,
                717.94,
                710.38,
                702.77,
                695.11,
                687.39,
                677.94,
                667.31,
                656.72,
                646.16,
                635.64,
                625.15,
                614.7,
                604.28,
                593.9,
                583.55,
                573.24,
                562.96,
                552.71,
                542.51,
                532.33,
                522.2,
                512.09,
                502.03,
                491.99,
                482.86,
                478.68,
                474.54,
                470.43,
                466.35,
                462.3,
                458.29,
                454.31,
                450.36,
                446.45,
                442.56,
                438.71,
                434.89,
                431.11,
                427.36,
                423.64,
                419.95,
                416.29,
                412.67,
                409.08,
                407.42,
                406.52,
                405.6,
                404.67,
                403.73,
                402.77,
                401.79,
                400.8,
                399.8,
                398.78,
                397.75,
                396.7,
                395.64,
                394.56,
                393.47,
                392.36,
                391.24,
                390.11,
                388.96,
                388.03,
                387.73,
                387.43,
                387.11,
                386.79,
                386.46,
                386.12,
                385.78,
                385.42,
                385.06,
                384.69,
                384.31,
                383.93,
                383.53,
                383.13,
                382.72,
                382.3,
                381.87,
                381.44,
                381.0,
                388.49,
                397.72,
                407.03,
                416.4,
                425.85,
                435.37,
                444.96,
                454.62,
                464.36,
                474.17,
                484.05,
                494.0,
                504.02,
                514.12,
                524.29,
                534.53,
                544.84,
                555.23,
                565.69,
                570.37,
                565.37,
                560.33,
                555.24,
                550.1,
                544.91,
                538.81,
                532.43,
                526.04,
                519.65,
                513.25,
                506.84,
                500.43,
                494.01,
                487.58,
                481.14,
                474.7,
                468.25,
                461.8,
                455.34,
                447.3,
                439.17,
                431.08,
                423.04,
                415.04,
                407.08,
                399.16,
                391.28,
                383.45,
                375.66,
                367.91,
                360.2,
                352.53,
                344.91,
                337.33,
                329.79,
                322.3,
                314.84,
                307.43,
                303.7,
                304.09,
                304.48,
                304.88,
                305.28,
                305.68,
                306.08,
                306.48,
                306.89,
                307.29,
                307.7,
                308.11,
                308.52,
                308.94,
                309.35,
                309.77,
                310.19,
                310.61,
                311.03,
                311.46,
                311.42,
                311.39,
                311.38,
                311.36,
                311.36,
                311.37,
                311.38,
                311.4,
                311.42,
                311.46,
                311.5,
                311.55,
                311.61,
                311.68,
                311.75,
                311.83,
                311.92,
                312.02,
                312.13,
                311.45,
                310.13,
                308.82,
                307.51,
                306.22,
                304.94,
                303.67,
                302.41,
                301.16,
                299.92,
                298.69,
                297.47,
                296.26,
                295.06,
                293.88,
                292.7,
                291.54,
                290.38,
                289.24,
                288.08,
                286.56,
                285.05,
                283.54,
                282.03,
                280.52,
                279.01,
                277.51,
                276.01,
                274.51,
                273.01,
                271.52,
                270.02,
                268.53,
                267.04,
                265.56,
                264.07,
                262.59,
                261.11,
                259.63,
                258.75,
                258.26,
                1397.94,
                1392.79,
                1387.63,
                1382.46,
                1377.28,
                1371.95,
                1363.24,
                1354.54,
                1345.84,
                1337.15,
                1328.47,
                1319.8,
                1311.14,
                1302.49,
                1293.84,
                1285.2,
                1276.57,
                1267.95,
                1259.33,
                1250.72,
                1242.12,
                1233.53,
                1224.95,
                1216.38,
                1207.81,
                1196.22,
                1182.72,
                1169.14,
                1155.5,
                1141.78,
                1127.99,
                1114.12,
                1100.18,
                1086.17,
                1072.09,
                1057.93,
                1043.7,
                1029.4,
                1015.02,
                1000.57,
                986.05,
                971.45,
                956.78,
                942.04,
                929.1,
                923.91,
                918.7,
                913.49,
                908.27,
                903.03,
                897.79,
                892.53,
                887.26,
                881.98,
                876.7,
                871.4,
                866.09,
                860.76,
                855.43,
                850.09,
                844.73,
                839.37,
                833.99,
                828.61,
                821.42,
                813.64,
                805.8,
                797.91,
                789.96,
                781.95,
                773.89,
                765.77,
                757.6,
                749.37,
                741.09,
                732.75,
                724.35,
                715.9,
                707.39,
                698.83,
                690.21,
                681.53,
                672.8,
                663.61,
                653.61,
                643.65,
                633.72,
                623.83,
                613.97,
                604.15,
                594.36,
                584.61,
                574.89,
                565.21,
                555.56,
                545.95,
                536.37,
                526.82,
                517.32,
                507.84,
                498.41,
                489.0,
                479.64,
                475.43,
                471.87,
                468.33,
                464.83,
                461.36,
                457.92,
                454.51,
                451.14,
                447.8,
                444.49,
                441.22,
                437.98,
                434.77,
                431.59,
                428.44,
                425.33,
                422.25,
                419.21,
                416.19,
                414.04,
                412.88,
                411.71,
                410.53,
                409.33,
                408.11,
                406.88,
                405.64,
                404.38,
                403.1,
                401.82,
                400.51,
                399.2,
                397.86,
                396.52,
                395.16,
                393.78,
                392.39,
                390.99,
                389.58,
                389.14,
                388.7,
                388.24,
                387.78,
                387.31,
                386.84,
                386.35,
                385.86,
                385.36,
                384.85,
                384.81,
                384.74,
                384.64,
                384.52,
                384.38,
                384.22,
                384.03,
                383.82,
                383.59,
                389.07,
                398.95,
                408.73,
                418.41,
                427.99,
                437.48,
                446.87,
                456.15,
                465.34,
                474.43,
                483.42,
                492.31,
                501.11,
                509.8,
                518.39,
                526.89,
                535.28,
                543.58,
                551.77,
                558.27,
                551.83,
                545.38,
                538.92,
                532.45,
                525.98,
                519.5,
                513.01,
                506.52,
                500.02,
                493.51,
                487.0,
                480.48,
                473.96,
                467.42,
                460.88,
                454.34,
                447.78,
                441.22,
                434.66,
                427.51,
                420.08,
                412.7,
                405.35,
                398.05,
                390.8,
                383.58,
                376.41,
                369.27,
                362.18,
                355.14,
                348.13,
                341.17,
                334.25,
                327.37,
                320.53,
                313.74,
                306.99,
                300.28,
                294.98,
                295.41,
                295.83,
                296.26,
                296.69,
                297.12,
                297.56,
                297.99,
                298.43,
                298.87,
                299.31,
                299.75,
                300.19,
                300.64,
                301.09,
                301.54,
                301.99,
                302.44,
                302.89,
                303.35,
                303.54,
                303.64,
                303.75,
                303.87,
                303.99,
                304.12,
                304.26,
                304.41,
                304.56,
                304.73,
                304.9,
                305.07,
                305.26,
                305.45,
                305.66,
                305.86,
                306.08,
                306.31,
                306.54,
                306.41,
                305.26,
                304.11,
                302.98,
                301.86,
                300.75,
                299.64,
                298.55,
                297.47,
                296.4,
                295.34,
                294.3,
                293.26,
                292.23,
                291.21,
                290.21,
                289.21,
                288.23,
                287.25,
                286.29,
                284.92,
                283.44,
                281.96,
                280.48,
                279.0,
                277.53,
                276.06,
                274.59,
                273.12,
                271.66,
                270.19,
                268.73,
                267.27,
                265.82,
                264.36,
                262.91,
                261.46,
                260.01,
                258.56,
                257.42,
                256.96,
                1430.99,
                1425.69,
                1420.38,
                1415.07,
                1409.74,
                1404.41,
                1396.47,
                1387.91,
                1379.35,
                1370.8,
                1362.27,
                1353.73,
                1345.21,
                1336.7,
                1328.19,
                1319.69,
                1311.2,
                1302.72,
                1294.24,
                1285.78,
                1277.32,
                1268.87,
                1260.42,
                1251.99,
                1243.56,
                1232.71,
                1217.83,
                1202.88,
                1187.86,
                1172.77,
                1157.61,
                1142.37,
                1127.06,
                1111.68,
                1096.22,
                1080.69,
                1065.09,
                1049.41,
                1033.66,
                1017.84,
                1001.94,
                985.97,
                969.93,
                953.81,
                937.62,
                931.69,
                926.29,
                920.87,
                915.45,
                910.01,
                904.57,
                899.11,
                893.64,
                888.16,
                882.67,
                877.17,
                871.66,
                866.13,
                860.6,
                855.05,
                849.5,
                843.93,
                838.35,
                832.77,
                825.54,
                816.74,
                807.89,
                798.99,
                790.03,
                781.01,
                771.94,
                762.81,
                753.63,
                744.39,
                735.09,
                725.74,
                716.34,
                706.88,
                697.36,
                687.78,
                678.15,
                668.47,
                658.72,
                648.96,
                639.59,
                630.26,
                620.97,
                611.7,
                602.48,
                593.28,
                584.12,
                575.0,
                565.91,
                556.86,
                547.84,
                538.86,
                529.91,
                521.0,
                512.12,
                503.27,
                494.47,
                485.69,
                476.96,
                471.89,
                468.9,
                465.94,
                463.01,
                460.12,
                457.25,
                454.42,
                451.63,
                448.86,
                446.13,
                443.43,
                440.76,
                438.13,
                435.53,
                433.01,
                430.5,
                428.0,
                425.5,
                423.0,
                420.76,
                419.54,
                418.36,
                417.23,
                416.14,
                415.09,
                414.09,
                413.13,
                412.21,
                411.34,
                410.5,
                409.71,
                408.97,
                408.26,
                407.6,
                406.99,
                406.41,
                405.88,
                405.39,
                404.95,
                404.68,
                404.43,
                404.17,
                403.88,
                403.56,
                403.23,
                402.87,
                402.48,
                402.08,
                401.65,
                401.2,
                400.72,
                400.22,
                399.7,
                399.16,
                398.59,
                398.0,
                397.38,
                396.74,
                398.74,
                406.94,
                415.04,
                423.04,
                430.94,
                438.75,
                446.45,
                454.06,
                461.57,
                468.98,
                476.29,
                483.5,
                490.61,
                497.62,
                504.54,
                511.35,
                518.06,
                524.68,
                531.19,
                537.61,
                533.1,
                526.54,
                519.97,
                513.4,
                506.83,
                500.24,
                493.65,
                487.05,
                480.45,
                473.84,
                467.22,
                460.59,
                453.96,
                447.33,
                440.68,
                434.03,
                427.37,
                420.71,
                414.04,
                407.33,
                400.61,
                393.92,
                387.28,
                380.69,
                374.13,
                367.62,
                361.14,
                354.71,
                348.33,
                341.98,
                335.68,
                329.41,
                323.2,
                317.02,
                310.88,
                304.79,
                298.74,
                292.73,
                286.77,
                286.71,
                287.17,
                287.63,
                288.09,
                288.55,
                289.02,
                289.48,
                289.95,
                290.42,
                290.89,
                291.37,
                291.84,
                292.32,
                292.8,
                293.28,
                293.77,
                294.25,
                294.74,
                295.22,
                295.59,
                295.82,
                296.06,
                296.3,
                296.55,
                296.81,
                297.08,
                297.35,
                297.63,
                297.92,
                298.22,
                298.52,
                298.84,
                299.16,
                299.49,
                299.82,
                300.17,
                300.52,
                300.88,
                301.25,
                300.29,
                299.31,
                298.35,
                297.4,
                296.46,
                295.53,
                294.6,
                293.69,
                292.79,
                291.9,
                291.03,
                290.16,
                289.3,
                288.45,
                287.62,
                286.79,
                285.98,
                285.17,
                284.38,
                283.26,
                281.81,
                280.36,
                278.91,
                277.47,
                276.03,
                274.59,
                273.15,
                271.72,
                270.28,
                268.85,
                267.42,
                266.0,
                264.57,
                263.15,
                261.73,
                260.31,
                258.89,
                257.47,
                256.09,
                255.64,
                1464.11,
                1458.66,
                1453.21,
                1447.74,
                1442.27,
                1436.79,
                1429.63,
                1421.21,
                1412.79,
                1404.39,
                1395.99,
                1387.6,
                1379.21,
                1370.84,
                1362.47,
                1354.11,
                1345.76,
                1337.42,
                1329.09,
                1320.76,
                1312.44,
                1304.13,
                1295.83,
                1287.53,
                1279.25,
                1269.86,
                1253.61,
                1237.29,
                1220.9,
                1204.43,
                1187.9,
                1171.29,
                1154.6,
                1137.85,
                1121.02,
                1104.11,
                1087.14,
                1070.09,
                1052.97,
                1035.77,
                1018.51,
                1001.16,
                983.75,
                966.26,
                948.7,
                939.58,
                933.97,
                928.36,
                922.73,
                917.09,
                911.44,
                905.79,
                900.11,
                894.43,
                888.74,
                883.04,
                877.33,
                871.6,
                865.87,
                860.12,
                854.36,
                848.59,
                842.82,
                837.03,
                830.15,
                820.35,
                810.49,
                800.57,
                790.6,
                780.57,
                770.49,
                760.36,
                750.16,
                739.91,
                729.61,
                719.25,
                708.83,
                698.36,
                687.83,
                677.25,
                666.61,
                656.34,
                646.04,
                635.7,
                627.03,
                618.72,
                610.43,
                602.15,
                593.88,
                585.62,
                577.38,
                569.15,
                560.94,
                552.74,
                544.55,
                536.38,
                528.22,
                520.07,
                511.94,
                503.83,
                495.72,
                487.63,
                479.56,
                473.65,
                471.15,
                468.67,
                466.18,
                463.71,
                461.24,
                458.77,
                456.31,
                453.86,
                451.41,
                448.97,
                446.53,
                444.1,
                441.67,
                439.25,
                436.84,
                434.43,
                432.03,
                429.63,
                427.23,
                426.65,
                426.22,
                425.83,
                425.48,
                425.18,
                424.92,
                424.7,
                424.53,
                424.4,
                424.31,
                424.27,
                424.26,
                424.31,
                424.39,
                424.52,
                424.69,
                424.9,
                425.16,
                425.46,
                425.32,
                424.67,
                424.0,
                423.31,
                422.59,
                421.85,
                421.09,
                420.3,
                419.49,
                418.66,
                417.8,
                416.92,
                416.02,
                415.1,
                414.15,
                413.18,
                412.18,
                411.16,
                410.12,
                409.31,
                415.83,
                422.24,
                428.57,
                434.79,
                440.91,
                446.94,
                452.87,
                458.69,
                464.42,
                470.05,
                475.59,
                481.02,
                486.35,
                491.58,
                496.72,
                501.75,
                506.69,
                511.52,
                516.26,
                514.42,
                507.76,
                501.09,
                494.41,
                487.73,
                481.04,
                474.34,
                467.64,
                460.93,
                454.22,
                447.49,
                440.76,
                434.03,
                427.29,
                420.54,
                413.78,
                407.02,
                400.25,
                393.47,
                386.77,
                380.75,
                374.77,
                368.83,
                362.93,
                357.08,
                351.27,
                345.5,
                339.77,
                334.08,
                328.44,
                322.83,
                317.27,
                311.76,
                306.28,
                300.85,
                295.46,
                290.11,
                284.8,
                279.54,
                277.99,
                278.48,
                278.98,
                279.47,
                279.96,
                280.46,
                280.96,
                281.46,
                281.96,
                282.46,
                282.97,
                283.48,
                283.99,
                284.5,
                285.01,
                285.53,
                286.04,
                286.56,
                287.08,
                287.57,
                287.93,
                288.29,
                288.66,
                289.04,
                289.42,
                289.82,
                290.22,
                290.63,
                291.05,
                291.47,
                291.9,
                292.34,
                292.79,
                293.25,
                293.71,
                294.18,
                294.66,
                295.15,
                295.64,
                295.22,
                294.42,
                293.63,
                292.84,
                292.07,
                291.31,
                290.56,
                289.82,
                289.09,
                288.37,
                287.66,
                286.96,
                286.27,
                285.59,
                284.93,
                284.27,
                283.63,
                282.99,
                282.37,
                281.58,
                280.16,
                278.74,
                277.33,
                275.92,
                274.51,
                273.1,
                271.7,
                270.29,
                268.89,
                267.49,
                266.1,
                264.7,
                263.31,
                261.92,
                260.53,
                259.14,
                257.75,
                256.37,
                254.99,
                254.32,
                1497.3,
                1491.71,
                1486.1,
                1480.49,
                1474.87,
                1469.24,
                1462.73,
                1454.44,
                1446.17,
                1437.9,
                1429.64,
                1421.39,
                1413.15,
                1404.92,
                1396.69,
                1388.47,
                1380.26,
                1372.06,
                1363.86,
                1355.68,
                1347.5,
                1339.33,
                1331.16,
                1323.01,
                1314.86,
                1306.73,
                1290.05,
                1272.36,
                1254.6,
                1236.76,
                1218.85,
                1200.87,
                1182.81,
                1164.68,
                1146.48,
                1128.21,
                1109.86,
                1091.44,
                1072.95,
                1054.38,
                1035.74,
                1017.03,
                998.24,
                979.38,
                961.14,
                949.13,
                944.08,
                938.96,
                933.76,
                928.49,
                923.16,
                917.75,
                912.26,
                906.71,
                901.09,
                895.39,
                889.62,
                883.78,
                877.87,
                871.89,
                865.83,
                859.7,
                853.5,
                847.23,
                840.79,
                830.44,
                820.06,
                809.64,
                799.19,
                788.69,
                778.17,
                767.6,
                757.0,
                746.36,
                735.69,
                724.97,
                714.23,
                703.44,
                692.62,
                681.76,
                670.86,
                659.93,
                648.96,
                637.95,
                628.63,
                620.57,
                612.52,
                604.48,
                596.46,
                588.45,
                580.46,
                572.48,
                564.51,
                556.55,
                548.61,
                540.69,
                532.77,
                524.88,
                516.99,
                509.12,
                501.26,
                493.42,
                485.59,
                478.49,
                476.09,
                473.7,
                471.32,
                468.94,
                466.56,
                464.2,
                461.83,
                459.47,
                457.12,
                454.78,
                452.43,
                450.1,
                447.77,
                445.44,
                443.13,
                440.81,
                438.5,
                436.2,
                433.91,
                433.37,
                433.68,
                434.04,
                434.44,
                434.88,
                435.36,
                435.89,
                436.46,
                437.08,
                437.73,
                438.43,
                439.17,
                439.96,
                440.79,
                441.66,
                442.57,
                443.53,
                444.53,
                445.58,
                446.18,
                445.13,
                444.05,
                442.95,
                441.83,
                440.69,
                439.52,
                438.33,
                437.12,
                435.88,
                434.62,
                433.34,
                432.03,
                430.7,
                429.35,
                427.98,
                426.58,
                425.15,
                423.71,
                422.24,
                425.61,
                430.35,
                434.99,
                439.53,
                443.98,
                448.32,
                452.57,
                456.72,
                460.77,
                464.72,
                468.57,
                472.33,
                475.98,
                479.54,
                482.99,
                486.35,
                489.6,
                492.76,
                495.82,
                495.8,
                489.04,
                482.26,
                475.48,
                468.69,
                461.9,
                455.1,
                448.29,
                441.47,
                434.65,
                427.83,
                420.99,
                414.15,
                407.3,
                400.45,
                393.59,
                386.72,
                379.84,
                372.96,
                366.07,
                360.51,
                355.23,
                349.99,
                344.8,
                339.64,
                334.53,
                329.46,
                324.44,
                319.45,
                314.51,
                309.61,
                304.75,
                299.93,
                295.16,
                290.42,
                285.74,
                281.09,
                276.48,
                271.92,
                269.26,
                269.78,
                270.31,
                270.83,
                271.36,
                271.89,
                272.42,
                272.95,
                273.48,
                274.02,
                274.55,
                275.09,
                275.64,
                276.18,
                276.72,
                277.27,
                277.82,
                278.37,
                278.92,
                279.47,
                279.96,
                280.45,
                280.95,
                281.45,
                281.97,
                282.49,
                283.02,
                283.55,
                284.1,
                284.65,
                285.21,
                285.78,
                286.35,
                286.93,
                287.52,
                288.12,
                288.73,
                289.34,
                289.96,
                290.06,
                289.43,
                288.8,
                288.19,
                287.59,
                287.0,
                286.41,
                285.84,
                285.28,
                284.73,
                284.19,
                283.66,
                283.15,
                282.64,
                282.14,
                281.65,
                281.18,
                280.71,
                280.26,
                279.82,
                278.49,
                277.11,
                275.73,
                274.35,
                272.97,
                271.6,
                270.22,
                268.85,
                267.48,
                266.11,
                264.75,
                263.39,
                262.02,
                260.67,
                259.31,
                257.95,
                256.6,
                255.25,
                253.9,
                252.98,
                1530.57,
                1524.82,
                1519.07,
                1513.31,
                1507.54,
                1501.76,
                1494.9,
                1485.61,
                1476.32,
                1467.02,
                1457.71,
                1448.4,
                1439.08,
                1429.76,
                1420.42,
                1411.08,
                1401.73,
                1392.38,
                1383.02,
                1373.65,
                1364.28,
                1354.89,
                1345.51,
                1336.11,
                1326.71,
                1317.3,
                1302.2,
                1284.29,
                1266.52,
                1248.9,
                1231.42,
                1214.1,
                1196.91,
                1179.88,
                1162.99,
                1146.25,
                1129.65,
                1113.2,
                1096.9,
                1080.75,
                1064.74,
                1048.89,
                1033.18,
                1017.61,
                1002.2,
                988.88,
                982.5,
                976.04,
                969.52,
                962.92,
                956.25,
                949.51,
                942.7,
                935.81,
                928.86,
                921.83,
                914.73,
                907.56,
                900.32,
                893.01,
                885.62,
                878.16,
                870.63,
                863.03,
                855.35,
                845.07,
                834.02,
                822.93,
                811.81,
                800.65,
                789.45,
                778.21,
                766.94,
                755.64,
                744.29,
                732.91,
                721.49,
                710.04,
                698.55,
                687.02,
                675.46,
                663.85,
                652.22,
                640.54,
                630.11,
                622.29,
                614.49,
                606.7,
                598.92,
                591.16,
                583.41,
                575.67,
                567.95,
                560.24,
                552.55,
                544.87,
                537.2,
                529.55,
                521.91,
                514.29,
                506.68,
                499.08,
                491.49,
                483.92,
                480.99,
                478.69,
                476.4,
                474.12,
                471.84,
                469.57,
                467.3,
                465.04,
                462.78,
                460.53,
                458.29,
                456.05,
                453.81,
                451.59,
                449.36,
                447.15,
                444.93,
                442.73,
                440.53,
                439.7,
                440.76,
                441.86,
                443.0,
                444.19,
                445.42,
                446.69,
                448.01,
                449.36,
                450.76,
                452.21,
                453.69,
                455.22,
                456.79,
                458.41,
                460.07,
                461.77,
                463.52,
                465.3,
                467.14,
                465.79,
                464.31,
                462.81,
                461.29,
                459.74,
                458.17,
                456.57,
                454.96,
                453.32,
                451.65,
                449.97,
                448.26,
                446.53,
                444.77,
                442.99,
                441.19,
                439.36,
                437.51,
                435.64,
                436.29,
                439.35,
                442.31,
                445.17,
                447.94,
                450.61,
                453.18,
                455.65,
                458.02,
                460.29,
                462.46,
                464.54,
                466.51,
                468.39,
                470.17,
                471.84,
                473.42,
                474.9,
                476.28,
                477.24,
                470.37,
                463.49,
                456.6,
                449.71,
                442.81,
                435.91,
                428.99,
                422.07,
                415.15,
                408.22,
                401.28,
                394.33,
                387.38,
                380.42,
                373.45,
                366.48,
                359.5,
                352.51,
                345.52,
                339.88,
                335.3,
                330.77,
                326.27,
                321.82,
                317.41,
                313.04,
                308.72,
                304.43,
                300.19,
                295.99,
                291.84,
                287.72,
                283.65,
                279.61,
                275.63,
                271.68,
                267.78,
                263.91,
                260.51,
                261.06,
                261.62,
                262.18,
                262.73,
                263.29,
                263.86,
                264.42,
                264.99,
                265.55,
                266.12,
                266.69,
                267.27,
                267.84,
                268.42,
                268.99,
                269.57,
                270.16,
                270.74,
                271.32,
                271.93,
                272.54,
                273.17,
                273.8,
                274.44,
                275.09,
                275.74,
                276.41,
                277.08,
                277.76,
                278.44,
                279.14,
                279.84,
                280.55,
                281.27,
                281.99,
                282.73,
                283.47,
                284.22,
                284.8,
                284.34,
                283.89,
                283.44,
                283.01,
                282.59,
                282.17,
                281.77,
                281.38,
                281.0,
                280.63,
                280.27,
                279.92,
                279.59,
                279.26,
                278.94,
                278.63,
                278.34,
                278.05,
                277.78,
                276.81,
                275.46,
                274.11,
                272.76,
                271.41,
                270.07,
                268.73,
                267.39,
                266.05,
                264.72,
                263.38,
                262.05,
                260.72,
                259.4,
                258.07,
                256.75,
                255.43,
                254.11,
                252.79,
                251.63,
                1528.99,
                1521.62,
                1514.21,
                1506.78,
                1499.3,
                1491.8,
                1484.26,
                1475.11,
                1465.7,
                1456.27,
                1446.84,
                1437.4,
                1427.95,
                1418.5,
                1409.04,
                1399.57,
                1390.1,
                1380.62,
                1371.13,
                1361.64,
                1352.14,
                1342.63,
                1333.11,
                1323.59,
                1314.06,
                1304.53,
                1292.58,
                1277.41,
                1262.39,
                1247.51,
                1232.77,
                1218.19,
                1203.74,
                1189.45,
                1175.3,
                1161.3,
                1147.45,
                1133.74,
                1120.18,
                1106.77,
                1093.5,
                1080.38,
                1067.42,
                1054.59,
                1041.92,
                1029.4,
                1021.57,
                1013.78,
                1005.92,
                998.0,
                990.0,
                981.93,
                973.78,
                965.57,
                957.29,
                948.93,
                940.5,
                932.0,
                923.43,
                914.78,
                906.07,
                897.28,
                888.42,
                879.49,
                870.48,
                860.03,
                848.31,
                836.55,
                824.76,
                812.93,
                801.06,
                789.16,
                777.22,
                765.25,
                753.23,
                741.18,
                729.1,
                716.97,
                704.82,
                692.62,
                680.39,
                668.12,
                655.81,
                643.46,
                631.46,
                623.89,
                616.33,
                608.79,
                601.26,
                593.74,
                586.24,
                578.75,
                571.27,
                563.81,
                556.36,
                548.93,
                541.51,
                534.1,
                526.71,
                519.33,
                511.96,
                504.61,
                497.27,
                489.95,
                485.83,
                483.63,
                481.44,
                479.25,
                477.07,
                474.89,
                472.72,
                470.55,
                468.39,
                466.24,
                464.09,
                461.95,
                459.81,
                457.68,
                455.55,
                453.43,
                451.32,
                449.21,
                447.1,
                445.65,
                447.45,
                449.3,
                451.18,
                453.11,
                455.09,
                457.1,
                459.16,
                461.26,
                463.41,
                465.59,
                467.82,
                470.1,
                472.41,
                474.77,
                477.17,
                479.62,
                482.11,
                484.64,
                487.21,
                486.67,
                484.79,
                482.88,
                480.95,
                479.0,
                477.03,
                475.03,
                473.01,
                470.97,
                468.9,
                466.81,
                464.7,
                462.56,
                460.4,
                458.22,
                456.01,
                453.78,
                451.53,
                449.26,
                447.86,
                449.24,
                450.52,
                451.71,
                452.8,
                453.79,
                454.68,
                455.47,
                456.16,
                456.76,
                457.25,
                457.65,
                457.95,
                458.15,
                458.24,
                458.24,
                458.14,
                457.94,
                457.65,
                457.25,
                451.76,
                444.78,
                437.79,
                430.79,
                423.78,
                416.77,
                409.75,
                402.73,
                395.7,
                388.66,
                381.62,
                374.57,
                367.51,
                360.44,
                353.37,
                346.29,
                339.21,
                332.12,
                325.02,
                318.87,
                314.99,
                311.16,
                307.37,
                303.62,
                299.91,
                296.24,
                292.62,
                289.03,
                285.49,
                281.99,
                278.54,
                275.12,
                271.75,
                268.42,
                265.13,
                261.88,
                258.68,
                255.52,
                252.4,
                252.33,
                252.91,
                253.5,
                254.09,
                254.68,
                255.28,
                255.87,
                256.47,
                257.07,
                257.67,
                258.27,
                258.88,
                259.48,
                260.09,
                260.7,
                261.31,
                261.93,
                262.54,
                263.16,
                263.82,
                264.56,
                265.31,
                266.07,
                266.84,
                267.62,
                268.4,
                269.19,
                269.99,
                270.79,
                271.61,
                272.43,
                273.26,
                274.09,
                274.94,
                275.79,
                276.65,
                277.52,
                278.39,
                279.28,
                279.16,
                278.87,
                278.6,
                278.33,
                278.08,
                277.84,
                277.61,
                277.38,
                277.17,
                276.97,
                276.78,
                276.6,
                276.44,
                276.28,
                276.13,
                275.99,
                275.87,
                275.75,
                275.65,
                275.1,
                273.78,
                272.47,
                271.15,
                269.84,
                268.53,
                267.22,
                265.91,
                264.6,
                263.3,
                262.0,
                260.7,
                259.4,
                258.11,
                256.81,
                255.52,
                254.23,
                252.95,
                251.66,
                250.38,
                1522.2,
                1514.2,
                1506.17,
                1498.1,
                1490.0,
                1481.87,
                1473.7,
                1464.68,
                1455.13,
                1445.58,
                1436.02,
                1426.46,
                1416.88,
                1407.31,
                1397.72,
                1388.13,
                1378.53,
                1368.92,
                1359.31,
                1349.69,
                1340.06,
                1330.42,
                1320.78,
                1311.13,
                1301.48,
                1291.82,
                1281.64,
                1269.21,
                1256.92,
                1244.79,
                1232.79,
                1220.95,
                1209.24,
                1197.69,
                1186.28,
                1175.02,
                1163.91,
                1152.94,
                1142.12,
                1131.45,
                1120.92,
                1110.54,
                1100.31,
                1090.23,
                1080.3,
                1070.51,
                1061.29,
                1052.17,
                1042.98,
                1033.73,
                1024.4,
                1015.0,
                1005.53,
                995.98,
                986.37,
                976.68,
                966.93,
                957.1,
                947.19,
                937.22,
                927.18,
                917.06,
                906.87,
                896.61,
                886.27,
                875.32,
                862.93,
                850.51,
                838.05,
                825.55,
                813.01,
                800.44,
                787.83,
                775.19,
                762.51,
                749.79,
                737.04,
                724.25,
                711.42,
                698.55,
                685.65,
                672.71,
                659.74,
                646.73,
                633.68,
                625.36,
                618.05,
                610.75,
                603.47,
                596.2,
                588.94,
                581.7,
                574.47,
                567.25,
                560.05,
                552.86,
                545.69,
                538.53,
                531.38,
                524.25,
                517.13,
                510.02,
                502.93,
                495.85,
                490.62,
                488.52,
                486.42,
                484.33,
                482.25,
                480.17,
                478.09,
                476.02,
                473.96,
                471.9,
                469.85,
                467.8,
                465.76,
                463.72,
                461.69,
                459.67,
                457.65,
                455.63,
                453.62,
                451.62,
                453.76,
                456.35,
                458.98,
                461.65,
                464.37,
                467.13,
                469.93,
                472.77,
                475.66,
                478.59,
                481.56,
                484.58,
                487.64,
                490.74,
                493.89,
                497.08,
                500.31,
                503.58,
                506.9,
                507.76,
                505.47,
                503.16,
                500.83,
                498.48,
                496.1,
                493.7,
                491.27,
                488.83,
                486.36,
                483.87,
                481.35,
                478.81,
                476.25,
                473.66,
                471.05,
                468.42,
                465.77,
                463.09,
                460.39,
                460.03,
                459.64,
                459.14,
                458.55,
                457.86,
                457.08,
                456.19,
                455.21,
                454.12,
                452.94,
                451.66,
                450.28,
                448.8,
                447.22,
                445.55,
                443.77,
                441.89,
                439.92,
                437.84,
                433.21,
                426.12,
                419.02,
                411.92,
                404.81,
                397.7,
                390.57,
                383.44,
                376.31,
                369.17,
                362.02,
                354.86,
                347.7,
                340.53,
                333.35,
                326.17,
                318.98,
                311.78,
                304.58,
                297.48,
                294.3,
                291.17,
                288.08,
                285.03,
                282.02,
                279.05,
                276.13,
                273.25,
                270.41,
                267.61,
                264.85,
                262.14,
                259.46,
                256.83,
                254.25,
                251.7,
                249.2,
                246.74,
                244.32,
                243.57,
                244.19,
                244.81,
                245.43,
                246.06,
                246.68,
                247.31,
                247.94,
                248.57,
                249.2,
                249.84,
                250.47,
                251.11,
                251.75,
                252.39,
                253.03,
                253.68,
                254.33,
                254.98,
                255.64,
                256.51,
                257.39,
                258.28,
                259.17,
                260.07,
                260.98,
                261.9,
                262.82,
                263.76,
                264.7,
                265.65,
                266.6,
                267.57,
                268.54,
                269.52,
                270.5,
                271.5,
                272.5,
                273.51,
                273.87,
                273.76,
                273.66,
                273.56,
                273.48,
                273.4,
                273.34,
                273.29,
                273.25,
                273.22,
                273.2,
                273.19,
                273.19,
                273.19,
                273.19,
                273.19,
                273.19,
                273.19,
                273.18,
                273.07,
                271.77,
                270.47,
                269.17,
                267.89,
                266.61,
                265.34,
                264.07,
                262.82,
                261.56,
                260.32,
                259.08,
                257.85,
                256.62,
                255.4,
                254.19,
                252.99,
                251.79,
                250.6,
                249.41,
                1515.7,
                1507.08,
                1498.42,
                1489.72,
                1481.0,
                1472.24,
                1463.44,
                1454.3,
                1444.63,
                1434.95,
                1425.27,
                1415.57,
                1405.88,
                1396.17,
                1386.46,
                1376.74,
                1367.01,
                1357.28,
                1347.54,
                1337.79,
                1328.04,
                1318.28,
                1308.51,
                1298.74,
                1288.96,
                1279.17,
                1269.37,
                1259.68,
                1250.13,
                1240.74,
                1231.48,
                1222.37,
                1213.41,
                1204.6,
                1195.93,
                1187.41,
                1179.03,
                1170.8,
                1162.72,
                1154.79,
                1147.0,
                1139.36,
                1131.87,
                1124.53,
                1117.33,
                1110.28,
                1101.66,
                1091.22,
                1080.7,
                1070.11,
                1059.46,
                1048.73,
                1037.93,
                1027.05,
                1016.11,
                1005.09,
                994.01,
                982.85,
                971.62,
                960.32,
                948.94,
                937.5,
                925.98,
                914.39,
                902.73,
                890.95,
                877.89,
                864.8,
                851.67,
                838.5,
                825.3,
                812.06,
                798.78,
                785.47,
                772.12,
                758.73,
                745.31,
                731.85,
                718.36,
                704.82,
                691.25,
                677.65,
                664.0,
                650.32,
                636.61,
                626.71,
                619.65,
                612.59,
                605.56,
                598.53,
                591.52,
                584.52,
                577.54,
                570.57,
                563.61,
                556.67,
                549.74,
                542.83,
                535.92,
                529.04,
                522.16,
                515.3,
                508.46,
                501.63,
                495.37,
                493.36,
                491.36,
                489.36,
                487.37,
                485.39,
                483.41,
                481.44,
                479.47,
                477.51,
                475.55,
                473.6,
                471.65,
                469.71,
                467.78,
                465.85,
                463.93,
                462.01,
                460.1,
                458.19,
                459.68,
                463.01,
                466.38,
                469.8,
                473.26,
                476.76,
                480.31,
                483.9,
                487.53,
                491.2,
                494.92,
                498.68,
                502.48,
                506.33,
                510.21,
                514.15,
                518.12,
                522.14,
                526.2,
                529.06,
                526.37,
                523.66,
                520.92,
                518.16,
                515.38,
                512.58,
                509.75,
                506.9,
                504.03,
                501.13,
                498.22,
                495.27,
                492.31,
                489.32,
                486.31,
                483.27,
                480.21,
                477.13,
                474.03,
                471.71,
                469.64,
                467.47,
                465.2,
                462.84,
                460.37,
                457.81,
                455.15,
                452.39,
                449.53,
                446.57,
                443.52,
                440.36,
                437.11,
                433.75,
                430.3,
                426.75,
                423.09,
                419.34,
                414.71,
                407.52,
                400.32,
                393.11,
                385.9,
                378.68,
                371.45,
                364.21,
                356.97,
                349.73,
                342.47,
                335.21,
                327.94,
                320.67,
                313.39,
                306.1,
                298.81,
                291.51,
                284.2,
                276.88,
                273.23,
                270.8,
                268.4,
                266.06,
                263.75,
                261.48,
                259.26,
                257.08,
                254.94,
                252.84,
                250.78,
                248.77,
                246.79,
                244.87,
                242.98,
                241.13,
                239.33,
                237.57,
                235.85,
                234.8,
                235.45,
                236.1,
                236.76,
                237.41,
                238.07,
                238.73,
                239.39,
                240.05,
                240.72,
                241.38,
                242.05,
                242.72,
                243.39,
                244.06,
                244.74,
                245.42,
                246.09,
                246.87,
                247.77,
                248.91,
                250.11,
                251.3,
                252.48,
                253.66,
                254.83,
                255.99,
                257.14,
                258.29,
                259.43,
                260.56,
                261.68,
                262.8,
                263.91,
                265.01,
                266.11,
                267.2,
                268.28,
                269.35,
                270.11,
                270.1,
                270.1,
                270.09,
                270.09,
                270.08,
                270.07,
                270.06,
                270.05,
                270.03,
                270.02,
                270.0,
                269.98,
                269.97,
                269.94,
                269.92,
                269.9,
                269.87,
                269.85,
                269.82,
                268.86,
                267.67,
                266.48,
                265.3,
                264.13,
                262.97,
                261.81,
                260.65,
                259.51,
                258.37,
                257.24,
                256.11,
                254.99,
                253.88,
                252.77,
                251.67,
                250.58,
                249.5,
                248.42,
                1509.51,
                1500.25,
                1490.96,
                1481.64,
                1472.29,
                1462.9,
                1453.48,
                1443.98,
                1434.19,
                1424.38,
                1414.57,
                1404.75,
                1394.93,
                1385.1,
                1375.26,
                1365.41,
                1355.56,
                1345.7,
                1335.84,
                1325.96,
                1316.08,
                1306.2,
                1296.3,
                1286.4,
                1276.5,
                1266.58,
                1256.66,
                1248.82,
                1242.02,
                1235.36,
                1228.84,
                1222.47,
                1216.25,
                1210.17,
                1204.24,
                1198.46,
                1192.82,
                1187.33,
                1181.99,
                1176.79,
                1171.74,
                1166.84,
                1162.09,
                1157.48,
                1153.02,
                1148.71,
                1142.69,
                1130.91,
                1119.07,
                1107.15,
                1095.17,
                1083.11,
                1070.98,
                1058.78,
                1046.51,
                1034.16,
                1021.75,
                1009.26,
                996.7,
                984.07,
                971.37,
                958.59,
                945.75,
                932.83,
                919.84,
                906.78,
                893.18,
                879.42,
                865.62,
                851.78,
                837.91,
                824.01,
                810.06,
                796.08,
                782.06,
                768.01,
                753.92,
                739.79,
                725.63,
                711.43,
                697.19,
                682.92,
                668.61,
                654.26,
                639.87,
                627.94,
                621.12,
                614.31,
                607.52,
                600.74,
                593.97,
                587.22,
                580.48,
                573.76,
                567.05,
                560.35,
                553.67,
                547.0,
                540.34,
                533.7,
                527.07,
                520.46,
                513.86,
                507.27,
                500.7,
                498.15,
                496.25,
                494.35,
                492.45,
                490.56,
                488.68,
                486.8,
                484.93,
                483.07,
                481.21,
                479.35,
                477.5,
                475.66,
                473.82,
                471.98,
                470.16,
                468.34,
                466.52,
                464.71,
                465.21,
                469.29,
                473.4,
                477.56,
                481.77,
                486.01,
                490.3,
                494.63,
                499.01,
                503.42,
                507.88,
                512.38,
                516.93,
                521.52,
                526.15,
                530.82,
                535.54,
                540.3,
                545.11,
                549.96,
                547.48,
                544.36,
                541.23,
                538.06,
                534.88,
                531.67,
                528.44,
                525.19,
                521.92,
                518.62,
                515.29,
                511.95,
                508.58,
                505.19,
                501.77,
                498.34,
                494.88,
                491.39,
                487.88,
                484.29,
                480.54,
                476.7,
                472.75,
                468.71,
                464.57,
                460.33,
                455.99,
                451.55,
                447.02,
                442.39,
                437.65,
                432.82,
                427.89,
                422.86,
                417.73,
                412.5,
                407.17,
                401.75,
                396.22,
                388.98,
                381.67,
                374.36,
                367.04,
                359.71,
                352.38,
                345.04,
                337.7,
                330.34,
                322.98,
                315.62,
                308.25,
                300.87,
                293.48,
                286.09,
                278.69,
                271.29,
                263.87,
                256.45,
                251.77,
                250.04,
                248.35,
                246.7,
                245.38,
                244.33,
                243.28,
                242.24,
                241.21,
                240.19,
                239.18,
                238.17,
                237.17,
                236.18,
                235.19,
                234.22,
                233.25,
                232.29,
                231.34,
                230.39,
                231.3,
                232.2,
                233.11,
                234.01,
                234.9,
                235.79,
                236.67,
                237.55,
                238.43,
                239.3,
                240.16,
                241.03,
                241.88,
                242.74,
                243.58,
                244.43,
                245.27,
                246.1,
                246.93,
                247.89,
                248.97,
                250.04,
                251.11,
                252.17,
                253.22,
                254.26,
                255.3,
                256.33,
                257.35,
                258.37,
                259.37,
                260.38,
                261.37,
                262.36,
                263.33,
                264.31,
                265.27,
                266.23,
                267.17,
                267.14,
                267.12,
                267.09,
                267.06,
                267.03,
                267.0,
                266.97,
                266.94,
                266.9,
                266.87,
                266.83,
                266.79,
                266.75,
                266.71,
                266.66,
                266.62,
                266.57,
                266.53,
                266.48,
                265.9,
                264.81,
                263.73,
                262.66,
                261.59,
                260.53,
                259.48,
                258.43,
                257.39,
                256.36,
                255.33,
                254.31,
                253.3,
                252.29,
                251.29,
                250.3,
                249.31,
                248.33,
                247.36,
                1503.6,
                1493.72,
                1483.81,
                1473.86,
                1463.88,
                1453.87,
                1443.82,
                1433.74,
                1423.8,
                1413.87,
                1403.94,
                1393.99,
                1384.04,
                1374.08,
                1364.12,
                1354.15,
                1344.17,
                1334.19,
                1324.19,
                1314.19,
                1304.19,
                1294.18,
                1284.16,
                1274.13,
                1264.1,
                1254.05,
                1244.01,
                1236.64,
                1232.58,
                1228.65,
                1224.88,
                1221.24,
                1217.76,
                1214.42,
                1211.23,
                1208.18,
                1205.28,
                1202.53,
                1199.92,
                1197.46,
                1195.15,
                1192.98,
                1190.96,
                1189.09,
                1187.37,
                1185.8,
                1184.36,
                1171.26,
                1158.09,
                1144.85,
                1131.53,
                1118.14,
                1104.69,
                1091.16,
                1077.56,
                1063.89,
                1050.14,
                1036.33,
                1022.44,
                1008.48,
                994.45,
                980.35,
                966.18,
                951.93,
                937.61,
                923.22,
                908.8,
                894.37,
                879.9,
                865.4,
                850.86,
                836.29,
                821.68,
                807.03,
                792.34,
                777.62,
                762.87,
                748.07,
                733.24,
                718.37,
                703.47,
                688.53,
                673.55,
                658.53,
                643.48,
                629.04,
                622.46,
                615.9,
                609.36,
                602.82,
                596.3,
                589.8,
                583.3,
                576.83,
                570.36,
                563.91,
                557.47,
                551.05,
                544.64,
                538.24,
                531.86,
                525.49,
                519.14,
                512.8,
                506.47,
                502.89,
                501.08,
                499.28,
                497.48,
                495.69,
                493.9,
                492.12,
                490.35,
                488.58,
                486.81,
                485.05,
                483.3,
                481.55,
                479.81,
                478.07,
                476.34,
                474.61,
                472.89,
                471.18,
                470.36,
                475.18,
                480.04,
                484.94,
                489.89,
                494.87,
                499.91,
                504.98,
                510.1,
                515.26,
                520.46,
                525.7,
                530.99,
                536.32,
                541.7,
                547.11,
                552.57,
                558.08,
                563.62,
                569.21,
                568.8,
                565.28,
                561.74,
                558.18,
                554.59,
                550.98,
                547.35,
                543.69,
                540.01,
                536.31,
                532.59,
                528.84,
                525.07,
                521.27,
                517.46,
                513.62,
                509.75,
                505.86,
                501.95,
                497.76,
                492.34,
                486.81,
                481.19,
                475.47,
                469.66,
                463.74,
                457.73,
                451.62,
                445.33,
                438.89,
                432.48,
                426.08,
                419.71,
                413.35,
                407.01,
                400.69,
                394.39,
                388.11,
                381.85,
                375.16,
                368.29,
                361.43,
                354.59,
                347.77,
                340.95,
                334.16,
                327.37,
                320.6,
                313.85,
                307.11,
                300.38,
                293.67,
                286.98,
                280.3,
                273.63,
                266.98,
                260.34,
                253.72,
                248.31,
                247.36,
                246.41,
                245.47,
                244.53,
                243.61,
                242.69,
                241.78,
                240.88,
                239.98,
                239.09,
                238.21,
                237.34,
                236.48,
                235.62,
                234.77,
                233.93,
                233.1,
                232.27,
                231.45,
                231.83,
                232.67,
                233.49,
                234.32,
                235.14,
                235.95,
                236.76,
                237.57,
                238.37,
                239.17,
                239.96,
                240.74,
                241.53,
                242.31,
                243.08,
                243.85,
                244.61,
                245.37,
                246.13,
                246.93,
                247.89,
                248.85,
                249.8,
                250.74,
                251.68,
                252.6,
                253.52,
                254.44,
                255.34,
                256.24,
                257.13,
                258.02,
                258.89,
                259.76,
                260.63,
                261.48,
                262.33,
                263.17,
                264.0,
                264.19,
                264.15,
                264.1,
                264.05,
                264.0,
                263.95,
                263.89,
                263.84,
                263.78,
                263.73,
                263.67,
                263.61,
                263.55,
                263.48,
                263.42,
                263.35,
                263.29,
                263.22,
                263.15,
                262.87,
                261.89,
                260.92,
                259.95,
                258.99,
                258.03,
                257.09,
                256.15,
                255.21,
                254.29,
                253.36,
                252.45,
                251.54,
                250.64,
                249.75,
                248.86,
                247.98,
                247.11,
                246.24,
                1498.0,
                1487.49,
                1476.95,
                1466.38,
                1455.77,
                1445.13,
                1434.46,
                1423.75,
                1413.48,
                1403.42,
                1393.36,
                1383.29,
                1373.22,
                1363.13,
                1353.04,
                1342.94,
                1332.84,
                1322.73,
                1312.61,
                1302.49,
                1292.35,
                1282.21,
                1272.07,
                1261.92,
                1251.76,
                1241.59,
                1231.42,
                1223.14,
                1221.81,
                1220.62,
                1219.58,
                1218.69,
                1217.94,
                1217.33,
                1216.88,
                1216.57,
                1216.4,
                1216.38,
                1216.51,
                1216.79,
                1217.21,
                1217.78,
                1218.5,
                1219.37,
                1220.38,
                1221.54,
                1222.85,
                1212.26,
                1197.76,
                1183.19,
                1168.55,
                1153.83,
                1139.05,
                1124.19,
                1109.27,
                1094.27,
                1079.2,
                1064.05,
                1048.84,
                1033.55,
                1018.2,
                1002.77,
                987.26,
                971.69,
                956.05,
                940.33,
                924.75,
                909.66,
                894.52,
                879.35,
                864.15,
                848.91,
                833.63,
                818.31,
                802.96,
                787.57,
                772.14,
                756.68,
                741.18,
                725.65,
                710.08,
                694.47,
                678.82,
                663.14,
                647.42,
                631.66,
                623.69,
                617.37,
                611.07,
                604.78,
                598.51,
                592.25,
                586.0,
                579.77,
                573.55,
                567.34,
                561.15,
                554.97,
                548.81,
                542.66,
                536.52,
                530.4,
                524.29,
                518.19,
                512.11,
                507.59,
                505.87,
                504.17,
                502.46,
                500.77,
                499.08,
                497.39,
                495.71,
                494.04,
                492.37,
                490.7,
                489.05,
                487.39,
                485.75,
                484.11,
                482.47,
                480.84,
                479.22,
                477.6,
                475.98,
                480.68,
                486.29,
                491.93,
                497.62,
                503.35,
                509.12,
                514.94,
                520.8,
                526.7,
                532.65,
                538.63,
                544.66,
                550.73,
                556.04,
                561.29,
                566.48,
                571.62,
                576.69,
                581.71,
                582.74,
                577.88,
                573.03,
                568.2,
                563.38,
                558.57,
                553.78,
                549.0,
                544.23,
                539.48,
                534.74,
                530.01,
                525.29,
                520.59,
                515.91,
                511.23,
                506.57,
                501.93,
                497.29,
                492.67,
                486.53,
                480.27,
                474.03,
                467.81,
                461.61,
                455.43,
                449.27,
                443.13,
                437.0,
                430.9,
                424.81,
                418.75,
                412.7,
                406.67,
                400.67,
                394.68,
                388.71,
                382.76,
                376.83,
                370.6,
                363.98,
                357.36,
                350.77,
                344.19,
                337.62,
                331.07,
                324.53,
                318.0,
                311.5,
                305.0,
                298.52,
                292.05,
                285.6,
                279.17,
                272.75,
                266.34,
                259.95,
                253.57,
                247.2,
                246.06,
                245.23,
                244.42,
                243.61,
                242.82,
                242.03,
                241.24,
                240.47,
                239.7,
                238.94,
                238.19,
                237.44,
                236.71,
                235.98,
                235.26,
                234.54,
                233.84,
                233.14,
                232.45,
                232.41,
                233.17,
                233.92,
                234.67,
                235.42,
                236.16,
                236.89,
                237.62,
                238.35,
                239.07,
                239.79,
                240.5,
                241.21,
                241.92,
                242.62,
                243.31,
                244.0,
                244.69,
                245.37,
                246.04,
                246.88,
                247.72,
                248.56,
                249.38,
                250.2,
                251.01,
                251.81,
                252.61,
                253.4,
                254.18,
                254.96,
                255.72,
                256.48,
                257.24,
                257.98,
                258.72,
                259.45,
                260.18,
                260.89,
                261.25,
                261.19,
                261.12,
                261.05,
                260.98,
                260.9,
                260.83,
                260.75,
                260.68,
                260.6,
                260.52,
                260.44,
                260.35,
                260.27,
                260.18,
                260.1,
                260.01,
                259.92,
                259.83,
                259.74,
                258.91,
                258.04,
                257.18,
                256.32,
                255.48,
                254.64,
                253.8,
                252.97,
                252.15,
                251.34,
                250.53,
                249.73,
                248.93,
                248.15,
                247.36,
                246.59,
                245.82,
                245.06,
                1492.7,
                1481.56,
                1470.39,
                1459.19,
                1447.96,
                1436.69,
                1425.39,
                1414.06,
                1403.22,
                1393.04,
                1382.85,
                1372.65,
                1362.45,
                1352.24,
                1342.02,
                1331.8,
                1321.57,
                1311.33,
                1301.09,
                1290.84,
                1280.58,
                1270.31,
                1260.04,
                1249.76,
                1239.48,
                1229.18,
                1218.89,
                1208.58,
                1209.71,
                1211.26,
                1212.96,
                1214.8,
                1216.79,
                1218.92,
                1221.2,
                1223.62,
                1226.19,
                1228.91,
                1231.77,
                1234.79,
                1237.94,
                1241.25,
                1244.7,
                1248.3,
                1252.05,
                1255.95,
                1259.99,
                1253.92,
                1238.09,
                1222.19,
                1206.22,
                1190.18,
                1174.07,
                1157.88,
                1141.63,
                1125.3,
                1108.9,
                1092.43,
                1075.89,
                1059.28,
                1042.6,
                1025.84,
                1009.01,
                992.11,
                975.14,
                958.09,
                941.04,
                925.28,
                909.48,
                893.64,
                877.77,
                861.86,
                845.91,
                829.93,
                813.91,
                797.85,
                781.76,
                765.63,
                749.46,
                733.26,
                717.02,
                700.75,
                684.43,
                668.08,
                651.7,
                635.27,
                624.78,
                618.71,
                612.66,
                606.62,
                600.59,
                594.57,
                588.57,
                582.58,
                576.61,
                570.65,
                564.7,
                558.77,
                552.85,
                546.95,
                541.06,
                535.18,
                529.37,
                523.65,
                517.97,
                512.75,
                511.22,
                509.67,
                508.1,
                506.51,
                504.9,
                503.27,
                501.61,
                499.93,
                498.24,
                496.52,
                494.77,
                493.01,
                491.23,
                489.42,
                487.59,
                485.74,
                483.87,
                481.98,
                480.07,
                482.47,
                487.47,
                492.41,
                497.29,
                502.12,
                506.89,
                511.6,
                516.25,
                520.85,
                525.39,
                529.87,
                534.29,
                538.66,
                542.97,
                547.22,
                551.41,
                555.55,
                559.63,
                563.65,
                566.41,
                561.78,
                557.17,
                552.57,
                547.99,
                543.41,
                538.85,
                534.31,
                529.77,
                525.25,
                520.75,
                516.26,
                511.78,
                507.31,
                502.86,
                498.42,
                493.99,
                489.58,
                485.18,
                480.79,
                475.39,
                469.46,
                463.55,
                457.66,
                451.79,
                445.94,
                440.11,
                434.3,
                428.5,
                422.73,
                416.97,
                411.24,
                405.52,
                399.82,
                394.14,
                388.48,
                382.85,
                377.23,
                371.62,
                365.91,
                359.53,
                353.16,
                346.81,
                340.48,
                334.15,
                327.85,
                321.55,
                315.27,
                309.01,
                302.76,
                296.52,
                290.3,
                284.1,
                277.91,
                271.73,
                265.57,
                259.42,
                253.29,
                247.17,
                244.69,
                243.99,
                243.31,
                242.63,
                241.96,
                241.29,
                240.64,
                239.99,
                239.35,
                238.72,
                238.09,
                237.47,
                236.86,
                236.26,
                235.67,
                235.08,
                234.5,
                233.93,
                233.37,
                233.03,
                233.71,
                234.39,
                235.07,
                235.74,
                236.4,
                237.07,
                237.72,
                238.37,
                239.02,
                239.67,
                240.3,
                240.94,
                241.57,
                242.19,
                242.81,
                243.43,
                244.04,
                244.65,
                245.25,
                245.94,
                246.66,
                247.38,
                248.09,
                248.79,
                249.48,
                250.17,
                250.85,
                251.52,
                252.19,
                252.85,
                253.5,
                254.14,
                254.78,
                255.41,
                256.03,
                256.64,
                257.25,
                257.85,
                258.33,
                258.24,
                258.15,
                258.06,
                257.97,
                257.87,
                257.78,
                257.68,
                257.58,
                257.48,
                257.38,
                257.28,
                257.18,
                257.07,
                256.96,
                256.86,
                256.75,
                256.64,
                256.52,
                256.41,
                255.87,
                255.1,
                254.35,
                253.6,
                252.86,
                252.12,
                251.39,
                250.67,
                249.96,
                249.25,
                248.55,
                247.85,
                247.16,
                246.48,
                245.81,
                245.14,
                244.47,
                243.82,
                1487.69,
                1475.93,
                1464.13,
                1452.31,
                1440.45,
                1428.55,
                1416.63,
                1404.67,
                1393.02,
                1382.71,
                1372.39,
                1362.07,
                1351.74,
                1341.41,
                1331.06,
                1320.72,
                1310.36,
                1300.0,
                1289.63,
                1279.25,
                1268.87,
                1258.47,
                1248.08,
                1237.67,
                1227.26,
                1216.84,
                1206.42,
                1195.98,
                1196.3,
                1200.58,
                1205.01,
                1209.59,
                1214.31,
                1219.17,
                1224.19,
                1229.35,
                1234.65,
                1240.1,
                1245.7,
                1251.45,
                1257.34,
                1263.38,
                1269.56,
                1275.9,
                1282.38,
                1289.01,
                1295.79,
                1296.22,
                1279.07,
                1261.84,
                1244.54,
                1227.18,
                1209.74,
                1192.23,
                1174.64,
                1156.99,
                1139.27,
                1121.47,
                1103.6,
                1085.66,
                1067.65,
                1049.57,
                1031.42,
                1013.19,
                994.89,
                976.52,
                957.16,
                939.3,
                921.93,
                904.61,
                887.36,
                870.17,
                853.04,
                835.98,
                818.97,
                802.03,
                785.15,
                768.34,
                751.58,
                734.89,
                718.27,
                701.7,
                685.2,
                668.76,
                652.38,
                636.07,
                623.24,
                617.6,
                612.0,
                606.45,
                600.94,
                595.47,
                590.04,
                584.65,
                579.3,
                574.0,
                568.74,
                563.52,
                558.35,
                553.21,
                548.12,
                543.07,
                538.07,
                533.1,
                528.18,
                523.3,
                520.94,
                519.01,
                517.06,
                515.08,
                513.09,
                511.07,
                509.04,
                506.98,
                504.9,
                502.79,
                500.67,
                498.53,
                496.36,
                494.17,
                491.96,
                489.73,
                487.48,
                485.2,
                482.91,
                482.91,
                486.91,
                490.85,
                494.73,
                498.56,
                502.33,
                506.04,
                509.69,
                513.29,
                516.83,
                520.32,
                523.74,
                527.11,
                530.42,
                533.67,
                536.87,
                540.01,
                543.08,
                546.11,
                549.07,
                545.57,
                541.19,
                536.82,
                532.47,
                528.13,
                523.81,
                519.49,
                515.2,
                510.91,
                506.64,
                502.38,
                498.13,
                493.9,
                489.68,
                485.48,
                481.28,
                477.11,
                472.94,
                468.79,
                464.07,
                458.48,
                452.9,
                447.34,
                441.8,
                436.28,
                430.77,
                425.29,
                419.83,
                414.38,
                408.95,
                403.55,
                398.16,
                392.79,
                387.44,
                382.12,
                376.81,
                371.51,
                366.24,
                360.99,
                354.95,
                348.83,
                342.72,
                336.63,
                330.55,
                324.49,
                318.44,
                312.41,
                306.39,
                300.38,
                294.39,
                288.42,
                282.46,
                276.51,
                270.58,
                264.66,
                258.76,
                252.87,
                247.0,
                243.25,
                242.68,
                242.12,
                241.57,
                241.03,
                240.49,
                239.96,
                239.44,
                238.93,
                238.42,
                237.93,
                237.44,
                236.95,
                236.48,
                236.01,
                235.55,
                235.1,
                234.66,
                234.22,
                233.79,
                234.3,
                234.9,
                235.5,
                236.1,
                236.69,
                237.28,
                237.86,
                238.44,
                239.01,
                239.58,
                240.15,
                240.71,
                241.26,
                241.81,
                242.36,
                242.9,
                243.44,
                243.97,
                244.5,
                245.06,
                245.67,
                246.27,
                246.86,
                247.44,
                248.02,
                248.59,
                249.15,
                249.71,
                250.26,
                250.8,
                251.34,
                251.86,
                252.38,
                252.9,
                253.4,
                253.9,
                254.39,
                254.87,
                255.35,
                255.31,
                255.19,
                255.08,
                254.97,
                254.85,
                254.74,
                254.62,
                254.5,
                254.38,
                254.26,
                254.13,
                254.01,
                253.88,
                253.75,
                253.62,
                253.49,
                253.36,
                253.23,
                253.09,
                252.76,
                252.11,
                251.46,
                250.82,
                250.18,
                249.55,
                248.93,
                248.31,
                247.7,
                247.1,
                246.5,
                245.91,
                245.33,
                244.75,
                244.18,
                243.62,
                243.07,
                242.52,
                1482.98,
                1470.59,
                1458.17,
                1445.72,
                1433.23,
                1420.71,
                1408.16,
                1395.57,
                1382.96,
                1372.44,
                1362.0,
                1351.55,
                1341.1,
                1330.64,
                1320.17,
                1309.69,
                1299.21,
                1288.72,
                1278.23,
                1267.72,
                1257.21,
                1246.7,
                1236.17,
                1225.64,
                1215.1,
                1204.56,
                1194.01,
                1183.45,
                1181.74,
                1189.65,
                1197.43,
                1205.08,
                1212.6,
                1219.99,
                1227.25,
                1234.39,
                1241.39,
                1248.26,
                1255.01,
                1261.62,
                1268.11,
                1274.46,
                1280.68,
                1286.78,
                1292.74,
                1298.57,
                1304.27,
                1309.2,
                1289.66,
                1270.1,
                1250.52,
                1230.92,
                1211.3,
                1191.66,
                1172.0,
                1152.32,
                1132.61,
                1112.89,
                1093.15,
                1073.38,
                1053.6,
                1033.8,
                1013.97,
                994.12,
                974.26,
                954.37,
                934.46,
                916.55,
                900.31,
                884.14,
                868.02,
                851.97,
                835.99,
                820.06,
                804.2,
                788.4,
                772.66,
                756.98,
                741.37,
                725.82,
                710.33,
                694.91,
                679.54,
                664.24,
                649.01,
                633.83,
                619.47,
                614.59,
                609.74,
                604.94,
                600.19,
                595.47,
                590.8,
                586.17,
                581.58,
                577.03,
                572.52,
                568.06,
                563.64,
                559.26,
                554.92,
                550.63,
                546.38,
                542.17,
                538.0,
                533.88,
                530.85,
                528.54,
                526.2,
                523.85,
                521.47,
                519.08,
                516.66,
                514.21,
                511.75,
                509.27,
                506.76,
                504.24,
                501.69,
                499.12,
                496.53,
                493.91,
                491.28,
                488.62,
                485.94,
                483.86,
                486.86,
                489.8,
                492.69,
                495.51,
                498.29,
                501.0,
                503.66,
                506.26,
                508.8,
                511.28,
                513.71,
                516.08,
                518.39,
                520.65,
                522.85,
                524.98,
                527.07,
                529.09,
                531.06,
                529.22,
                525.08,
                520.95,
                516.83,
                512.73,
                508.63,
                504.56,
                500.49,
                496.44,
                492.4,
                488.38,
                484.37,
                480.37,
                476.38,
                472.41,
                468.45,
                464.51,
                460.58,
                456.66,
                452.58,
                447.31,
                442.07,
                436.84,
                431.62,
                426.43,
                421.26,
                416.11,
                410.97,
                405.86,
                400.76,
                395.68,
                390.63,
                385.59,
                380.57,
                375.57,
                370.59,
                365.63,
                360.69,
                355.76,
                350.24,
                344.36,
                338.5,
                332.65,
                326.82,
                321.0,
                315.2,
                309.41,
                303.64,
                297.88,
                292.13,
                286.4,
                280.68,
                274.98,
                269.29,
                263.62,
                257.96,
                252.32,
                246.69,
                241.74,
                241.3,
                240.87,
                240.44,
                240.03,
                239.62,
                239.22,
                238.82,
                238.44,
                238.06,
                237.69,
                237.33,
                236.97,
                236.62,
                236.28,
                235.95,
                235.63,
                235.31,
                235.0,
                234.7,
                234.92,
                235.45,
                235.98,
                236.5,
                237.02,
                237.53,
                238.04,
                238.55,
                239.05,
                239.54,
                240.03,
                240.52,
                241.0,
                241.48,
                241.95,
                242.41,
                242.88,
                243.34,
                243.79,
                244.25,
                244.74,
                245.22,
                245.69,
                246.16,
                246.62,
                247.08,
                247.53,
                247.97,
                248.4,
                248.82,
                249.24,
                249.65,
                250.05,
                250.45,
                250.84,
                251.22,
                251.6,
                251.96,
                252.32,
                252.38,
                252.25,
                252.12,
                251.98,
                251.85,
                251.71,
                251.57,
                251.43,
                251.29,
                251.14,
                251.0,
                250.85,
                250.71,
                250.56,
                250.41,
                250.25,
                250.1,
                249.95,
                249.79,
                249.6,
                249.05,
                248.51,
                247.97,
                247.44,
                246.91,
                246.4,
                245.89,
                245.38,
                244.89,
                244.4,
                243.91,
                243.44,
                242.97,
                242.5,
                242.04,
                241.59,
                241.15,
                1495.09,
                1483.38,
                1471.6,
                1459.74,
                1447.8,
                1435.78,
                1423.69,
                1411.52,
                1399.28,
                1389.04,
                1379.55,
                1370.03,
                1360.47,
                1350.87,
                1341.24,
                1331.58,
                1321.87,
                1312.14,
                1302.36,
                1292.56,
                1282.71,
                1272.83,
                1262.91,
                1252.96,
                1242.97,
                1232.95,
                1222.89,
                1212.79,
                1206.68,
                1212.16,
                1217.51,
                1222.73,
                1227.83,
                1232.79,
                1237.63,
                1242.33,
                1246.91,
                1251.36,
                1255.68,
                1259.87,
                1263.92,
                1267.85,
                1271.65,
                1275.32,
                1278.85,
                1282.26,
                1285.53,
                1288.68,
                1273.78,
                1253.85,
                1233.9,
                1213.93,
                1193.94,
                1173.93,
                1153.9,
                1133.85,
                1113.78,
                1093.69,
                1073.58,
                1053.45,
                1033.29,
                1013.12,
                992.93,
                972.71,
                952.48,
                932.22,
                911.95,
                893.22,
                878.12,
                863.09,
                848.12,
                833.21,
                818.36,
                803.58,
                788.85,
                774.19,
                759.59,
                745.06,
                730.58,
                716.17,
                701.82,
                687.54,
                673.31,
                659.15,
                645.06,
                631.02,
                617.05,
                611.19,
                607.1,
                603.06,
                599.06,
                595.09,
                591.18,
                587.3,
                583.46,
                579.67,
                575.92,
                572.21,
                568.55,
                564.92,
                561.34,
                557.8,
                554.31,
                550.85,
                547.44,
                544.07,
                540.96,
                538.26,
                535.55,
                532.81,
                530.05,
                527.27,
                524.47,
                521.65,
                518.81,
                515.94,
                513.05,
                510.14,
                507.21,
                504.26,
                501.29,
                498.3,
                495.28,
                492.24,
                489.18,
                486.1,
                487.33,
                489.27,
                491.16,
                492.99,
                494.77,
                496.48,
                498.14,
                499.74,
                501.29,
                502.77,
                504.2,
                505.57,
                506.89,
                508.15,
                509.35,
                510.49,
                511.57,
                512.6,
                513.57,
                512.76,
                508.85,
                504.95,
                501.07,
                497.2,
                493.34,
                489.5,
                485.66,
                481.85,
                478.04,
                474.25,
                470.48,
                466.71,
                462.96,
                459.22,
                455.5,
                451.79,
                448.09,
                444.41,
                440.74,
                435.98,
                431.06,
                426.16,
                421.28,
                416.41,
                411.57,
                406.75,
                401.94,
                397.15,
                392.39,
                387.64,
                382.91,
                378.2,
                373.51,
                368.84,
                364.19,
                359.56,
                354.95,
                350.36,
                345.4,
                339.76,
                334.15,
                328.55,
                322.96,
                317.38,
                311.83,
                306.28,
                300.75,
                295.24,
                289.73,
                284.25,
                278.78,
                273.32,
                267.88,
                262.45,
                257.03,
                251.64,
                246.25,
                240.88,
                239.85,
                239.54,
                239.25,
                238.96,
                238.68,
                238.4,
                238.14,
                237.88,
                237.63,
                237.38,
                237.15,
                236.92,
                236.7,
                236.49,
                236.28,
                236.08,
                235.89,
                235.71,
                235.54,
                235.59,
                236.05,
                236.5,
                236.95,
                237.39,
                237.83,
                238.26,
                238.69,
                239.12,
                239.54,
                239.96,
                240.37,
                240.78,
                241.18,
                241.58,
                241.97,
                242.36,
                242.74,
                243.12,
                243.5,
                243.87,
                244.24,
                244.6,
                244.95,
                245.29,
                245.63,
                245.96,
                246.28,
                246.6,
                246.91,
                247.21,
                247.51,
                247.79,
                248.07,
                248.34,
                248.61,
                248.87,
                249.12,
                249.36,
                249.47,
                249.32,
                249.16,
                249.01,
                248.85,
                248.69,
                248.53,
                248.37,
                248.21,
                248.04,
                247.88,
                247.71,
                247.54,
                247.37,
                247.2,
                247.03,
                246.85,
                246.68,
                246.5,
                246.32,
                245.93,
                245.49,
                245.06,
                244.64,
                244.22,
                243.81,
                243.4,
                243.01,
                242.61,
                242.23,
                241.85,
                241.48,
                241.12,
                240.76,
                240.41,
                240.06,
                239.72,
                1551.88,
                1538.67,
                1525.39,
                1512.03,
                1498.6,
                1485.08,
                1471.49,
                1457.82,
                1444.08,
                1432.06,
                1421.88,
                1411.67,
                1401.42,
                1391.13,
                1380.81,
                1370.46,
                1360.07,
                1349.64,
                1339.17,
                1328.68,
                1318.14,
                1307.57,
                1296.96,
                1286.32,
                1275.64,
                1264.93,
                1254.18,
                1243.39,
                1232.79,
                1235.84,
                1238.77,
                1241.57,
                1244.23,
                1246.77,
                1249.18,
                1251.46,
                1253.62,
                1255.64,
                1257.53,
                1259.29,
                1260.93,
                1262.43,
                1263.8,
                1265.04,
                1266.15,
                1267.13,
                1267.98,
                1268.7,
                1258.08,
                1237.78,
                1217.47,
                1197.13,
                1176.77,
                1156.39,
                1135.99,
                1115.57,
                1095.13,
                1074.67,
                1054.19,
                1033.69,
                1013.17,
                992.63,
                972.07,
                951.49,
                930.88,
                910.26,
                889.61,
                869.33,
                855.37,
                841.48,
                827.65,
                813.88,
                800.17,
                786.52,
                772.94,
                759.42,
                745.96,
                732.56,
                719.23,
                705.95,
                692.74,
                679.6,
                666.51,
                653.49,
                640.53,
                627.64,
                614.81,
                607.41,
                604.08,
                600.79,
                597.54,
                594.34,
                591.17,
                588.05,
                584.97,
                581.93,
                578.94,
                575.98,
                573.07,
                570.2,
                567.38,
                564.59,
                561.85,
                559.15,
                556.49,
                553.88,
                551.26,
                548.19,
                545.09,
                541.97,
                538.83,
                535.67,
                532.49,
                529.28,
                526.06,
                522.81,
                519.54,
                516.25,
                512.94,
                509.6,
                506.25,
                502.87,
                499.48,
                496.06,
                492.61,
                489.15,
                488.32,
                489.27,
                490.16,
                490.99,
                491.76,
                492.48,
                493.14,
                493.75,
                494.29,
                494.78,
                495.22,
                495.59,
                495.91,
                496.17,
                496.37,
                496.51,
                496.6,
                496.63,
                496.6,
                496.17,
                492.49,
                488.83,
                485.18,
                481.54,
                477.92,
                474.31,
                470.71,
                467.13,
                463.56,
                460.0,
                456.46,
                452.93,
                449.41,
                445.91,
                442.42,
                438.94,
                435.48,
                432.03,
                428.59,
                424.46,
                419.87,
                415.3,
                410.75,
                406.22,
                401.7,
                397.21,
                392.73,
                388.28,
                383.84,
                379.42,
                375.02,
                370.64,
                366.28,
                361.94,
                357.62,
                353.32,
                349.04,
                344.77,
                340.42,
                335.03,
                329.66,
                324.3,
                318.96,
                313.63,
                308.32,
                303.02,
                297.73,
                292.46,
                287.21,
                281.96,
                276.74,
                271.52,
                266.33,
                261.14,
                255.97,
                250.82,
                245.68,
                240.55,
                238.33,
                238.15,
                237.98,
                237.82,
                237.67,
                237.52,
                237.38,
                237.25,
                237.12,
                237.01,
                236.9,
                236.8,
                236.7,
                236.62,
                236.54,
                236.47,
                236.41,
                236.35,
                236.31,
                236.3,
                236.68,
                237.06,
                237.43,
                237.8,
                238.17,
                238.53,
                238.88,
                239.23,
                239.58,
                239.92,
                240.26,
                240.59,
                240.92,
                241.25,
                241.56,
                241.88,
                242.19,
                242.5,
                242.8,
                243.07,
                243.32,
                243.56,
                243.8,
                244.03,
                244.25,
                244.46,
                244.67,
                244.87,
                245.06,
                245.25,
                245.43,
                245.6,
                245.76,
                245.92,
                246.06,
                246.21,
                246.34,
                246.47,
                246.57,
                246.4,
                246.22,
                246.05,
                245.87,
                245.69,
                245.51,
                245.32,
                245.14,
                244.95,
                244.77,
                244.58,
                244.39,
                244.2,
                244.01,
                243.81,
                243.62,
                243.42,
                243.22,
                243.02,
                242.75,
                242.42,
                242.09,
                241.78,
                241.46,
                241.16,
                240.86,
                240.57,
                240.28,
                240.04,
                239.81,
                239.58,
                239.36,
                239.14,
                238.92,
                238.71,
                238.49,
                1609.38,
                1594.67,
                1579.89,
                1565.04,
                1550.1,
                1535.09,
                1520.0,
                1504.84,
                1489.59,
                1475.41,
                1464.54,
                1453.64,
                1442.7,
                1431.72,
                1420.71,
                1409.67,
                1398.59,
                1387.47,
                1376.32,
                1365.13,
                1353.9,
                1342.64,
                1331.35,
                1320.02,
                1308.65,
                1297.24,
                1285.8,
                1274.33,
                1262.82,
                1260.7,
                1261.2,
                1261.57,
                1261.82,
                1261.93,
                1261.92,
                1261.77,
                1261.5,
                1261.1,
                1260.56,
                1259.9,
                1259.11,
                1258.19,
                1257.14,
                1255.95,
                1254.64,
                1253.2,
                1251.62,
                1249.92,
                1242.56,
                1221.9,
                1201.21,
                1180.5,
                1159.78,
                1139.03,
                1118.26,
                1097.48,
                1076.67,
                1055.84,
                1034.99,
                1014.12,
                993.23,
                972.32,
                951.39,
                930.44,
                909.47,
                888.48,
                867.46,
                846.43,
                832.05,
                819.3,
                806.6,
                793.97,
                781.4,
                768.9,
                756.45,
                744.07,
                731.75,
                719.49,
                707.3,
                695.16,
                683.09,
                671.08,
                659.14,
                647.26,
                635.44,
                623.68,
                611.99,
                603.25,
                600.68,
                598.14,
                595.65,
                593.2,
                590.79,
                588.42,
                586.09,
                583.81,
                581.57,
                579.37,
                577.21,
                575.1,
                573.02,
                570.99,
                569.01,
                567.06,
                565.16,
                563.3,
                561.48,
                558.3,
                554.82,
                551.32,
                547.8,
                544.26,
                540.69,
                537.11,
                533.5,
                529.87,
                526.22,
                522.55,
                518.86,
                515.14,
                511.41,
                507.65,
                503.87,
                500.07,
                496.25,
                492.4,
                489.83,
                489.78,
                489.67,
                489.5,
                489.28,
                489.0,
                488.67,
                488.27,
                487.82,
                487.32,
                486.75,
                486.13,
                485.45,
                484.71,
                483.91,
                483.06,
                482.15,
                481.18,
                480.16,
                479.07,
                476.02,
                472.59,
                469.17,
                465.77,
                462.38,
                459.0,
                455.64,
                452.29,
                448.95,
                445.63,
                442.32,
                439.03,
                435.74,
                432.47,
                429.22,
                425.97,
                422.74,
                419.53,
                416.33,
                412.77,
                408.51,
                404.27,
                400.05,
                395.85,
                391.66,
                387.5,
                383.35,
                379.22,
                375.11,
                371.03,
                366.96,
                362.91,
                358.88,
                354.86,
                350.87,
                346.9,
                342.94,
                339.01,
                335.1,
                330.17,
                325.04,
                319.93,
                314.83,
                309.75,
                304.68,
                299.62,
                294.58,
                289.56,
                284.54,
                279.55,
                274.56,
                269.6,
                264.64,
                259.7,
                254.78,
                249.87,
                244.97,
                240.09,
                236.74,
                236.69,
                236.65,
                236.61,
                236.58,
                236.56,
                236.55,
                236.55,
                236.55,
                236.56,
                236.58,
                236.6,
                236.64,
                236.68,
                236.73,
                236.79,
                236.85,
                236.92,
                237.0,
                237.09,
                237.36,
                237.66,
                237.96,
                238.25,
                238.55,
                238.83,
                239.11,
                239.39,
                239.66,
                239.93,
                240.19,
                240.45,
                240.71,
                240.96,
                241.2,
                241.44,
                241.68,
                241.91,
                242.14,
                242.34,
                242.47,
                242.6,
                242.72,
                242.83,
                242.93,
                243.03,
                243.12,
                243.2,
                243.28,
                243.35,
                243.41,
                243.47,
                243.51,
                243.55,
                243.58,
                243.65,
                243.74,
                243.82,
                243.9,
                243.83,
                243.7,
                243.57,
                243.45,
                243.32,
                243.18,
                243.05,
                242.92,
                242.78,
                242.65,
                242.51,
                242.37,
                242.23,
                242.09,
                241.94,
                241.8,
                241.65,
                241.5,
                241.35,
                241.19,
                240.98,
                240.77,
                240.56,
                240.35,
                240.15,
                239.94,
                239.74,
                239.54,
                239.35,
                239.15,
                238.96,
                238.77,
                238.58,
                238.39,
                238.21,
                238.03,
                1667.59,
                1651.39,
                1635.11,
                1618.75,
                1602.32,
                1585.81,
                1569.23,
                1552.56,
                1535.82,
                1519.09,
                1507.53,
                1495.94,
                1484.31,
                1472.64,
                1460.94,
                1449.21,
                1437.44,
                1425.63,
                1413.79,
                1401.91,
                1390.0,
                1378.05,
                1366.06,
                1354.04,
                1341.98,
                1329.89,
                1317.76,
                1305.6,
                1293.4,
                1286.73,
                1284.81,
                1282.76,
                1280.58,
                1278.27,
                1275.83,
                1273.26,
                1270.56,
                1267.74,
                1264.78,
                1261.69,
                1258.48,
                1255.13,
                1251.66,
                1248.05,
                1244.32,
                1240.45,
                1236.45,
                1232.33,
                1227.22,
                1206.19,
                1185.14,
                1164.06,
                1142.97,
                1121.85,
                1100.72,
                1079.56,
                1058.38,
                1037.19,
                1015.97,
                994.73,
                973.48,
                952.2,
                930.9,
                909.58,
                888.24,
                866.88,
                845.5,
                824.09,
                808.16,
                796.55,
                784.99,
                773.5,
                762.07,
                750.7,
                739.4,
                728.15,
                716.97,
                705.85,
                694.8,
                683.8,
                672.87,
                662.0,
                651.19,
                640.45,
                629.77,
                619.15,
                608.59,
                598.71,
                596.89,
                595.11,
                593.37,
                591.67,
                590.02,
                588.4,
                586.83,
                585.3,
                583.82,
                582.37,
                580.97,
                579.61,
                578.29,
                577.01,
                575.78,
                574.59,
                573.44,
                572.33,
                571.27,
                568.62,
                564.75,
                560.87,
                556.97,
                553.05,
                549.1,
                545.13,
                541.14,
                537.13,
                533.1,
                529.05,
                524.97,
                520.88,
                516.76,
                512.62,
                508.46,
                504.28,
                500.08,
                495.85,
                491.85,
                490.8,
                489.7,
                488.54,
                487.32,
                486.04,
                484.71,
                483.32,
                481.87,
                480.37,
                478.81,
                477.19,
                475.51,
                473.77,
                471.98,
                470.13,
                468.23,
                466.26,
                464.24,
                462.16,
                459.42,
                456.22,
                453.04,
                449.87,
                446.71,
                443.57,
                440.44,
                437.33,
                434.23,
                431.14,
                428.06,
                425.0,
                421.95,
                418.91,
                415.89,
                412.88,
                409.89,
                406.9,
                403.93,
                400.91,
                396.98,
                393.07,
                389.17,
                385.3,
                381.44,
                377.61,
                373.79,
                369.99,
                366.21,
                362.45,
                358.71,
                354.99,
                351.29,
                347.61,
                343.95,
                340.3,
                336.68,
                333.07,
                329.49,
                325.18,
                320.29,
                315.42,
                310.57,
                305.73,
                300.91,
                296.09,
                291.3,
                286.52,
                281.75,
                277.0,
                272.26,
                267.53,
                262.82,
                258.13,
                253.45,
                248.78,
                244.13,
                239.5,
                235.08,
                235.16,
                235.24,
                235.33,
                235.43,
                235.54,
                235.65,
                235.78,
                235.91,
                236.04,
                236.19,
                236.34,
                236.5,
                236.67,
                236.85,
                237.03,
                237.22,
                237.42,
                237.63,
                237.84,
                238.07,
                238.3,
                238.51,
                238.69,
                238.88,
                239.07,
                239.25,
                239.44,
                239.62,
                239.81,
                240.0,
                240.18,
                240.37,
                240.55,
                240.74,
                240.93,
                241.11,
                241.3,
                241.48,
                241.67,
                241.75,
                241.83,
                241.92,
                242.0,
                242.08,
                242.17,
                242.25,
                242.33,
                242.42,
                242.5,
                242.58,
                242.67,
                242.75,
                242.83,
                242.92,
                243.0,
                243.08,
                243.17,
                243.25,
                243.22,
                243.08,
                242.93,
                242.78,
                242.63,
                242.47,
                242.32,
                242.16,
                242.01,
                241.85,
                241.69,
                241.53,
                241.37,
                241.21,
                241.04,
                240.87,
                240.71,
                240.54,
                240.37,
                240.2,
                240.02,
                239.84,
                239.66,
                239.49,
                239.31,
                239.14,
                238.97,
                238.81,
                238.64,
                238.48,
                238.32,
                238.16,
                238.0,
                237.85,
                237.69,
                237.54,
                1726.5,
                1708.81,
                1691.03,
                1673.18,
                1655.25,
                1637.25,
                1619.16,
                1601.0,
                1582.77,
                1564.45,
                1550.85,
                1538.56,
                1526.25,
                1513.89,
                1501.5,
                1489.08,
                1476.62,
                1464.12,
                1451.59,
                1439.02,
                1426.42,
                1413.78,
                1401.11,
                1388.4,
                1375.65,
                1362.87,
                1350.05,
                1337.2,
                1324.31,
                1313.94,
                1309.59,
                1305.11,
                1300.51,
                1295.78,
                1290.92,
                1285.92,
                1280.8,
                1275.56,
                1270.18,
                1264.67,
                1259.03,
                1253.26,
                1247.36,
                1241.34,
                1235.18,
                1228.89,
                1222.47,
                1215.92,
                1209.24,
                1190.66,
                1169.24,
                1147.8,
                1126.34,
                1104.85,
                1083.35,
                1061.83,
                1040.28,
                1018.72,
                997.13,
                975.53,
                953.9,
                932.25,
                910.59,
                888.9,
                867.19,
                845.46,
                823.71,
                801.94,
                783.71,
                773.23,
                762.82,
                752.46,
                742.17,
                731.94,
                721.77,
                711.67,
                701.62,
                691.64,
                681.72,
                671.87,
                662.07,
                652.34,
                642.67,
                633.07,
                623.52,
                614.04,
                604.63,
                595.27,
                592.72,
                591.7,
                590.71,
                589.77,
                588.87,
                588.01,
                587.19,
                586.42,
                585.68,
                584.99,
                584.34,
                583.73,
                583.17,
                582.65,
                582.17,
                581.73,
                581.33,
                580.98,
                580.67,
                579.12,
                574.88,
                570.62,
                566.33,
                562.03,
                557.7,
                553.35,
                548.98,
                544.59,
                540.18,
                535.74,
                531.29,
                526.81,
                522.31,
                517.79,
                513.25,
                508.69,
                504.1,
                499.5,
                494.87,
                492.35,
                490.25,
                488.09,
                485.87,
                483.6,
                481.27,
                478.88,
                476.44,
                473.94,
                471.38,
                468.77,
                466.09,
                463.36,
                460.57,
                457.73,
                454.82,
                451.86,
                448.84,
                445.77,
                442.69,
                439.73,
                436.78,
                433.85,
                430.93,
                428.02,
                425.12,
                422.24,
                419.37,
                416.52,
                413.67,
                410.85,
                408.03,
                405.23,
                402.44,
                399.66,
                396.9,
                394.15,
                391.42,
                388.7,
                385.27,
                381.69,
                378.12,
                374.58,
                371.05,
                367.54,
                364.05,
                360.59,
                357.14,
                353.71,
                350.29,
                346.9,
                343.53,
                340.18,
                336.84,
                333.53,
                330.23,
                326.95,
                323.7,
                320.05,
                315.41,
                310.79,
                306.18,
                301.58,
                297.0,
                292.43,
                287.88,
                283.54,
                279.2,
                274.84,
                270.46,
                266.07,
                261.66,
                257.23,
                252.79,
                248.34,
                243.87,
                239.38,
                234.88,
                233.99,
                234.18,
                234.37,
                234.56,
                234.74,
                234.93,
                235.11,
                235.3,
                235.48,
                235.66,
                235.84,
                236.03,
                236.21,
                236.39,
                236.57,
                236.74,
                236.92,
                237.1,
                237.28,
                237.45,
                237.64,
                237.83,
                238.01,
                238.2,
                238.38,
                238.57,
                238.76,
                238.94,
                239.13,
                239.31,
                239.5,
                239.69,
                239.87,
                240.06,
                240.24,
                240.43,
                240.62,
                240.8,
                240.99,
                241.1,
                241.18,
                241.26,
                241.35,
                241.43,
                241.51,
                241.6,
                241.68,
                241.76,
                241.85,
                241.93,
                242.01,
                242.1,
                242.18,
                242.26,
                242.35,
                242.43,
                242.51,
                242.6,
                242.63,
                242.46,
                242.29,
                242.12,
                241.95,
                241.78,
                241.6,
                241.42,
                241.25,
                241.07,
                240.89,
                240.71,
                240.52,
                240.34,
                240.15,
                239.97,
                239.78,
                239.59,
                239.4,
                239.2,
                239.04,
                238.89,
                238.75,
                238.6,
                238.46,
                238.32,
                238.19,
                238.05,
                237.92,
                237.79,
                237.66,
                237.53,
                237.41,
                237.28,
                237.16,
                237.04,
                1786.13,
                1766.94,
                1747.67,
                1728.32,
                1708.89,
                1689.39,
                1669.81,
                1650.15,
                1630.42,
                1610.61,
                1594.49,
                1581.52,
                1568.51,
                1555.47,
                1542.39,
                1529.28,
                1516.13,
                1502.94,
                1489.72,
                1476.47,
                1463.18,
                1449.85,
                1436.49,
                1423.09,
                1409.65,
                1396.18,
                1382.68,
                1369.13,
                1355.55,
                1342.32,
                1335.55,
                1328.65,
                1321.62,
                1314.46,
                1307.18,
                1299.77,
                1292.22,
                1284.55,
                1276.75,
                1268.82,
                1260.76,
                1252.57,
                1244.25,
                1235.8,
                1227.22,
                1218.51,
                1209.67,
                1200.7,
                1191.6,
                1175.32,
                1153.53,
                1131.72,
                1109.89,
                1088.04,
                1066.17,
                1044.27,
                1022.36,
                1000.43,
                978.48,
                956.5,
                934.51,
                912.49,
                890.46,
                868.4,
                846.33,
                824.23,
                802.11,
                779.97,
                758.69,
                749.35,
                740.07,
                730.86,
                721.7,
                712.61,
                703.58,
                694.61,
                685.71,
                676.86,
                668.08,
                659.36,
                650.71,
                642.11,
                633.58,
                625.11,
                616.71,
                608.36,
                600.08,
                591.87,
                588.18,
                587.9,
                587.67,
                587.48,
                587.34,
                587.23,
                587.17,
                587.14,
                587.16,
                587.23,
                587.33,
                587.48,
                587.67,
                587.9,
                588.17,
                588.49,
                588.84,
                589.24,
                589.69,
                589.83,
                585.2,
                580.56,
                575.89,
                571.21,
                566.5,
                561.77,
                557.02,
                552.24,
                547.45,
                542.63,
                537.8,
                532.94,
                528.06,
                523.16,
                518.24,
                513.29,
                508.33,
                503.34,
                498.33,
                494.41,
                491.31,
                488.16,
                484.95,
                481.68,
                478.35,
                474.97,
                471.53,
                468.03,
                464.48,
                460.87,
                457.2,
                453.47,
                449.69,
                445.84,
                441.94,
                437.99,
                433.97,
                429.9,
                425.85,
                423.12,
                420.4,
                417.7,
                415.02,
                412.34,
                409.68,
                407.03,
                404.4,
                401.77,
                399.17,
                396.57,
                394.14,
                392.49,
                390.82,
                389.12,
                387.38,
                385.62,
                383.84,
                382.02,
                379.73,
                377.04,
                374.32,
                371.57,
                368.8,
                365.99,
                363.16,
                360.29,
                357.4,
                354.48,
                351.54,
                348.56,
                345.55,
                342.52,
                339.46,
                336.37,
                333.25,
                330.1,
                326.93,
                323.69,
                319.21,
                314.71,
                310.19,
                305.66,
                301.11,
                296.55,
                291.97,
                287.38,
                282.77,
                278.15,
                273.51,
                268.85,
                264.18,
                259.49,
                254.79,
                250.07,
                245.33,
                240.58,
                235.81,
                233.52,
                233.7,
                233.87,
                234.05,
                234.22,
                234.4,
                234.57,
                234.75,
                234.92,
                235.09,
                235.26,
                235.43,
                235.6,
                235.77,
                235.94,
                236.11,
                236.28,
                236.44,
                236.61,
                236.77,
                236.96,
                237.15,
                237.33,
                237.52,
                237.7,
                237.89,
                238.08,
                238.26,
                238.45,
                238.63,
                238.82,
                239.0,
                239.19,
                239.38,
                239.56,
                239.75,
                239.93,
                240.12,
                240.31,
                240.44,
                240.53,
                240.61,
                240.69,
                240.78,
                240.86,
                240.94,
                241.03,
                241.11,
                241.19,
                241.28,
                241.36,
                241.44,
                241.53,
                241.61,
                241.69,
                241.78,
                241.86,
                241.94,
                242.03,
                241.86,
                241.67,
                241.48,
                241.28,
                241.09,
                240.89,
                240.69,
                240.5,
                240.3,
                240.09,
                239.89,
                239.69,
                239.48,
                239.28,
                239.07,
                238.86,
                238.65,
                238.44,
                238.22,
                238.04,
                237.93,
                237.82,
                237.7,
                237.59,
                237.49,
                237.38,
                237.28,
                237.18,
                237.08,
                236.98,
                236.88,
                236.79,
                236.7,
                236.61,
                236.52,
                1846.46,
                1825.77,
                1805.01,
                1784.16,
                1763.24,
                1742.24,
                1721.17,
                1700.02,
                1678.79,
                1657.48,
                1638.47,
                1624.8,
                1611.11,
                1597.38,
                1583.61,
                1569.81,
                1555.97,
                1542.1,
                1528.19,
                1514.24,
                1500.26,
                1486.25,
                1472.2,
                1458.11,
                1443.98,
                1429.83,
                1415.63,
                1401.4,
                1387.13,
                1372.83,
                1362.67,
                1353.35,
                1343.9,
                1334.33,
                1324.62,
                1314.79,
                1304.82,
                1294.73,
                1284.51,
                1274.16,
                1263.67,
                1253.06,
                1242.32,
                1231.45,
                1220.45,
                1209.32,
                1198.06,
                1186.67,
                1175.15,
                1160.16,
                1138.0,
                1115.82,
                1093.62,
                1071.4,
                1049.16,
                1026.9,
                1004.62,
                982.32,
                960.0,
                937.66,
                915.3,
                892.92,
                870.51,
                848.09,
                825.64,
                803.18,
                780.69,
                758.19,
                735.66,
                724.9,
                716.76,
                708.68,
                700.67,
                692.71,
                684.82,
                676.99,
                669.22,
                661.51,
                653.87,
                646.29,
                638.77,
                631.31,
                623.92,
                616.59,
                609.32,
                602.11,
                594.97,
                587.89,
                583.25,
                583.73,
                584.25,
                584.82,
                585.42,
                586.07,
                586.76,
                587.49,
                588.26,
                589.08,
                589.94,
                590.84,
                591.78,
                592.76,
                593.79,
                594.86,
                595.97,
                597.12,
                598.32,
                599.56,
                595.72,
                590.69,
                585.65,
                580.58,
                575.49,
                570.38,
                565.25,
                560.09,
                554.92,
                549.72,
                544.5,
                539.27,
                534.0,
                528.72,
                523.42,
                518.22,
                513.57,
                508.88,
                504.15,
                499.83,
                496.55,
                493.26,
                489.97,
                486.67,
                483.37,
                480.07,
                476.76,
                473.44,
                470.12,
                466.8,
                463.46,
                460.13,
                456.79,
                453.44,
                450.09,
                446.74,
                443.38,
                440.01,
                436.64,
                434.51,
                432.68,
                430.83,
                428.95,
                427.03,
                425.1,
                423.13,
                421.13,
                419.11,
                417.06,
                414.98,
                412.88,
                410.74,
                408.58,
                406.39,
                404.17,
                401.93,
                399.65,
                397.35,
                394.79,
                391.61,
                388.4,
                385.17,
                381.9,
                378.61,
                375.29,
                371.94,
                368.57,
                365.16,
                361.73,
                358.26,
                354.77,
                351.25,
                347.7,
                344.13,
                340.52,
                336.89,
                333.22,
                329.53,
                325.04,
                320.27,
                315.49,
                310.7,
                305.89,
                301.06,
                296.22,
                291.36,
                286.49,
                281.6,
                276.7,
                271.78,
                266.84,
                261.89,
                256.92,
                251.94,
                246.94,
                241.92,
                236.89,
                233.05,
                233.21,
                233.38,
                233.55,
                233.71,
                233.87,
                234.04,
                234.2,
                234.36,
                234.52,
                234.68,
                234.84,
                235.0,
                235.16,
                235.32,
                235.48,
                235.64,
                235.79,
                235.95,
                236.1,
                236.28,
                236.46,
                236.65,
                236.84,
                237.02,
                237.21,
                237.39,
                237.58,
                237.77,
                237.95,
                238.14,
                238.32,
                238.51,
                238.69,
                238.88,
                239.07,
                239.25,
                239.44,
                239.62,
                239.79,
                239.88,
                239.96,
                240.04,
                240.12,
                240.21,
                240.29,
                240.37,
                240.46,
                240.54,
                240.62,
                240.71,
                240.79,
                240.87,
                240.96,
                241.04,
                241.12,
                241.21,
                241.29,
                241.37,
                241.27,
                241.06,
                240.84,
                240.63,
                240.41,
                240.2,
                239.98,
                239.76,
                239.54,
                239.31,
                239.09,
                238.87,
                238.64,
                238.41,
                238.18,
                237.95,
                237.72,
                237.49,
                237.25,
                237.03,
                236.95,
                236.86,
                236.78,
                236.71,
                236.63,
                236.56,
                236.49,
                236.42,
                236.35,
                236.28,
                236.22,
                236.16,
                236.1,
                236.04,
                235.98,
                1907.51,
                1885.32,
                1863.06,
                1840.72,
                1818.3,
                1795.81,
                1773.24,
                1750.59,
                1727.87,
                1705.06,
                1682.77,
                1668.42,
                1654.03,
                1639.61,
                1625.16,
                1610.67,
                1596.14,
                1581.58,
                1566.98,
                1552.35,
                1537.68,
                1522.98,
                1508.24,
                1493.46,
                1478.65,
                1463.8,
                1448.92,
                1434.0,
                1419.04,
                1404.05,
                1390.97,
                1379.23,
                1367.36,
                1355.36,
                1343.24,
                1330.98,
                1318.6,
                1306.08,
                1293.44,
                1280.67,
                1267.77,
                1254.74,
                1241.58,
                1228.29,
                1214.87,
                1201.32,
                1187.63,
                1173.82,
                1159.88,
                1145.18,
                1122.65,
                1100.1,
                1077.54,
                1054.95,
                1032.34,
                1009.71,
                987.07,
                964.4,
                941.71,
                919.0,
                896.27,
                873.52,
                850.75,
                827.96,
                805.15,
                782.31,
                759.46,
                736.59,
                713.69,
                699.88,
                692.88,
                685.94,
                679.06,
                672.24,
                665.49,
                658.8,
                652.16,
                645.6,
                639.09,
                632.64,
                626.26,
                619.94,
                613.68,
                607.49,
                601.36,
                595.29,
                589.28,
                583.39,
                578.81,
                580.88,
                583.0,
                585.16,
                587.37,
                589.63,
                591.94,
                594.3,
                596.7,
                599.16,
                601.66,
                604.21,
                606.81,
                609.46,
                612.15,
                614.9,
                617.69,
                620.54,
                623.43,
                626.37,
                625.03,
                620.25,
                615.43,
                610.56,
                605.65,
                600.69,
                595.69,
                590.65,
                585.56,
                580.42,
                575.24,
                570.01,
                564.74,
                559.43,
                554.07,
                548.66,
                543.21,
                537.72,
                532.18,
                526.69,
                523.32,
                519.94,
                516.55,
                513.16,
                509.77,
                506.37,
                502.96,
                499.55,
                496.13,
                492.71,
                489.29,
                485.86,
                482.42,
                478.98,
                475.54,
                472.09,
                468.63,
                465.17,
                461.71,
                458.86,
                456.55,
                454.2,
                451.83,
                449.44,
                447.01,
                444.56,
                442.08,
                439.57,
                437.03,
                434.46,
                431.87,
                429.25,
                426.6,
                423.92,
                421.22,
                418.48,
                415.72,
                412.93,
                410.11,
                406.44,
                402.75,
                399.02,
                395.27,
                391.5,
                387.69,
                383.85,
                379.99,
                376.1,
                372.18,
                368.23,
                364.25,
                360.24,
                356.21,
                352.15,
                348.05,
                343.93,
                339.78,
                335.61,
                331.01,
                325.98,
                320.94,
                315.88,
                310.8,
                305.71,
                300.61,
                295.49,
                290.35,
                285.2,
                280.03,
                274.85,
                269.65,
                264.43,
                259.2,
                253.95,
                248.69,
                243.41,
                238.12,
                232.81,
                232.74,
                232.89,
                233.05,
                233.2,
                233.36,
                233.51,
                233.66,
                233.81,
                233.96,
                234.11,
                234.26,
                234.41,
                234.56,
                234.71,
                234.85,
                235.0,
                235.15,
                235.29,
                235.44,
                235.6,
                235.78,
                235.97,
                236.16,
                236.34,
                236.53,
                236.71,
                236.9,
                237.08,
                237.27,
                237.46,
                237.64,
                237.83,
                238.01,
                238.2,
                238.38,
                238.57,
                238.76,
                238.94,
                239.13,
                239.22,
                239.31,
                239.39,
                239.47,
                239.55,
                239.64,
                239.72,
                239.8,
                239.89,
                239.97,
                240.05,
                240.14,
                240.22,
                240.3,
                240.39,
                240.47,
                240.55,
                240.64,
                240.72,
                240.69,
                240.46,
                240.22,
                239.99,
                239.75,
                239.51,
                239.27,
                239.03,
                238.79,
                238.55,
                238.3,
                238.06,
                237.81,
                237.56,
                237.31,
                237.06,
                236.8,
                236.55,
                236.29,
                236.04,
                235.95,
                235.9,
                235.85,
                235.8,
                235.76,
                235.71,
                235.67,
                235.64,
                235.6,
                235.57,
                235.53,
                235.5,
                235.48,
                235.45,
                235.43,
                1969.26,
                1945.58,
                1921.82,
                1897.98,
                1874.07,
                1850.08,
                1826.02,
                1801.88,
                1777.66,
                1753.36,
                1728.98,
                1712.36,
                1697.29,
                1682.18,
                1667.03,
                1651.85,
                1636.64,
                1621.39,
                1606.1,
                1590.78,
                1575.43,
                1560.03,
                1544.61,
                1529.14,
                1513.64,
                1498.11,
                1482.54,
                1466.93,
                1451.29,
                1435.61,
                1420.44,
                1406.28,
                1391.99,
                1377.57,
                1363.03,
                1348.35,
                1333.55,
                1318.62,
                1303.55,
                1288.36,
                1273.04,
                1257.59,
                1242.01,
                1226.3,
                1210.46,
                1194.49,
                1178.39,
                1162.16,
                1145.8,
                1129.31,
                1107.48,
                1084.61,
                1061.83,
                1039.14,
                1016.52,
                993.99,
                971.54,
                949.18,
                926.9,
                904.71,
                882.6,
                860.57,
                838.63,
                816.78,
                795.0,
                773.32,
                751.72,
                730.2,
                708.77,
                692.72,
                687.67,
                682.66,
                677.69,
                672.77,
                667.9,
                663.07,
                658.28,
                653.54,
                648.85,
                644.2,
                639.6,
                635.04,
                630.53,
                626.06,
                621.64,
                617.27,
                612.94,
                608.65,
                604.41,
                606.04,
                609.02,
                612.04,
                615.11,
                618.23,
                621.4,
                624.61,
                627.88,
                631.19,
                634.55,
                637.96,
                641.42,
                644.92,
                648.48,
                652.08,
                655.74,
                659.44,
                663.19,
                666.99,
                667.96,
                662.38,
                656.75,
                651.08,
                645.36,
                639.6,
                633.79,
                627.94,
                622.05,
                616.11,
                610.12,
                604.09,
                598.02,
                591.9,
                585.73,
                579.52,
                573.27,
                566.97,
                560.62,
                554.23,
                550.14,
                546.66,
                543.18,
                539.7,
                536.21,
                532.71,
                529.21,
                525.71,
                522.2,
                518.68,
                515.16,
                511.64,
                508.11,
                504.57,
                501.03,
                497.49,
                493.94,
                490.38,
                486.82,
                483.47,
                480.67,
                477.84,
                474.98,
                472.1,
                469.18,
                466.24,
                463.27,
                460.28,
                457.25,
                454.2,
                451.12,
                448.01,
                444.88,
                441.71,
                438.52,
                435.3,
                432.05,
                428.78,
                425.47,
                421.53,
                417.35,
                413.14,
                408.9,
                404.64,
                400.35,
                396.02,
                391.67,
                387.29,
                382.89,
                378.45,
                373.99,
                369.5,
                364.98,
                360.43,
                355.85,
                351.24,
                346.61,
                341.94,
                337.12,
                331.83,
                326.52,
                321.2,
                315.86,
                310.51,
                305.14,
                299.76,
                294.36,
                288.94,
                283.51,
                278.06,
                272.6,
                267.12,
                261.62,
                256.11,
                250.59,
                245.04,
                239.49,
                233.91,
                232.27,
                232.41,
                232.56,
                232.7,
                232.84,
                232.99,
                233.13,
                233.27,
                233.41,
                233.55,
                233.69,
                233.83,
                233.96,
                234.1,
                234.24,
                234.37,
                234.51,
                234.64,
                234.78,
                234.92,
                235.1,
                235.29,
                235.47,
                235.66,
                235.85,
                236.03,
                236.22,
                236.4,
                236.59,
                236.77,
                236.96,
                237.15,
                237.33,
                237.52,
                237.7,
                237.89,
                238.07,
                238.26,
                238.45,
                238.57,
                238.65,
                238.74,
                238.82,
                238.9,
                238.98,
                239.07,
                239.15,
                239.23,
                239.32,
                239.4,
                239.48,
                239.57,
                239.65,
                239.73,
                239.82,
                239.9,
                239.98,
                240.07,
                240.12,
                239.87,
                239.62,
                239.36,
                239.1,
                238.84,
                238.58,
                238.32,
                238.06,
                237.79,
                237.52,
                237.26,
                236.99,
                236.72,
                236.45,
                236.17,
                235.9,
                235.63,
                235.35,
                235.07,
                234.93,
                234.91,
                234.89,
                234.88,
                234.86,
                234.85,
                234.85,
                234.84,
                234.84,
                234.83,
                234.83,
                234.83,
                234.84,
                234.84,
                234.85,
                2031.71,
                2006.54,
                1981.29,
                1955.96,
                1930.55,
                1905.07,
                1879.51,
                1853.87,
                1828.16,
                1802.37,
                1776.5,
                1756.63,
                1740.87,
                1724.39,
                1707.54,
                1690.64,
                1673.68,
                1656.65,
                1639.57,
                1622.42,
                1605.22,
                1587.95,
                1570.62,
                1553.23,
                1535.78,
                1518.26,
                1500.69,
                1483.05,
                1465.36,
                1447.6,
                1429.89,
                1413.22,
                1396.54,
                1379.85,
                1363.16,
                1346.45,
                1329.74,
                1313.02,
                1296.29,
                1279.55,
                1262.8,
                1246.04,
                1229.28,
                1212.5,
                1195.72,
                1178.93,
                1162.13,
                1145.32,
                1128.5,
                1111.68,
                1092.17,
                1070.86,
                1049.63,
                1028.49,
                1007.43,
                986.45,
                965.56,
                944.75,
                924.02,
                903.38,
                882.83,
                862.36,
                841.97,
                821.66,
                801.45,
                781.31,
                761.26,
                741.3,
                721.42,
                702.86,
                698.62,
                694.43,
                690.28,
                686.18,
                682.12,
                678.11,
                674.14,
                670.22,
                666.35,
                662.51,
                658.73,
                654.99,
                651.3,
                647.65,
                644.04,
                640.49,
                636.97,
                633.51,
                630.09,
                630.77,
                634.61,
                638.49,
                642.42,
                646.39,
                650.42,
                654.49,
                658.61,
                662.79,
                667.0,
                671.27,
                675.59,
                679.95,
                684.37,
                688.83,
                693.34,
                697.9,
                702.51,
                707.17,
                711.3,
                704.92,
                698.48,
                692.01,
                685.49,
                678.92,
                672.31,
                665.65,
                658.95,
                652.21,
                645.42,
                638.59,
                631.71,
                624.78,
                617.81,
                610.8,
                603.74,
                596.63,
                589.48,
                582.29,
                577.01,
                573.44,
                569.86,
                566.28,
                562.7,
                559.11,
                555.51,
                551.91,
                548.31,
                544.7,
                541.08,
                537.46,
                533.84,
                530.21,
                526.57,
                522.93,
                519.29,
                515.64,
                511.99,
                508.33,
                505.04,
                501.73,
                498.38,
                495.01,
                491.61,
                488.18,
                484.73,
                481.25,
                477.74,
                474.2,
                470.63,
                467.04,
                463.41,
                459.76,
                456.09,
                452.38,
                448.64,
                444.88,
                441.09,
                436.88,
                432.21,
                427.52,
                422.79,
                418.04,
                413.26,
                408.45,
                403.62,
                398.75,
                393.86,
                388.94,
                383.99,
                379.01,
                374.0,
                368.97,
                363.9,
                358.81,
                353.69,
                348.54,
                343.36,
                337.82,
                332.25,
                326.67,
                321.07,
                315.45,
                309.82,
                304.17,
                298.5,
                292.82,
                287.13,
                281.42,
                275.69,
                269.95,
                264.19,
                258.42,
                252.63,
                246.82,
                241.0,
                235.16,
                231.81,
                231.94,
                232.07,
                232.21,
                232.34,
                232.47,
                232.6,
                232.73,
                232.86,
                232.99,
                233.12,
                233.25,
                233.37,
                233.5,
                233.62,
                233.75,
                233.87,
                234.0,
                234.12,
                234.24,
                234.42,
                234.61,
                234.79,
                234.98,
                235.16,
                235.35,
                235.54,
                235.72,
                235.91,
                236.09,
                236.28,
                236.46,
                236.65,
                236.84,
                237.02,
                237.21,
                237.39,
                237.58,
                237.76,
                237.92,
                238.0,
                238.08,
                238.17,
                238.25,
                238.33,
                238.41,
                238.5,
                238.58,
                238.66,
                238.75,
                238.83,
                238.91,
                239.0,
                239.08,
                239.16,
                239.25,
                239.33,
                239.41,
                239.49,
                239.3,
                239.02,
                238.74,
                238.46,
                238.18,
                237.9,
                237.62,
                237.33,
                237.05,
                236.76,
                236.47,
                236.18,
                235.89,
                235.6,
                235.3,
                235.01,
                234.71,
                234.41,
                234.12,
                233.89,
                233.9,
                233.92,
                233.94,
                233.95,
                233.98,
                234.0,
                234.02,
                234.05,
                234.08,
                234.11,
                234.14,
                234.18,
                234.22,
                234.26,
                2038.77,
                2013.01,
                1987.39,
                1961.91,
                1936.58,
                1911.39,
                1886.35,
                1861.46,
                1836.7,
                1812.1,
                1787.64,
                1765.62,
                1747.74,
                1729.81,
                1711.81,
                1693.75,
                1675.63,
                1657.45,
                1639.21,
                1620.91,
                1602.55,
                1584.13,
                1565.65,
                1547.1,
                1528.49,
                1509.83,
                1491.1,
                1472.31,
                1453.45,
                1434.54,
                1415.56,
                1398.42,
                1381.58,
                1364.74,
                1347.89,
                1331.03,
                1314.16,
                1297.28,
                1280.39,
                1263.5,
                1246.59,
                1229.68,
                1212.76,
                1195.83,
                1178.89,
                1161.94,
                1144.98,
                1128.02,
                1111.04,
                1094.06,
                1076.1,
                1056.34,
                1036.67,
                1017.08,
                997.57,
                978.15,
                958.81,
                939.55,
                920.38,
                901.29,
                882.29,
                863.37,
                844.53,
                825.78,
                807.12,
                788.54,
                770.04,
                751.63,
                733.3,
                715.06,
                709.16,
                705.79,
                702.46,
                699.18,
                695.94,
                692.74,
                689.59,
                686.49,
                683.43,
                680.42,
                677.45,
                674.53,
                671.65,
                668.82,
                666.03,
                663.29,
                660.6,
                657.95,
                655.35,
                655.07,
                659.76,
                664.5,
                669.29,
                674.12,
                679.01,
                683.94,
                688.92,
                693.95,
                699.02,
                704.15,
                709.32,
                714.55,
                719.82,
                725.14,
                730.51,
                735.93,
                741.39,
                746.91,
                752.47,
                747.87,
                740.63,
                733.35,
                726.02,
                718.65,
                711.24,
                703.78,
                696.27,
                688.73,
                681.13,
                673.49,
                665.81,
                658.08,
                650.31,
                642.49,
                634.63,
                626.72,
                618.76,
                610.76,
                603.93,
                600.26,
                596.59,
                592.92,
                589.24,
                585.55,
                581.86,
                578.17,
                574.47,
                570.76,
                567.05,
                563.34,
                559.62,
                555.9,
                552.17,
                548.43,
                544.69,
                540.95,
                537.2,
                533.45,
                529.67,
                525.87,
                522.04,
                518.18,
                514.3,
                510.38,
                506.44,
                502.47,
                498.47,
                494.45,
                490.4,
                486.32,
                482.21,
                478.07,
                473.91,
                469.71,
                465.49,
                461.24,
                456.97,
                452.48,
                447.33,
                442.15,
                436.94,
                431.7,
                426.44,
                421.14,
                415.82,
                410.47,
                405.09,
                399.69,
                394.25,
                388.79,
                383.29,
                377.77,
                372.22,
                366.64,
                361.04,
                355.4,
                349.74,
                343.96,
                338.12,
                332.27,
                326.41,
                320.53,
                314.63,
                308.72,
                302.79,
                296.85,
                290.89,
                284.92,
                278.93,
                272.92,
                266.9,
                260.86,
                254.81,
                248.74,
                242.66,
                236.56,
                231.35,
                231.47,
                231.59,
                231.72,
                231.84,
                231.96,
                232.08,
                232.2,
                232.32,
                232.44,
                232.55,
                232.67,
                232.79,
                232.9,
                233.02,
                233.13,
                233.25,
                233.36,
                233.47,
                233.58,
                233.74,
                233.93,
                234.11,
                234.3,
                234.48,
                234.67,
                234.85,
                235.04,
                235.23,
                235.41,
                235.6,
                235.78,
                235.97,
                236.15,
                236.34,
                236.53,
                236.71,
                236.9,
                237.08,
                237.26,
                237.35,
                237.43,
                237.51,
                237.6,
                237.68,
                237.76,
                237.84,
                237.93,
                238.01,
                238.09,
                238.18,
                238.26,
                238.34,
                238.43,
                238.51,
                238.59,
                238.68,
                238.76,
                238.84,
                238.73,
                238.43,
                238.14,
                237.84,
                237.53,
                237.23,
                236.93,
                236.62,
                236.31,
                236.01,
                235.7,
                235.39,
                235.07,
                234.76,
                234.45,
                234.13,
                233.81,
                233.49,
                233.17,
                232.85,
                232.88,
                232.93,
                232.98,
                233.03,
                233.08,
                233.13,
                233.19,
                233.25,
                233.31,
                233.37,
                233.44,
                233.5,
                233.57,
                233.64,
                2018.5,
                1995.52,
                1972.68,
                1949.98,
                1927.42,
                1905.01,
                1882.75,
                1860.63,
                1838.66,
                1816.83,
                1795.14,
                1773.9,
                1754.87,
                1735.78,
                1716.63,
                1697.41,
                1678.14,
                1658.81,
                1639.41,
                1619.96,
                1600.44,
                1580.86,
                1561.23,
                1541.53,
                1521.77,
                1501.94,
                1482.06,
                1462.12,
                1442.11,
                1422.04,
                1401.91,
                1383.69,
                1366.7,
                1349.7,
                1332.69,
                1315.68,
                1298.65,
                1281.62,
                1264.57,
                1247.52,
                1230.46,
                1213.39,
                1196.32,
                1179.23,
                1162.13,
                1145.03,
                1127.92,
                1110.79,
                1093.66,
                1076.52,
                1059.27,
                1041.06,
                1022.94,
                1004.9,
                986.95,
                969.08,
                951.29,
                933.59,
                915.97,
                898.43,
                880.98,
                863.61,
                846.33,
                829.13,
                812.02,
                794.99,
                778.05,
                761.19,
                744.41,
                727.72,
                719.3,
                716.75,
                714.23,
                711.77,
                709.35,
                706.97,
                704.64,
                702.35,
                700.11,
                697.91,
                695.76,
                693.66,
                691.6,
                689.58,
                687.61,
                685.69,
                683.81,
                681.98,
                680.19,
                678.93,
                684.48,
                690.08,
                695.72,
                701.42,
                707.16,
                712.95,
                718.78,
                724.67,
                730.61,
                736.59,
                742.62,
                748.7,
                754.83,
                761.01,
                767.24,
                773.51,
                779.84,
                786.21,
                792.63,
                791.23,
                783.19,
                775.1,
                766.97,
                758.8,
                750.58,
                742.32,
                734.01,
                725.66,
                717.26,
                708.82,
                700.33,
                691.8,
                683.22,
                674.6,
                665.93,
                657.22,
                648.46,
                639.65,
                630.9,
                627.14,
                623.37,
                619.6,
                615.83,
                612.05,
                608.26,
                604.47,
                600.68,
                596.88,
                593.08,
                589.27,
                585.45,
                581.63,
                577.81,
                573.98,
                570.15,
                566.31,
                562.46,
                558.61,
                554.56,
                550.27,
                545.95,
                541.61,
                537.24,
                532.84,
                528.41,
                523.95,
                519.47,
                514.96,
                510.42,
                505.85,
                501.26,
                496.64,
                491.99,
                487.31,
                482.6,
                477.86,
                473.1,
                468.31,
                462.71,
                457.04,
                451.35,
                445.63,
                439.87,
                434.09,
                428.29,
                422.45,
                416.59,
                410.69,
                404.77,
                398.82,
                392.84,
                386.84,
                380.8,
                374.74,
                368.64,
                362.52,
                356.37,
                350.23,
                344.14,
                338.02,
                331.9,
                325.75,
                319.59,
                313.42,
                307.23,
                301.02,
                294.8,
                288.56,
                282.31,
                276.04,
                269.75,
                263.45,
                257.14,
                250.8,
                244.46,
                238.09,
                231.71,
                231.01,
                231.12,
                231.23,
                231.34,
                231.45,
                231.56,
                231.67,
                231.78,
                231.89,
                232.0,
                232.1,
                232.21,
                232.31,
                232.42,
                232.52,
                232.62,
                232.73,
                232.83,
                232.93,
                233.06,
                233.25,
                233.43,
                233.62,
                233.8,
                233.99,
                234.17,
                234.36,
                234.55,
                234.73,
                234.92,
                235.1,
                235.29,
                235.47,
                235.66,
                235.84,
                236.03,
                236.22,
                236.4,
                236.59,
                236.69,
                236.78,
                236.86,
                236.94,
                237.02,
                237.11,
                237.19,
                237.27,
                237.36,
                237.44,
                237.52,
                237.61,
                237.69,
                237.77,
                237.86,
                237.94,
                238.02,
                238.1,
                238.19,
                238.18,
                237.86,
                237.54,
                237.22,
                236.9,
                236.57,
                236.25,
                235.92,
                235.59,
                235.27,
                234.94,
                234.6,
                234.27,
                233.95,
                233.66,
                233.38,
                233.1,
                232.82,
                232.54,
                232.26,
                232.25,
                232.38,
                232.5,
                232.62,
                232.74,
                232.85,
                232.97,
                233.08,
                233.2,
                233.31,
                233.42,
                233.53,
                233.64,
                233.74,
                1996.92,
                1976.71,
                1956.65,
                1936.73,
                1916.95,
                1897.32,
                1877.83,
                1858.48,
                1839.29,
                1820.23,
                1801.32,
                1782.56,
                1762.55,
                1742.3,
                1722.0,
                1701.63,
                1681.2,
                1660.71,
                1640.17,
                1619.56,
                1598.89,
                1578.16,
                1557.36,
                1536.51,
                1515.59,
                1494.62,
                1473.58,
                1452.48,
                1431.32,
                1410.1,
                1388.82,
                1369.04,
                1351.9,
                1334.74,
                1317.58,
                1300.4,
                1283.22,
                1266.03,
                1248.83,
                1231.62,
                1214.41,
                1197.18,
                1179.95,
                1162.71,
                1145.45,
                1128.19,
                1110.92,
                1093.65,
                1076.36,
                1059.06,
                1041.76,
                1025.02,
                1008.45,
                991.96,
                975.56,
                959.24,
                943.01,
                926.85,
                910.79,
                894.8,
                878.91,
                863.09,
                847.36,
                831.71,
                816.15,
                800.67,
                785.28,
                769.97,
                754.75,
                739.61,
                729.04,
                727.3,
                725.6,
                723.95,
                722.35,
                720.79,
                719.27,
                717.8,
                716.38,
                715.0,
                713.66,
                712.37,
                711.13,
                709.93,
                708.78,
                707.67,
                706.61,
                705.6,
                704.63,
                703.7,
                708.77,
                715.22,
                721.72,
                728.27,
                734.87,
                741.52,
                748.22,
                754.96,
                761.75,
                768.59,
                775.48,
                782.42,
                789.41,
                796.44,
                803.53,
                810.66,
                817.84,
                825.07,
                832.35,
                835.0,
                826.16,
                817.27,
                808.34,
                799.36,
                790.33,
                781.27,
                772.16,
                763.0,
                753.8,
                744.55,
                735.26,
                725.92,
                716.54,
                707.12,
                697.65,
                688.13,
                678.57,
                668.96,
                659.31,
                654.06,
                650.2,
                646.33,
                642.46,
                638.59,
                634.71,
                630.83,
                626.94,
                623.04,
                619.15,
                615.24,
                611.33,
                607.42,
                603.5,
                599.58,
                595.65,
                591.71,
                587.78,
                583.83,
                579.7,
                574.93,
                570.12,
                565.29,
                560.43,
                555.55,
                550.64,
                545.69,
                540.72,
                535.73,
                530.7,
                525.65,
                520.57,
                515.46,
                510.32,
                505.16,
                499.97,
                494.74,
                489.5,
                484.22,
                478.35,
                472.2,
                466.02,
                459.81,
                453.57,
                447.3,
                441.01,
                434.69,
                428.34,
                421.96,
                415.55,
                409.12,
                402.65,
                396.16,
                389.64,
                383.09,
                376.51,
                369.91,
                363.27,
                356.65,
                350.29,
                343.92,
                337.53,
                331.12,
                324.7,
                318.26,
                311.8,
                305.33,
                298.85,
                292.35,
                285.83,
                279.3,
                272.75,
                266.19,
                259.61,
                253.01,
                246.4,
                239.77,
                233.13,
                230.55,
                230.65,
                230.75,
                230.85,
                230.95,
                231.05,
                231.15,
                231.25,
                231.35,
                231.44,
                231.54,
                231.63,
                231.73,
                231.82,
                231.92,
                232.01,
                232.1,
                232.19,
                232.28,
                232.38,
                232.56,
                232.75,
                232.94,
                233.12,
                233.31,
                233.49,
                233.68,
                233.86,
                234.05,
                234.24,
                234.42,
                234.61,
                234.79,
                234.98,
                235.16,
                235.35,
                235.53,
                235.72,
                235.9,
                236.07,
                236.2,
                236.33,
                236.46,
                236.58,
                236.7,
                236.82,
                236.94,
                237.05,
                237.16,
                237.27,
                237.37,
                237.48,
                237.58,
                237.67,
                237.77,
                237.86,
                237.95,
                238.04,
                238.12,
                237.86,
                237.58,
                237.31,
                237.03,
                236.76,
                236.49,
                236.22,
                235.96,
                235.69,
                235.43,
                235.17,
                234.91,
                234.66,
                234.4,
                234.15,
                233.9,
                233.65,
                233.4,
                233.16,
                233.05,
                233.15,
                233.25,
                233.35,
                233.45,
                233.54,
                233.64,
                233.73,
                233.82,
                233.91,
                234.0,
                234.09,
                234.18,
                234.26,
                1974.03,
                1956.6,
                1939.31,
                1922.16,
                1905.16,
                1888.3,
                1871.59,
                1855.02,
                1838.6,
                1822.32,
                1806.18,
                1790.19,
                1770.78,
                1749.38,
                1727.92,
                1706.4,
                1684.82,
                1663.17,
                1641.47,
                1619.71,
                1597.88,
                1576.0,
                1554.05,
                1532.05,
                1509.98,
                1487.85,
                1465.66,
                1443.41,
                1421.09,
                1398.72,
                1376.28,
                1354.47,
                1337.17,
                1319.86,
                1302.54,
                1285.21,
                1267.87,
                1250.52,
                1233.17,
                1215.8,
                1198.43,
                1181.05,
                1163.66,
                1146.26,
                1128.85,
                1111.43,
                1094.01,
                1076.57,
                1059.13,
                1041.68,
                1024.22,
                1008.21,
                993.19,
                978.26,
                963.41,
                948.64,
                933.96,
                919.36,
                904.84,
                890.41,
                876.06,
                861.8,
                847.62,
                833.53,
                819.51,
                805.59,
                791.75,
                777.99,
                764.32,
                750.73,
                738.36,
                737.44,
                736.56,
                735.73,
                734.94,
                734.19,
                733.5,
                732.84,
                732.23,
                731.67,
                731.15,
                730.68,
                730.26,
                729.87,
                729.54,
                729.25,
                729.0,
                728.8,
                728.65,
                728.54,
                732.62,
                739.93,
                747.29,
                754.7,
                762.16,
                769.66,
                777.21,
                784.82,
                792.46,
                800.16,
                807.91,
                815.71,
                823.55,
                831.44,
                839.38,
                847.37,
                855.41,
                863.5,
                871.64,
                879.19,
                869.54,
                859.85,
                850.11,
                840.33,
                830.5,
                820.63,
                810.72,
                800.76,
                790.75,
                780.7,
                770.61,
                760.47,
                750.28,
                740.05,
                729.78,
                719.46,
                709.1,
                698.69,
                688.23,
                681.03,
                677.07,
                673.12,
                669.15,
                665.18,
                661.21,
                657.23,
                653.25,
                649.26,
                645.26,
                641.27,
                637.26,
                633.25,
                629.24,
                625.22,
                621.2,
                617.17,
                613.14,
                609.1,
                605.06,
                599.84,
                594.55,
                589.23,
                583.89,
                578.52,
                573.12,
                567.69,
                562.23,
                556.75,
                551.24,
                545.7,
                540.14,
                534.54,
                528.92,
                523.27,
                517.59,
                511.88,
                506.15,
                500.38,
                494.25,
                487.61,
                480.94,
                474.25,
                467.52,
                460.77,
                454.0,
                447.19,
                440.35,
                433.49,
                426.6,
                419.67,
                412.73,
                405.75,
                398.74,
                391.71,
                384.64,
                377.55,
                370.43,
                363.28,
                356.59,
                349.95,
                343.3,
                336.63,
                329.94,
                323.24,
                316.52,
                309.79,
                303.04,
                296.28,
                289.5,
                282.7,
                275.89,
                269.06,
                262.22,
                255.36,
                248.49,
                241.6,
                234.69,
                230.1,
                230.19,
                230.28,
                230.37,
                230.46,
                230.55,
                230.77,
                231.03,
                231.27,
                231.49,
                231.7,
                231.89,
                232.06,
                232.21,
                232.35,
                232.47,
                232.57,
                232.66,
                232.73,
                232.78,
                232.97,
                233.21,
                233.45,
                233.69,
                233.92,
                234.15,
                234.38,
                234.6,
                234.82,
                235.04,
                235.26,
                235.47,
                235.69,
                235.9,
                236.1,
                236.31,
                236.51,
                236.71,
                236.9,
                237.07,
                237.16,
                237.25,
                237.33,
                237.42,
                237.49,
                237.57,
                237.64,
                237.71,
                237.78,
                237.85,
                237.91,
                237.97,
                238.03,
                238.09,
                238.14,
                238.19,
                238.24,
                238.28,
                238.32,
                238.17,
                237.93,
                237.68,
                237.44,
                237.2,
                236.96,
                236.72,
                236.49,
                236.26,
                236.03,
                235.8,
                235.57,
                235.35,
                235.12,
                234.9,
                234.68,
                234.47,
                234.25,
                234.04,
                233.86,
                233.94,
                234.02,
                234.1,
                234.17,
                234.25,
                234.32,
                234.39,
                234.46,
                234.53,
                234.6,
                234.67,
                234.73,
                234.8,
                1949.83,
                1935.17,
                1920.65,
                1906.28,
                1892.05,
                1877.97,
                1864.03,
                1850.24,
                1836.59,
                1823.08,
                1809.72,
                1796.51,
                1779.55,
                1757.0,
                1734.39,
                1711.71,
                1688.98,
                1666.18,
                1643.33,
                1620.41,
                1597.44,
                1574.4,
                1551.3,
                1528.14,
                1504.92,
                1481.64,
                1458.29,
                1434.89,
                1411.42,
                1387.89,
                1364.3,
                1340.65,
                1322.51,
                1305.05,
                1287.57,
                1270.08,
                1252.59,
                1235.09,
                1217.58,
                1200.06,
                1182.53,
                1164.99,
                1147.44,
                1129.89,
                1112.32,
                1094.75,
                1077.17,
                1059.58,
                1041.98,
                1024.37,
                1006.76,
                990.64,
                977.18,
                963.8,
                950.5,
                937.28,
                924.15,
                911.1,
                898.13,
                885.25,
                872.46,
                859.74,
                847.11,
                834.57,
                822.11,
                809.73,
                797.44,
                785.23,
                773.11,
                761.07,
                749.12,
                747.17,
                747.11,
                747.09,
                747.12,
                747.19,
                747.31,
                747.48,
                747.68,
                747.94,
                748.24,
                748.58,
                748.97,
                749.41,
                749.89,
                750.41,
                750.98,
                751.6,
                752.26,
                752.97,
                756.04,
                764.21,
                772.43,
                780.69,
                789.0,
                797.37,
                805.78,
                814.23,
                822.74,
                831.3,
                839.9,
                848.55,
                857.25,
                866.0,
                874.8,
                883.65,
                892.54,
                901.49,
                910.48,
                919.52,
                913.33,
                902.83,
                892.29,
                881.71,
                871.08,
                860.41,
                849.69,
                838.92,
                828.12,
                817.26,
                806.37,
                795.42,
                784.44,
                773.41,
                762.33,
                751.21,
                740.04,
                728.83,
                717.57,
                708.05,
                704.0,
                699.95,
                695.89,
                691.82,
                687.76,
                683.68,
                679.6,
                675.52,
                671.43,
                667.34,
                663.24,
                659.14,
                655.03,
                650.92,
                646.8,
                642.68,
                638.55,
                634.42,
                630.28,
                625.0,
                619.23,
                613.43,
                607.6,
                601.74,
                595.85,
                589.94,
                584.0,
                578.03,
                572.04,
                566.01,
                559.96,
                553.88,
                547.77,
                541.63,
                535.47,
                529.28,
                523.06,
                516.81,
                510.4,
                503.28,
                496.13,
                488.95,
                481.74,
                474.5,
                467.24,
                459.95,
                452.63,
                445.28,
                437.9,
                430.46,
                422.96,
                415.42,
                407.85,
                400.24,
                392.59,
                384.91,
                377.2,
                369.45,
                362.26,
                355.45,
                348.65,
                341.88,
                335.12,
                328.38,
                321.66,
                314.96,
                308.27,
                301.6,
                294.95,
                288.32,
                281.71,
                275.11,
                268.53,
                261.97,
                255.43,
                248.9,
                242.4,
                236.31,
                236.38,
                236.44,
                236.48,
                236.5,
                236.51,
                236.5,
                236.47,
                236.43,
                236.36,
                236.28,
                236.19,
                236.07,
                235.94,
                235.8,
                235.63,
                235.45,
                235.25,
                235.03,
                234.8,
                234.78,
                234.97,
                235.17,
                235.36,
                235.55,
                235.74,
                235.92,
                236.11,
                236.29,
                236.46,
                236.64,
                236.81,
                236.98,
                237.15,
                237.31,
                237.47,
                237.63,
                237.79,
                237.94,
                238.1,
                238.15,
                238.19,
                238.23,
                238.27,
                238.31,
                238.34,
                238.38,
                238.4,
                238.43,
                238.45,
                238.47,
                238.49,
                238.51,
                238.52,
                238.53,
                238.54,
                238.54,
                238.55,
                238.55,
                238.46,
                238.25,
                238.04,
                237.83,
                237.62,
                237.41,
                237.21,
                237.0,
                236.8,
                236.6,
                236.41,
                236.21,
                236.02,
                235.83,
                235.64,
                235.45,
                235.27,
                235.08,
                234.9,
                234.72,
                234.74,
                234.8,
                234.85,
                234.91,
                234.96,
                235.02,
                235.07,
                235.12,
                235.16,
                235.21,
                235.26,
                235.3,
                235.35,
                1924.31,
                1912.43,
                1900.68,
                1889.09,
                1877.63,
                1866.32,
                1855.16,
                1844.13,
                1833.26,
                1822.53,
                1811.94,
                1801.5,
                1788.88,
                1765.18,
                1741.41,
                1717.58,
                1693.7,
                1669.75,
                1645.74,
                1621.67,
                1597.54,
                1573.35,
                1549.1,
                1524.79,
                1500.41,
                1475.98,
                1451.48,
                1426.92,
                1402.3,
                1377.62,
                1352.88,
                1328.08,
                1307.94,
                1290.31,
                1272.68,
                1255.04,
                1237.39,
                1219.73,
                1202.06,
                1184.39,
                1166.7,
                1149.01,
                1131.3,
                1113.59,
                1095.87,
                1078.14,
                1060.41,
                1042.66,
                1024.91,
                1007.14,
                989.37,
                972.32,
                960.4,
                948.57,
                936.82,
                925.15,
                913.57,
                902.07,
                890.66,
                879.33,
                868.08,
                856.92,
                845.84,
                834.85,
                823.93,
                813.11,
                802.37,
                791.71,
                781.14,
                770.65,
                760.25,
                756.5,
                757.25,
                758.05,
                758.9,
                759.79,
                760.72,
                761.7,
                762.73,
                763.8,
                764.91,
                766.07,
                767.28,
                768.53,
                769.82,
                771.16,
                772.55,
                773.98,
                775.46,
                776.99,
                779.03,
                788.05,
                797.13,
                806.25,
                815.42,
                824.64,
                833.9,
                843.22,
                852.58,
                861.99,
                871.45,
                880.96,
                890.52,
                900.12,
                909.78,
                919.48,
                929.23,
                939.03,
                948.88,
                958.78,
                957.53,
                946.23,
                934.89,
                923.5,
                912.07,
                900.59,
                889.07,
                877.51,
                865.9,
                854.24,
                842.54,
                830.8,
                819.01,
                807.17,
                795.29,
                783.37,
                771.4,
                759.38,
                747.32,
                735.22,
                730.97,
                726.83,
                722.67,
                718.51,
                714.35,
                710.18,
                706.01,
                701.83,
                697.65,
                693.46,
                689.27,
                685.07,
                680.87,
                676.66,
                672.43,
                667.99,
                663.54,
                659.07,
                654.58,
                649.23,
                642.81,
                636.37,
                629.93,
                623.49,
                617.04,
                610.58,
                604.12,
                597.65,
                591.18,
                584.7,
                578.22,
                571.72,
                565.23,
                558.72,
                552.22,
                545.7,
                539.18,
                532.66,
                526.12,
                518.53,
                510.78,
                502.99,
                495.17,
                487.31,
                479.42,
                471.49,
                463.53,
                455.53,
                447.5,
                439.43,
                431.32,
                423.18,
                415.01,
                406.8,
                398.56,
                390.27,
                381.96,
                373.61,
                365.87,
                359.35,
                352.86,
                346.38,
                339.92,
                333.48,
                327.05,
                320.65,
                314.26,
                307.89,
                301.53,
                295.2,
                288.88,
                282.58,
                276.3,
                270.04,
                263.79,
                257.56,
                251.35,
                245.16,
                243.69,
                243.46,
                243.22,
                242.96,
                242.68,
                242.38,
                242.07,
                241.74,
                241.39,
                241.03,
                240.65,
                240.25,
                239.83,
                239.4,
                238.95,
                238.48,
                238.0,
                237.5,
                236.98,
                236.6,
                236.76,
                236.91,
                237.06,
                237.21,
                237.35,
                237.5,
                237.64,
                237.77,
                237.91,
                238.04,
                238.17,
                238.3,
                238.42,
                238.55,
                238.67,
                238.78,
                238.9,
                239.01,
                239.12,
                239.16,
                239.16,
                239.16,
                239.16,
                239.15,
                239.14,
                239.13,
                239.12,
                239.1,
                239.08,
                239.06,
                239.04,
                239.01,
                238.98,
                238.95,
                238.92,
                238.88,
                238.84,
                238.8,
                238.74,
                238.55,
                238.37,
                238.19,
                238.02,
                237.84,
                237.67,
                237.5,
                237.33,
                237.16,
                237.0,
                236.83,
                236.67,
                236.51,
                236.36,
                236.2,
                236.05,
                235.9,
                235.75,
                235.6,
                235.55,
                235.59,
                235.62,
                235.66,
                235.69,
                235.72,
                235.75,
                235.78,
                235.81,
                235.83,
                235.86,
                235.88,
                235.9,
                1897.48,
                1888.37,
                1879.4,
                1870.58,
                1861.9,
                1853.36,
                1844.97,
                1836.72,
                1828.61,
                1820.65,
                1812.84,
                1805.17,
                1797.64,
                1773.9,
                1748.98,
                1724.0,
                1698.96,
                1673.86,
                1648.7,
                1623.48,
                1598.2,
                1572.86,
                1547.45,
                1521.99,
                1496.46,
                1470.87,
                1445.22,
                1419.51,
                1393.74,
                1367.91,
                1342.02,
                1316.06,
                1293.44,
                1275.66,
                1257.87,
                1240.07,
                1222.26,
                1204.45,
                1186.62,
                1168.79,
                1150.95,
                1133.1,
                1115.24,
                1097.38,
                1079.5,
                1061.61,
                1043.72,
                1025.82,
                1007.91,
                989.99,
                972.06,
                954.12,
                942.86,
                932.58,
                922.38,
                912.26,
                902.23,
                892.28,
                882.42,
                872.64,
                862.94,
                853.33,
                843.8,
                834.35,
                824.99,
                815.72,
                806.52,
                797.42,
                788.39,
                779.46,
                770.6,
                765.42,
                766.99,
                768.61,
                770.27,
                771.97,
                773.72,
                775.52,
                777.36,
                779.24,
                781.17,
                783.15,
                785.17,
                787.24,
                789.35,
                791.51,
                793.71,
                795.96,
                798.25,
                800.59,
                802.97,
                811.46,
                821.39,
                831.37,
                841.4,
                851.47,
                861.6,
                871.77,
                881.99,
                892.26,
                902.57,
                912.94,
                923.35,
                933.81,
                944.32,
                954.88,
                965.49,
                976.14,
                986.85,
                997.27,
                1001.11,
                988.39,
                975.69,
                962.99,
                950.31,
                937.64,
                924.98,
                912.33,
                899.69,
                887.07,
                874.45,
                861.85,
                849.25,
                836.67,
                824.1,
                811.55,
                799.0,
                786.46,
                773.94,
                761.42,
                754.78,
                750.28,
                745.77,
                741.25,
                736.71,
                732.16,
                727.6,
                723.02,
                718.42,
                713.82,
                709.19,
                704.56,
                699.91,
                695.24,
                690.56,
                685.87,
                681.16,
                676.44,
                671.71,
                666.63,
                660.1,
                653.56,
                647.02,
                640.47,
                633.92,
                627.35,
                620.79,
                614.22,
                607.64,
                601.05,
                594.46,
                587.87,
                581.27,
                574.66,
                568.05,
                561.43,
                554.8,
                548.17,
                541.54,
                533.83,
                525.48,
                517.09,
                508.67,
                500.21,
                491.71,
                483.18,
                474.62,
                466.02,
                457.39,
                448.72,
                440.01,
                431.27,
                422.5,
                413.69,
                404.84,
                395.96,
                387.04,
                378.09,
                369.31,
                363.1,
                356.9,
                350.72,
                344.56,
                338.41,
                332.29,
                326.18,
                320.08,
                314.01,
                307.95,
                301.92,
                295.89,
                289.89,
                283.91,
                277.94,
                271.99,
                266.06,
                260.15,
                254.25,
                251.16,
                250.65,
                250.12,
                249.57,
                249.01,
                248.43,
                247.83,
                247.21,
                246.58,
                245.93,
                245.26,
                244.58,
                243.88,
                243.16,
                242.43,
                241.67,
                240.9,
                240.12,
                239.31,
                238.49,
                238.56,
                238.67,
                238.78,
                238.89,
                238.99,
                239.09,
                239.19,
                239.28,
                239.38,
                239.47,
                239.56,
                239.64,
                239.72,
                239.8,
                239.88,
                239.96,
                240.03,
                240.1,
                240.16,
                240.19,
                240.15,
                240.11,
                240.06,
                240.01,
                239.96,
                239.91,
                239.85,
                239.8,
                239.73,
                239.67,
                239.61,
                239.54,
                239.47,
                239.39,
                239.31,
                239.24,
                239.15,
                239.07,
                238.98,
                238.84,
                238.69,
                238.55,
                238.4,
                238.26,
                238.12,
                237.98,
                237.84,
                237.7,
                237.57,
                237.44,
                237.31,
                237.18,
                237.06,
                236.93,
                236.81,
                236.69,
                236.57,
                236.46,
                236.38,
                236.39,
                236.41,
                236.42,
                236.43,
                236.44,
                236.45,
                236.46,
                236.46,
                236.47,
                236.47,
                236.47,
                236.48,
                1869.35,
                1863.01,
                1856.81,
                1850.76,
                1844.85,
                1839.08,
                1833.46,
                1827.98,
                1822.65,
                1817.46,
                1812.41,
                1807.52,
                1802.76,
                1783.18,
                1757.11,
                1730.97,
                1704.78,
                1678.53,
                1652.22,
                1625.84,
                1599.41,
                1572.91,
                1546.36,
                1519.74,
                1493.06,
                1466.32,
                1439.52,
                1412.66,
                1385.74,
                1358.76,
                1331.71,
                1304.6,
                1279.01,
                1261.07,
                1243.13,
                1225.18,
                1207.21,
                1189.24,
                1171.26,
                1153.27,
                1135.28,
                1117.27,
                1099.26,
                1081.23,
                1063.2,
                1045.16,
                1027.11,
                1009.05,
                990.99,
                972.91,
                954.83,
                936.73,
                924.56,
                915.83,
                907.18,
                898.61,
                890.13,
                881.73,
                873.42,
                865.18,
                857.04,
                848.97,
                840.99,
                833.1,
                825.28,
                817.56,
                809.91,
                802.35,
                794.88,
                787.49,
                780.19,
                773.94,
                776.32,
                779.22,
                782.28,
                785.39,
                788.57,
                791.81,
                795.11,
                798.46,
                801.88,
                805.36,
                808.9,
                812.51,
                816.17,
                819.89,
                823.68,
                827.52,
                831.43,
                835.4,
                839.43,
                846.9,
                856.96,
                866.91,
                876.78,
                886.55,
                896.22,
                905.81,
                915.29,
                924.69,
                933.98,
                943.19,
                952.3,
                961.31,
                970.23,
                979.05,
                987.78,
                996.42,
                1004.96,
                1013.4,
                1021.34,
                1008.82,
                996.31,
                983.81,
                971.32,
                958.84,
                946.37,
                933.91,
                921.47,
                909.04,
                896.61,
                884.2,
                871.8,
                859.41,
                847.04,
                834.67,
                822.32,
                809.97,
                797.64,
                785.32,
                776.59,
                771.84,
                767.08,
                762.3,
                757.51,
                752.71,
                747.89,
                743.06,
                738.21,
                733.35,
                728.47,
                723.59,
                718.68,
                713.76,
                708.83,
                703.89,
                698.93,
                693.95,
                688.96,
                683.96,
                677.45,
                670.81,
                664.16,
                657.51,
                650.85,
                644.18,
                637.51,
                630.83,
                624.15,
                617.46,
                610.77,
                604.07,
                597.36,
                590.65,
                583.93,
                577.21,
                570.48,
                563.74,
                557.0,
                549.45,
                540.5,
                531.51,
                522.48,
                513.43,
                504.33,
                495.2,
                486.04,
                476.84,
                467.6,
                458.33,
                449.03,
                439.68,
                430.31,
                420.9,
                411.45,
                401.97,
                392.45,
                382.9,
                373.31,
                366.69,
                360.78,
                354.9,
                349.03,
                343.19,
                337.36,
                331.54,
                325.75,
                319.97,
                314.21,
                308.47,
                302.75,
                297.04,
                291.35,
                285.68,
                280.03,
                274.4,
                268.78,
                263.18,
                258.78,
                257.99,
                257.17,
                256.34,
                255.49,
                254.62,
                253.74,
                252.84,
                251.92,
                250.99,
                250.04,
                249.07,
                248.08,
                247.08,
                246.06,
                245.02,
                243.97,
                242.9,
                241.81,
                240.7,
                240.39,
                240.46,
                240.53,
                240.59,
                240.65,
                240.71,
                240.77,
                240.82,
                240.87,
                240.92,
                240.96,
                241.01,
                241.05,
                241.08,
                241.12,
                241.15,
                241.18,
                241.21,
                241.23,
                241.25,
                241.16,
                241.08,
                240.99,
                240.9,
                240.81,
                240.71,
                240.61,
                240.51,
                240.41,
                240.31,
                240.2,
                240.09,
                239.97,
                239.86,
                239.74,
                239.62,
                239.49,
                239.37,
                239.24,
                239.11,
                238.99,
                238.88,
                238.76,
                238.65,
                238.54,
                238.44,
                238.33,
                238.23,
                238.12,
                238.02,
                237.93,
                237.83,
                237.74,
                237.64,
                237.55,
                237.47,
                237.38,
                237.3,
                237.22,
                237.21,
                237.2,
                237.19,
                237.18,
                237.17,
                237.16,
                237.15,
                237.13,
                237.12,
                237.1,
                237.08,
                237.06,
                1839.91,
                1836.34,
                1832.91,
                1829.63,
                1826.49,
                1823.49,
                1820.64,
                1817.93,
                1815.37,
                1812.95,
                1810.67,
                1808.55,
                1806.56,
                1793.0,
                1765.78,
                1738.5,
                1711.15,
                1683.75,
                1656.28,
                1628.76,
                1601.17,
                1573.52,
                1545.82,
                1518.05,
                1490.22,
                1462.33,
                1434.38,
                1406.36,
                1378.29,
                1350.16,
                1321.96,
                1293.7,
                1265.38,
                1246.57,
                1228.47,
                1210.5,
                1193.84,
                1177.2,
                1160.6,
                1144.02,
                1127.48,
                1110.97,
                1094.49,
                1078.04,
                1061.62,
                1045.23,
                1028.87,
                1012.55,
                996.25,
                979.99,
                963.76,
                947.56,
                935.18,
                928.61,
                922.0,
                915.35,
                908.65,
                901.92,
                895.14,
                888.32,
                881.46,
                874.56,
                867.61,
                860.63,
                853.6,
                846.53,
                839.42,
                832.27,
                825.07,
                817.83,
                810.55,
                803.23,
                805.61,
                809.69,
                813.84,
                818.05,
                822.31,
                826.64,
                831.03,
                835.48,
                839.99,
                844.56,
                849.19,
                853.88,
                858.63,
                863.45,
                868.32,
                873.26,
                878.25,
                883.31,
                888.43,
                894.61,
                902.96,
                911.21,
                919.37,
                927.43,
                935.4,
                943.28,
                951.06,
                958.74,
                966.34,
                973.83,
                981.24,
                988.54,
                995.76,
                1002.88,
                1009.9,
                1016.83,
                1023.66,
                1030.4,
                1037.04,
                1029.15,
                1016.83,
                1004.52,
                992.22,
                979.94,
                967.66,
                955.4,
                943.15,
                930.91,
                918.68,
                906.46,
                894.25,
                882.05,
                869.87,
                857.7,
                845.53,
                833.38,
                821.24,
                809.12,
                798.53,
                793.53,
                788.51,
                783.49,
                778.44,
                773.39,
                768.31,
                763.23,
                758.13,
                753.02,
                747.89,
                742.75,
                737.59,
                732.42,
                727.23,
                722.04,
                716.82,
                711.59,
                706.35,
                701.1,
                694.86,
                688.11,
                681.36,
                674.6,
                667.83,
                661.06,
                654.29,
                647.51,
                640.72,
                633.92,
                627.13,
                620.32,
                613.51,
                606.69,
                599.87,
                593.04,
                586.21,
                579.37,
                572.52,
                565.4,
                555.84,
                546.25,
                536.63,
                526.96,
                517.27,
                507.54,
                497.77,
                487.97,
                478.14,
                468.27,
                458.36,
                448.42,
                438.44,
                428.43,
                418.38,
                408.3,
                398.18,
                388.03,
                377.84,
                370.11,
                364.51,
                358.92,
                353.35,
                347.8,
                342.27,
                336.75,
                331.25,
                325.77,
                320.31,
                314.86,
                309.44,
                304.03,
                298.64,
                293.26,
                287.91,
                282.57,
                277.25,
                271.95,
                266.66,
                265.48,
                264.38,
                263.26,
                262.13,
                260.98,
                259.81,
                258.63,
                257.42,
                256.2,
                254.97,
                253.71,
                252.44,
                251.16,
                249.85,
                248.53,
                247.19,
                245.83,
                244.46,
                243.07,
                242.25,
                242.27,
                242.3,
                242.32,
                242.34,
                242.35,
                242.36,
                242.38,
                242.38,
                242.39,
                242.39,
                242.39,
                242.39,
                242.39,
                242.38,
                242.37,
                242.36,
                242.35,
                242.33,
                242.31,
                242.2,
                242.07,
                241.94,
                241.81,
                241.68,
                241.54,
                241.4,
                241.26,
                241.11,
                240.96,
                240.81,
                240.66,
                240.5,
                240.35,
                240.19,
                240.02,
                239.86,
                239.69,
                239.52,
                239.36,
                239.28,
                239.19,
                239.11,
                239.03,
                238.95,
                238.88,
                238.8,
                238.73,
                238.66,
                238.59,
                238.53,
                238.46,
                238.4,
                238.34,
                238.28,
                238.22,
                238.17,
                238.12,
                238.07,
                238.03,
                238.01,
                237.98,
                237.95,
                237.91,
                237.88,
                237.85,
                237.81,
                237.77,
                237.74,
                237.7,
                237.66,
                1814.36,
                1814.66,
                1814.91,
                1815.11,
                1815.27,
                1815.38,
                1815.43,
                1815.44,
                1815.41,
                1815.32,
                1815.18,
                1815.0,
                1814.77,
                1808.45,
                1781.32,
                1754.24,
                1727.2,
                1700.22,
                1673.28,
                1646.39,
                1619.55,
                1592.76,
                1566.01,
                1539.31,
                1512.67,
                1486.07,
                1459.51,
                1433.01,
                1406.56,
                1380.15,
                1353.79,
                1327.48,
                1301.22,
                1281.91,
                1265.75,
                1249.62,
                1233.52,
                1217.45,
                1201.41,
                1185.4,
                1169.42,
                1153.47,
                1137.56,
                1121.67,
                1105.82,
                1089.99,
                1074.2,
                1058.44,
                1042.71,
                1027.01,
                1011.34,
                995.71,
                981.34,
                973.99,
                966.6,
                959.16,
                951.68,
                944.17,
                936.61,
                929.0,
                921.36,
                913.68,
                905.95,
                898.18,
                890.37,
                882.52,
                874.62,
                866.69,
                858.71,
                850.69,
                842.63,
                834.52,
                834.44,
                839.62,
                844.86,
                850.15,
                855.51,
                860.93,
                866.41,
                871.94,
                877.54,
                883.2,
                888.92,
                894.71,
                900.55,
                906.45,
                912.42,
                918.44,
                924.53,
                930.68,
                936.89,
                943.18,
                949.82,
                956.37,
                962.82,
                969.18,
                975.44,
                981.61,
                987.69,
                993.67,
                999.55,
                1005.35,
                1011.04,
                1016.65,
                1022.15,
                1027.57,
                1032.88,
                1038.11,
                1043.24,
                1048.27,
                1053.2,
                1049.38,
                1037.25,
                1025.14,
                1013.03,
                1000.94,
                988.86,
                976.78,
                964.73,
                952.68,
                940.64,
                928.61,
                916.6,
                904.6,
                892.6,
                880.62,
                868.65,
                856.69,
                844.75,
                832.81,
                820.89,
                815.35,
                810.08,
                804.8,
                799.51,
                794.19,
                788.87,
                783.53,
                778.18,
                772.81,
                767.43,
                762.04,
                756.63,
                751.2,
                745.77,
                740.32,
                734.85,
                729.37,
                723.87,
                718.37,
                712.31,
                705.46,
                698.61,
                691.74,
                684.87,
                678.0,
                671.12,
                664.23,
                657.34,
                650.44,
                643.54,
                636.63,
                629.72,
                622.79,
                615.87,
                608.93,
                602.0,
                595.05,
                588.1,
                581.15,
                571.5,
                561.31,
                551.09,
                540.83,
                530.53,
                520.2,
                509.83,
                499.43,
                489.0,
                478.52,
                468.02,
                457.47,
                446.9,
                436.28,
                425.64,
                414.95,
                404.23,
                393.48,
                382.69,
                373.38,
                368.07,
                362.78,
                357.51,
                352.25,
                347.01,
                341.8,
                336.59,
                331.41,
                326.24,
                321.1,
                315.96,
                310.85,
                305.76,
                300.68,
                295.62,
                290.58,
                285.56,
                280.55,
                275.56,
                273.13,
                271.74,
                270.34,
                268.92,
                267.49,
                266.04,
                264.57,
                263.08,
                261.58,
                260.05,
                258.52,
                256.96,
                255.39,
                253.8,
                252.19,
                250.57,
                248.93,
                247.27,
                245.59,
                244.12,
                244.11,
                244.09,
                244.07,
                244.04,
                244.02,
                243.99,
                243.96,
                243.92,
                243.89,
                243.85,
                243.81,
                243.76,
                243.72,
                243.67,
                243.62,
                243.56,
                243.5,
                243.44,
                243.38,
                243.26,
                243.09,
                242.92,
                242.75,
                242.57,
                242.39,
                242.21,
                242.02,
                241.84,
                241.65,
                241.45,
                241.26,
                241.06,
                240.86,
                240.66,
                240.45,
                240.24,
                240.03,
                239.82,
                239.6,
                239.54,
                239.49,
                239.44,
                239.39,
                239.35,
                239.3,
                239.26,
                239.22,
                239.18,
                239.14,
                239.11,
                239.08,
                239.04,
                239.02,
                238.99,
                238.96,
                238.94,
                238.92,
                238.9,
                238.87,
                238.82,
                238.77,
                238.72,
                238.67,
                238.62,
                238.56,
                238.5,
                238.45,
                238.39,
                238.33,
                238.26,
                1842.31,
                1841.68,
                1840.99,
                1840.26,
                1839.48,
                1838.66,
                1837.78,
                1836.86,
                1835.89,
                1834.87,
                1833.8,
                1832.68,
                1831.52,
                1830.3,
                1804.38,
                1778.21,
                1752.08,
                1726.0,
                1699.97,
                1673.98,
                1648.05,
                1622.16,
                1596.32,
                1570.53,
                1544.79,
                1519.09,
                1493.45,
                1467.85,
                1442.3,
                1416.8,
                1391.35,
                1365.94,
                1340.59,
                1319.63,
                1304.03,
                1288.46,
                1272.92,
                1257.42,
                1241.94,
                1226.5,
                1211.08,
                1195.7,
                1180.35,
                1165.03,
                1149.74,
                1134.48,
                1119.25,
                1104.05,
                1088.89,
                1073.75,
                1058.65,
                1043.58,
                1028.54,
                1019.76,
                1011.58,
                1003.36,
                995.1,
                986.8,
                978.46,
                970.07,
                961.65,
                953.18,
                944.67,
                936.12,
                927.53,
                918.89,
                910.21,
                901.5,
                892.73,
                883.93,
                875.09,
                866.2,
                862.74,
                869.0,
                875.33,
                881.71,
                888.16,
                894.67,
                901.24,
                907.87,
                914.55,
                921.3,
                928.12,
                934.99,
                941.92,
                948.91,
                955.97,
                963.08,
                970.26,
                977.5,
                984.79,
                992.15,
                997.55,
                1002.39,
                1007.13,
                1011.79,
                1016.34,
                1020.81,
                1025.18,
                1029.45,
                1033.64,
                1037.72,
                1041.72,
                1045.61,
                1049.42,
                1053.13,
                1056.74,
                1060.26,
                1063.68,
                1067.01,
                1070.24,
                1069.51,
                1057.58,
                1045.65,
                1033.74,
                1021.84,
                1009.95,
                998.07,
                986.2,
                974.35,
                962.5,
                950.67,
                938.85,
                927.04,
                915.24,
                903.45,
                891.67,
                879.91,
                868.15,
                856.41,
                844.68,
                837.3,
                831.78,
                826.25,
                820.7,
                815.14,
                809.56,
                803.97,
                798.36,
                792.74,
                787.11,
                781.46,
                775.8,
                770.12,
                764.43,
                758.73,
                753.01,
                747.28,
                741.53,
                735.77,
                729.83,
                722.87,
                715.91,
                708.94,
                701.97,
                694.99,
                688.01,
                681.02,
                674.02,
                667.02,
                660.01,
                652.99,
                645.98,
                638.95,
                631.92,
                624.88,
                617.84,
                610.79,
                603.74,
                596.68,
                587.48,
                576.69,
                565.87,
                555.01,
                544.11,
                533.18,
                522.21,
                511.21,
                500.17,
                489.1,
                478.0,
                466.85,
                455.68,
                444.46,
                433.22,
                421.93,
                410.61,
                399.26,
                387.87,
                376.48,
                371.47,
                366.48,
                361.5,
                356.54,
                351.6,
                346.68,
                341.77,
                336.89,
                332.02,
                327.17,
                322.33,
                317.52,
                312.72,
                307.94,
                303.17,
                298.43,
                293.7,
                288.99,
                284.3,
                280.93,
                279.27,
                277.58,
                275.88,
                274.16,
                272.42,
                270.66,
                268.89,
                267.1,
                265.3,
                263.48,
                261.64,
                259.78,
                257.9,
                256.01,
                254.1,
                252.18,
                250.24,
                248.28,
                246.3,
                245.96,
                245.9,
                245.84,
                245.77,
                245.71,
                245.63,
                245.56,
                245.49,
                245.41,
                245.33,
                245.24,
                245.16,
                245.07,
                244.98,
                244.88,
                244.79,
                244.69,
                244.59,
                244.48,
                244.35,
                244.14,
                243.92,
                243.7,
                243.49,
                243.26,
                243.04,
                242.81,
                242.58,
                242.35,
                242.12,
                241.88,
                241.64,
                241.4,
                241.15,
                240.9,
                240.65,
                240.4,
                240.14,
                239.89,
                239.79,
                239.77,
                239.75,
                239.73,
                239.72,
                239.71,
                239.7,
                239.69,
                239.68,
                239.67,
                239.67,
                239.67,
                239.67,
                239.67,
                239.68,
                239.69,
                239.7,
                239.71,
                239.72,
                239.73,
                239.66,
                239.58,
                239.51,
                239.44,
                239.36,
                239.29,
                239.21,
                239.13,
                239.05,
                238.97,
                238.92,
                1870.7,
                1869.13,
                1867.52,
                1865.85,
                1864.14,
                1862.38,
                1860.57,
                1858.72,
                1856.81,
                1854.86,
                1852.86,
                1850.81,
                1848.71,
                1846.56,
                1827.01,
                1801.74,
                1776.52,
                1751.35,
                1726.22,
                1701.14,
                1676.11,
                1651.13,
                1626.2,
                1601.31,
                1576.47,
                1551.68,
                1526.94,
                1502.25,
                1477.61,
                1453.01,
                1428.47,
                1403.97,
                1379.52,
                1357.06,
                1342.03,
                1327.03,
                1312.05,
                1297.11,
                1282.2,
                1267.32,
                1252.47,
                1237.65,
                1222.86,
                1208.11,
                1193.38,
                1178.69,
                1164.02,
                1149.39,
                1134.79,
                1120.22,
                1105.68,
                1091.17,
                1076.7,
                1065.9,
                1056.95,
                1047.94,
                1038.9,
                1029.82,
                1020.69,
                1011.53,
                1002.32,
                993.07,
                983.78,
                974.44,
                965.07,
                955.65,
                946.19,
                936.69,
                927.15,
                917.56,
                907.94,
                898.27,
                890.49,
                897.84,
                905.26,
                912.73,
                920.27,
                927.87,
                935.52,
                943.24,
                951.02,
                958.86,
                966.76,
                974.72,
                982.74,
                990.82,
                998.97,
                1007.17,
                1015.44,
                1023.76,
                1032.15,
                1040.6,
                1046.13,
                1049.27,
                1052.31,
                1055.26,
                1058.11,
                1060.87,
                1063.54,
                1066.11,
                1068.59,
                1070.97,
                1073.26,
                1075.45,
                1077.55,
                1079.55,
                1081.46,
                1083.28,
                1085.0,
                1086.62,
                1088.15,
                1089.54,
                1077.8,
                1066.07,
                1054.35,
                1042.64,
                1030.94,
                1019.26,
                1007.58,
                995.92,
                984.27,
                972.63,
                961.0,
                949.38,
                937.77,
                926.18,
                914.59,
                903.02,
                891.46,
                879.9,
                868.36,
                859.39,
                853.61,
                847.83,
                842.02,
                836.21,
                830.38,
                824.53,
                818.68,
                812.8,
                806.92,
                801.02,
                795.1,
                789.17,
                783.23,
                777.27,
                771.3,
                765.32,
                759.32,
                753.3,
                747.27,
                740.34,
                733.27,
                726.2,
                719.12,
                712.04,
                704.95,
                697.85,
                690.75,
                683.65,
                676.53,
                669.41,
                662.29,
                655.16,
                648.03,
                640.88,
                633.74,
                626.58,
                619.42,
                612.26,
                603.79,
                592.4,
                580.97,
                569.51,
                558.01,
                546.48,
                534.91,
                523.31,
                511.68,
                500.0,
                488.3,
                476.55,
                464.78,
                452.96,
                441.12,
                429.23,
                417.31,
                405.36,
                393.37,
                381.34,
                374.71,
                370.02,
                365.34,
                360.67,
                356.03,
                351.4,
                346.8,
                342.2,
                337.63,
                333.08,
                328.54,
                324.02,
                319.52,
                315.03,
                310.57,
                306.12,
                301.69,
                297.27,
                292.88,
                288.9,
                286.94,
                284.97,
                282.98,
                280.98,
                278.96,
                276.92,
                274.86,
                272.79,
                270.7,
                268.59,
                266.47,
                264.32,
                262.17,
                259.99,
                257.8,
                255.59,
                253.36,
                251.11,
                248.85,
                247.84,
                247.74,
                247.64,
                247.53,
                247.42,
                247.31,
                247.19,
                247.07,
                246.95,
                246.83,
                246.7,
                246.57,
                246.44,
                246.31,
                246.17,
                246.03,
                245.89,
                245.75,
                245.6,
                245.45,
                245.2,
                244.95,
                244.69,
                244.43,
                244.16,
                243.89,
                243.63,
                243.35,
                243.08,
                242.8,
                242.52,
                242.24,
                241.96,
                241.67,
                241.38,
                241.09,
                240.79,
                240.59,
                240.38,
                240.3,
                240.4,
                240.49,
                240.57,
                240.66,
                240.73,
                240.81,
                240.88,
                240.95,
                241.01,
                241.07,
                241.13,
                241.18,
                241.23,
                241.27,
                241.31,
                241.34,
                241.38,
                241.4,
                241.43,
                241.39,
                241.34,
                241.28,
                241.23,
                241.17,
                241.11,
                241.05,
                240.99,
                240.93,
                240.86,
                240.8,
                1899.53,
                1897.03,
                1894.48,
                1891.88,
                1889.24,
                1886.54,
                1883.8,
                1881.01,
                1878.18,
                1875.29,
                1872.36,
                1869.38,
                1866.35,
                1863.27,
                1849.21,
                1824.84,
                1800.53,
                1776.26,
                1752.04,
                1727.87,
                1703.74,
                1679.66,
                1655.64,
                1631.66,
                1607.72,
                1583.84,
                1560.0,
                1536.22,
                1512.48,
                1488.79,
                1465.15,
                1441.55,
                1418.01,
                1394.51,
                1379.76,
                1365.32,
                1350.91,
                1336.53,
                1322.19,
                1307.87,
                1293.58,
                1279.33,
                1265.11,
                1250.91,
                1236.75,
                1222.62,
                1208.52,
                1194.45,
                1180.42,
                1166.41,
                1152.43,
                1138.49,
                1124.58,
                1112.44,
                1102.7,
                1092.91,
                1083.09,
                1073.22,
                1063.32,
                1053.37,
                1043.38,
                1033.35,
                1023.27,
                1013.16,
                1003.0,
                992.8,
                982.56,
                972.28,
                961.95,
                951.59,
                941.18,
                930.73,
                920.23,
                926.14,
                934.64,
                943.21,
                951.83,
                960.52,
                969.27,
                978.07,
                986.94,
                995.87,
                1004.86,
                1013.91,
                1023.02,
                1032.19,
                1041.42,
                1050.71,
                1060.07,
                1069.48,
                1078.96,
                1088.49,
                1095.58,
                1097.01,
                1098.35,
                1099.59,
                1100.74,
                1101.8,
                1102.76,
                1103.63,
                1104.4,
                1105.08,
                1105.66,
                1106.15,
                1106.55,
                1106.85,
                1107.06,
                1107.17,
                1107.18,
                1107.1,
                1106.93,
                1106.66,
                1097.93,
                1086.39,
                1074.86,
                1063.35,
                1051.84,
                1040.35,
                1028.86,
                1017.39,
                1005.93,
                994.48,
                983.05,
                971.62,
                960.21,
                948.8,
                937.41,
                926.03,
                914.66,
                903.3,
                891.95,
                881.6,
                875.58,
                869.53,
                863.48,
                857.41,
                851.33,
                845.23,
                839.12,
                833.0,
                826.86,
                820.7,
                814.54,
                808.36,
                802.16,
                795.95,
                789.73,
                783.49,
                777.24,
                770.97,
                764.69,
                757.86,
                750.69,
                743.51,
                736.33,
                729.14,
                721.95,
                714.75,
                707.54,
                700.33,
                693.11,
                685.89,
                678.66,
                671.43,
                664.19,
                656.94,
                649.69,
                642.43,
                635.17,
                627.9,
                620.41,
                608.42,
                596.39,
                584.33,
                572.23,
                560.1,
                547.94,
                535.73,
                523.5,
                511.23,
                498.92,
                486.58,
                474.2,
                461.79,
                449.34,
                436.86,
                424.34,
                411.78,
                399.19,
                386.57,
                377.79,
                373.39,
                369.01,
                364.65,
                360.3,
                355.97,
                351.66,
                347.36,
                343.08,
                338.82,
                334.58,
                330.36,
                326.15,
                321.97,
                317.8,
                313.64,
                309.51,
                305.39,
                301.29,
                297.21,
                294.77,
                292.52,
                290.25,
                287.96,
                285.65,
                283.33,
                280.99,
                278.63,
                276.25,
                273.86,
                271.45,
                269.03,
                266.58,
                264.12,
                261.65,
                259.15,
                256.64,
                254.11,
                251.57,
                249.75,
                249.6,
                249.46,
                249.31,
                249.24,
                249.22,
                249.2,
                249.18,
                249.16,
                249.14,
                249.11,
                249.09,
                249.06,
                249.03,
                249.0,
                248.97,
                248.94,
                248.9,
                248.87,
                248.84,
                248.69,
                248.49,
                248.28,
                248.07,
                247.84,
                247.61,
                247.37,
                247.13,
                246.88,
                246.61,
                246.35,
                246.07,
                245.78,
                245.49,
                245.19,
                244.89,
                244.57,
                244.25,
                243.92,
                243.63,
                243.66,
                243.68,
                243.71,
                243.73,
                243.74,
                243.75,
                243.76,
                243.77,
                243.77,
                243.76,
                243.76,
                243.74,
                243.73,
                243.71,
                243.69,
                243.66,
                243.63,
                243.59,
                243.55,
                243.49,
                243.42,
                243.34,
                243.27,
                243.19,
                243.11,
                243.03,
                242.95,
                242.86,
                242.78,
                242.69,
                1928.8,
                1925.37,
                1921.88,
                1918.35,
                1914.78,
                1911.15,
                1907.48,
                1903.76,
                1899.99,
                1896.17,
                1892.3,
                1888.39,
                1884.43,
                1880.41,
                1870.97,
                1847.51,
                1824.1,
                1800.74,
                1777.42,
                1754.16,
                1730.94,
                1707.77,
                1684.64,
                1661.57,
                1638.54,
                1615.56,
                1592.63,
                1569.75,
                1546.91,
                1524.13,
                1501.39,
                1478.7,
                1456.06,
                1433.47,
                1417.21,
                1403.34,
                1389.49,
                1375.68,
                1361.9,
                1348.15,
                1334.42,
                1320.73,
                1307.07,
                1293.44,
                1279.85,
                1266.28,
                1252.74,
                1239.24,
                1225.77,
                1212.32,
                1198.91,
                1185.53,
                1172.18,
                1159.35,
                1148.83,
                1138.27,
                1127.66,
                1117.01,
                1106.32,
                1095.59,
                1084.82,
                1074.01,
                1063.15,
                1052.26,
                1041.32,
                1030.34,
                1019.31,
                1008.25,
                997.14,
                985.99,
                974.8,
                963.57,
                952.3,
                953.89,
                963.49,
                973.14,
                982.85,
                992.63,
                1002.46,
                1012.36,
                1022.31,
                1032.33,
                1042.41,
                1052.55,
                1062.74,
                1073.0,
                1083.32,
                1093.7,
                1104.15,
                1114.65,
                1125.21,
                1135.84,
                1145.88,
                1145.61,
                1145.25,
                1144.79,
                1144.23,
                1143.58,
                1142.84,
                1142.01,
                1141.08,
                1140.05,
                1138.94,
                1137.72,
                1136.42,
                1135.01,
                1133.52,
                1131.93,
                1130.24,
                1128.46,
                1126.58,
                1124.61,
                1117.96,
                1106.61,
                1095.27,
                1083.95,
                1072.64,
                1061.34,
                1050.05,
                1038.77,
                1027.5,
                1016.24,
                1005.0,
                993.76,
                982.54,
                971.33,
                960.13,
                948.94,
                937.76,
                926.6,
                915.44,
                904.3,
                897.67,
                891.38,
                885.07,
                878.75,
                872.41,
                866.06,
                859.7,
                853.32,
                846.93,
                840.52,
                834.1,
                827.67,
                821.22,
                814.76,
                808.28,
                801.79,
                795.29,
                788.77,
                782.24,
                775.43,
                768.16,
                760.87,
                753.59,
                746.3,
                739.0,
                731.69,
                724.38,
                717.07,
                709.75,
                702.42,
                695.09,
                687.75,
                680.4,
                673.05,
                665.7,
                658.34,
                650.97,
                643.59,
                636.22,
                624.76,
                612.13,
                599.47,
                586.78,
                574.04,
                561.28,
                548.48,
                535.64,
                522.77,
                509.86,
                496.92,
                483.94,
                470.93,
                457.89,
                444.8,
                431.68,
                418.53,
                405.34,
                392.12,
                380.72,
                376.61,
                372.53,
                368.46,
                364.4,
                360.37,
                356.35,
                352.36,
                348.38,
                344.44,
                340.8,
                337.17,
                333.55,
                329.92,
                326.31,
                322.69,
                319.08,
                315.48,
                311.88,
                308.28,
                305.46,
                303.09,
                300.7,
                298.27,
                295.82,
                293.33,
                290.82,
                288.28,
                285.71,
                283.11,
                280.48,
                277.83,
                275.14,
                272.42,
                269.68,
                266.9,
                264.1,
                261.27,
                258.4,
                255.61,
                255.57,
                255.54,
                255.5,
                255.46,
                255.42,
                255.38,
                255.34,
                255.29,
                255.25,
                255.2,
                255.16,
                255.11,
                255.06,
                255.01,
                254.96,
                254.9,
                254.85,
                254.79,
                254.74,
                254.56,
                254.24,
                253.9,
                253.56,
                253.21,
                252.86,
                252.49,
                252.12,
                251.74,
                251.35,
                250.96,
                250.56,
                250.15,
                249.73,
                249.3,
                248.87,
                248.42,
                247.98,
                247.52,
                247.05,
                246.95,
                246.92,
                246.88,
                246.83,
                246.79,
                246.74,
                246.68,
                246.62,
                246.56,
                246.49,
                246.42,
                246.35,
                246.27,
                246.18,
                246.1,
                246.01,
                245.91,
                245.81,
                245.71,
                245.61,
                245.51,
                245.42,
                245.32,
                245.22,
                245.12,
                245.02,
                244.92,
                244.81,
                244.71,
                244.6,
                1958.51,
                1954.14,
                1949.73,
                1945.27,
                1940.76,
                1936.2,
                1931.59,
                1926.94,
                1922.24,
                1917.49,
                1912.69,
                1907.84,
                1902.95,
                1898.01,
                1892.31,
                1869.75,
                1847.25,
                1824.79,
                1802.38,
                1780.01,
                1757.7,
                1735.43,
                1713.21,
                1691.04,
                1668.92,
                1646.85,
                1624.82,
                1602.84,
                1580.91,
                1559.03,
                1537.2,
                1515.41,
                1493.68,
                1471.99,
                1454.4,
                1441.09,
                1427.8,
                1414.55,
                1401.34,
                1388.15,
                1374.99,
                1361.86,
                1348.77,
                1335.7,
                1322.67,
                1309.66,
                1296.69,
                1283.75,
                1270.84,
                1257.96,
                1245.11,
                1232.29,
                1219.51,
                1206.75,
                1195.35,
                1184.0,
                1172.62,
                1161.19,
                1149.72,
                1138.2,
                1126.65,
                1115.06,
                1103.42,
                1091.74,
                1080.02,
                1068.26,
                1056.45,
                1044.61,
                1032.72,
                1020.79,
                1008.82,
                996.81,
                984.75,
                981.11,
                991.79,
                1002.53,
                1013.33,
                1024.19,
                1035.12,
                1046.1,
                1057.14,
                1068.25,
                1079.41,
                1090.64,
                1101.93,
                1113.27,
                1124.68,
                1136.15,
                1147.68,
                1159.27,
                1170.92,
                1182.64,
                1194.41,
                1195.07,
                1193.0,
                1190.84,
                1188.59,
                1186.23,
                1183.79,
                1181.25,
                1178.62,
                1175.89,
                1173.07,
                1170.16,
                1167.15,
                1164.04,
                1160.85,
                1157.55,
                1154.16,
                1150.68,
                1147.1,
                1143.43,
                1137.88,
                1126.73,
                1115.59,
                1104.46,
                1093.34,
                1082.23,
                1071.13,
                1060.04,
                1048.97,
                1037.9,
                1026.85,
                1015.81,
                1004.78,
                993.76,
                982.75,
                971.75,
                960.77,
                949.79,
                938.83,
                927.88,
                919.89,
                913.35,
                906.79,
                900.21,
                893.63,
                887.02,
                880.41,
                873.78,
                867.13,
                860.48,
                853.8,
                847.12,
                840.42,
                833.7,
                826.97,
                820.23,
                813.47,
                806.7,
                799.91,
                793.06,
                785.68,
                778.29,
                770.9,
                763.51,
                756.1,
                748.7,
                741.28,
                733.86,
                726.44,
                719.01,
                711.57,
                704.13,
                696.68,
                689.02,
                680.73,
                672.47,
                664.22,
                655.99,
                647.78,
                636.78,
                623.24,
                609.77,
                596.39,
                583.09,
                569.88,
                556.74,
                543.69,
                530.72,
                517.84,
                505.03,
                492.31,
                479.68,
                467.12,
                454.65,
                442.26,
                429.96,
                417.74,
                405.6,
                393.55,
                389.4,
                385.79,
                382.19,
                378.6,
                375.01,
                371.42,
                367.84,
                364.26,
                360.69,
                357.12,
                353.55,
                349.99,
                346.43,
                342.88,
                339.33,
                335.78,
                332.24,
                328.7,
                325.17,
                321.88,
                319.03,
                316.15,
                313.24,
                310.3,
                307.33,
                304.34,
                301.31,
                298.26,
                295.17,
                292.06,
                288.92,
                285.74,
                282.54,
                279.31,
                276.06,
                272.77,
                269.45,
                266.1,
                262.73,
                261.87,
                261.81,
                261.75,
                261.69,
                261.63,
                261.57,
                261.5,
                261.44,
                261.37,
                261.31,
                261.24,
                261.17,
                261.1,
                261.03,
                260.95,
                260.88,
                260.8,
                260.73,
                260.65,
                260.51,
                260.06,
                259.6,
                259.13,
                258.66,
                258.17,
                257.68,
                257.18,
                256.68,
                256.16,
                255.64,
                255.11,
                254.58,
                254.03,
                253.48,
                252.92,
                252.35,
                251.78,
                251.19,
                250.6,
                250.29,
                250.19,
                250.09,
                249.98,
                249.87,
                249.75,
                249.64,
                249.51,
                249.39,
                249.26,
                249.12,
                248.98,
                248.84,
                248.7,
                248.55,
                248.39,
                248.24,
                248.07,
                247.91,
                247.74,
                247.62,
                247.5,
                247.38,
                247.26,
                247.14,
                247.02,
                246.9,
                246.77,
                246.64,
                246.52],
            "height": 135 //151
        });
        this.map.addLayer(demLayer);
        this.map.currentBasemap = demLayer;

        this.gridlayer = new GridLayer({ center: this.map.center, name: "coordinate grid", appWidth: this.width, appHeight: this.height });
        this.map.addLayer(this.gridlayer);

        new LayerControl(this.map.layers, {
            collapsed: true,
            parentDiv: 'layer-control-parent-id'
        }).addTo(this.map);

        this.basemapControl = new BasemapControl('Baselayer', {
            position: 'topright'
        }).addTo(this.map);

        //slider for scaling z value
        this.slider = new SliderControl({ layers: this.map.layers }).addTo(this.map);

        //slice on x and y axes:
        // this.slicer = new SlicerControl({ parentDiv: 'slicer-control' }).addTo(this.map);


        this.capsScene.add(this.selectionBox.boxMesh);
        // this.scene.add(this.selection.displayMeshes);
        // this.scene.add(this.selection.touchMeshes);

        this.start();
    }

    keydown(e) {
        if (e.ctrlKey || e.altKey) return;
        let keyPressed = e.which;
        if (!e.shiftKey) {
            //if (keyPressed == 27) app.closePopup(); // ESC           
            if (keyPressed === 87) {
                this.setWireframeMode();    // W
            }
        }
    }

    setWireframeMode() {
        let wireframe = !this.wireframeMode;
        if (wireframe === this.wireframeMode) return;

        for (var key in this.map._layers) {
            let layer = this.map._layers[key];
            layer.setWireframeMode(wireframe);
        }

        this.wireframeMode = wireframe;
        this.animate();
    }

    onWindowResize() {
        if (this._fullWindow) {
            this._setCanvasSize(window.innerWidth, window.innerHeight);
        }
        else {
            this._setCanvasSize(this.container.clientWidth, this.container.clientHeight);
        }
    }

    _setCanvasSize(width, height) {
        this.width = width;
        this.height = height;
        this.map.camera.aspect = width / height;
        this.map.camera.updateProjectionMatrix();
        this.renderer.setSize(width, height);
        this.animate();
    }

    _buildDefaultLights(scene) {
        let deg2rad = Math.PI / 180;

        // ambient light
        scene.add(new AmbientLight(0x999999));
        //scene.add(new THREE.AmbientLight(0xeeeeee));

        // directional lights
        let opt = {
            azimuth: 220,   // note: default light azimuth of gdaldem hillshade is 315.
            altitude: 45    // altitude angle
        };
        //appSettings.Options.light.directional;
        let lambda = (90 - opt.azimuth) * deg2rad;
        let phi = opt.altitude * deg2rad;

        let x = Math.cos(phi) * Math.cos(lambda),
            y = Math.cos(phi) * Math.sin(lambda),
            z = Math.sin(phi);

        let light1 = new DirectionalLight(0xffffff, 0.5);
        light1.position.set(x, y, z);
        scene.add(light1);

        // thin light from the opposite direction
        let light2 = new DirectionalLight(0xffffff, 0.1);
        light2.position.set(-x, -y, -z);
        scene.add(light2);
    }

    start() {
        this.running = true;

        this.map.addListener('change', this.animate, this); // add this only if there is no animation loop (requestAnimationFrame)
        this.animate();
    }

    deferringThrottle = _.throttle(this.animate, 40);

    animate() {
        this.renderer.clear();

        // The HTML5 Canvas's 'webgl' context obtained from the canvas where the renderer will draw.
        let gl = this.renderer.context;

        if (this.showCaps && gl != undefined) {

            // enable stencil test
            gl.enable(gl.STENCIL_TEST);
            // gl.stencilFunc( gl.ALWAYS, 1, 0xff );
            // gl.stencilOp( gl.REPLACE, gl.REPLACE, gl.REPLACE );

            this.renderer.state.setStencilTest(true);

            gl.StencilFunc(gl.ALWAYS, 1, 0xff);
            gl.StencilOp(gl.KEEP, gl.KEEP, gl.INCR);
            // this.renderer.render(this.backStencil, this.camera);

            gl.stencilFunc(gl.ALWAYS, 1, 0xff);
            gl.StencilOp(gl.KEEP, gl.KEEP, gl.DECR);
            // this.renderer.render(this.frontStencil, this.camera);

            gl.StencilFunc(gl.EQUAL, 1, 0xff);
            gl.StencilOp(gl.KEEP, gl.KEEP, gl.KEEP);
            this.renderer.render(this.capsScene, this.camera);

            this.renderer.state.setStencilTest(false);

            // disable stencil test
            gl.disable(gl.STENCIL_TEST);
        }

        this.renderer.render(this.scene, this.map.camera);
        this.northArrow.animate();
        this.gridlayer.animate();
    }

    addEventListeners() {
        domEvent.on(window, 'resize', this.onWindowResize, this);
        domEvent.on(window, 'keydown', this.keydown, this);

        // let inputNodes = [this.$topTextInput, this.$bottomTextInput, this.$imageInput];
        // inputNodes.forEach(element => domEvent.on(element, 'keyup', this.createMeme, this));
        // //if image is changed
        // inputNodes.forEach(element => domEvent.on(element, 'change', this.createMeme, this));

        domEvent.on(this.downloadButton, 'click', this.downloadMapImage, this);

        domEvent.on(this.aboutIcon, 'click', function (e) {
            e.preventDefault();
            this.dialog.show();
        }, this);

        // domEvent.on(this.menuIcon, 'click', function (e) {
        //     e.preventDefault();
        //     this.navigation.classList.toggle("active");
        // }, this);

        const $navbarBurgers = Array.prototype.slice.call(document.querySelectorAll('.navbar-burger'), 0);
        // Check if there are any navbar burgers
        if ($navbarBurgers.length > 0) {
            // Add a click event on each of them
            $navbarBurgers.forEach(el => {
                el.addEventListener('click', () => {

                    // Get the target from the "data-target" attribute
                    const target = el.dataset.target;
                    const $target = document.getElementById(target);

                    // Toggle the "is-active" class on both the "navbar-burger" and the "navbar-menu"
                    el.classList.toggle('is-active');
                    $target.classList.toggle('is-active');

                });
            });
        }

        let tabButtons = [].slice.call(document.querySelectorAll('ul.tab-nav li'));

        tabButtons.map(function (button) {
            button.addEventListener('click', function () {
                document.querySelector('li.is-active').classList.remove('is-active');
                button.classList.add('is-active');

                document.querySelector('.tab-pane.active').classList.remove('active');
                document.querySelector(button.getAttribute('name')).classList.add('active');
            })
        });

        //toggle GridLayer
        let chkGrid = document.getElementById("chkGrid");
        domEvent.on(chkGrid, 'click', function (e) {
            this.gridlayer.toggle();
        }, this);

         //toggle SlicingBox
         let chkSlicingBox = document.getElementById("chkSlicingBox");
         domEvent.on(chkSlicingBox, 'click', function (e) {
             this.selectionBox.toggle();
         }, this);

    }

    downloadMapImage() {
        // if(!this.$imageInput.files[0]) {
        //   this.$imageInput.parentElement.classList.add('has-error');
        //   return;
        // }
        // if(this.$bottomTextInput.value === '') {
        //   this.$imageInput.parentElement.classList.remove('has-error');
        //   this.$bottomTextInput.parentElement.classList.add('has-error');
        //   return;
        // }
        // this.$imageInput.parentElement.classList.remove('has-error');
        // this.$bottomTextInput.parentElement.classList.remove('has-error');

        // const imageSource = this.renderer.domElement.toDataURL('image/png');
        // const att = document.createAttribute('href');
        // att.value = imageSource.replace(/^data:image\/[^;]/, 'data:application/octet-stream');
        // this.downloadButton.setAttributeNode(att);
        // this.renderer.preserveDrawingBuffer = true;
        // this.renderer.render(this.scene, this.camera);
        this.saveCanvasImage(this.renderer.domElement);
    }

    saveCanvasImage(canvas) {
        // !HTMLCanvasElement.prototype.toBlob
        // https://developer.mozilla.org/en-US/docs/Web/API/HTMLCanvasElement.toBlob
        // decode the String
        var binStr = atob(canvas.toDataURL("image/png").split(',')[1]);
        var len = binStr.length;
        var arr = new Uint8Array(len);

        for (var i = 0; i < len; i++) {
            arr[i] = binStr.charCodeAt(i);
        }

        this.saveBlob(new Blob([arr], { type: "image/png" }));


    }

    saveBlob(blob) {
        // ie
        if (window.navigator.msSaveBlob !== undefined) {
            window.navigator.msSaveBlob(blob, filename);
            //app.popup.hide();
        }
        else {
            // create object url
            if (this._canvasImageUrl) {
                URL.revokeObjectURL(this._canvasImageUrl);
            }
            this._canvasImageUrl = URL.createObjectURL(blob);
            // display a link to save the image
            var e = this.downloadButton;//document.createElement("a");           
            e.href = this._canvasImageUrl;

        }
    }

    createMeme() {
        let context = this.$canvas.getContext('2d');

        // font size of top and bottom text
        let fontSize = ((this.canvas.width + this.canvas.height) / 2) * 4 / 100;
        context.font = `${fontSize}pt sans-serif`;
        context.textAlign = 'center';
        context.textBaseline = 'top';

        /**
         * Stroke Text Style
         */
        context.lineWidth = fontSize / 5;
        context.strokeStyle = 'black';
        /**
         * Fill Text Style
         */
        context.fillStyle = 'white';
        // Fix lines over M
        context.lineJoin = 'round';

        // get he value of the top text an dbottom text from the input fields
        let topText = this.$topTextInput.value.toUpperCase();
        let bottomText = this.$bottomTextInput.value.toUpperCase();

        // Top Text: first parameter text, second and thir parameters contain location where the text should start rendering
        context.strokeText(topText, this.canvas.width / 2, this.canvas.height * (5 / 100));
        context.fillText(topText, this.canvas.width / 2, this.canvas.height * (5 / 100));

        // Bottom Text
        context.strokeText(bottomText, this.canvas.width / 2, this.canvas.height * (90 / 100));
        context.fillText(bottomText, this.canvas.width / 2, this.canvas.height * (90 / 100));
    }

}

let container = document.getElementById("webgl");
let app = new Application(container);
app.build();
